#include "stdafx.h"
#include "DateExtractor.h"
#include "FileUtils.h"
#include <string>


TDateExtractor::TDateExtractor(void)
{
	DebugOutputSucc = true;
	DebugOutputFail = true;
}


TDateExtractor::~TDateExtractor(void)
{
}

void TDateExtractor::DecodeExtractedDates(const TStr& EncodedDates, TVec<TExtractedDate>& ExtractedDates)
{
	PJsonVal ArrayJson = TJsonVal::GetValFromStr(EncodedDates);
	if (!ArrayJson->IsArr())
		return;
	int Len = ArrayJson->GetArrVals();
	for (int N=0; N < Len; N++) {
		PJsonVal DateJson = ArrayJson->GetArrVal(N);
		bool Amb = DateJson->GetObjBool("Amb", true);
		if (DateJson->IsObjKey("Date")) {
			ExtractedDates.Add(TExtractedDate(DateJson->GetObjStr("Date"), "", Amb));
		}
		else if (DateJson->IsObjKey("DateStart") && DateJson->IsObjKey("DateEnd")) {
			ExtractedDates.Add(TExtractedDate(DateJson->GetObjStr("DateStart"), DateJson->GetObjStr("DateEnd"), "", Amb));
		}
	}
}

TStr TDateExtractor::EncodeExtractedDates(const TVec<TExtractedDate>& ExtractedDates)
{
	if (ExtractedDates.Len() == 0)
		return "";
	PJsonVal DateArr = TJsonVal::NewArr();
	for (int N=0; N < ExtractedDates.Len(); N++) {
		TExtractedDate Date = ExtractedDates[N];
		PJsonVal DateJson = TJsonVal::NewObj("Amb", Date.Ambiguous);
		if (Date.IsDateRange) {
			DateJson->AddToObj("DateStart", Date.DateStart);
			DateJson->AddToObj("DateEnd", Date.DateEnd);
		}
		else {
			DateJson->AddToObj("Date", Date.Date);
		}
		DateArr->AddToArr(DateJson);
	}
	return TJsonVal::GetStrFromVal(DateArr);
}


void TDateExtractor::ExtractDates(const TStr& Text, const ETextLang& TextLang, TVec<TExtractedDate>& ExtractedDates)
{
	if (TextLang == LangEng)
		ExtractDatesEng(Text, ExtractedDates);
}

void TDateExtractor::ExtractDatesEng(const TStr& Text, TVec<TExtractedDate>& ExtractedDates)
{
	TVec<TIntPr> SeenMatchesV;
	const StringPiece SPText(Text.CStr());
	RE2::Options ReOptions;
	ReOptions.set_case_sensitive(false);
	ReOptions.set_utf8(false);

	map<string, int> Groups;
	int off;
	StringPiece Matches[10];
	//
	// extract date ranges (e.g. Jan. 14-16)
	//

	// date range 
	// match: jan 1. - 5. | Jan. 5 and 6 | February 6. to 12.
	RE2 ReGr1("\\b(?P<month1>jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[a-z]*\\.?\\s+(?P<day1>\\d{1,2})\\.?(?:\\s+to\\s+|\\s*-\\s*|\\s+and\\s+)(?P<day2>\\d{1,2})\\.?(?:[\\s*,]+(?P<year>\\d{2}|\\d{4}))?\\b", ReOptions);
    Groups = ReGr1.NamedCapturingGroups();
    off = 0;
	while(ReGr1.Match(SPText, off, SPText.size(), RE2::UNANCHORED, Matches, 10)) {
		AddDateRange(SPText, LangEng, &ReGr1, Matches, Groups, ExtractedDates, SeenMatchesV);
        off = Matches[0].end() - SPText.begin();
    } 

	// date range jan 1. to feb 4 2010
	RE2 ReGr2("\\b(?P<month1>jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[a-z]*\\.?\\s+(?P<day1>\\d{1,2})\\.?(?:\\s+to\\s+|\\s*-\\s*)(?P<month2>jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[a-z]*\\.?\\s+(?P<day2>\\d{1,2})\\.?(?:[\\s*,]+(?P<year>\\d{2}|\\d{4}))?\\b", ReOptions);
    Groups = ReGr2.NamedCapturingGroups();
    off = 0;
	while(ReGr2.Match(SPText, off, SPText.size(), RE2::UNANCHORED, Matches, 10)) {
		AddDateRange(SPText, LangEng, &ReGr2, Matches, Groups, ExtractedDates, SeenMatchesV);
        off = Matches[0].end() - SPText.begin();
    } 

	//
	// extract date ranges (e.g. Jan. 14-16)
	//
	// recognize: 2013-3-14 | 2011/3/14 | 
	// don't match: 2013-3/14
	// find all numeric occurences of dates: dddd[-/ ]dd[.-/ ]dd[-/ ]. 
	RE2 Re1("\\b(?P<year>\\d{4})(?:\\s*[-\\/]\\s*)(?P<month>\\d{1,2})(?:\\s*[-\\/]\\s*)(?P<day>\\d{1,2})\\b");
    Groups = Re1.NamedCapturingGroups();
    off = 0;
	while(Re1.Match(SPText, off, SPText.size(), RE2::UNANCHORED, Matches, 10)) {
		AddDate(SPText, LangEng, &Re1, Matches, Groups, ExtractedDates, SeenMatchesV);
        off = Matches[0].end() - SPText.begin();
    } 
	
	// find all numeric occurences of dates: dd[.-/ ]dd[.-/ ](dddd)?. 
	// recognize: 3/14/2011 | 3/14/11 | 03/14/11 | 14/03/2013 | 14/03/11 | 03/14/11 
	// ignore: 3/14 | 3.14 |	
	RE2 Re2("\\b(?P<day>\\d{1,2})(?:\\s*[-\\/\\.]\\s*)(?P<month>\\d{1,2})(?:\\s*[-\\/\\.]\\s*(?P<year>\\d{2}|\\d{4}))\\b");
    Groups = Re2.NamedCapturingGroups();
    off = 0;
	while(Re2.Match(SPText, off, SPText.size(), RE2::UNANCHORED, Matches, 10)) {
		AddDate(SPText, LangEng, &Re2, Matches, Groups, ExtractedDates, SeenMatchesV);
        off = Matches[0].end() - SPText.begin();
    }  

	// recognize: 14. 1. | 14.3. | 13.2.
	// ignore: 13.3 | 13/3/2013 | 14. 1. 2013 (covered before)
	RE2 Re2_1("\\b(?P<day>\\d{1,2})\\.\\s?(?P<month>\\d{1,2})\\.(?P<year>)");
    Groups = Re2_1.NamedCapturingGroups();
    off = 0;
	while(Re2_1.Match(SPText, off, SPText.size(), RE2::UNANCHORED, Matches, 10)) {
		AddDate(SPText, LangEng, &Re2_1, Matches, Groups, ExtractedDates, SeenMatchesV);
        off = Matches[0].end() - SPText.begin();
    }  
		
	// find all numeric occurences of dates: day month(in words) (dddd)?
	// match: 22nd September ,2006 | 22nd September 2006 | 22 September 2006 | 2nd February 2007 |  18 October 
	RE2 Re3("\\b(?P<day>\\d{1,2})(?:\\s*(\\.|st|nd|rd|th)?\\s*)(?P<month>jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[a-z]*\\.?\\s*,?\\s*(?P<year>\\d{2}|\\d{4})?\\b", ReOptions);
    Groups = Re3.NamedCapturingGroups();
    off = 0;
	while(Re3.Match(SPText, off, SPText.size(), RE2::UNANCHORED, Matches, 10)) {
		AddDate(SPText, LangEng, &Re3, Matches, Groups, ExtractedDates, SeenMatchesV);
        off = Matches[0].end() - SPText.begin();
    } 

	// find all occurences of dates: month(in words) day (dddd)?
	RE2 Re4("\\b(?P<month>jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[a-z]*\\.?\\s*(the|-)?\\s*(?P<day>\\d{1,2})(?:\\.|st|nd|rd|th)?(?:[\\s*,]+(?P<year>\\d{2}|\\d{4}))?\\b", ReOptions);
    Groups = Re4.NamedCapturingGroups();
    off = 0;
	while(Re4.Match(SPText, off, SPText.size(), RE2::UNANCHORED, Matches, 10)) {
		AddDate(SPText, LangEng, &Re4, Matches, Groups, ExtractedDates, SeenMatchesV);
        off = Matches[0].end() - SPText.begin();
    } 
	
	// find all occurences of dates: month(in words) dddd
	// here we don't want to match Jan. 12 (which would mean Jan 2012) - year has to be 4 digits
	RE2 Re5("(?P<day>)\\b(?P<month>jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[a-z]*\\.?\\s*(?P<year>\\d{4})\\b", ReOptions);
    Groups = Re5.NamedCapturingGroups();
    off = 0;
	while(Re5.Match(SPText, off, SPText.size(), RE2::UNANCHORED, Matches, 10)) {
		AddDate(SPText, LangEng, &Re5, Matches, Groups, ExtractedDates, SeenMatchesV);
        off = Matches[0].end() - SPText.begin();
    } 
	int i=0;
}

void TDateExtractor::AddDate(const StringPiece& SPText, const ETextLang& TextLang, re2::RE2* RegExpr, StringPiece* Matches, map<string, int>& Groups, TVec<TExtractedDate>& ExtractedDates, TVec<TIntPr>& SeenMatchesV)
{
	// make sure we haven't match this date already with some previous regex
	// regex expressions are ordered from most to least specific so we don't want to match multiple times cases such as 15 August 2013
	int MatchStart = (int) (Matches[0].begin() - SPText.begin());
	int MatchEnd = (int) (Matches[0].end() - SPText.begin());
	for (int SeenN = 0; SeenN < SeenMatchesV.Len(); SeenN++) {
		if (SeenMatchesV[SeenN].Val1 <= MatchStart && MatchStart <= SeenMatchesV[SeenN].Val2) {
			if (DebugOutputFail)
				FileUtils::LogToFileFmt("skipping match '%s' since already seen", Matches[0].ToString().c_str());
			return;
		}
	}
	TStr Day = Matches[Groups["day"]].ToString().c_str();
	TStr Month = Matches[Groups["month"]].ToString().c_str();
	TStr Year = Matches[Groups["year"]].ToString().c_str();
	bool Ambiguous = false;
	
	if (Day != "" && !Day.IsInt() && !Month.IsInt()) return;
	if (!Day.IsInt() && Month.IsInt()) { TStr Tmp = Day; Day = Month; Month = Tmp; }
	// check if we have valid values for month and day and if we need to switch month and day due to their values
	if (Month.IsInt() && Day.IsInt()) {
		int M = Month.GetInt(-1);
		int D = Day.GetInt(-1);
		Ambiguous = D <= 12 && M <= 12;
		if (M > 12 && D <= 12) { 
			int T = D; D = M; M = T; 
			TStr Tmp = Day; Day =Month; Month = Tmp;
		}
	}
	if (!Month.IsInt())
		Month = GetMonthAsInt(TextLang, Month);
	
	// we treat as valid values only "" and range (-2000, 3000)
	if (Year != "" && (!Year.IsInt() || Year.GetInt(0) < -2000 || Year.GetInt(0) > 3000)) {
		if (DebugOutputFail)
			FileUtils::LogToFileFmt("skipping invalid date match '%s'", Matches[0].ToString().c_str());
		return;
	}
	if (Day != "" && (!Day.IsInt() || Day.GetInt(0) > 31 || Day.GetInt(0) < 1)) {
		if (DebugOutputFail)
			FileUtils::LogToFileFmt("skipping invalid date match '%s'", Matches[0].ToString().c_str());
		return;		// invalid value for day found - exit
	}
	if (!Month.IsInt() || Month.GetInt(0) < 1 || Month.GetInt(0) > 12) {
		if (DebugOutputFail)
			FileUtils::LogToFileFmt("skipping invalid date match '%s'", Matches[0].ToString().c_str());
		return;		// invalid value for month found - exit
	}

	int StartPrint = max(0, MatchStart-10);
	int EndPrint = min(SPText.length() - StartPrint, MatchEnd-MatchStart + 20);
	TStr PrintOrigContext = SPText.substr(StartPrint, EndPrint).ToString().c_str();
	TStr OriginalDate = Matches[0].ToString().c_str();
	if (Year.IsInt() && Year.Len() == 2) 
		Year = "20" + Year;
	if (Day != "" && Day.Len() < 2)
		Day = "0" + Day;
	TStr NormalizedDate = TStr::Fmt("%s-%02d-%s", Year.CStr(), Month.GetInt(0), Day.CStr());
	if (DebugOutputSucc)
		FileUtils::LogToFileFmt("found date: %s in text ...%s...", NormalizedDate.CStr(), PrintOrigContext.CStr());
	ExtractedDates.Add(TExtractedDate(NormalizedDate, OriginalDate, Ambiguous));
	SeenMatchesV.Add(TIntPr(MatchStart, MatchEnd));
}

void TDateExtractor::AddDateRange(const StringPiece& SPText, const ETextLang& TextLang, re2::RE2* RegExpr, StringPiece* Matches, map<string, int>& Groups, TVec<TExtractedDate>& ExtractedDates, TVec<TIntPr>& SeenMatchesV)
{
	int MatchStart = (int) (Matches[0].begin() - SPText.begin());
	int MatchEnd = (int) (Matches[0].end() - SPText.begin());
	for (int SeenN = 0; SeenN < SeenMatchesV.Len(); SeenN++) {
		if (SeenMatchesV[SeenN].Val1 <= MatchStart && MatchStart <= SeenMatchesV[SeenN].Val2) {
			if (DebugOutputFail)
				FileUtils::LogToFileFmt("skipping match '%s' since already seen", Matches[0].ToString().c_str());
			return;
		}
	}
	TStr Day1 = Matches[Groups["day1"]].ToString().c_str();
	TStr Day2 = Matches[Groups["day2"]].ToString().c_str();
	TStr Month1 = Matches[Groups["month1"]].ToString().c_str();
	TStr Month2 = Matches[Groups["month2"]].ToString().c_str();
	TStr Year = Matches[Groups["year"]].ToString().c_str();
	
	if (!Day1.IsInt()  || !Day2.IsInt()) 
		return;
	if (!Month1.IsInt())
		Month1 = GetMonthAsInt(TextLang, Month1);
	if (Month2 == "")
		Month2 = Month1;
	else if (!Month2.IsInt())
		Month2 = GetMonthAsInt(TextLang, Month2);
	
	// we treat as valid values only "" and range (-2000, 3000)
	if (Year != "" && (!Year.IsInt() || Year.GetInt(0) < -2000 || Year.GetInt(0) > 3000)) {
		if (DebugOutputFail)
			FileUtils::LogToFileFmt("skipping invalid date range match '%s'", Matches[0].ToString().c_str());
		return;
	}
	if (!Day1.IsInt() || Day1.GetInt(0) > 31 || Day1.GetInt(0) < 1 || !Day2.IsInt() || Day2.GetInt(0) > 31 || Day2.GetInt(0) < 1) {
		if (DebugOutputFail)
			FileUtils::LogToFileFmt("skipping invalid date range match '%s'", Matches[0].ToString().c_str());
		return;		// invalid value for day found - exit
	}
	if (!Month1.IsInt() || Month1.GetInt(0) < 1 || Month1.GetInt(0) > 12) {
		if (DebugOutputFail)
			FileUtils::LogToFileFmt("skipping invalid date range match '%s'", Matches[0].ToString().c_str());
		return;		// invalid value for month found - exit
	}
	if (!Month2.IsInt() || Month2.GetInt(0) < 1 || Month2.GetInt(0) > 12) {
		if (DebugOutputFail)
			FileUtils::LogToFileFmt("skipping invalid date range match '%s'", Matches[0].ToString().c_str());
		return;		// invalid value for month found - exit
	}

	int StartPrint = max(0, MatchStart-10);
	int EndPrint = min(SPText.length() - StartPrint, MatchEnd-MatchStart + 20);
	TStr PrintOrigContext = SPText.substr(StartPrint, EndPrint).ToString().c_str();
	TStr OriginalDate = Matches[0].ToString().c_str();
	if (Year.IsInt() && Year.Len() == 2) 
		Year = "20" + Year;
	TStr NormalizedDateStart = TStr::Fmt("(%s-%02d-%02d)", Year.CStr(), Month1.GetInt(0), Day1.GetInt(0));
	TStr NormalizedDateEnd = TStr::Fmt("(%s-%02d-%02d)", Year.CStr(), Month2.GetInt(0), Day2.GetInt(0));
	if (DebugOutputSucc)
		FileUtils::LogToFileFmt("found date range: (%s, %s) in text ...%s...", NormalizedDateStart.CStr(), NormalizedDateEnd.CStr(), PrintOrigContext.CStr());
	ExtractedDates.Add(TExtractedDate(NormalizedDateStart, NormalizedDateEnd, OriginalDate, false));
	SeenMatchesV.Add(TIntPr(MatchStart, MatchEnd));
}

TStr TDateExtractor::GetMonthAsInt(const ETextLang& TextLang, const TStr& Text)
{
	if (Text == "")
		return "";
	const TStr Month = Text.GetLc();
	if (TextLang == LangEng) {
		if (Month.SearchStr("jan") >= 0) return "1";
		else if (Month.SearchStr("feb") >= 0) return "2";
		else if (Month.SearchStr("mar") >= 0) return "3";
		else if (Month.SearchStr("apr") >= 0) return "4";
		else if (Month.SearchStr("may") >= 0) return "5";
		else if (Month.SearchStr("jun") >= 0) return "6";
		else if (Month.SearchStr("jul") >= 0) return "7";
		else if (Month.SearchStr("aug") >= 0) return "8";
		else if (Month.SearchStr("sep") >= 0) return "9";
		else if (Month.SearchStr("oct") >= 0) return "10";
		else if (Month.SearchStr("nov") >= 0) return "11";
		else if (Month.SearchStr("dec") >= 0) return "12";
		else return ""; 	// invalid value for month found - exit
	}
	else return "";
}


void TDateExtractor::Test()
{
	TStr EngText =  "Jan. 11-15 | Feb 4. - 12. | "
					"Jan. 11- feb 15 | May 4 - Jun 12. | "
					"3/14 | 3/14/2011 | 3/14/11 | 03/14/11 | 14/03/2013 | 14/03/11 | 03/14/11 | "
					"14. 1. | 14.3. | 13.2. | "
					"2013-3-14 | 2011/3/14 | "
					"22nd September ,2006 | 22nd September 2006 | 22 September 2006 | 2nd February 2007 |  18 October | 18 October, 2009 | 14 March 2011 | 18th November, 2009 (OR 18 October, 2009) | 4th October, 2005 (OR 4 October, 2005) | "
					"March 14, 2011 |  March 14th | March 14 | September 22 2006 (without mma) | August 15, 1990. | September 22, 2006 | September 22nd, 2006 | September 22, 2006 | March 14th, 2011 | April-1 | "
					"Jan. 2012 | Feb 2013 | "
				"March the Fourteenth | A the Fourteenth of March, 2011	March the Fourteenth, 2011 | B	14th March 2011 | "
				"C	 | D	14/3/2011	3/14/2011 | E	14/3/11	3/14/11 | "
				" |  30.3.2005 | 19/2/2002 | 30-2-1999 | 26 May 1979  | 30 October 2003 | 18 October, 2003 | 29 January, 2008 | My son was born on 18 October, 2003. April-1";
	TVec<TExtractedDate> Dates;
	ExtractDates(EngText, LangEng, Dates);
}

void TDateExtractor::RETest()
{
	int i;
	std::string s;
	bool ok = RE2::FullMatch("ruby:1234", "(\\w+):(\\d+)", &s, &i);
	ok = (s == "ruby");
	ok = (i == 1234);

	RE2 re("(?P<N1>is) (?P<N2>\\d+)|(?P<N3>car)|(object.*movie)");
    re2::StringPiece sp = "car is 12 the animation creation with the object as cars, its a very fine movie";
    int off = 0;
    re2::StringPiece matches[5];
    while(re.Match(sp, off, sp.size(), RE2::UNANCHORED, matches, 5)) {
		map<string, int> map = re.NamedCapturingGroups();
		int ind = map["N3"];
		re2::StringPiece carMatch = matches[ind];
		re2::StringPiece wholeMatch = matches[0];
        printf("match at %d,%d: %s\n", (int)(wholeMatch.begin() - sp.begin()), (int)(wholeMatch.end() - sp.begin()), wholeMatch.as_string().c_str());
        off = wholeMatch.end() - sp.begin();
    }

	//RE2::Arg Month, Day;
	std::string Month, Day, X;
	//RE2::Arg Month, Day, X;
	ok = RE2::PartialMatch("test March 14 20", "(Jan|(March|April)) (\\d+)(th)?", &Month, &Day, &X);

	// Fails: "ruby" cannot be parsed as an integer.
	ok = !RE2::FullMatch("ruby", "(.+)", &i);

	// Success; does not extract the number.
	ok = RE2::FullMatch("ruby:1234", "(\\w+):(\\d+)", &s);

	// Success; skips NULL argument.
	ok = RE2::FullMatch("ruby:1234", "(\\w+):(\\d+)", (void*)NULL, &i);

	// Fails: integer overflow keeps value from being stored in i.
	ok = !RE2::FullMatch("ruby:123456789123", "(\\w+):(\\d+)", &s, &i);

	//std::string s ("Bojan Jan., 10th has a submarine as a subsequence");
	//std::smatch m;
	//std::regex monthsLong("(\\b)(january|february|march|april|may|june|july|august|september|october|november|december)(\\b)",  std::regex_constants::ECMAScript | std::regex_constants::icase );
	//std::regex monthsShort("(\\b)(jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[.]?(\\b)",  std::regex_constants::ECMAScript | std::regex_constants::icase );
	//std::regex months("(\\b)((january|february|march|april|may|june|july|august|september|october|november|december)|(jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)[.]?)(\\b)",  std::regex_constants::ECMAScript | std::regex_constants::icase );

	//std::cout << "Target sequence: " << s << std::endl;
	//std::cout << "The following matches and submatches were found:" << std::endl;

	//while (std::regex_search (s, m, months)) {
	//	int len = m.length();
	//	std::string x = m[1];
	//	/*for (auto x:m) 
	//		std::cout << x << " ";*/
	//	std::cout << std::endl;
	//	s = m.suffix().str();
	//}
}