#pragma once

#include "base.h"
#include "re2/re2.h"

using std::string;
using std::map;
using re2::StringPiece;

//typedef TTriple<TStr, TStr, TBool> TExtractedDate;		// contains the detected string, the normalized string in the form (YYYY)?-MM-DD, ambiguity status (is it possible that DD-MM is switched with MM-DD)

class TExtractedDate 
{
public:
	TStr DetectedDate;
	TStr Date;
	TStr DateStart;
	TStr DateEnd;
	bool Ambiguous;
	bool IsDateRange;

	TExtractedDate() { }

	TExtractedDate(TStr _Date, TStr _DetectedDate, bool _Ambiguous) 
	{
		Date = _Date;
		DetectedDate = _DetectedDate;
		Ambiguous = _Ambiguous;
		IsDateRange = false;
	}

	TExtractedDate(TStr _DateStart, TStr _DateEnd, TStr _DetectedDate, bool _Ambiguous) 
	{
		DateStart = _DateStart;
		DateEnd = _DateEnd;
		DetectedDate = _DetectedDate;
		Ambiguous = _Ambiguous;
		IsDateRange = true;
	}

};

class TDateExtractor
{
public:
	enum ETextLang { LangEng, LangSpa, LangDeu, LangZho, LangSlv };
	
	TDateExtractor();
	~TDateExtractor();

	void ExtractDates(const TStr& Text, const ETextLang& TextLang, TVec<TExtractedDate>& ExtractedDates);
	
	void ExtractDatesEng(const TStr& Text, TVec<TExtractedDate>& ExtractedDates);
	void AddDate(const StringPiece& SPText, const ETextLang& TextLang, re2::RE2* RegExpr, StringPiece* Matches, map<string, int>& Groups, TVec<TExtractedDate>& ExtractedDates, TVec<TIntPr>& SeenMatchesV);
	void AddDateRange(const StringPiece& SPText, const ETextLang& TextLang, re2::RE2* RegExpr, StringPiece* Matches, map<string, int>& Groups, TVec<TExtractedDate>& ExtractedDates, TVec<TIntPr>& SeenMatchesV);
	
	TStr GetMonthAsInt(const ETextLang& TextLang, const TStr& Text);
	
	TStr EncodeExtractedDates(const TVec<TExtractedDate>& ExtractedDates);
	void DecodeExtractedDates(const TStr& EncodedDates, TVec<TExtractedDate>& ExtractedDates);	

	void Test();
	void RETest();

	bool DebugOutputSucc;		// print info when we find a match that we can convert to a date
	bool DebugOutputFail;		// print info when we find a match that we cannot convert correctly to date
};

