#pragma once

#include "base.h"
#include "re2/re2.h"

using std::string;
using std::map;
using re2::StringPiece;

//typedef TTriple<TStr, TStr, TBool> TExtractedDate;		// contains the detected string, the normalized string in the form (YYYY)?-MM-DD, ambiguity status (is it possible that DD-MM is switched with MM-DD)

class TExtractedDate 
{
private:
	bool BIsDateRange;

public:
	TStr DetectedDate;
	TStr Date;
	TStr DateEnd;
	bool Ambiguous;
	int Freq;

	TExtractedDate() { }

	TExtractedDate(TStr _Date, TStr _DetectedDate, bool _Ambiguous) 
	{
		Date = _Date;
		DetectedDate = _DetectedDate;
		Ambiguous = _Ambiguous;
		BIsDateRange = false;
		Freq = 1;
	}

	TExtractedDate(TStr _DateStart, TStr _DateEnd, TStr _DetectedDate, bool _Ambiguous) 
	{
		Date = _DateStart;
		DateEnd = _DateEnd;
		DetectedDate = _DetectedDate;
		Ambiguous = _Ambiguous;
		BIsDateRange = true;
		Freq = 1;
	}

	TExtractedDate(PJsonVal DateJson);

	bool IsDateRange() { return BIsDateRange; }
	void IncreaseFreq(const int& _Freq = 1) { Freq += _Freq; }
	void SetFreq(const int _Freq) { Freq = _Freq; }
	PJsonVal GetJson(const bool& IncludeDetectedDateStr = false);
};

class TExtractedDateColl
{
public:
	TVec<TExtractedDate> ExtractedDateV;

	TExtractedDateColl() { }
	TExtractedDateColl(const TStr& EncodedDateColl);

	void AddExtractedDate(const TExtractedDate& Date);
	void AddColl(const TExtractedDateColl& Coll);

	void SortByFreq(const bool& AscSort=true);
	void ImputeMissingYears(PNotify Notify = NULL);

	//void DecodeExtractedDates(const TStr& EncodedDateColl);
	TStr EncodeExtractedDates(const int& MaxCount = -1);
	PJsonVal EncodeExtractedDatesAsJson(const int& MaxCount = -1);
	
	int Len() const { return ExtractedDateV.Len(); } ;
	TExtractedDate GetExtractedDate(const int& DateN) const { return ExtractedDateV[DateN]; }
};


class TDateExtractor
{
public:
	//enum ETextLang { LangEng, LangSpa, LangDeu, LangZho, LangSlv };
	
	THash<TStr, TStrV> DateRangeH;
	THash<TStr, TStrV> DateH;

	TDateExtractor();
	~TDateExtractor();

	void SetRegEx(const TStr& TextLang, const TStrV& DateRangeV, const TStrV DateV);
	void AddRegExDate(const TStr& TextLang, const TStr& Date);
	void AddRegExDateRange(const TStr& TextLang, const TStr& DateRange);
	
	//void ExtractDates(const TStr& Text, const ETextLang& TextLang, TExtractedDateColl& ExtractedDateColl);
	void ExtractDates(const TStr& Text, const TStr& TextLang, TExtractedDateColl& ExtractedDateColl);
		
	void AddIndividualDates(const StringPiece SPText, const char* RegEx, const RE2::Options& ReOptions, const TStr& TextLang, TExtractedDateColl& ExtractedDateColl, TVec<TIntPr>& SeenMatchesV);
	void AddDateRanges(const StringPiece SPText, const char* RegEx, const RE2::Options& ReOptions, const TStr& TextLang, TExtractedDateColl& ExtractedDateColl, TVec<TIntPr>& SeenMatchesV);
	
	TStr GetMonthAsInt(const TStr& TextLang, const TStr& Text);

	void NotifyInfo(const char *FmtStr, ...);
	void NotifyErr(const char *FmtStr, ...);
	void Notify(const TNotifyType& Type, const char *FmtStr, va_list argptr);
	PNotify NotifyInst;
	char NotifyBuf[10*1024];

	void Test();
	void RETest();

	static TStr EncodeExtractedDates(const TExtractedDateColl& ExtractedDates);
	static void DecodeExtractedDates(const TStr& EncodedDates, TExtractedDateColl& ExtractedDates);	
};

