#include "FileUtils.h"

//////////////////////////////////////
// File-Notify
TFileNotifyNoTime::TFileNotifyNoTime(const TStr& _FileName) : FileName(_FileName) { 
	File = TFOut::New(_FileName, true);
}

void TFileNotifyNoTime::OnStatus(const TStr& MsgStr) {
	File->PutStrFmt("%s\n", 
		MsgStr.CStr());
	File->Flush();
}

void TFileNotifyNoTime::OnNotify(const TNotifyType& Type, const TStr& MsgStr){
	TStr TypeStr = "";
	if (Type==ntInfo) TypeStr = "INFO";
	else if (Type == ntErr) TypeStr = "ERROR";
	else if (Type == ntWarn) TypeStr = "WARNING";
	else if (Type == ntStat) TypeStr = "STAT";

	TTm NowTm = TTm::GetCurLocTm();
	
	File->PutStrFmt("%s: %s\n", 
		TypeStr.CStr(),
		MsgStr.CStr());
	File->Flush();
}

TFileNotifyNoTime* TFileUtils::FileNotify = NULL;

TStr TFileUtils::ReadTextFile(const TStr& FileName, const TStr& DefVal)
{
	try 
	{
		bool Opened;
		TFIn FIn(FileName, Opened, true);
		TChA Content;
		while (true) {
			TStr Line;
			bool ok = FIn.GetNextLn(Line);
			if (!ok)
				break;
			Content += Line + "\n";
		}
		return Content;
	}
	catch (PExcept E)
	{
		return DefVal;
	}
}

// skip the byte-order-mask if it exists at the beginning of the file
PSIn TFileUtils::OpenUnicodeFile(const TStr& FName)
{
	PSIn File = TFIn::New(FName);
	char* Prefix = new char[3];
	memset(Prefix, 0, 3);
	File->GetBf(Prefix, 3);
	if (Prefix[0] != (char)0xEF || Prefix[1] !=(char)0xBB || Prefix[2] !=(char)0xBF)
		File->Reset();
	delete Prefix;
	return File;
}

// lines starting with CommentPrefix are not read
TStr TFileUtils::ReadTextFileWithoutComments(const TStr& FileName, const TStr& DefVal, const TStr& CommentPrefix)
{
	try 
	{
		TFIn FIn(FileName);
		TChA Content;
		while (true) {
			TStr Line;
			bool ok = FIn.GetNextLn(Line);
			if (!ok)
				break;
			if (CommentPrefix != "" && Line.SearchStr(CommentPrefix) == 0)
				continue;
			Content += Line;
		}
		return Content;
	}
	catch (PExcept E)
	{
		return DefVal;
	}
}
