﻿// concepts with these prefixes will be shown in the entities list
var entityPrefixes = { "pm_org": true, "pm_person": true, "pm_location": true, "pm_cf": true };
// concepts with these prefixes will be shown in the keywords list
var keywordPrefixes = { "pm_category": true, "pm_keyword": true };
//var catToImage = { "pm_category": "topic", "pm_keyword": "topic", "pm_location": "location", "pm_person": "person", "pm_org": "organization", "pm_cf": "folder" }

// what the category changed - anything edited, removed, ...
var categoryChanged = false;
// this contains the object for the category that we are currently displaying (original, not edited)
var loadedCategory = null;

// this is the version of the category used for showing the articles in the bottom part
var editedCategory = null;

var articleCols = 4;
var articlePage = 0;
var articlePageCount = 10 * articleCols;

var defaultConceptWgt = 10;
var defaultSourceWgt = 100;
var defaultLocationWgt = 100;

// load the list of all concept folders (categories)
function loadAllCategories()
{
    editedCategory = null;
    loadedCategory = null;
    $.get("/admin/json/listFolders", function (data) {
        var catList = jQuery.parseJSON(data);
        var output = "";
        $.each(catList, function (index, element) {
            output += '<li id="' + encodeURIComponent(element.conceptFolderUri) + '"><a href="#" onclick="displayCategory(\'' + encodeURIComponent(element.conceptFolderUri) + '\');">' + element.folderName + '</a></li>';
        });
        $("#wrapper #main #sidebar ul.categories").html(output);
        if (catList.length > 0) {
            displayCategory(encodeURIComponent(catList[0].conceptFolderUri));
        }
    });
}

// based on the user input return string telling what to do if content of the category changed:
// "noChanges" - no changes were done, no saving is needed
// "save" - save the changes to the category
// "discard" - discard the changes to the category
// "cancel" - the user does not want to save or discard the changes
function checkForCategoryChanges(callback)
{
    if (categoryChanged) {
        $("#dialog-saveChanges").dialog({
            buttons: [{
                text: "Save changes", click: function () {
                    $(this).dialog("close");
                    callback("save");
                }
            },
            {
                text: "Discard changes", click: function () {
                    $(this).dialog("close");
                    callback("discard");
                }
            },
            {
                text: "Cancel", click: function () {
                    $(this).dialog("close");
                    callback("cancel");
                }
            }]
        });
        $("#dialog-saveChanges").dialog("open");
    } else {
        callback("noChanges");
    }
    
}

// build html for displaying entities and keywords in the list box
function buildConceptLI(conceptUri, conceptLabel, conceptWgt)
{
    var encodedConceptUri = encodeURIComponent(conceptUri);
    var conceptType = conceptUri.split(":")[0];
    var href = (conceptUri.split(":")[1].indexOf("http") == 0) ? conceptUri.replace(conceptType + ":", "") + '" target="_blank' : "#";
    var line = '<li class="removableItem" id="' + encodedConceptUri + '">\n';
    line += '\t<strong class="name ' + conceptType + '" title="' + conceptUri + '"><a href="' + href + '">' + conceptLabel + '</a></strong>\n';
    line += '\t<input type="text" value="' + conceptWgt + '" />\n';
    line += '\t<div class="del"><a href="#">-</a></div>\n';
    line += '</li>';
    return line;
}

// build html for displaying sources in the list box
function buildSourceLI(sourceUri, sourceLabel, sourceWgt) {
    var encodedSourceUri = encodeURIComponent(sourceUri);
    var line = '<li class="removableItem" id="' + encodedSourceUri + '">\n';
    line += '\t<strong class="name"><a title="' + sourceUri + '" href="' + sourceUri + '" target="_blank">' + sourceLabel + '</a></strong>\n';
    line += '\t<div class="select-box"></div>\n';
    line += '\t<input type="text" value="' + sourceWgt + '" />\n';
    line += '\t<div class="del"><a href="#">-</a></div>\n';
    line += '</li>';
    return line;
}

// build html for displaying sources in the list box
function buildLocationLI(location) {
    var line = '<li class="removableItem" id="' + location + '">\n';
    line += '\t<strong class="name location"><a title="' + location + '" target="_blank">' + location + '</a></strong>\n';
    line += '\t<div class="select-box"></div>\n';
    
    line += '\t<div class="del"><a href="#">-</a></div>\n';
    line += '</li>';
    return line;
}


// the user clicked one of the categories (conceptFolderUri). display the info about the category and mark it as selected
function displayCategory(encodedConceptFolderUri)
{
    checkForCategoryChanges(function (whatToDo) {
        if (whatToDo == "cancel")
            return;
        else if (whatToDo == "save")
            saveCurrentCategory();

        clearInputTextBoxes();
        hideAutoSuggestions();
        $.get("/admin/json/folderJson?conceptFolderUri=" + encodedConceptFolderUri, function (data) {
            var folder = jQuery.parseJSON(data);
            loadedCategory = folder;
            editedCategory = folder;
            $("h1 #selectedCategoryTitle").html(folder["folderName"]);
            $("#sidebar ul.categories li").removeAttr("class");
            $("#sidebar ul.categories li[id='" + encodedConceptFolderUri + "'] ").attr("class", "active");

            // build text for the concepts and sources
            var entities = "";
            var keywords = "";
            var sources = "";
            var locations = "";

            if (folder["hasConceptWgts"].length == folder["hasConceptUris"].length && folder["hasConceptWgts"].length == folder["hasConceptLabels"].length && folder["hasConceptWgts"].length > 0) {
                $.each(folder["hasConceptUris"], function (index, conceptUri) {
                    var conceptLabel = folder["hasConceptLabels"][index];
                    var conceptWgt = folder["hasConceptWgts"][index];
                    var conceptType = conceptUri.split(":")[0];
                    if (entityPrefixes[conceptType])
                        entities += buildConceptLI(conceptUri, conceptLabel, conceptWgt);
                    else if (keywordPrefixes[conceptType])
                        keywords += buildConceptLI(conceptUri, conceptLabel, conceptWgt);
                });
            }
            
            if (folder.validSourceLabels != undefined && folder.validSourceWgts != undefined && folder.validSourceUris != undefined && folder.validSourceLabels.length == folder.validSourceWgts.length) {
                $.each(folder.validSourceLabels, function (index, sourceLabel) {
                    var sourceUri = folder.validSourceUris[index];
                    var sourceWgt = folder.validSourceWgts[index];
                    sources += buildSourceLI(sourceUri, sourceLabel, sourceWgt);;
                });
            }

            if (folder.validLocations != undefined) {
                $.each(folder.validLocations, function (index, location) {
                    locations += buildLocationLI(location);;
                });
            }

            $("#entityList").html(entities);
            $("#keywordList").html(keywords);
            $("#sourcesList").html(sources);
            $("#locationsList").html(locations);

            $('#popup-edit input[name="categoryName"]').val(loadedCategory.folderName);
            setSwitcherVal('#popup-edit .switchers #catVisibility', loadedCategory.visibility);
            setSwitcherVal('#popup-edit .switchers #catAutoAdd', loadedCategory.autoAddArticles == true ? "1" : "0");

            // mark category as changed if a category or source is removed
            updateKwEntSrcHandlers();

            categoryChanged = false;
            updateArticles();
        });
    });
}

// create a new category categoryName and display it in the list (don't yet change the focus since the current category might have changed)
function createNewCategory(categoryName, visibility) {
    var folder = { folderName: categoryName, visibility: visibility, autoAddArticles: true};
    var query = "/admin/do/addFolder?folder=" + encodeURIComponent(JSON.stringify(folder));
    $.get(query, function (data) {
        var newCategory = jQuery.parseJSON(data);
        if (typeof newCategory == "object") {
            var line = '<li id="' + encodeURIComponent(newCategory["conceptFolderUri"]) + '"><a href="#" onclick="displayCategory(\'' + encodeURIComponent(newCategory["conceptFolderUri"]) + '\');">' + newCategory["folderName"] + '</a></li>';
            $("#wrapper #main #sidebar ul.categories").append(line);
        }
    });
}

// take the content of the currently displayed category and save it
function saveCurrentCategory()
{
    if (categoryChanged == false)
        return;
    // update the edited category with the latest content
    updateKwEntSrc();
    var query = "/admin/do/modifyFolder?folder=" + encodeURIComponent(JSON.stringify(editedCategory));
    $.get(query, function (data) {
        // here we could check data.success. if 0 then data.statusMsg could have explanation
        categoryChanged = false;
    });
}

// udpate the delete and changed events for entities, keywords and sources
function updateKwEntSrcHandlers() {
    $("li.removableItem .del").unbind("click");
    $("li.removableItem .del").click(function () {
        $(this).parent().remove();
        categoryChanged = true;
    });

    // mark category as changed if text in any input boxes changes 
    $("li.removableItem input").unbind("change");
    $("li.removableItem input").change(function () {
        categoryChanged = true;
    });
}

// update the editedCategory with the values in entity, keyword and source lists
function updateKwEntSrc()
{
    editedCategory.hasConceptLabels = new Array();
    editedCategory.hasConceptUris = new Array();
    editedCategory.hasConceptWgts = new Array();
    editedCategory.fixedConceptUris = new Array();
    editedCategory.validSourceLabels = new Array();
    editedCategory.validSourceUris = new Array();
    editedCategory.validSourceWgts = new Array();
    editedCategory.validLocations = new Array();

    $("ul.conceptList li.removableItem").each(function () {
        var encodedConceptUri = $(this).prop('id');
        var conceptUri = decodeURIComponent(encodedConceptUri);
        var conceptLabel = $(this).find(".name a").html();
        var conceptWgtStr = $(this).find("input").val().replace(",", ".");;
        var conceptWgt = parseFloat(conceptWgtStr);
        editedCategory.hasConceptLabels.push(conceptLabel);
        editedCategory.hasConceptUris.push(conceptUri);
        editedCategory.hasConceptWgts.push(conceptWgt);
        if ($.inArray(conceptLabel, loadedCategory.fixedConceptUris) != -1)
            editedCategory.fixedConceptUris.push(conceptUri);
    });
    $("#sourcesList li.removableItem").each(function () {
        var encodedSourceUri = $(this).prop('id');
        var sourceUri = decodeURIComponent(encodedSourceUri);
        var sourceLabel = $(this).find(".name a").html();
        var sourceWgtStr = $(this).find("input").val().replace(",", ".");
        var sourceWgt = parseFloat(sourceWgtStr);
        editedCategory.validSourceLabels.push(sourceLabel);
        editedCategory.validSourceUris.push(sourceUri);
        editedCategory.validSourceWgts.push(sourceWgt);
    });
    $("#locationsList li.removableItem").each(function () {
        editedCategory.validLocations.push($(this).prop('id'));
    });
}

// use the content in all the columns, create from it folder definition and use it to get and show articles for it
function updateArticles()
{
    updateKwEntSrc();
    articlePage = 0;
    var cols = "";
    for (var i = 0; i < articleCols; i++)
        cols += '<div class="col" id="articleCol' + i + 1 + '"></div>\n';
    // clear existing articles
    $("#content .articles .articles-holder").html(cols);
    displayNextArticlePage();
}

function displayNextArticlePage() {
    if (editedCategory == null)
        return;
    // for a category with empty definition don't show any articles
    if (editedCategory.hasConceptUris.length == 0 && editedCategory.validSourceUris.length == 0)
        return;
    var firstPage = articlePage == 0;
    var searchParams = { page: articlePage, count: articlePageCount, relatedConceptsCount: firstPage ? 100 : 0, relatedSourcesCount: firstPage ? 100 : 0 };
    query = "/json/articlesForFolder?folder=" + encodeURIComponent(JSON.stringify(editedCategory)) + "&searchParams=" + encodeURIComponent(JSON.stringify(searchParams));
    $.getJSON(query, function (data) {
        if (data.results == undefined || data.results.articles == undefined)
            return;
        $.each(data.results.articles, function (i, item) {
            var article = '';
            var item_class = "news_item" + (item.illustration != '' ? " illustrated" : "");
            article += '<div class="' + item_class + '" id="' + encodeURIComponent(item.uri) + '">';
            article += '<div class="content">';
            if (item.source_title.length > 15) {
                var tText = item.source_title.substring(0, 15) + '...';
                article += '<span class="source" title="' + item.source_title + '">' + tText + '</span>';
            } else {
                article += '<span class="source">' + item.source_title + '</span>';
            }
            article += '<div class="actions"><a class="add" onclick="addArticleToFolder(this);" ></a>';
            article += '<a class="like" onclick="removeArticleFromFolder(this);"></a></div>';
            if (item.image_uri) article += '<div class="illustration"><img src="' + item.image_uri + '"/></div>';
            article += '<h3><a href="/article/' + encodeURIComponent(item.uri) + '">' + item.title + '</a></h3>';
            article += '<div class="dek">' + item.body + '</div>';
            article += '<p class="time">' + item.time + '</p>';
            article += '</div>';
            //article += '<div class="social"><ul><li class="comments"><a></a> <span>0 Notes</span></li><li class="email"><a></a></li><li class="facebook"><a></a></li><li class="twitter"><a></a></li></ul></div>';
            //article += '<div class="notes"><ul><li class="note"><ul><li class="thumb"><img src="/img/sample/icon_natalie.jpg" width="27" height="27" alt=""></li><li class="ncontent"><strong>HEATHER LEO</strong>  What an inspiring article! Can\'t wait to share this!</li><li class="clear"></li></ul></li></ul></div>';
            article += '<div class="shadow"><img src="/img/backgrounds/news_item_shadow.png" width="220" height="14" alt=""></div>';
            article += '</div>';

            $('#content .articles .articles-holder #articleCol' + (i % articleCols) + 1).append(article);
        });
        // clean previous suggestions
        if (firstPage)
            $("popup-suggestions ul.suggestions").html("");
        // add suggested keywords and entities to the dialogs
        if (data.relatedConcepts != undefined) {
            $.each(data.relatedConcepts, function (i, concept) {
                var line = buildConceptLI(concept.uri, concept.label, 10);
                line = line.replace('<div class="del"><a href="#">-</a></div>', '<a class="add-link ' + concept.type + '" href="#">+</a>');
                var conceptType = concept.uri.split(":")[0];
                if (entityPrefixes[conceptType])
                    $("#popup-entity-suggestions ul.suggestions").append(line);
                else 
                    $("#popup-keyword-suggestions ul.suggestions").append(line);
            });
        }
        // add sources
        if (data.relatedSources != undefined) {
            $.each(data.relatedSources, function (i, source) {
                var line = buildSourceLI(source.uri, source.title, 100);
                line = line.replace('<div class="del"><a href="#">-</a></div>', '<a class="add-link source" href="#">+</a>');
                $("#popup-source-suggestions ul.suggestions").append(line);
            });
        }
        if (firstPage) {
            $('.popup-suggestions li a.add-link').off("click"); // remove previous handlers
            $('.popup-suggestions li a.add-link').click(function () {
                var encodedUri = $(this).parent().prop("id");
                var label = $(this).parent().find(".name a").html();
                var uri = decodeURIComponent(encodedUri);
                var wgt = $(this).parent().find("input").val();
                addEntKwSrc($(this).prop('class'), uri, label, wgt);
                $(this).parent().remove();
            });
        }
        articlePage++;
    });
}

function addArticleToFolder(selectedItem) {
    var articleUri = decodeURIComponent($(selectedItem).parents(".news_item").prop('id'));
    var folderUri = editedCategory.conceptFolderUri;
    var wgt = 100;
    $.getJSON("/admin/do/addArticleToFolder", { folderUri: folderUri, articleUri: articleUri, articleWgt: wgt }, function (result) {
        if (!result.success)
            alert(result.statusMsg);
        // todo: add some visual notification if all went ok. show result.statusMsg value
    });
}

function removeArticleFromFolder(selectedItem) {
    var articleUri = decodeURIComponent($(selectedItem).parents(".news_item").prop('id'));
    var folderUri = editedCategory.conceptFolderUri;
    $.getJSON("/admin/do/removeArticleFromFolder", { folderUri: folderUri, articleUri: articleUri }, function (result) {
        if (!result.success)
            alert(result.statusMsg);
        // todo: add some visual notification if all went ok. show result.statusMsg value
    });
}

// //////////////
// autosuggest functionality

// get and show suggestions for entities, keywords or sources
function showSuggestions(methodName, inputbox, prefix) {
    if (prefix == "") {
        var p = inputbox.parent().children('.suggestions');
        $(p).hide();
        return;
    }

    query = "/json/" + methodName + "?prefix=" + prefix;
    $.get(query, function (data) {
        var suggList = jQuery.parseJSON(data);
        var output = '';
        if (methodName == "findSources") {
            $.each(suggList, function(index, element) {
                output += '<li id="' + encodeURIComponent(element.uri) + '" class="source" title="' + element.uri + '">' + element.title + '</li>';
            });
        }
        else if (methodName == "findLocations") {
            $.each(suggList, function (index, element) {
                output += '<li id="' + encodeURIComponent(element) + '" class="location" title="' + element + '">' + element + '</li>';
            });
        }
        else {
            $.each(suggList, function (index, element) {
                output += '<li id="' + encodeURIComponent(element.uri) + '" class="' + element.type + '" title="' + element.uri + '">' + element.label + '</li>';
            });
        }
        
        inputbox.parent().children('.suggestions').html(output);
        var p = inputbox.parent().children('.suggestions');
        $(p).show();
        var pl = $(p).children('li');

        // the function that is called when we select an item from the auto-suggest list
        $(pl).click(function () {
            var encodedUri = $(this).prop('id');
            var label = $(this).html();
            var uri = decodeURIComponent(encodedUri);
            addEntKwSrc($(this).prop('class'), uri, label);
            clearInputTextBoxes();
            hideAutoSuggestions();
            $(this).parent().parent().children(".suggestions").hide();  // hide suggestions list
        });
    });
}

// add entity, keyword or source into the list
// called after clicking autosuggested items or machine generated suggestions
function addEntKwSrc(itemClass, uri, label, wgt)
{
    if (editedCategory == null) {
        alert("You need to create first a category, before you can edit it.");
        return;
    }
    if (entityPrefixes[itemClass]) {
        var line = buildConceptLI(uri, label, wgt != undefined ? wgt : defaultConceptWgt);
        $("#entityList").append(line);
    }
    else if (keywordPrefixes[itemClass]) {
        var line = buildConceptLI(uri, label, wgt != undefined ? wgt : defaultConceptWgt);
        $("#keywordList").append(line);
    }
    else if (itemClass == 'source') {
        var line = buildSourceLI(uri, label, wgt != undefined ? wgt : defaultSourceWgt);
        $("#sourcesList").append(line);
    }
    else if (itemClass == 'location') {
        var line = buildLocationLI(uri, label, wgt != undefined ? wgt : defaultLocationWgt);
        $("#locationsList").append(line);
    }
    // add event handlers for the new item
    updateKwEntSrcHandlers();
    categoryChanged = true;
}

function clearInputTextBoxes()
{
    $(".getSuggestion input").val("");
}

function hideAutoSuggestions() {
    $(".getSuggestion .suggestions").hide();
}

function getSwitcherVal(selector) {
    return $(selector + " input").val();
}

// set the value of the switcher in the interface
// selector - string used to identify the switcher div
// value - the value that should be set ("1" or "0")
function setSwitcherVal(selector, value) {
    $(selector + " input").val(value);
    $(selector).prop("class", "switcher" + ((value == 0) ? " off" : ""));
    $(selector + " div a").css("left", (value == 0) ? "0%" : "100%");
}