// create a dummy element, apply the appropriate classes,
// and then measure the element
function measureTextWidth(text, classname) {
    if (!text || text.length === 0) return 0;
    var container = d3.select('body').append('svg').attr('class', classname);
    container.append('text').attr({ x: -1000, y: -1000 }).text(text);
    var bbox = container.node().getBBox();
    container.remove();
    return bbox.width;
};


function EventCtrl($scope, $http) {
    
    $scope.allArticlesLoaded = false;
    $scope.queryParams = {};
    $scope.articleList = [];
    $scope.displayedArticleIndex = -1;
    $scope.hasData = false;
    $scope.error = undefined;
    $scope.medoidArticle = undefined;
    $scope.concepts = [];
    $scope.eventInfo = {};

    $scope.displayedEntities = [];
    $scope.displayedKeywords = [];
    $scope.displayedEntitiesLabelWidth = 0;
    $scope.displayedKeywordsLabelWidth = 0;

    $scope.updateArticlesForLang = function() {
        if ($scope.queryParams.searchParams == undefined)
            $scope.queryParams.searchParams = {};

        $scope.queryParams.searchParams.page = 0;
        $scope.allArticlesLoaded = false;
        
        $scope.pushUrl();
        
        $scope.queryParams.langs = getURLParameterByName("lang");
        $scope.queryParams.searchParams.sort = getURLParameterByName("sort");

        $scope.articleList = [];
        $scope.displayedArticleIndex = -1;

        $scope.loadNextArticlePage();
    };

    $scope.loadEventInfo = function () {
        var eventId = $("#eventIdInput").val();
        $scope.resetData();
        if (isNaN(parseFloat(eventId))) {
            $scope.error = "Please enter a valid number of the event id";
            return;
        }
        $scope.queryParams = { searchParams: { page: 0, count: 30 }, eventId: eventId };
        $.getJSON("/json/getEventInfo", { params: JSON.stringify($scope.queryParams) }, function (eventData) {
            $scope.hasData = (eventData.error == undefined);
            if (eventData.error)
                $scope.error = eventData.error;
            else {
                $scope.eventInfo = eventData.eventInfo;
                $scope.eventInfo.mainMedoidArticle = $scope.getArticleForLanguage(eventData.eventInfo.medoidArticles, $scope.queryParams.langs);
                $scope.concepts = eventData.eventInfo.concepts;
                $scope.showMoreConcepts();

                $scope.$apply();
                
                $scope.drawConcepts();
            }
        });
        $scope.updateArticlesForLang();
    };
    
    $scope.resetData = function () {
        // clear existing content
        $scope.queryParams = {};
        $scope.allArticlesLoaded = false;
        
        $scope.hasData = false;
        $scope.error = undefined;
        $scope.medoidArticle = undefined;
        $scope.displayedEntities = [];
        $scope.displayedKeywords = [];
        $scope.concepts = [];
        $scope.eventInfo = {};
    };

    $scope.loadNextArticlePage = function () {
        if ($scope.allArticlesLoaded == true)
            return;
        if ($scope.queryParams == {})
            return;
        if (!$scope.queryParams.searchParams)
            return;

        $.getJSON("/json/getEventArticles", { params: JSON.stringify($scope.queryParams) }, function (eventData) {
            if (eventData.error) {
                $scope.allArticlesLoaded = true;
                return;
            }
            if (!eventData.articles || eventData.articles.length == 0)
                $scope.allArticlesLoaded = true;
            if (eventData.articles) {
                $.each(eventData.articles, function (ind, item) {
                    $scope.articleList.push(item);
                });
                if ($scope.articleList.length == eventData.totalCount)
                    $scope.allArticlesLoaded = true;
            }
            
            // show other parameters of the event
            if ($scope.queryParams.searchParams.page == 0) {
            //    $("#event-info-details").append('<div class="row-fluid"><div class="span12"><a class="accordion-toggle" data-toggle="collapse" style="line-height: 26px;" data-target="#advEventDetails" >Toggle advanced event details</a>');
            //    $("#event-info-details").append('<div class="row-fluid" id="advEventDetails" class="accordion-body collapse"><p></p></div>');
            //    $('#advEventDetails').collapse("toggle");
            //    $('#advEventDetails').on('show', function () {
            //        $.getJSON("/json/getEventInternals", { params: JSON.stringify({ eventId: $scope.queryParams.eventId }) }, function (eventDetails) {
            //            $("#advEventDetails").html("");
            //            $.each(eventDetails.stories, function (index, story) {
            //                var out = '<div class="row-fluid"><div class="span12">';
            //                if (story.error) {
            //                    out += '<h4>' + story.error + '</h4>\n';
            //                }
            //                else {
            //                    out += '<h3>' + story.name + '</h3>\n';
            //                    out += '<p>Articles: ' + story.articleCount + ', total weight: ' + story.totalWeight + ', centroid has ' + story.centroidFeatures + ' features.</p>';
            //                    out += '<p>Variance = ' + story.variance + ', StdDev = ' + story.stDev + ', Avg cos to centroid = ' + story.avgCosToCentroid + '</p>\n';
            //                    if (story.oldestArticleTime)
            //                        out += '<p>Oldest article time: ' + story.oldestArticleTime + '</p>\n<p>Newest article time: ' + story.latestArticleTime + '</p>\n';
            //                    out += '<p>Latest addition: ' + story.latestAddition + '</p>\n';
            //                    out += '<h4>Articles:</h4>\n';
            //                    $.each(story.articles, function (i, art) {
            //                        out += '<p>cos sim: ' + art.cos + ', ' + art.title + ' (time: ' + art.time + ', time weight: ' + art.timeWeight + ', sumWgt: ' + art.timeSumWgt + '</p>\n';
            //                    });
            //                }
            //                out += '</div></div>';
            //                $("#advEventDetails").append(out);
            //            });
            //        });
            //    });
            }

            $scope.queryParams.searchParams.page++;
            // update the binding values
            $scope.$apply();
            
            // udpate tooltips
            $("[data-toggle=tooltip]").tooltip({ "html": true });
            initLightbox();
        });
    };

    $scope.getConceptTooltip = function(concept) {
        var dmoz = "";
        if (concept.label.indexOf("/") >= 0) {
            dmoz = "<br/><b>DMoz category</b>: " + concept.label;
        }
        return '<B>Importance</B>:' + concept.score + dmoz;
    };

    $scope.getConceptLabel = function(concept) {
        return concept.label.substring(concept.label.lastIndexOf("/") + 1, concept.label.length);
        //if (concept["label_" + lang] != undefined)
        //    return concept["label_" + lang];
        //return concept.label;
    };
    
    $scope.getConceptLabels = function (concepts, lang) {
        return $.map(concepts, function (item, i) {
            if (item["label_" + lang] != undefined)
                return item["label_" + lang];
            return item.label;
        });
    };

    $scope.showMoreConcepts = function () {
        var addedKeywords = 0, addedEntities = 0;
        var remaining = [];
        $.each($scope.concepts, function (index, concept) {
            if (entityTypes[concept.type] && addedEntities < 10) {
                $scope.displayedEntities.push(concept);
                addedEntities++;
            } else if (keywordTypes[concept.type] && addedKeywords < 10) {
                $scope.displayedKeywords.push(concept);
                addedKeywords++;
            } else
                remaining.push(concept);
        });
        $scope.displayedEntitiesLabelWidth = 0;
        $scope.displayedKeywordsLabelWidth = 0;
        if ($scope.displayedEntities.length > 0)
            $scope.displayedEntitiesLabelWidth = d3.max($.map($scope.displayedEntities, function (val, i) { return measureTextWidth(val.label, "text.name"); }));
        if ($scope.displayedKeywords.length > 0)
            $scope.displayedKeywordsLabelWidth = d3.max($.map($scope.displayedKeywords, function (val, i) { return measureTextWidth(val.label, "text.name"); }));
        
        $scope.concepts = remaining;
        $scope.drawConcepts();
        
    };

    $scope.drawConcepts = function () {
        var bar_height = 20,
            gap = 2;
        var lang = getURLParameterByName("lang");
        var entityLabels = $scope.getConceptLabels($scope.displayedEntities, lang);
        // ENTITIES
        var graphWidth = $("#entities").parent().width(),
            entitiesHeight = (bar_height + 2 * gap) * $scope.displayedEntities.length,
            entitiesOffset = $scope.displayedEntitiesLabelWidth + 10

        d3.select("#entities svg").remove();
        var chart = d3.select("#entities")
            .append('svg')
            .attr('class', 'chart')
            .attr('width', graphWidth)
            .attr('height', entitiesHeight);
        
        var x = d3.scale.linear()
           .domain([0, 100])
           .range([0, graphWidth - entitiesOffset]);

        var y = d3.scale.ordinal()
            .domain(d3.range(0, $scope.displayedEntities.length))
            .rangeBands([0, entitiesHeight]);
        
        // rect
        chart.selectAll("#entities rect")
            .data($scope.displayedEntities)
            .enter().append("rect")
            .attr("x", entitiesOffset)
            .attr("y", function (val, ind) { return y(ind) + gap; })
            .attr("width", function (val) { return d3.max([x(val.score), 0]); })
            .attr("height", bar_height);
        
        // scores
        chart.selectAll("#entities text.score")
            .data($scope.displayedEntities)
            .enter().append("text")
            .attr('class', 'score')
            .attr("x", function (val) { return x(val.score) + entitiesOffset; })
            .attr("y", function (val, ind) { return y(ind) + y.rangeBand() / 2; })
            .attr("dx", -5)
            .attr("dy", ".36em")
            .attr("text-anchor", "end")
            .text(function (val) { return val.score; });

        // labels
        chart.selectAll("#entities text.name")
            .data(entityLabels)
            .enter().append("text")
            .attr('class', 'name')
            .attr("x", entitiesOffset - 10)
            .attr("y", function (val, ind) { return y(ind) + y.rangeBand() / 2; })
            .attr("dy", ".36em")
            .attr("text-anchor", "end")
            .text(String);


        // KEYWORDS
        var keywordLabels = $scope.getConceptLabels($scope.displayedKeywords, lang);
        d3.select("#keywords svg").remove();
        graphWidth = $("#keywords").parent().width();
        var keywordsHeight = (bar_height + 2 * gap) * $scope.displayedKeywords.length,
            keywordsOffset = $scope.displayedKeywordsLabelWidth + 10

        chart = d3.select("#keywords")
            .append('svg')
            .attr('class', 'chart')
            .attr('width', graphWidth)
            .attr('height', keywordsHeight);

        x = d3.scale.linear()
           .domain([0, 100])
           .range([0, graphWidth - keywordsOffset]);

        y = d3.scale.ordinal()
            .domain(d3.range(0, $scope.displayedKeywords.length))
            .rangeBands([0, keywordsHeight]);

        // rect
        chart.selectAll("#keywords rect")
            .data($scope.displayedKeywords)
            .enter().append("rect")
            .attr("x", keywordsOffset)
            .attr("y", function (val, ind) { return y(ind) + gap; })
            .attr("width", function (val) { return d3.max([x(val.score), 0]); })
            .attr("height", bar_height);

        // scores
        chart.selectAll("#keywords text.score")
            .data($scope.displayedKeywords)
            .enter().append("text")
            .attr('class', 'score')
            .attr("x", function (val) { return x(val.score) + keywordsOffset; })
            .attr("y", function (val, ind) { return y(ind) + y.rangeBand() / 2; })
            .attr("dx", -5)
            .attr("dy", ".36em")
            .attr("text-anchor", "end")
            .text(function (val) { return val.score; });

        // labels
        chart.selectAll("#keywords text.name")
            .data(keywordLabels)
            .enter().append("text")
            .attr('class', 'name')
            .attr("x", keywordsOffset - 10)
            .attr("y", function (val, ind) { return y(ind) + y.rangeBand() / 2; })
            .attr("dy", ".36em")
            .attr("text-anchor", "end")
            .text(String);

    };

    $scope.getArticleDate = function (article) {
        var date = Date.parse(article.date + " " + article.time);
        return date.toString('MMM. d, HH:mm');
    };

    $scope.getDateStr = function(dateStr) {
        // todo: figure out what to do when date doesn't have a day. for now ignore the date
        if (!dateStr || dateStr.lastIndexOf('-') == dateStr.length - 1)
            return "";
        var skipYear = false;
        if (dateStr[0] == '-') {
            dateStr = "2000" + dateStr;
            skipYear = true;
        }
        var date = Date.parse(dateStr);
        if (skipYear)
            return date.toString('d MMM');
        else
            return date.toString('d MMM yyyy');
    };

    $scope.getEventDate = function () {
        if (!$scope.eventInfo || !$scope.eventInfo.primaryStory)
            return "unknown";
        var story = $scope.eventInfo.primaryStory;
        var dateStart = $scope.getDateStr(story.extractedDate);
        var dateEnd = "";
        if (story.extractedDateEnd && story.extractedDateEnd != "")
            dateEnd = $scope.getDateStr(story.extractedDateEnd);
        if (dateStart != "") {
            var ret = dateStart;
            if (dateEnd != "")
                ret += " - " + dateEnd;
            return ret;
        }
        else
            return $scope.getDateStr(story.averageDate);
    };

    $scope.getArticleCounts = function () {
        if (!$scope.eventInfo || !$scope.eventInfo.stories)
            return "";
        var articleCounts = {};
        var totCount = 0;
        $.each($scope.eventInfo.stories, function (index, story) {
            totCount += story.articleCount;
            var lang = story.language;
            articleCounts[lang] = (articleCounts[lang] != undefined) ? articleCounts[lang] + story.articleCount : story.articleCount;
        });
        var cntPerLang = $.map(langList, function (lang, ind) {
            return (articleCounts[lang] ? articleCounts[lang] : 0) + " " + lang;
        });
        return totCount + " (" + cntPerLang.join(", ") + ")";
    };

    $scope.getArticleBody = function (article) {
        var parBody = "";
        parBody += '<div class="body-text">';
        $.each(article.body.split("\n"), function (index, par) {
            parBody += "<p>" + par + "</p>";
        });
        parBody += '</div>';
        return parBody;
    };

    $scope.getArticleForLanguage = function (medoidArticles, preferredLang) {
        var articles = $.grep(medoidArticles, function (article, index) {
            return article.lang == preferredLang;
        });
        if (articles.length > 0)
            return articles[0];
        articles = $.grep(medoidArticles, function (article, index) {
            return article.lang == "eng";
        });
        if (articles.length > 0)
            return articles[0];
        return (medoidArticles.length > 0) ? medoidArticles[0] : undefined;
    };

    // /////////////////////
    // DISPLAYING ARTICLE CONTENT
    $scope.showArticle = function (articleIndex) {
        $scope.displayedArticleIndex = articleIndex;
        displayArticle($scope.articleList[articleIndex]);
    };
    
    $scope.displayNextArticle = function () {
        var next = $scope.displayedArticleIndex + 1;
        next = next % $scope.articleList.length;
        $scope.displayedArticleIndex = next;
        if (next >= $scope.articleList.length || next < 0)
            return;
        displayArticle($scope.articleList[next]);
    };

    $scope.displayPrevArticle = function () {
        var prev = $scope.displayedArticleIndex - 1;
        if (prev < 0) prev = $scope.articleList.length - 1;
        $scope.displayedArticleIndex = prev;
        if (prev >= $scope.articleList.length || prev < 0)
            return;
        displayArticle($scope.articleList[prev]);
    };
    
    // push settings to url
    $scope.pushUrl = function () {
        var eventId = $("#eventIdInput").val();
        var selectedLang = "eng";
        if ($("#article-langs a.active").length > 0)
            selectedLang = $("#article-langs a.active").attr("id").replace("lang-", "");
        var sort = $("#sort-time.active").length > 0 ? "time" : "rel";
        window.history.replaceState({}, "", window.location.pathname + "?eventId=" + encodeURIComponent(eventId) + "&lang=" + encodeURIComponent(selectedLang) + "&sort=" + sort);
    };

    // Initializations come after function definitions 
    
    // Init tooltips
    $("[data-toggle=tooltip]").tooltip();

    // load the event specified in the url parameters
    var eventId = getURLParameterByName("eventId");
    var lang = getURLParameterByName("lang");
    var sort = getURLParameterByName("sort");
    if (eventId != "")
        $("#eventIdInput").val(eventId);
    if (lang != "") {
        $("#article-langs a").removeClass("active");
        if (langList.indexOf(lang) >= 0)
            $($("#article-langs a")[langList.indexOf(lang)]).addClass("active");
    }
    if (sort != "") {
        $("#article-sorting a").removeClass("active");
        $("#sort-" + sort).addClass("active");
    }

    $("#eventIdInput").keypress(function (event) {
        if (event.which == 13)
            $scope.loadEventInfo();
    });

    if (eventId != "")
        $scope.loadEventInfo();

    // when we are 700px from the bottom of the page start loading new articles
    //$(window).scroll(function () {
    //    if ($(window).scrollTop() + $(window).height() + 700 > $(document).height())
    //        eventViewer.loadNextArticlePage();
    //});
    $("#prevArticle").click(function () {
        $scope.displayPrevArticle();
    });
    $("#nextArticle").click(function () {
        $scope.displayNextArticle();
    });

    //// when we are 700px from the bottom of the page start loading new articles
    //$(window).scroll(function () {
    //    if ($(window).scrollTop() + $(window).height() + 700 > $(document).height())
    //        $scope.loadNextArticlePage();
    //});
    $(window).resize(function () {
        $scope.drawConcepts();
    });
};


function updateEventInfo() {
    var enteredEventId = $("#eventIdInput").val();
    $.get("/do/eventAdmin?actioin=updateEventInfo?eventId=" + enteredEventId, function (foo) {
    });
}

function getDateTimeStr(datetime) {
    return datetime.toString("yyyy-MM-ddTHH:mm:ss");
}
