eventViewer = new EventViewer();

$().ready(function () {
       
    // Init tooltips
    $("[data-toggle=tooltip]").tooltip();

    // load the event specified in the url parameters
    var eventId = getURLParameterByName("eventId");
    var langs = getURLParameterByName("langs");
    if (eventId != "")
        $("#eventIdInput").val(eventId);

    $("#searchButton").click(function () { eventViewer.loadArticlesForEvent($("#eventIdInput").val(), ""); });
    $("#eventIdInput").keypress(function(event) {
        if (event.which == 13)
            eventViewer.loadArticlesForEvent($("#eventIdInput").val(), "");
    });
    
    if (eventId != "")
        eventViewer.loadArticlesForEvent(eventId, langs);

    // when we are 700px from the bottom of the page start loading new articles
    $(window).scroll(function () {
        if ($(window).scrollTop() + $(window).height() + 700 > $(document).height())
            eventViewer.loadNextArticlePage();
    });
    $("#prevArticle").click(function () {
        eventViewer.displayPrevArticle();
    });
    $("#nextArticle").click(function () {
        eventViewer.displayNextArticle();
    });
});


function updateEventInfo() {
    var enteredEventId = $("#eventIdInput").val();
    $.get("/do/updateEventInfo?eventId=" + enteredEventId, function (foo) {
    });
}

function getDateTimeStr(datetime) {
    return datetime.toString("yyyy-MM-ddTHH:mm:ss");
}

function EventViewer() {
    
    this.allArticlesLoaded = false;
    this.queryParams = undefined;
    this.articleList = [];
    this.displayedArticleIndex = -1;
    

    this.resetData = function () {
        // clear existing content
        $("#error-container").css("display", "none");
        $("#article-container").html("");
        $("#event-info h2").html("");
        $("#event-info-details").html("");
        this.queryParams = undefined;
        this.allArticlesLoaded = false;
        this.articleList = [];
        this.displayedArticleIndex = -1;
    };
    
    this.loadArticlesForEvent = function (eventId, langs) {
        this.resetData();
        if (isNaN(parseFloat(eventId))) {
            $("#error-container").css("display", "block");
            $("#error-text").html("Please enter a valid number of the event id");
            return;
        }
        // show the main part
        $("#event-info").css("display", "block");
        
        var params = { searchParams: { page: 0, count: 30 }, eventId: eventId, langs: langs };
        this.queryParams = params;
        this.loadNextArticlePage();
    };
    
    this.loadNextArticlePage = function () {
        var self = this;
        if (this.allArticlesLoaded == true)
            return;
        if (this.queryParams == undefined)
            return;
        if (!this.queryParams.searchParams)
            return;
        $.getJSON("/json/getEventArticles", { params: JSON.stringify(this.queryParams) }, function (eventData) {
            if (eventData.error) {
                self.allArticlesLoaded = true;
                $("#error-container").css("display", "block");
                $("#error-text").html(eventData.error);
                return;
            }
            var articlesHtml = "";
            if (!eventData.articles || eventData.articles.length == 0)
                self.allArticlesLoaded = true;
            if (eventData.articles) {
                $.each(eventData.articles, function (ind, item) {
                    var articleId = "article" + self.articleList.length;
                    self.articleList.push(item);
                    articlesHtml += createHtmlForArticle(item, false, articleId);
                });
            }
            $("#article-container").append(articlesHtml);

            initLightbox();
            $("a.open-gallery").click(function (event) {
                var articleIndex = $(event.target).parents(".article").prop('id');
                articleIndex = parseInt(articleIndex.replace("article", ""));
                self.displayedArticleIndex = articleIndex;
                if (articleIndex >= self.articleList.length || articleIndex < 0)
                    return;
                displayArticle(self.articleList[articleIndex]);
            });
            
            // show other parameters of the event
            if (self.queryParams.searchParams.page == 0) {
                var article = self.getArticleForLanguage(eventData.eventInfo.medoid_articles, self.queryParams.lang);
                if (article) {
                    $("#event-info h2").html(article.title);
                }
                
                var entities = "", keywords = "";
                $.each(eventData.eventInfo.concepts, function(index, concept) {
                    var dmoz = "";
                    if (concept.label.indexOf("/") >= 0) {
                        dmoz = "<br/><b>DMoz category</b>: " + concept.label;
                        concept.label = concept.label.substring(concept.label.lastIndexOf("/") + 1, concept.label.length);
                    }
                    var el = '<a href="#" class="label arrowed" data-toggle="tooltip" title="" data-original-title="<B>Importance</B>:' + concept.score + dmoz + '">' + concept.label + '</a>';
                    if (entityTypes[concept.type])
                        entities += el;
                    else
                        keywords += el;
                });
                var date = Date.parse(article.date + " " + article.time);
                $("#event-info-details").html("");
                $("#event-info-details").append('<div class="row-fluid"><div class="span12"><div class="entities"><span class="event-param">Event time: </span>' + date.toString('MMMM d, yyyy') + '</div></div></div>\n');

                // print article counts per language
                var articleCounts = {};
                var totCount = 0;
                $.each(eventData.eventInfo.stories, function (index, story) {
                    totCount += story.article_count;
                    var lang = story.language;
                    articleCounts[lang] = (articleCounts[lang] != undefined) ? articleCounts[lang] + story.article_count : story.article_count;
                });
                var cntPerLang = $.map(langList, function (lang, ind) {
                    return lang + ": " + (articleCounts[lang] ? articleCounts[lang] : 0);
                });
                // add article counts
                $("#event-info-details").append('<div class="row-fluid"><div class="span12"><div class="articleCounts"><span class="event-param">Nr. of articles: </span>' + totCount + " (" + cntPerLang.join(", ") + ")</div>\n");

                // add entities, keywords
                $("#event-info-details").append('<div class="row-fluid"><div class="span12"><div class="entities"><span class="event-param">Entities: </span>' + entities + '</div></div></div>\n');
                $("#event-info-details").append('<div class="row-fluid"><div class="span12"><div class="entities"><span class="event-param">Keywords: </span>' + keywords + '</div></div></div>\n');
                
                // udpate tooltips
                $("[data-toggle=tooltip]").tooltip({ "html": true });
                
                $("#event-info-details").append('<div class="row-fluid"><div class="span12"><a class="accordion-toggle" data-toggle="collapse" style="line-height: 26px;" data-target="#advEventDetails" >Toggle advanced event details</a>');
                $("#event-info-details").append('<div class="row-fluid" id="advEventDetails" class="accordion-body collapse"><p></p></div>');
                $('#advEventDetails').collapse("toggle");
                $('#advEventDetails').on('show', function () {
                    $.getJSON("/json/getEventInternals", { params: JSON.stringify({ eventId: self.queryParams.eventId }) }, function (eventDetails) {
                        $("#advEventDetails").html("");
                        $.each(eventDetails.stories, function (index, story) {
                            var out = '<div class="row-fluid"><div class="span12">';
                            if (story.error) {
                                out += '<h4>' + story.error + '</h4>\n';
                            }
                            else {
                                out += '<h3>' + story.name + '</h3>\n';
                                out += '<p>Articles: ' + story.articleCount + ', total weight: ' + story.totalWeight + ', centroid has ' + story.centroidFeatures + ' features.</p>';
                                out += '<p>Variance = ' + story.variance + ', StdDev = ' + story.stDev + ', Avg cos to centroid = ' + story.avgCosToCentroid + '</p>\n';
                                if (story.oldestArticleTime)
                                    out += '<p>Oldest article time: ' + story.oldestArticleTime + '</p>\n<p>Newest article time: ' + story.latestArticleTime + '</p>\n';
                                out += '<p>Latest addition: ' + story.latestAddition + '</p>\n';
                                out += '<h4>Articles:</h4>\n';
                                $.each(story.articles, function(i, art) {
                                    out += '<p>cos sim: ' + art.cos + ', ' + art.title + ' (time: ' + art.time + ', time weight: ' + art.timeWeight + ', sumWgt: ' + art.timeSumWgt + '</p>\n';
                                });
                            }
                            out += '</div></div>';
                            $("#advEventDetails").append(out);
                        });
                    });
                });
            }

            self.queryParams.searchParams.page++;
        });
    };

    
    this.getArticleForLanguage = function (medoidArticles, preferredLang) {
        var articles = $.grep(medoidArticles, function (article, index) {
            return article.lang == preferredLang;
        });
        if (articles.length > 0)
            return articles[0];
        articles = $.grep(medoidArticles, function (article, index) {
            return article.lang == "eng";
        });
        if (articles.length > 0)
            return articles[0];
        return (medoidArticles.length > 0) ? medoidArticles[0] : undefined;
    };
    
    // /////////////////////
    // DISPLAYING ARTICLE CONTENT
    this.displayNextArticle = function () {
        var next = this.displayedArticleIndex + 1;
        next = next % this.articleList.length;
        this.displayedArticleIndex = next;
        if (next >= this.articleList.length || next < 0)
            return;
        displayArticle(this.articleList[next]);
    };

    this.displayPrevArticle = function () {
        var prev = this.displayedArticleIndex - 1;
        if (prev < 0) prev = this.articleList.length - 1;
        this.displayedArticleIndex = prev;
        if (prev >= this.articleList.length || prev < 0)
            return;
        displayArticle(this.articleList[prev]);
    };
};