var langList = ["eng", "spa", "deu", "zho", "slv"];

// render the individual concepts shown in autocomplete textbox
$.widget("ui.autocomplete", $.ui.autocomplete, {
    _renderItem: function (ul, element) {
        return $('<li id="' + encodeURIComponent(element.uri) + '" class="' + element.type + ' ' + element.detailed_type + '"><a title="' + element.uri +'">' + element.label + '</a></li>').appendTo(ul);
    }
});


$().ready(function () {
    // build the tags input box for entering concepts, ...
    $('#searchConditions').tagsInput({ defaultText: "What topics are you interested in?", minInputWidth: 140 });
    $('#searchButton').click(function () { searchArticles.searchForArticles(); });
    $("#searchConditions_tag").keypress(function (event) {
        if (event.which == 13)
            searchArticles.searchForArticles();
    });
    
    $('#advSearchOptions').collapse("toggle");

    // Init tooltips
    $("[data-toggle=tooltip]").tooltip();

    // set up the autocomplete handlers
    $("#searchConditions" + "_tag").autocomplete({
	    source: function (request, response) {
	        var query = "/json/findConcepts?prefix=" + request.term;
	        $.getJSON(query, function(suggList) {
	            response(suggList);
	        });          
	    },

	    select: function (event, ui) {
	        var join = { $name: "hasArticle", $query: { $from: "Concept", "URI": ui.item.uri } };
	        $('#searchConditions').addTag(ui.item.label, { focus: true, type: "concept", join: join });
	        return false;
	    }
    });
    $(window).scroll(function () {
        // when we are 700px from the bottom of the page start loading new articles
        if ($(window).scrollTop() + $(window).height() + 700 > $(document).height())
            searchArticles.loadNextArticlePage();
    });
    $("#prevArticle").click(function () {
        searchArticles.displayPrevArticle();
    });
    $("#nextArticle").click(function () {
        searchArticles.displayNextArticle();
    });
});

function getDateTimeStr(datetime) {
    return datetime.toString("yyyy-MM-ddTHH:mm:ss");
}

function SearchArticles() {

    this.allArticlesLoaded = false;
    this.queryParams = undefined;
    this.articleList = [];
    this.displayedArticleIndex = -1;
    
    // when a concept in an event is clicked it is added to the conditions
    this.addConceptConstraint = function(element) {
        var uri = $(element).attr("data-uri");
        var label = $(element).text();
        var join = { $name: "hasArticle", $query: { $from: "Concept", "URI": uri } };
        $('#searchConditions').addTag(label, { focus: true, type: "concept", join: join });
    };

    this.resetData = function() {
        // clear existing content
        $("#article-container").html("");
        $("#result-info").html("");
        this.queryParams = undefined;
        this.allArticlesLoaded = false;
        this.articleList = [];
        this.displayedArticleIndex = -1;
    };

    // //////////////////
    // event handler for clicking the search button
    this.searchForArticles = function () {
        this.resetData();
        
        var conditions = {};
        var queryTime;
        var joins = [];
        var self = this;

        $.each($("#searchConditions" + "_tagList .tag"), function (i, element) {
            if ($(element).data("options").join)
                joins.push($(element).data("options").join);
            else if ($(element).data("options").queryTime)
                queryTime = $(element).data("options").queryTime;
            else
                // if we have any conditions then extend the conditions object
                $.extend(conditions, $(element).data("options").conditions);
        });
        // if not all languages are checked then add the condition for language
        if ($("#validLanguages input:checked").length != $("#validLanguages input")) {
            var langs = $.map($("#validLanguages input:checked"), function (item, ind) {
                return $(item).val();
            });
            conditions["Language"] = { $or: langs };
        }

        
        if (joins.length == 0 && Object.keys(conditions).length == 0 && queryTime == undefined)
            return;
        var minerQuery = {};
        if (joins.length > 0)
            minerQuery["$join"] = joins;
        if (Object.keys(conditions).length > 0) {
            $.extend(minerQuery, conditions);
            minerQuery["$from"] = "Article";
        }

        var params = { searchParams: { page: 0, count: 30 } };
        if (Object.keys(minerQuery).length > 0)
            params.query = minerQuery;
        if (queryTime)
            params.queryTime = queryTime;
        
        this.queryParams = params;
        this.loadNextArticlePage();
    };

    this.loadNextArticlePage = function () {
        var self = this;
        if (this.allArticlesLoaded == true)
            return;
        if (this.queryParams == undefined)
            return;
        if (!this.queryParams.searchParams)
            return;
        $.get("/json/getArticles", { params: JSON.stringify(this.queryParams) }, function (articleDataStr) {
            var articleData = JSON.parse(articleDataStr);
            if (articleData.error) {
                self.allArticlesLoaded = true;
                $("#error-container").css("display", "block");
                $("#error-text").html(articleData.error);
                return;
            }
            var articlesHtml = "";
            if (self.queryParams.searchParams.page == 0) {
                var results = (articleData.resultInfo && articleData.resultInfo.resultCount) ? articleData.resultInfo.resultCount : 0;
                $("#result-info").css("display", "block");
                $("#result-info").html(results + " results found");
            }
            if (!articleData.results || articleData.results.length == 0)
                self.allArticlesLoaded = true;
            if (articleData.results) {
                $.each(articleData.results, function (ind, item) {
                    var articleId = "article" + self.articleList.length;
                    self.articleList.push(item);
                    articlesHtml += createHtmlForArticle(item, true, articleId);
                });
            }
            $("#article-container").append(articlesHtml);

            // udpate tooltips
            $("[data-toggle=tooltip]").tooltip({ "html": true });
            initLightbox();
            $("a.open-gallery").click(function (event) {
                var articleIndex = $(event.target).parents(".article").prop('id');
                articleIndex = parseInt(articleIndex.replace("article", ""));
                self.displayedArticleIndex = articleIndex;
                if (articleIndex >= self.articleList.length || articleIndex < 0)
                    return;
                displayArticle(self.articleList[articleIndex]);
            });

            self.queryParams.searchParams.page++;
        });
    };
    
    // /////////////////////
    // DISPLAYING ARTICLE CONTENT
    this.displayNextArticle = function () {
        var next = this.displayedArticleIndex + 1;
        next = next % this.articleList.length;
        this.displayedArticleIndex = next;
        if (next >= this.articleList.length || next < 0)
            return;
        displayArticle(this.articleList[next]);
    };

    this.displayPrevArticle = function () {
        var prev = this.displayedArticleIndex - 1;
        if (prev < 0) prev = this.articleList.length - 1;
        this.displayedArticleIndex = prev;
        if (prev >= this.articleList.length || prev < 0)
            return;
        displayArticle(this.articleList[prev]);
    };
};

searchArticles = new SearchArticles();