
// render the individual concepts shown in autocomplete textbox
$.widget("ui.autocomplete", $.ui.autocomplete, {
    _renderItem: function (ul, element) {
        return $('<li id="' + encodeURIComponent(element.uri) + '" class="' + element.type + ' ' + element.detailed_type + '" title="' + element.uri + '"><a title="' + element.uri + '">' + element.label + '</a></li>').appendTo(ul);
    }
});

function updateUrl() {
    var params = { conditions: [] };
    $.each($("#eventConditions" + "_tagList .tag"), function (i, element) {
        params.conditions.push($(element).data("options"));
    });
    // if not all languages are checked then add the condition for language
    if ($("#validLanguages label[class*='checked']") != $("#validLanguages label")) {
        params.validLangs = $.map($("#validLanguages label[class*='checked']"), function (item, ind) {
            return $(item).children("input").val();
        });
    }
    params.minArticles = parseInt($("#min-articles").val());
    params.sortBy = $("i.icon-time").parent().hasClass("active") ? "date" : "rel";
    window.history.replaceState(params, "", window.location.pathname + "?query=" + encodeURIComponent(JSON.stringify(params)));
};

$().ready(function () {
    // build the tags input box for entering concepts, ...
    $('#eventConditions').tagsInput({ defaultText: "What events are you looking for...", minInputWidth: 140 });
    $('#eventSearchButton').click(function () { searchEvents.searchForEvents(); });
    $("#button-sortBy a.btn").click(function() {
        searchEvents.searchForEvents();
    });
    $("#eventConditions_tag").keypress(function (event) {
        if (event.which == 13)
            searchEvents.searchForEvents();
    });
    
    $('#advSearchOptions').collapse("toggle");

    // Init tooltips
    $("[data-toggle=tooltip]").tooltip();

    // set up the autocomplete handlers
    $("#eventConditions" + "_tag").autocomplete({
	    source: function (request, response) {
	        query = "/json/findConcepts?prefix=" + request.term;
	        $.getJSON(query, function(suggList) {
	            response(suggList);
	        });          
	    },

	    select: function (event, ui) {
	        $('#eventConditions').addTag(ui.item.label, { focus: true, type: "concept", uri: ui.item.uri, label: ui.item.label });
	        return false;
	    }
    });
    // init min articles drop down
    $("#min-articles").dropkick();
    
    // date picker for date range
    $('#dp-start').datepicker({ format: 'dd-mm-yyyy' }).on('changeDate', validateDate);
    $('#dp-end').datepicker({ format: 'dd-mm-yyyy' }).on('changeDate', validateDate);
    $('#dp-set').click(function () {
        if ($("#dp-set").hasClass("disabled"))
            return;
        var start = $('#dp-start').data('datepicker').date;
        var end = $('#dp-end').data('datepicker').date;
        searchEvents.addTimeConstraint(start, end);
    });

    // scrolling of the page should load next batch of articles
    $(window).scroll(function () {
        // when we are 700px from the bottom of the page start loading new events
        if ($(window).scrollTop() + $(window).height() + 700 > $(document).height())
            searchEvents.loadNextEventPage();
    });
    
    // //////////
    // if we have query arguments then parse them and do the search
    var queryStr = decodeURIComponent(getURLParameterByName("query"));
    if (queryStr != "") {
        var query = JSON.parse(queryStr);
        $.each(query.conditions, function (i, element) {
            if (element.type == "concept")
                $('#eventConditions').addTag(element.label, { focus: true, type: "concept", uri: element.uri, label: element.label });
            else if (element.type == "date")
                searchEvents.addTimeConstraint(element.time.start, element.time.end);
        });

        $.each(langList, function (i, element) {
            if (query.validLangs == [] || query.validLangs.indexOf(element) >= 0)
                $("lang-" + element).parent().addClass("checked");
            else
                $("lang-" + element).parent().removeClass("checked");
        });
        $("#min-articles").dropkick("select", query.minArticles);
        searchEvents.searchForEvents();
    }
});

function validateDate(ev) {
    var start = $('#dp-start').data('datepicker').date.valueOf();
    var end = $('#dp-end').data('datepicker').date.valueOf();
    if ($('#dp-start').val() != "" && $('#dp-end').val() != "" && start < end)
        $("#dp-set").removeClass("disabled");
    else
        $("#dp-set").addClass("disabled");   
};

function getDateTimeStr(datetime) {
    return datetime.toString("yyyy-MM-ddTHH:mm:ss");
}

function SearchEvents() {
    this.allArticlesLoaded = false;
    this.queryArgs = "";
    this.queryPage = 0;
    this.queryCount = 25;
    this.validLangs = [];
    
    this.resetData = function () {
        $("#event-container").html("");
        $("#result-info").html("");
        this.allArticlesLoaded = false;
        this.queryArgs = "";
        this.queryPage = 0;
        this.validLangs = [];
    };

    this.addTimeConstraintBySpan = function(timeSpanDays) {
        var now = new Date.today();
        var start = Date.today().addDays(-timeSpanDays);
        this.addTimeConstraint(start);
    };

    // handler for clicking a link for adding time condition
    this.addTimeConstraint = function(start, end) {
        $.each($("#eventConditions" + "_tagList .tag"), function (i, element) {
            if ($(element).data("options").time)
                $("#eventConditions").removeTag($(element));
        });
        var label = (end != undefined) ? start.toString("yyyy-MM-dd") + " - " + end.toString("yyyy-MM-dd") : "since " + start.toString("yyyy-MM-dd");
        if (end == undefined)
            end = new Date.today();
        $('#eventConditions').addTag(label, { focus: true, type: "date", time: { start: start, end: end } });
    };

    // when a concept in an event is clicked it is added to the conditions
    this.addConceptConstraint = function(element) {
        var uri = $(element).attr("data-uri");
        $('#eventConditions').addTag(label, { focus: true, type: "concept", uri: uri, label: $(element).text() });
    };

    // //////////////////
    // event handler for clicking the search button
    this.searchForEvents = function () {
        var self = this;
        updateUrl();
        // clear existing content
        this.resetData();
        var args = "";
        var queryStr = decodeURIComponent(getURLParameterByName("query"));
        var query = queryStr != "" ? JSON.parse(queryStr) : {};
        $.each(query.conditions, function (i, element) {
            if (element.type == "concept")
                args += "conceptUri=" + encodeURIComponent(element.uri) + "&";
            else if (element.type == "date")
                args += "timeStart=" + encodeURIComponent(element.time.start) + "&timeEnd=" + encodeURIComponent(element.time.end) + "&";
            else {
                // if we have any conditions then extend the conditions object
                alert(element);
                //$.extend(conditions, $(element).data("options").conditions);
            }
        });
        // if not all languages are checked then add the condition for language
        if (query.validLangs) {
            $.each(query.validLangs, function (ind, item) { args += "lang=" + encodeURIComponent(item) + "&"; });
            self.validLangs = query.validLangs;     // remember langs so that we can specify the on eventViewer page
        }
        if (query.minArticles && query.minArticles > 0)
            args += "minArticlesInEvent=" + query.minArticles + "&";
        if (args == "")
            return;

        args += "sortBy=" + query.sortBy;
        this.queryArgs = args;

        this.loadNextEventPage();
    };

    this.loadNextEventPage = function() {
        var self = this;
        if (this.allArticlesLoaded == true)
            return;
        if (this.queryArgs == "")
            return;

        var args = this.queryArgs + "&page=" + this.queryPage + "&count=" + this.queryCount;
        //args += "&callback=?";

        $.getJSON("/json/getEvents?" + args, function (eventData) {
            if (eventData.error) {
                self.allArticlesLoaded = true;
                $("#error-container").css("display", "block");
                $("#error-text").html(eventData.error);
                return;
            } else {
                $("#error-container").css("display", "none");
            }

            var preferredLang = $("#preferredLanguage input:checked").val();
            var eventHtml = "";
            if (self.queryPage == 0) {
                var results = (eventData.resultInfo && eventData.resultInfo.resultCount) ? eventData.resultInfo.resultCount : 0;
                $("#result-info").css("display", "block");
                $("#result-info").html(results + " events found");
            }
            if (!eventData.results || eventData.results.length == 0)
                self.allArticlesLoaded = true;
            if (eventData.results) {
                $.each(eventData.results, function(ind, item) {
                    eventHtml += self.createHtmlForEvent(item, preferredLang);
                    console.log("story id: " + item.primaryStory.id + "; lang=" + item.medoidArticles[0].lang + "; " + item.medoidArticles[0].title);
                });
            }
            $("#event-container").append(eventHtml);

            // udpate tooltips
            $("[data-toggle=tooltip]").tooltip({ "html": true });
            $.each($("#event-container .event"), function(ind, item) {
                var eventId = $(this).attr("id");
                $(item).find("img").click(function() {
                    window.open(window.location.host + "/eventViewer?eventId=" + eventId + "&lang=" + self.validLangs.join(","));
                });
            });
            self.queryPage++;
        });
    };

    // //////////////////
    // create html for a single event
    this.createHtmlForEvent = function (event, preferredLang) {
        var article = this.getArticleForLanguage(event.medoidArticles, preferredLang);
        if (!article)
            return "";
        var out = '<div class="row-fluid" >';
        out += '<div class="event span12" id="' + event.id + '">';
        out += '<div class="pull-left" >';
        //out += ' <a href="/eventViewer?eventId=' + event.id + '&lang=' + this.validLangs.join(",") + '" target="_blank">\n';
        if (article.image_uri)
            out += '<img class="thumbnail media-object" src="' + article.image_uri + '">';
        else
            out += '<img class="thumbnail media-object" src="/img/noimage64.png">';
        //out += '</a>';
        out += '</div>';
        out += '<div class="media-body">';
        out += '<h4 class="media-heading"><a href="/eventViewer?eventId=' + event.id + '&lang=' + preferredLang + '" target="_blank">' + article.title + '</a></h4>';
        
        var date = Date.parse(article.date + " " + article.time);
        
        // print article counts per language
        var totCount = event.articleCounts.total;
        var cntPerLang = $.map(langList, function(lang, ind) {
            return lang + ": " + (event.articleCounts[lang] ? event.articleCounts[lang] : 0);
        });

        var addedEntities = 0, addedKeywords = 0;
        var entities = "", keywords = "";
        $.each(event.concepts, function (index, concept) {
            var dmoz = "";
            if (concept.label.indexOf("/") >= 0) {
                dmoz = "<br/><B>DMoz category</b>: " + concept.label;
                concept.label = concept.label.substring(concept.label.lastIndexOf("/")+1, concept.label.length);
            }
            var el = '<a href="#" onclick="searchEvents.addConceptConstraint(this);" data-uri="' + concept.uri + '" data-toggle="tooltip" title="" data-original-title="<b>Importance</b>: ' + concept.score + dmoz + '"><span class="label arrowed">' + concept.label + '</span></a>';
            if (entityTypes[concept.type] && addedEntities < 10) {
                entities += el;
                addedEntities++;
            }
            else if (keywordTypes[concept.type] && addedKeywords < 10) {
                keywords += el;
                addedKeywords++;
            }
        });

        out += '<div class="time">When: ' + date.toString('MMMM d, yyyy') + '</span></div>';
        out += '<div class="articleCounts">Article counts: ' + totCount + " (" + cntPerLang.join(", ") + ")</div>";
        out += '<div class="entities">Entities:' + entities + '</div>';
        out += '<div class="topics">Keywords:' + keywords + '</div>';
        out += article.body;
        out += '</div>';    // media-body
        out += '</div>';    // span12
        out += '</div>';    // event
        return out;
    };

    this.getArticleForLanguage = function (medoidArticles, preferredLang) {
        var articles = $.grep(medoidArticles, function (article, index) {
            return article.lang == preferredLang;
        });
        if (articles.length > 0)
            return articles[0];
        articles = $.grep(medoidArticles, function (article, index) {
            return article.lang == "eng";
        });
        if (articles.length > 0)
            return articles[0];
        return (medoidArticles.length > 0) ? medoidArticles[0] : undefined;
    };
};

searchEvents = new SearchEvents();