// concepts with these prefixes will be shown in the entities list
var entityPrefixes = { "pm_org": true, "pm_person": true, "pm_location": true, "pm_cf": true };
// concepts with these prefixes will be shown in the keywords list
var keywordPrefixes = { "pm_category": true, "pm_keyword": true };
var catToImage = { "pm_category": "topic", "pm_keyword": "topic", "pm_location": "location", "pm_person": "person", "pm_org": "organization", "pm_cf": "folder" }

var defaultConceptWgt = 10;
var defaultSourceWgt = 100;
var defaultLocationWgt = 100;

var userInfo = null;

function UserInfo() {
    this.userProfile = { };
    this.folderIdPrefix = "folderId_";
    this.articleDateIdPrefix = "articleDate_";
    this.folderJson = null;
    this.tempFolder = null;
    this.changedTempFolder = false;     // did we make any changes to the temp folder that we would need to save?
    this.folderId = "";
    this.loadingPage = -1;
    this.showSaved = false;
    this.nextArticlePage = 0;
    this.articlePageCount = 20;
    this.articles = {};     // dictionary of uri -> article object
    this.articleUriList = [];   // list of uris. used for next, prev article
    this.displayedArticleIndex = -1;
    this.dailyTopics = { };
    this.dailyArticleObjs = { };
    this.relatedConcepts = new Array();
    this.relatedSources = new Array();
    this.storedArticles = {};
    
    this.showingCustomPage = false;
    this.pageType = null;
    this.pageTypeInfo = null;

    this.clearLoadedChannelData = function () {
        this.folderJson = null;
        this.tempFolder = null;
        this.changedTempFolder = false;
        this.loadingPage = -1;
        this.nextArticlePage = 0;
        this.articles = {};
        this.articleUriList = [];  
        this.dailyTopics = {};
        this.dailyArticleObjs = {};
        this.relatedConcepts = new Array();
        this.relatedSources = new Array();
        $("#content #article-data").html("");
    };

    this.clearUserInfo = function (clearChannelList) {
        this.clearLoadedChannelData();
        this.showingCustomPage = false;
        this.pageType = null;
        this.pageTypeInfo = null;
        this.folderId = "";
        this.showSaved = false;
        $("#wrapper #main #content .heading-block h1").html("");
        if (clearChannelList) 
            $("#sidebar .holder ul.channelList").html("");
    };

    // selectChannel can be
    // false if nothing is to be selected
    // true if the first channel is to be selected
    // folder id if a particular channels should be selected
    this.loadUserProfile = function (selectChannel) {
        var self = this;
        this.clearUserInfo(true);
        $.getJSON("/json/userProfile", function(userProfile) {
            self.userProfile = userProfile;

            var channelsStr = "";
            if (userProfile.folders) {
                $.each(userProfile.folders, function(index, element) {
                    channelsStr += '<li id="' + userInfo.folderIdPrefix + element.folderId + '"><a href="#" onclick="userInfo.displayChannel(\'' + element.folderId + '\');">' + element.folderName + '</a></li>\n';
                });
            }
            $("#sidebar .holder ul.channelList").html(channelsStr);
            if (selectChannel === true && userProfile.folders && userProfile.folders.length > 0)
                self.displayChannel(userProfile.folders[0].folderId);
            else if (typeof selectChannel == "number" && userProfile.folders && userProfile.folders.length > 0) {
                var found = $.grep(userProfile.folders, function (e) { return e.folderId == selectChannel; });
                if (found.length > 0)
                    self.displayChannel(selectChannel);
            }
        });
    };

    this.displayFavorites = function () {
        window.history.pushState({}, document.title, window.location.origin + "/me/favorites");
        this.clearUserInfo(false);
        this.folderId = "";
        $("#sidebar .holder ul.channelList li").removeClass("active");
        $("#wrapper #main #content .heading-block h1").html("Favorites");
        this.showSaved = true;
        this.loadNextArticlePage();
    };

    // display the provided channel
    this.displayChannel = function (folderId) {
        window.history.pushState({}, document.title, window.location.origin + "/me/channel/" + folderId);
        var self = this;
        this.clearUserInfo(false);
        // make sure we have a string with folderId and remove the prefix
        folderId = folderId.toString();
        if (folderId.search(this.folderIdPrefix) == 0)
            folderId = folderId.replace(this.folderIdPrefix, "");
        this.folderId = folderId;
        var queryArgs = { folderId: folderId };
        $("#sidebar .holder ul.channelList li").removeClass("active");
        var sidebarChannel = $("#sidebar .holder ul.channelList li#" + this.folderIdPrefix + folderId);
        sidebarChannel.addClass("active");
        $("#wrapper #main #content .heading-block h1").html(sidebarChannel.text());
        this.loadNextArticlePage();
        //$.getJSON("/json/folderJsonFull", queryArgs, function (folderJson) {
        //    self.folderJson = folderJson;
        //});
        if (!self.storedArticles[folderId]) {
            self.storedArticles[folderId] = {};
            $.getJSON("/json/getStoredArticleUris", { "folderId": folderId }, function(uriList) {
                for (var i = 0; i < uriList.length; i++)
                    self.storedArticles[folderId][encodeURIComponent(uriList[i])] = true;
                $.each($("#article-data .article"), function (index, article) {
                    if (self.storedArticles[folderId][$(article).prop("id")])
                        $(article).child("li.changeFavorite").addClass("active");
                });
            });
        }
    };

    // used to display a page for sources, entities, ...
    this.displayCustomPage = function (pageType, pageTypeInfo) {
        window.history.pushState({}, document.title, window.location.origin + "/" + pageType + "/" + encodeURIComponent(pageTypeInfo));
        this.showingCustomPage = true;
        this.nextArticlePage = 0;
        this.pageType = pageType;
        this.pageTypeInfo = pageTypeInfo;
        this.loadNextArticlePage(function (articleData) {
            if (!articleData)
                return;
            if (articleData.concept)
                $("#wrapper #main #content h1").html(articleData.concept.label);
            else if (articleData.source)
                $("#wrapper #main #content h1").html(articleData.source.Title);
            else if (articleData.story)
                $("#wrapper #main #content h1").html("Story Id: " + articleData.story.id + " (" + articleData.story.count + ")");
        });
    };

    // load next batch of the articles for the folder this.folderJson
    this.loadNextArticlePage = function (dataCallback) {
        // if we haven't yet finished loading of the previous page then don't do anything
        if (this.loadingPage == this.nextArticlePage)
            return;
        this.loadingPage = this.nextArticlePage;
        var self = this;
        var related = this.nextArticlePage == 0 && this.showingCustomPage == false ? 100 : 0;
        var searchParams = { page: this.nextArticlePage, count: this.articlePageCount, relatedConceptsCount: related, relatedSourcesCount: related, extractDailyTopics: true };
        var customInfo = { pageType: this.pageType, pageTypeInfo: this.pageTypeInfo };
        var queryArgs = this.showingCustomPage ? 
            { searchParams: JSON.stringify(searchParams), customInfo: JSON.stringify(customInfo) } : 
            { searchParams: JSON.stringify(searchParams), folderId: this.folderId, saved: this.showSaved ? 1 : 0 };
        var queryAddr = this.showingCustomPage ? "/json/articlesForCustomPage" : "/json/articlesForFolder";
        if (this.folderJson && !this.showingCustomPage)
            queryArgs.folder = JSON.stringify(this.folderJson);
        $.getJSON(queryAddr, queryArgs, function (articleData) {
            // cache the folder definition so that next time we don't have to ask the user server for the folder
            // if the person will like some articles we will still use the old definition
            if (!self.folderJson)
                self.folderJson = articleData.folder;
            if (articleData.dailyTopics != undefined)
                $.extend(self.dailyTopics, articleData.dailyTopics); // merge the old with the new daily topics
            if (articleData.relatedConcepts && articleData.relatedConcepts instanceof Array)
                self.relatedConcepts = articleData.relatedConcepts;
            if (articleData.relatedSources && articleData.relatedSources instanceof Array)
                self.relatedSources = articleData.relatedSources;
            self.displayArticles(articleData.results.articles, self.nextArticlePage);
            self.nextArticlePage++;
            
            $('.articles-container .popup-holder').contentPopup({
        		mode: 'click'
        	});   
            $('.popup .actions .cancel a').off('click.articleReset');
            $('.popup .actions .cancel a').on('click.articleReset', function () {
                if (!self.changedTempFolder)
                    return true;
                return confirm("Are you sure you want to discard changes?")
            });
            $(".addArticle a").off('click.addArticle');
            $(".addArticle a").on('click.addArticle', function () {
                var article = $(this).parents(".article");
                var channelsStr = "";
                if (self.userProfile.folders) {
                    $.each(self.userProfile.folders, function (index, element) {
                        channelsStr += '<option value="' + self.folderIdPrefix + element.folderId + '">' + element.folderName + '</option>\n';
                    });
                }
                $(article).find('.popup select').html(channelsStr);
            });
            if (dataCallback)
                dataCallback(articleData);
        });
    };

    // show articles from articleList
    this.displayArticles = function (articleList, page, showPlus, showStar) {
        showPlus = typeof showPlus !== "undefined" ? showPlus : true;
        showStar = typeof showStar !== "undefined" ? showStar : true;
        var self = this;
        if (!articleList instanceof Array)
            return;
        $.each(articleList, function (index, article) {
            self.articles[article.uri] = article;
            var arrayIndex = self.articleUriList.push(article.uri) - 1;

            // add a new daily tag and article area for the date
            if (self.dailyArticleObjs[article.date] == undefined) {
                var text = (self.showSaved == false) ? self.createDailyLine(article.date) : "";
                var id = self.articleDateIdPrefix + article.date;
                text += '<div id="' + id + '" class="articles-container"></div>';
                $("#article-data").append(text);
                self.dailyArticleObjs[article.date] = $("#article-data #" + id);
            }
            var output = "";
            output += '<div class="article popup-holder" id="article' + arrayIndex + '">\n';
            output += '<div class="holder">\n';
            output += '<div class="frame">\n';
            output += '<div class="title-block">\n';
            output += '<ul class="links">\n';
            if (showPlus)
                output += '<li class="addArticle"><a class="addA display-popup open">add article</a></li>\n';
            var activeClass = self.storedArticles[self.folderId] && self.storedArticles[self.folderId][encodeURIComponent(article.uri)] ? ' active' : '';
            if (showStar)
                output += '<li class="changeFavorite' + activeClass +'"><a onclick="userInfo.toggleArticleStored(this)" class="favorite">favorite</a></li>\n';
            //output += '<li><a href="#" class="close">close</a></li>\n';
            output += '</ul>\n';
            if (article.source_title.length > 36) {
                var tText = article.source_title.substring(0,36) + '...';
                output += '<h3><a href="/source/' + encodeURIComponent(article.source_uri) + '" title="' + article.source_title + '">' + tText + '</a></h3>\n';
            } else {
                output += '<h3><a href="/source/' + encodeURIComponent(article.source_uri) + '" title="' + article.source_title + '">' + article.source_title + '</a></h3>\n';
            }
            output += '</div>';
            if (article.image_uri)
                output += '<a onclick="userInfo.clickedArticle(this);" href="#popup-gallery" class="open-gallery image-crop"><img src="' + article.image_uri + '" alt="" /></a>\n';
            output += '<h2><a onclick="userInfo.clickedArticle(this);" href="#popup-gallery" class="open-gallery">' + article.title + '</a></h2>\n';
            output += '<p>' + article.body + '</p>\n';
            output += '<em class="published">' + article.date + " " + article.time + '</em>\n';
            if (article.story_id)
                output += '<div><a class="story" href="/story/' + article.story_id + '">Story: ' + article.story_id + '</a></div>\n';
            output += '</div>\n';
            output += '</div>\n';
            output += '<div class="popup add_article_box exempt">\n';
            output += '<h3>Add Article</h3>\n';
            output += '<div class="content-box settings">\n';
            output += '<h4>Select Channel</h4>\n';
            output += '<select>\n';
            output += '</select>\n';
            output += '<div class="back_line"><span>OR</span></div>\n';
            output += '<h4>Create New Folder</h4>\n';
            output += '<ul class="details">\n';
            output += '<li class="folder_name"><label>Name</label><span><input type="text" name="folder_name" value=""></span></li>\n';
            output += '</ul>\n';
            output += '<ul class="actions">\n';
            output += '<li class="save close"><a class="blue_button">Save</a></li>\n';
            output += '<li class="cancel close"><a class="grey_button">Cancel</a></li>\n';
            output += '</ul>\n';
            output += '</div>\n';
            output += '</div>\n';
            output += '</div>\n';
            self.dailyArticleObjs[article.date].append(output);
        });
        
        if (page == 0) {
            $('.articles-container').masonry({ itemSelector: '.article' });
            initLightbox();
        }
        else {
             $('.articles-container').masonry('reload');
             initLightbox();
        }
    };

    // create the daily line showing the topics of the day
    this.createDailyLine = function (date) {
        var text = "";
        text += '<div class="tags">\n\t<div class="holder">\n';
        text += '<p><strong>' + date + ' / </strong>\n';
        if (this.dailyTopics[date] != undefined) {
            var added = 0;
            $.each(this.dailyTopics[date], function (index, element) {
                text += '<a href="/concept/' + encodeURIComponent(element.uri) + '">' + element.label + '</a>,';
                added++;
                if (added >= 7) 
                    return false;   // break;
            });
        }
        if (text.length > 0 && text.substr(-1) === ",")
            text = text.substr(0, text.length - 1);
        text += '\n\t</div>\n</div>';
        return text;
    };

    // populate the add or edit popup with content from folder
    this.populatePopup = function (selector, folder) {
        var self = this;
        if (folder == null)
            return;
        if (folder.folderName)
            $(selector + " .channel-name input[type='text']").val(folder.folderName);

        var entities = "";
        var keywords = "";
        var sources = "";
        var locations = "";
        if (folder.hasConceptUris && folder.hasConceptLabels && folder.hasConceptUris.length == folder.hasConceptLabels.length) {
            $.each(folder.hasConceptUris, function (index, conceptUri) {
                if ($.inArray(conceptUri, folder.fixedConceptUris) == -1)
                    return true;    // skip automatically added concepts
                var conceptLabel = folder.hasConceptLabels[index];
                var conceptType = conceptUri.split(":")[0];
                var encodedUri = encodeURIComponent(conceptUri);
                if (entityPrefixes[conceptType])
                    entities += self.buildLI(encodedUri, conceptLabel, conceptType);
                else if (keywordPrefixes[conceptType])
                    keywords += self.buildLI(encodedUri, conceptLabel, conceptType);
            });
        }
        if (folder.validSourceLabels && folder.validSourceUris && folder.validSourceLabels.length == folder.validSourceUris.length) {
            $.each(folder.validSourceLabels, function (index, sourceLabel) {
                var encodedUri = encodeURIComponent(folder.validSourceUris[index]);
                sources += self.buildLI(encodedUri, sourceLabel, "source");
            });
        }

        if (folder.validLocations) {
            $.each(folder.validLocations, function (index, location) {
                locations += self.buildLI(location, location, "location");
            });
        }

        $(selector + " .list-entities").html(entities);
        $(selector + " .list-keywords").html(keywords);
        $(selector + " .list-sources").html(sources);
        $(selector + " .list-locations").html(locations);
        self.remakeLIRemoveHandlers($(selector + " .list-area ul.list li .ico-minus"));
    };
    
    // ///////////
    // Create, edit, delete channels

    // create a new channel
    this.createChannel = function () {
        var self = this;
        if (!self.tempFolder || self.tempFolder.folderName == undefined || self.tempFolder.folderName == "")
            return;     // todo: here we should show some error message
        var query = "/do/addFolder?folder=" + encodeURIComponent(JSON.stringify(self.tempFolder));
        $.getJSON(query, function(newFolderData) {
            var newChannelLine = '<li id="' + self.folderIdPrefix + newFolderData.folderId + '"><a href="#" onclick="userInfo.displayChannel(\'' + newFolderData.folderId + '\');">' + newFolderData.folderName + '</a></li>\n';
            self.userProfile.folders.push({folderId: newFolderData.folderId, folderName: newFolderData.folderName, visibility: newFolderData.visibility });
            $("#sidebar .holder ul.channelList").append(newChannelLine);
            if ($("#sidebar .holder ul.channelList li").length == 1)
                self.displayChannel(newFolderData.folderId);
        });
    };

    // save the folder changes done in the edit popup menu
    this.saveEditedChanges = function () {
        var self = this;
        if (!self.tempFolder)
            return;
        var folderId = self.tempFolder.folderId;
        var query = "/do/modifyFolder?folder=" + encodeURIComponent(JSON.stringify(self.tempFolder));
        $.get(query, function (foo) {
            $("#sidebar .holder ul.channelList #" + self.folderIdPrefix + self.tempFolder.folderId + " a").text(self.tempFolder.folderName);
            self.clearLoadedChannelData();
            self.displayChannel(folderId);
        });
    };

    // delete an existing channel
    this.deleteChannel = function () {
        var self = this;
        var query = "/do/removeFolder?folderId=" + this.folderId;
        $.get(query, function(foo) {
            $("#sidebar .holder ul.channelList #" + self.folderIdPrefix + self.folderId).remove();
            self.loadUserProfile(true);
        });
    };

    // /////////
    // SUGGESTIONS
    this.updateSuggestionsPopup = function () {
        var self = this;
        this.clearPopupContent(".sugg-channel .popup");
        this.tempFolder = JSON.parse(JSON.stringify(this.folderJson));
        var entities = "";
        var keywords = "";
        var sources = "";
        var locations = "";
        $.each(this.relatedConcepts, function (index, concept) {
            var encodedUri = encodeURIComponent(concept.uri);
            var conceptType = concept.uri.split(":")[0];
            if (self.folderJson && self.folderJson.hasConceptUris && $.inArray(concept.uri, self.folderJson.hasConceptUris) >= 0)
                return true;    // continue
            if (self.folderJson && self.folderJson.validSourceUris && $.inArray(concept.uri, self.folderJson.validSourceUris) >= 0)
                return true;    // continue
            if (entityPrefixes[conceptType])
                entities += self.buildLI(encodedUri, concept.label, conceptType, false, true);
            else if (keywordPrefixes[conceptType])
                keywords += self.buildLI(encodedUri, concept.label, conceptType, false, true);
        });
        $.each(this.relatedSources, function (index, source) {
            sources += self.buildLI(encodeURIComponent(source.uri), source.title, "source", false, true);
        });
        $(".sugg-channel .popup .list-entities").html(entities);
        $(".sugg-channel .popup .list-keywords").html(keywords);
        $(".sugg-channel .popup .list-sources").html(sources);
        $(".sugg-channel .popup .list-locations").html(locations);
        $(".sugg-channel .popup .list-area ul.list li .ico-plus").unbind("click");
        $(".sugg-channel .popup .list-area ul.list li .ico-plus").click(function () {
            var li = $(this).parent();
            var label = li.find("span").text();
            var uri = decodeURIComponent(li.prop('id'));
            var parentId = $(this).parents(".list-area").prop('id');
            if (self.addToTempFolder(uri, label, parentId) == false)
                return;
            li.remove();
        });
    };

    // the user pressed save in the suggestions popup dialog. send the modified folderJson to the server and reload the articles
    this.saveSuggestedChanges = function () {
        var self = this;
        if (!this.tempFolder)
            return;
        var folderId = this.tempFolder.folderId;
        var query = "/do/modifyFolder?folder=" + encodeURIComponent(JSON.stringify(this.tempFolder));
        $.get(query, function (foo) {
            self.clearLoadedChannelData();
            self.displayChannel(folderId);
        });
    };

    // ///////
    // favorites
    this.loadFavoritesPopup = function () {
        if (this.folderId == "")
            return;
        $.getJSON("/json/getStoredArticles", { "folderId": this.folderId }, function (articleList) {
            var output = "";
            $.each(articleList, function (index, article) {
                output += '<li>\n';
                if (article.image_uri)
                    output += '<div class="visual">\n<a href="#"><img src="' + article.image_uri + '" width="47" height="47" alt="" /></a>\n</div>\n';
                output += '<div class="text-block">\n';
                output += '<h4><a href="#">' + article.title + '</a></h4>';
                output += '<div class="info"><a href="#">' + article.source_title + '</a> - <em class="published">7 hours ago</em></div>';
                output += '</div>\n</li>\n';
            });
            $('.favorites ul.post-list').html(output);
        });
    };
    
    // /////////
    // DISPLAY SINGLE ARTICLE

    // called when the user clicks an article 
    this.clickedArticle = function (selectedItem) {
        var articleIndex = $(selectedItem).parents(".article").prop('id');
        articleIndex = articleIndex.replace("article", "");
        this.displayArticle(articleIndex);
    };

    this.displayArticle = function (articleIndex) {
        var self = this;
        // we could get the number as a string here so we need to convert to int
        articleIndex = parseInt(articleIndex);
        if (articleIndex >= this.articleUriList.length || articleIndex < 0)
            return;
        var uri = this.articleUriList[articleIndex];
        var encUri = encodeURIComponent(uri);
        var article = this.articles[uri];
        if (!article)
            return;

        this.displayedArticleIndex = articleIndex;
        var articleDom = $("#popup-gallery .slide .article-section");
        articleDom.prop("id", "displayedArticle" + articleIndex);
        if (self.storedArticles && self.storedArticles[self.folderId] && self.storedArticles[self.folderId][encUri])
            $("#popup-gallery .slide .article-section li.changeFavorite").addClass("active");
        else
            $("#popup-gallery .slide .article-section li.changeFavorite").removeClass("active");
        articleDom.children("h3.by").html(article.source_title);
        articleDom.children("h2").html(article.title);
        articleDom.children(".published").html(article.date + " " + article.time);
        articleDom.children("div.body").html("");
        $.getJSON("/json/articleJson", { articleUri: uri }, function (articleData) {
            var body = articleData.Body;
            var parBody = "";
            $.each(body.split("\n"), function (index, par) {
                parBody += "<p>" + par + "</p>";
            });
            $("#popup-gallery .slide .article-section div.body").html(parBody);
        });
    };

    this.displayNextArticle = function () {
        var next = this.displayedArticleIndex + 1;
        next = next % this.articleUriList.length;
        this.displayArticle(next);
    };

    this.displayPrevArticle = function () {
        var prev = this.displayedArticleIndex - 1;
        if (prev < 0) prev = this.articleUriList.length - 1;
        this.displayArticle(prev);
    };

    // ///////    
    // helper functions

    // build li text for a concept, source or location
    this.buildLI = function (uri, label, category, showMinus, showPlus) {
        var line = '<li class="' + category + '" id="' + uri + '">\n';
        if (showMinus === true || showMinus == undefined)
            line += '\t<a href="#" class="ico-minus">-</a>\n';
        if (showPlus === true)
            line += '\t<a href="#" class="ico-plus">-</a>\n';
        line += '\t<span class="name ' + catToImage[category] + '" /><span>' + label + '</span>\n';
        //line += '\t<span>' + label + '</span>\n';
        line += '</li>';
        return line;
    };

    // store or unstore the clicked article
    this.toggleArticleStored = function (element) {
        var self = this;
        var li = $(element).parents("li");
        var isStored = li.hasClass("active");
        var articleIndex = $(element).parents("div.article").prop("id");
        if (articleIndex == undefined)
            articleIndex = $(element).parents("div.article-section").prop("id");
        if (articleIndex == undefined)
            return;
        articleIndex = articleIndex.replace("displayedArticle", "").replace("article", "");
        articleIndex = parseInt(articleIndex);
        var articleUri = self.articleUriList[articleIndex];
        var query = (isStored) ?
            "/do/removeArticleFromFolder?articleUri=" + encodeURIComponent(articleUri) + "&folderId=" + encodeURIComponent(this.folderId) :
            "/do/addArticleToFolder?articleUri=" + encodeURIComponent(articleUri) + "&folderId=" + encodeURIComponent(this.folderId);
        $.get(query, function (data) { });
        $("#article" + articleIndex + " .changeFavorite").toggleClass("active");
        $("#displayedArticle" + articleIndex + " .changeFavorite").toggleClass("active");
        if (isStored)
            delete this.storedArticles[this.folderId][articleUri];
        else {
            this.storedArticles[this.folderId][articleUri] = true;
            // forget the save definition of the folder. staring an article will update folder concepts and we need to use the new definition for next articles (and for editing folder)
            this.folderJson = null;
        }
    };
    
    this.clearPopupContent = function (selector) {
        $(selector + " input[type='text']").val("");
        $(selector + " .list").html("");
    };
    
    // get and show suggestions for entities, keywords or sources
    this.showSuggestions = function(methodName, inputbox, prefix) {
        var self = this;
        if (prefix == "") {
            var p = inputbox.parent().children('.suggestions');
            $(p).hide();
            return;
        }

        query = "/json/" + methodName + "?prefix=" + prefix;
        $.getJSON(query, function(suggList) {
            var output = '';
            if (methodName == "findSources") {
                $.each(suggList, function(index, element) {
                    output += '<li id="' + encodeURIComponent(element["uri"]) + '" class="source" title="' + element["uri"] + '">' + element["title"] + '</li>';
                });
            } else if (methodName == "findLocations") {
                $.each(suggList, function(index, element) {
                    output += '<li id="' + encodeURIComponent(element) + '" class="location" title="' + element + '">' + element + '</li>';
                });
            } else {
                $.each(suggList, function(index, element) {
                    output += '<li id="' + encodeURIComponent(element["uri"]) + '" class="' + element.type + ' ' + element.detailed_type + '" title="' + element["uri"] + '">' + element["label"] + '</li>';
                });
            }

            inputbox.parent().children('.suggestions').html(output);
            var p = inputbox.parent().children('.suggestions');
            $(p).show();
            var pl = $(p).children('li');

            // the function that is called when we select an item from the auto-suggest list
            $(pl).click(function() {
                var encodedUri = $(this).prop('id');
                if (self.addToTempFolder(decodeURIComponent(encodedUri), $(this).html(), $(this).parents(".tab-edit").prop("id")) == false)
                    return;
                var li = self.buildLI(encodedUri, $(this).html(), $(this).prop('class'));
                $(this).parents(".tab-edit").find(".list-area ul.list").append(li);
                $(this).parents(".tab-edit").find(".row input[type='text']").val("");
                $(this).parents(".tab-edit").find(".suggestions").hide();
                self.remakeLIRemoveHandlers($(this).parents(".tab-edit").find(".list-area ul.list li .ico-minus"));
            });
        });
    };

    this.remakeLIRemoveHandlers = function(elements) {
        var self = this;
        elements.unbind("click");
        elements.click(function() {
            $(this).parent().remove();
            var id = decodeURIComponent($(this).parent().prop('id'));
            self.removeFromTempFolder(id);
        });
    };

    // when the user clicks a suggested element add it to the temp folder
    this.addToTempFolder = function (uri, label, parentId) {
        if (!this.tempFolder)
            return false;
        this.changedTempFolder = true;
        if (parentId.indexOf("entities") >= 0) {
            if (!this.tempFolder.hasConceptUris) this.tempFolder.hasConceptUris = [];
            if (!this.tempFolder.hasConceptWgts) this.tempFolder.hasConceptWgts = [];
            if (!this.tempFolder.hasConceptLabels) this.tempFolder.hasConceptLabels = [];
            if (!this.tempFolder.fixedConceptUris) this.tempFolder.fixedConceptUris = [];
            if (this.tempFolder.fixedConceptUris.length >= 50) {
                alert("Maximum number of concepts (50) were reached. You cannot add any more concepts.");
                return false;
            }
            this.tempFolder.hasConceptUris.push(uri);
            this.tempFolder.hasConceptWgts.push(defaultConceptWgt);
            this.tempFolder.hasConceptLabels.push(label);
            this.tempFolder.fixedConceptUris.push(uri);
        }
        else if (parentId.indexOf("sources") >= 0) {
            if (!this.tempFolder.validSourceUris) this.tempFolder.validSourceUris = [];
            if (!this.tempFolder.validSourceWgts) this.tempFolder.validSourceWgts = [];
            if (this.tempFolder.validSourceUris.length >= 50) {
                alert("Maximum number of news sources (50) were reached. You cannot add any more sources.");
                return false;
            }
            this.tempFolder.validSourceUris.push(uri);
            this.tempFolder.validSourceWgts.push(defaultSourceWgt);
        }
        else if (parentId.indexOf("locations") >= 0) {
            if (!this.tempFolder.validLocations) this.tempFolder.validLocations = [];
            if (!this.tempFolder.validLocationWgts) this.tempFolder.validLocationWgts = [];
            if (this.tempFolder.validLocations.length >= 50) {
                alert("Maximum number of geographic locations (50) were reached. You cannot add any more locations.");
                return false;
            }
            this.tempFolder.validLocations.push(uri);
            this.tempFolder.validLocationWgts.push(defaultLocationWgt);
        }
        return true;
    };
    
    // if user clicked minus on any type of elements (concepts, sources, locations, we remove them from the temp folder definition
    this.removeFromTempFolder = function (uri) {
        if (!this.tempFolder)
            return;
        this.changedTempFolder = true;
        var ind = $.inArray(uri, this.tempFolder.hasConceptUris);
        if (ind >= 0) {
            this.tempFolder.hasConceptUris.splice(ind, 1);
            this.tempFolder.hasConceptWgts.splice(ind, 1);
            this.tempFolder.hasConceptLabels.splice(ind, 1);
        }
        ind = $.inArray(uri, this.tempFolder.fixedConceptUris);
        if (ind >= 0)
            this.tempFolder.fixedConceptUris.splice(ind, 1);
        ind = $.inArray(uri, this.tempFolder.validSourceUris);
        if (ind >= 0) {
            this.tempFolder.validSourceUris.splice(ind, 1);
            this.tempFolder.validSourceWgts.splice(ind, 1);
        }
        ind = $.inArray(uri, this.tempFolder.validLocations);
        if (ind >= 0)
            this.tempFolder.validLocations.splice(ind, 1);
    };

    // /////////
    // CUSTOM PAGE SHOWING ARTICLES FOR ENTITIES, SOURCES, ...
    
    //// load next batch of the articles for the folder this.folderJson
    //this.loadNextCustomPage = function (dataCallback) {
    //    // if we haven't yet finished loading of the previous page then don't do anything
    //    if (this.loadingPage == this.nextArticlePage)
    //        return;
    //    this.loadingPage = this.nextArticlePage;
    //    var self = this;
    //    var searchParams = { page: this.nextArticlePage, count: this.articlePageCount, relatedConceptsCount: 0, relatedSourcesCount: 0, extractDailyTopics: true };
    //    var customInfo = { pageType: this.pageType, pageTypeInfo: this.pageTypeInfo }
    //    var queryArgs = { searchParams: JSON.stringify(searchParams), customInfo: JSON.stringify(customInfo) };
    //    $.getJSON("/json/articlesForCustomPage", queryArgs, function (articleData) {
    //        if (articleData.dailyTopics != undefined)
    //            $.extend(self.dailyTopics, articleData.dailyTopics); // merge the old with the new daily topics
    //        self.displayArticles(articleData.results.articles, self.nextArticlePage, true, false);
    //        self.nextArticlePage++;
            
    //        $('.articles-container .popup-holder').contentPopup({
    //            mode: 'click'
    //        });   
    //        $('.popup .actions .cancel a').off('click.articleReset');
    //        $('.popup .actions .cancel a').on('click.articleReset', function () {
    //            if (!self.changedTempFolder)
    //                return true;
    //            return confirm("Are you sure you want to discard changes?")
    //        });
    //        $(".addArticle a").off('click.addArticle');
    //        $(".addArticle a").on('click.addArticle', function () {
    //            var article = $(this).parents(".article");
    //            var channelsStr = "";
    //            if (self.userProfile.folders) {
    //                $.each(self.userProfile.folders, function (index, element) {
    //                    channelsStr += '<option value="' + self.folderIdPrefix + element.folderId + '">' + element.folderName + '</option>\n';
    //                });
    //            }
    //            $(article).find('.popup select').html(channelsStr);
    //        });
    //        if (dataCallback)
    //            dataCallback(articleData);
    //    });
    //};
}
/**
 * jQuery Masonry v2.1.08
 * A dynamic layout plugin for jQuery
 * The flip-side of CSS Floats
 * http://masonry.desandro.com
 *
 * Licensed under the MIT license.
 * Copyright 2012 David DeSandro
 */
;(function(e,t,n){"use strict";var r=t.event,i;r.special.smartresize={setup:function(){t(this).bind("resize",r.special.smartresize.handler)},teardown:function(){t(this).unbind("resize",r.special.smartresize.handler)},handler:function(e,t){var n=this,s=arguments;e.type="smartresize",i&&clearTimeout(i),i=setTimeout(function(){r.dispatch.apply(n,s)},t==="execAsap"?0:100)}},t.fn.smartresize=function(e){return e?this.bind("smartresize",e):this.trigger("smartresize",["execAsap"])},t.Mason=function(e,n){this.element=t(n),this._create(e),this._init()},t.Mason.settings={isResizable:!0,isAnimated:!1,animationOptions:{queue:!1,duration:500},gutterWidth:0,isRTL:!1,isFitWidth:!1,containerStyle:{position:"relative"}},t.Mason.prototype={_filterFindBricks:function(e){var t=this.options.itemSelector;return t?e.filter(t).add(e.find(t)):e},_getBricks:function(e){var t=this._filterFindBricks(e).css({position:"absolute"}).addClass("masonry-brick");return t},_create:function(n){this.options=t.extend(!0,{},t.Mason.settings,n),this.styleQueue=[];var r=this.element[0].style;this.originalStyle={height:r.height||""};var i=this.options.containerStyle;for(var s in i)this.originalStyle[s]=r[s]||"";this.element.css(i),this.horizontalDirection=this.options.isRTL?"right":"left";var o=this.element.css("padding-"+this.horizontalDirection),u=this.element.css("padding-top");this.offset={x:o?parseInt(o,10):0,y:u?parseInt(u,10):0},this.isFluid=this.options.columnWidth&&typeof this.options.columnWidth=="function";var a=this;setTimeout(function(){a.element.addClass("masonry")},0),this.options.isResizable&&t(e).bind("smartresize.masonry",function(){a.resize()}),this.reloadItems()},_init:function(e){this._getColumns(),this._reLayout(e)},option:function(e,n){t.isPlainObject(e)&&(this.options=t.extend(!0,this.options,e))},layout:function(e,t){for(var n=0,r=e.length;n<r;n++)this._placeBrick(e[n]);var i={};i.height=Math.max.apply(Math,this.colYs);if(this.options.isFitWidth){var s=0;n=this.cols;while(--n){if(this.colYs[n]!==0)break;s++}i.width=(this.cols-s)*this.columnWidth-this.options.gutterWidth}this.styleQueue.push({$el:this.element,style:i});var o=this.isLaidOut?this.options.isAnimated?"animate":"css":"css",u=this.options.animationOptions,a;for(n=0,r=this.styleQueue.length;n<r;n++)a=this.styleQueue[n],a.$el[o](a.style,u);this.styleQueue=[],t&&t.call(e),this.isLaidOut=!0},_getColumns:function(){var e=this.options.isFitWidth?this.element.parent():this.element,t=e.width();this.columnWidth=this.isFluid?this.options.columnWidth(t):this.options.columnWidth||this.$bricks.outerWidth(!0)||t,this.columnWidth+=this.options.gutterWidth,this.cols=Math.floor((t+this.options.gutterWidth)/this.columnWidth),this.cols=Math.max(this.cols,1)},_placeBrick:function(e){var n=t(e),r,i,s,o,u;r=Math.ceil(n.outerWidth(!0)/this.columnWidth),r=Math.min(r,this.cols);if(r===1)s=this.colYs;else{i=this.cols+1-r,s=[];for(u=0;u<i;u++)o=this.colYs.slice(u,u+r),s[u]=Math.max.apply(Math,o)}var a=Math.min.apply(Math,s),f=0;for(var l=0,c=s.length;l<c;l++)if(s[l]===a){f=l;break}var h={top:a+this.offset.y};h[this.horizontalDirection]=this.columnWidth*f+this.offset.x,this.styleQueue.push({$el:n,style:h});var p=a+n.outerHeight(!0),d=this.cols+1-c;for(l=0;l<d;l++)this.colYs[f+l]=p},resize:function(){var e=this.cols;this._getColumns(),(this.isFluid||this.cols!==e)&&this._reLayout()},_reLayout:function(e){var t=this.cols;this.colYs=[];while(t--)this.colYs.push(0);this.layout(this.$bricks,e)},reloadItems:function(){this.$bricks=this._getBricks(this.element.children())},reload:function(e){this.reloadItems(),this._init(e)},appended:function(e,t,n){if(t){this._filterFindBricks(e).css({top:this.element.height()});var r=this;setTimeout(function(){r._appended(e,n)},1)}else this._appended(e,n)},_appended:function(e,t){var n=this._getBricks(e);this.$bricks=this.$bricks.add(n),this.layout(n,t)},remove:function(e){this.$bricks=this.$bricks.not(e),e.remove()},destroy:function(){this.$bricks.removeClass("masonry-brick").each(function(){this.style.position="",this.style.top="",this.style.left=""});var n=this.element[0].style;for(var r in this.originalStyle)n[r]=this.originalStyle[r];this.element.unbind(".masonry").removeClass("masonry").removeData("masonry"),t(e).unbind(".masonry")}},t.fn.imagesLoaded=function(e){function u(){e.call(n,r)}function a(e){var n=e.target;n.src!==s&&t.inArray(n,o)===-1&&(o.push(n),--i<=0&&(setTimeout(u),r.unbind(".imagesLoaded",a)))}var n=this,r=n.find("img").add(n.filter("img")),i=r.length,s="data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///ywAAAAAAQABAAACAUwAOw==",o=[];return i||u(),r.bind("load.imagesLoaded error.imagesLoaded",a).each(function(){var e=this.src;this.src=s,this.src=e}),n};var s=function(t){e.console&&e.console.error(t)};t.fn.masonry=function(e){if(typeof e=="string"){var n=Array.prototype.slice.call(arguments,1);this.each(function(){var r=t.data(this,"masonry");if(!r){s("cannot call methods on masonry prior to initialization; attempted to call method '"+e+"'");return}if(!t.isFunction(r[e])||e.charAt(0)==="_"){s("no such method '"+e+"' for masonry instance");return}r[e].apply(r,n)})}else this.each(function(){var n=t.data(this,"masonry");n?(n.option(e||{}),n._init()):t.data(this,"masonry",new t.Mason(e,this))});return this}})(window,jQuery);

  // Masonry corner stamp modifications
  $.Mason.prototype.resize = function() {
    this._getColumns();
    this._reLayout();
  };
  
  $.Mason.prototype._reLayout = function( callback ) {
    var freeCols = this.cols;
    if ( this.options.cornerStampSelector ) {
      var $cornerStamp = this.element.find( this.options.cornerStampSelector ),
          cornerStampX = $cornerStamp.offset().left - 
            ( this.element.offset().left + this.offset.x + parseInt($cornerStamp.css('marginLeft')) );
      freeCols = Math.floor( cornerStampX / this.columnWidth );
    }
    // reset columns
    var i = this.cols;
    this.colYs = [];
    while (i--) {
      this.colYs.push( this.offset.y );
    }

    for ( i = freeCols; i < this.cols; i++ ) {
      this.colYs[i] = this.offset.y + $cornerStamp.outerHeight(true);
    }

    // apply layout logic to all bricks
    this.layout( this.$bricks, callback );
  };