﻿var entityTypes = { "org": true, "loc": true, "person": true };
var keywordTypes = { "wiki": true, "keyword": true };
var cfTypes = { "cf": true }

var langList = ["eng", "deu", "spa", "zho", "slv"];
var langToLabel = { "eng": "English", "deu": "German", "spa": "Spanish", "zho": "Chinese", "svn": "Slovene" };


String.prototype.trimLStr = function (prefix) {
    if (this.indexOf(prefix) == 0)
        return this.replace(prefix, "");
    return this.substring(0, this.length);
};

String.prototype.shorten = function(length) {
    if (this.length > length) {
        var lastSpace = this.substring(0, length).lastIndexOf(" ");
        if (lastSpace < 0) lastSpace = length;
        return this.substring(0, lastSpace) + "...";
    }
    return this.substring(0, length);
};

String.prototype.format = function () {
    var formatted = this;
    for (arg in arguments) {
        formatted = formatted.replace("{" + arg + "}", arguments[arg]);
    }
    return formatted;
};

// format number 1234 to 1,234
function numberWithCommas(x) {
    return x.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
}

// render the individual concepts shown in autocomplete textbox
var renderConceptItem = function (ul, element) {
    return $('<li id="' + encodeURIComponent(element.uri) + '" title="' + element.uri + '"><a title="' + element.uri + '" class="' + element.type + ' black concept">' + element.label + '</a></li>').appendTo(ul);
};
var renderLocationItem = function (ul, element) {
    return $('<li id="' + encodeURIComponent(element.wikiUri) + '" class="' + element.type + ' black concept" title="' + element.wikiUri + '"><a title="' + element.wikiUri + '">' + element.label + '</a></li>').appendTo(ul);
};
var renderPublisherItem = function (ul, element) {
    return $('<li id="' + encodeURIComponent(element.uri) + '" title="' + element.uri + '"><a title="' + element.uri + '">' + (element.title ? element.title : element.uri) + '</a></li>').appendTo(ul);
};


function showWait() {
    $("#wait").css("display", "block");
    $('.wait-circle, .wait-circle1').removeClass('wait-stop');
}

function hideWait() {
    $('.wait-circle, .wait-circle1').addClass('wait-stop');
    $("#wait").css("display", "none");
}

// create html for a single article
function createHtmlForArticle(article, showEventId, articleId) {
    if (!article)
        return "";
    var out = '<div class="row">\n';
    out += '<div class="col-md-12 article" id=' + articleId + '>\n';
    out += '<a class="pull-left open-gallery" href="#popup-gallery">\n';
    if (article.image_uri)
        out += '<img class="thumbnail media-object" src="' + article.image_uri + '">';
    else
        out += '<img class="thumbnail media-object" src="/img/noimage64.png">';
    out += '</a>';
    var date = Date.parse(article.date + " " + article.time);
        
    out += '<div class="media-body">';
    out += '<span class="rightInfo">';
    out += '<div class="time">' + date.toString('MMM. d, HH:mm') + '</div>';
    out += '<div class="source">' + article.source_title + '</div>';
    //out += '<div class="cossim">Sim to centroid: ' + article.sim + '</div>';
    
    out += '</span>';
    out += '<a href="#popup-gallery" class="open-gallery"><h4 class="media-heading">' + article.title + '</h4></a>';
    out += article.body;
    if (showEventId && article.story_id) {
        out += '<div><a target="_blank" href="/event/S' + article.story_id + '"><span class="badge">Story id: ' + article.story_id + '</span></a></div>';
    }
    if (article.isDuplicate)
        out += '<span class="badge">Article is a duplicate</badge>';
    out += '</div>';    // media-body
    out += '</div>';    // col-md-12
    out += '</div>';    // event
    return out;
};


function displayArticle(article) {
    var self = this;
    var uri = article.uri;

    $("#popup-gallery .slide .article-section .content h3.by").html(article.source_title);
    $("#popup-gallery .slide .article-section .content h2").html(article.title);
    $("#popup-gallery .slide .article-section .content .published").html(article.date + " " + article.time);
    $("#popup-gallery .slide .article-section .content div.body").html("");
    $.getJSON("/json/article?action=getArticle", { uri: uri }, function (articleData) {
        var body = articleData.body;
        var parBody = "";
        if (articleData.hasImage && articleData.hasImage.length > 0)
            parBody += '<img class="thumbnail media-object" width="200px" height="200px" src="' + articleData.hasImage[0].URI + '" style="float:right; margin-left: 20px; margin-bottom: 20px;">';
        parBody += '<div class="body-text">';
        $.each(body.split("\n"), function (index, par) {
            parBody += "<p>" + par + "</p>";
        });
        parBody += '</div>';
        var entities = "", keywords = "";
        $.each(articleData.concepts, function (index, concept) {
            var dmoz = "";
            if (concept.label.indexOf("/") >= 0) {
                dmoz = "<br/><b>DMoz category</b>: " + concept.label;
                concept.label = concept.label.substring(concept.label.lastIndexOf("/") + 1, concept.label.length);
            }
            var el = '<span class="badge" title="Importance: ' + concept.score + dmoz + ', ' + concept.uri + '">' + concept.label + '</span></a>';
            if (entityTypes[concept.type] == true)
                entities += el;
            else
                keywords += el;
        });
        parBody += '<p><a target="_blank" href="' + article.uri + '">View original article</a></p>';
        
        parBody += '<div class=""><div class="entities"><strong>Entities: </strong>' + entities + '</div></div>';
        parBody += '<div class=""><div class="entities"><strong>Keywords: </strong>' + keywords + '</div></div>';
        
        $("#popup-gallery .slide .article-section div.body").html(parBody);
        // udpate tooltips
        $("[data-toggle=tooltip]").tooltip({ "html": true });
    });
    $("#popup-gallery .slide .article-section .content .body").height($("#popup-gallery .slides").height() - 5 - $("#popup-gallery .slides .content h3").height() - $("#popup-gallery .slides .content h2").height() - $("#popup-gallery .slides .content .published").height() - 60);
};

// return event uri parsed from window url (e.g. http://eventregistry.ijs.si/event/8619?lang=eng -> 8619)
function getEventUri() {
    return window.location.pathname.replace(/\/event\//gi, "");
}

// get the value of an argument from the url
function getURLParameterByName(name, defaultVal) {
    if (arguments.length == 1)
        defaultVal = "";
    name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
    var regexS = "[\\?&]" + name + "=([^&#]*)";
    var regex = new RegExp(regexS);
    var results = regex.exec(window.location.href);
    if (results == null)
        return defaultVal;
    else
        return decodeURIComponent(results[1].replace(/\+/g, " "));
}

function displayStats() {
    $.getJSON("/json/getStatsRecent", function(data) {
        var report = "Number of articles: " + data.articles + "\n";
        report += "Number of concepts (entities + topics): " + data.concepts + "\n";
        report += "Number of new sources: " + data.sources + "\n";
        report += "Number of stories: " + data.stories + "\n";
        alert(report);
    });
}

function splitArticlesByLang(allArticles) {
    var artsByLang = {};
    $.each(allArticles, function (i, art) {
        var lang = art.lang;
        if (artsByLang[lang] == undefined)
            artsByLang[lang] = [];
        artsByLang[lang].push(art);
    })
    return artsByLang;
}

// adds window.readCookie and window.setCookie methods that can get and set cookie values
(function () {
    function setExpiration(cookieLife) {
        var today = new Date();
        var expr = new Date(today.getTime() + cookieLife * 24 * 60 * 60 * 1000);
        return expr.toGMTString();
    }
    
    function setCookie(name, value, expires, path, domain, secure) {
        document.cookie = name + "=" + escape(value) + "; ";

        if (expires == undefined)
            expires = 100;
        expires = setExpiration(expires);
        document.cookie += "expires=" + expires + "; ";
        if (path)
            document.cookie += "path=" + path + "; ";
        if (domain)
            document.cookie += "domain=" + domain + "; ";
        if (secure)
            document.cookie += "secure; ";
    }
    
    function readCookie(name) {
        c = document.cookie.split('; ');
        for (i = c.length - 1; i >= 0; i--) {
            C = c[i].split('=');
            if (C[0] == name && C[1] != "")
                return C[1];
        }
        return undefined;
    }

    window.readCookie = readCookie; // or expose it however you want
    window.setCookie = setCookie;
})();

function showWaitWhileLoading(idName, handler, object) {
    $(idName + " .wait-content").css("display", "block");
    $(idName + ' .wait-circle, ' + idName + ' .wait-circle1').removeClass('wait-stop');
    $(idName + " .content").css("display", "none");
    handler.call(object, function () {
        $(idName + " .wait-content").css("display", "none");
        $(idName + ' .wait-circle, ' + idName + ' .wait-circle1').addClass('wait-stop');
        $(idName + " .content").css("display", "block");
    });
}

// logout the current user
function logout() {
    $.get("/logout", function (foo) {
        //setCookie("fullName", "", -1);
        //setCookie("fbId", "", -1);
        //setCookie("role", "", -1);
        //updateHeader();
        window.location.reload();
    });
}

// show in the header login link or the logged in user + logout
function updateHeader() {
    var fullName = utils.getCurrentUserName();
    if (fullName) {
        fullName = decodeURIComponent(fullName);
        $("#login-info").html("<span>" + fullName + '</span>|<a href="#" style="display:inline;" onclick="logout();">Logout</a>');
    } else {
        $("#login-info").html('<a href="#">Login</a>');
        $("#login-info a").click(function() {
            window.location = window.location.origin + "/login?redirectUrl=" + window.location.pathname + window.location.search;
        });
    }
    $("#admin-dropdown-actions").css("display", utils.getCurrentUserRole() == 2 ? "show" : "none");
}

function Utils() {
    this.getTimeStamp = function () {
        return (new Date()).toString("H:mm:ss");
    }

    this.getCurrentUserName = function () {
        return readCookie("fullName");
    }
    this.getCurrentUserRole = function () {
        return readCookie("role");
    }
    this.isPowerUser = function () {
        return this.getCurrentUserRole() >= 1;
    }
    this.isAdmin = function () {
        return this.getCurrentUserRole() >= 2;
    }

    this.getCurrentUserImage = function (large) {
        var fbId = readCookie("fbId");
        if (fbId)
            return "http://graph.facebook.com/" + fbId + "/picture" + (large ? "?type=large" : "");
        return undefined;
    }

    this.getEntities = function (story, maxCount) {
        if (!story || !story.concepts) return [];
        var concepts = $.grep(story.concepts, function (concept, i) { return entityTypes[concept.type]; });
        concepts.sort(function (a, b) { return ((a.freq > b.freq) ? -1 : ((a.freq < b.freq) ? 1 : 0)); })
        return concepts.slice(0, maxCount);
    }

    this.getKeywords = function (story, maxCount) {
        if (!story || !story.concepts) return [];
        var concepts = $.grep(story.concepts, function (concept, i) { return keywordTypes[concept.type]; });
        return concepts.slice(0, maxCount);
    }

    this.getConceptLabel = function (concept, lang) {
        if (concept["label_" + lang] != undefined) return concept["label_" + lang];
        if (concept[lang] != undefined) return concept[lang];
        if (concept["label"] != undefined) return concept["label"];
        return "";
    }

    this.getArticleBody = function (article) {
        var parBody = "";
        parBody += '<div class="body-text">';
        $.each(article.body.split("\n"), function (index, par) {
            parBody += "<p>" + par + "</p>";
        });
        parBody += '</div>';
        return parBody;
    };

    this.getDateStr = function (dateStr, longFormat) {
        // todo: figure out what to do when date doesn't have a day. for now ignore the date
        if (!dateStr || dateStr.lastIndexOf('-') == dateStr.length - 1)
            return "";
        var skipYear = false;
        if (dateStr[0] == '-') {
            var date = d3.time.format("-%m-%d").parse(dateStr)
            return d3.time.format("%e %B")(date);
        }
        else {
            var date = d3.time.format("%Y-%m-%d").parse(dateStr)
            return d3.time.format(longFormat ? "%e %B, %Y" : "%d %b %Y")(date);
        }
    };

    this.getEventDate = function (event, longFormat) {
        if (!event) return "";
        var dateStr = this.getDateStr(event.eventDate, longFormat);
        return (event.eventDateEnd != "") ? dateStr + " - " + this.getDateStr(event.eventDateEnd, longFormat) : dateStr;
    }

    this.showWait = function (elContent, elWait) {
        $(elWait).text("");
        $(".wait-home .wait-content").clone().appendTo(elWait);
        $(elWait + " .wait-content").css("display", "block");
        $(elWait + " .wait-circle, .wait-circle1").removeClass('wait-stop');
        if (elContent)
            $(elContent).css("display", "none");
    }

    this.hideWait = function (elContent, elWait) {
        $(elWait).text("");
        if (elContent)
        $(elContent).css("display", "block");
    }

}
var utils = new Utils();

/*
    angular.treeview.js
*/
(function (l) {
    l.module("angularTreeview", []).directive("treeModel", function ($compile) {
        return {
            restrict: "A", link: function (a, g, c) {
                var e = c.treeModel, h = c.nodeLabel || "label", d = c.nodeChildren || "children", k = '<ul><li data-ng-repeat="node in ' + e + '"><i class="collapsed" data-ng-show="node.' + d + '.length && node.collapsed" data-ng-click="selectNodeHead(node, $event)"></i><i class="expanded" data-ng-show="node.' + d + '.length && !node.collapsed" data-ng-click="selectNodeHead(node, $event)"></i><i class="normal" data-ng-hide="node.' +
                d + '.length"></i> <span data-ng-class="node.selected" data-ng-click="selectNodeLabel(node, $event)">{{node.' + h + '}}</span><div data-ng-hide="node.collapsed" data-tree-model="node.' + d + '" data-node-id=' + (c.nodeId || "id") + " data-node-label=" + h + " data-node-children=" + d + "></div></li></ul>"; e && e.length && (c.angularTreeview ? (a.$watch(e, function (m, b) { g.empty().html($compile(k)(a)) }, !1), a.selectNodeHead = a.selectNodeHead || function (a, b) {
                    b.stopPropagation && b.stopPropagation(); b.preventDefault && b.preventDefault(); b.cancelBubble = !0; b.returnValue = !1; a.collapsed = !a.collapsed
                }, a.selectNodeLabel = a.selectNodeLabel || function (c, b) { b.stopPropagation && b.stopPropagation(); b.preventDefault && b.preventDefault(); b.cancelBubble = !0; b.returnValue = !1; a.currentNode && a.currentNode.selected && (a.currentNode.selected = void 0); c.selected = "selected"; a.currentNode = c }) : g.html($compile(k)(a)))
            }
        }
    })
})(angular);


var eventRegApp = angular.module('EventRegApp', ['ui.bootstrap', 'ui.slider', "google-maps", "angularTreeview"]);

// utility methods
eventRegApp.factory('utils', function () {
    return utils;
});

// utility methods
eventRegApp.factory('langOptions', function () {
    var langOptions = {};
    langOptions.langs = [ 
            { name: "English", val: "eng"},
            { name: "Spanish", val: "spa"},
            { name: "German",  val: "deu"},
            { name: "Chinese", val: "zho"},
            { name: "Slovene", val: "slv" }]
    langOptions.lang = langOptions.langs[0];
    return langOptions;
});

eventRegApp.controller("LangCtrl", function ($scope, $rootScope, langOptions) {
    $scope.langOptions = langOptions;
    $scope.langChanged = function () {
        $rootScope.$broadcast('langChangedBroadcast');
    }
});

eventRegApp.filter('nullToString', function () {
    var nullToStringFilter = function (input, val) {
        if (input) return input;
        return val;
    }
    return nullToStringFilter;
});
eventRegApp.filter('shorten', function () {
    return function(input, val) {
        if (!input)
            return "";
        return input.shorten(val);
    };
});
eventRegApp.filter('newlineToPar', function () {
    return function (input, val) {
        if (!input)
            return "";
        return "<p>" + input.replace(/\n/g, "</p><p>") + "</p>";
    }
});
eventRegApp.filter('numberWithCommas', function () {
    return function (input, val) {
        if (input == undefined)
            return "";
        return numberWithCommas(input);
    }
});
eventRegApp.filter('toFixed', function () {
    return function (input, val) {
        if (!input)
            return "";
        return input.toFixed(val);
    };
});
eventRegApp.filter('beautifyCategoryLabel', function () {
    return function (input, val) {
        if (!input)
            return "";
        return input.replace(/_/g, " ").trimLStr("/").replace(/\//g, " &rarr; ");
    }
});

eventRegApp.directive('conceptlist', function () {
    return {
        restrict: "E",
        replace: true,
        link: function (scope, element, attrs, controller) {
            var data = (attrs.source == "entities") ? scope.utils.getEntities(scope.event, 10) : scope.utils.getKeywords(scope.event, 10);
            var output = "";
            $.each(data, function(i, item) {
                output += '<a href="#" data-type="{0}" data-uri="{1}" tooltip-html-unsafe="<b>Importance</b>: {2}"><span class="label arrowed">{3}</span></a>'.format(item.type, item.uri, item.score, scope.utils.getConceptLabel(item, scope.langOptions.lang.val))
            })
            element.html(output);
        }
    };
});

eventRegApp.directive('articlecounts', function () {
    return {
        restrict: "E",
        replace: true,
        link: function (scope, element, attrs, controller) {
            var c = scope.event.articleCounts;
            var output = "<span>{0} ({1} eng, {2} ger, {3} spa, {4} chi, {5} slo)<span>".format(c.total ? c.total : 0, c.eng ? c.eng : 0, c.deu ? c.deu : 0, c.spa ? c.spa : 0, c.zho ? c.zho : 0, c.slv ? c.slv : 0);
            element.html(output);
        }
    };
});


$().ready(function () {
    try { updateHeader(); } catch (e) { }

    $(".dlg-scroll").on("show", function () {
        $("body").addClass("modal-open");
    }).on("hidden", function () {
        $("body").removeClass("modal-open")
    });
});