
eventRegApp.controller("CompareEventsCtrl", function ($scope, $http, $browser, $location, langOptions, utils) {
    $scope.utils = utils;
    $scope.langOptions = langOptions;

    $scope.error = undefined;
    $scope.tabs = { concepts: true, locations: false, time: false, categories: false }
    $scope.minArticleCountList = [0, 10, 20, 30, 40, 50, 70, 100, 150, 200];
    $scope.eventLangs = [{ name: "Any language", val: "any" }, { name: "English", val: "eng" }, { name: "Spanish", val: "spa" }, { name: "German", val: "deu" }, { name: "Chinese", val: "zho" }, { name: "Slovene", val: "slv" }];
    $scope.searchOpt1 = {
        isOpen: false, index: 0, label: "Left group", keywords: "", minArticleCount: 0       
    }
    $scope.searchOpt2 = {
        isOpen: false, index: 1, label: "Right group", keywords: "", minArticleCount: 0
    }
    $scope.searchOpt1.eventLang = $scope.eventLangs[0];
    $scope.searchOpt2.eventLang = $scope.eventLangs[0];
    $scope.searchTabArray = [$scope.searchOpt1, $scope.searchOpt2];

    $scope.mapLeft = { center: { latitude: 46, longitude: 15 }, markers: [], zoom: 3, bounds: {} }
    $scope.mapRight = { center: { latitude: 46, longitude: 15 }, markers: [], zoom: 3, bounds: {} }
    
    $scope.queryArgs = undefined;
    
    $scope.prepareQueryArgs = function() {
        $scope.queryArgs = "";
        var queryStr = $location.search().query;
        var queryArr = queryStr ? JSON.parse(queryStr) : [{}, {}];
        for (var i = 0; i < 2; i++) {
            var args = ""
            var query = queryArr[i];
            if (query.conditions) {
                $.each(query.conditions, function(foo, element) {
                    args += "conceptUri" + i + "=" + encodeURIComponent(element.uri) + "&";
                });
            }
            if (query.keywords)
                args += "keywords" + i + "=" + encodeURIComponent(query.keywords) + "&";
            if (query.locationUri)
                args += "locationUri" + i + "=" + encodeURIComponent(query.locationUri) + "&";
            if (query.publisherUri)
                args += "publisherUri" + i + "=" + encodeURIComponent(query.publisherUri) + "&";
            if (query.date)
                args += "timeStart" + i + "=" + encodeURIComponent(query.date.start) + "&timeEnd" + i + "=" + encodeURIComponent(query.date.end) + "&";
            if (query.lang)
                args += "lang" + i + "=" + encodeURIComponent(query.lang) + "&";
            if (query.categoryUri)
                args += "categoryUri" + i + "=" + encodeURIComponent(query.categoryUri) + "&";           
            // if not all languages are checked then add the condition for language
            if (query.minArticles && query.minArticles > 0)
                args += "minArticlesInEvent" + i + "=" + query.minArticles + "&";
            if (args == "") {
                $scope.queryArgs = undefined;
                alert("There are no conditions specified in Group " + (i + 1) + ". You have to specify some before you can make a comparison.");
                return;
            }
            $scope.queryArgs += args;
        }
    };

    $scope.makeComparison = function() {
        $scope.updateUrlFromForm();
        $scope.resetData();

        $scope.prepareQueryArgs();

        if ($scope.tabs.concepts)
            $scope.compareConcepts();
        else if ($scope.tabs.locations)
            $scope.compareLocations();
        else if ($scope.tabs.categories)
            $scope.compareCategories();
    };

    // compare concepts
    $scope.compareConcepts = function () {
        if ($scope.queryArgs == undefined)
            return;
        var args = $scope.queryArgs + "action=compareConcepts";
        $.each($(".entity-types label.checked"), function(i, item) {
            args += "&conceptType=" + $(item).attr("for");
        });
        $http.get("/json/eventCompare?" + args)
            .success(function (data, status, headers, config) {
                $scope.error = data.error;
                $scope.conceptData.data = data;
                $scope.showConcepts();
            });
    };

    $scope.showConcepts = function() {
        var data = $scope.conceptData.data;
        var lang = $scope.langOptions.lang.val;
        if (data.topConc0)
            $scope.conceptData.top0.setData(data.topConc0, lang);
        if (data.topConc1)
            $scope.conceptData.top1.setData(data.topConc1, lang);
        if (data.diffConc0)
            $scope.conceptData.diff0.setData(data.diffConc0, lang);
        if (data.diffConc1)
            $scope.conceptData.diff1.setData(data.diffConc1, lang);
    };

    // compare locations
    $scope.compareLocations = function () {
        if ($scope.queryArgs == undefined)
            return;
        var args = $scope.queryArgs + "action=compareLocations";
        // trigger the resize to fix the display: none;
        $.each(window.googlemaps ? window.googlemaps : [], function(i, item) { google.maps.event.trigger(item, "resize"); })
        $http.get("/json/eventCompare?" + args)
            .success(function(data, status, headers, config) {
                $scope.locationData.data = data;
                $scope.error = data.error;
                
                $scope.mapLeft.markers = [];
                $scope.mapLeft.bounds = {};
                $scope.mapRight.markers = [];
                $scope.mapRight.bounds = {};

                var bounds = new google.maps.LatLngBounds();
                $.each(data.group0, function(foo, item) {
                    bounds.extend(new google.maps.LatLng(item.location.lat, item.location.long));
                    for (var i = 0; i < item.count; i++)
                        $scope.mapLeft.markers.push({ latitude: item.location.lat, longitude: item.location.long, title: item.location.labelEng });
                });
                if (data.group0.length >= 2)
                    $scope.mapLeft.bounds = { northeast: { latitude: bounds.getNorthEast().lat(), longitude: bounds.getNorthEast().lng() }, southwest: { latitude: bounds.getSouthWest().lat(), longitude: bounds.getSouthWest().lng() } }

                bounds = new google.maps.LatLngBounds();
                $.each(data.group1, function(foo, item) {
                    bounds.extend(new google.maps.LatLng(item.location.lat, item.location.long));
                    for (var i = 0; i < item.count; i++)
                        $scope.mapRight.markers.push({ latitude: item.location.lat, longitude: item.location.long, title: item.location.labelEng });
                });
                if (data.group1.length >= 2)
                    $scope.mapRight.bounds = { northeast: { latitude: bounds.getNorthEast().lat(), longitude: bounds.getNorthEast().lng() }, southwest: { latitude: bounds.getSouthWest().lat(), longitude: bounds.getSouthWest().lng() } }
            });
    };

    // compare categories
    $scope.compareCategories = function() {
        if ($scope.queryArgs == undefined)
            return;
    };

    // compare timelines
    $scope.compareTime = function () {
        if ($scope.queryArgs == undefined)
            return;
        $http.get("/json/eventCompare?" + $scope.queryArgs + "action=compareTimeline")
            .success(function(data, status, headers, config) {
                $scope.timelineData.data = data;
                $scope.error = data.error;
                $scope.timelineData.graph.setData([{ data: data.group0, lineColor: "#0D6DD4", areaColor: "rgba(13, 109, 212, 0.37)", legendText: "Events from left group" }, { data: data.group1, lineColor: "#10B175", areaColor: "rgba(16, 177, 117, 0.37)", legendText: "Events from right group" }])
            });
    };

    $scope.getEventsTimelineTooltip = function (date, valueArray) {
        if (valueArray.length != 2)
            return "";
        return "Date: <b>{0}</b><br/>Events in left group: <b>{1}</b><br/>Events in right group: <b>{2}</b>".format(d3.time.format("%e %B %Y")(date), numberWithCommas(valueArray[0]), numberWithCommas(valueArray[1]));
    }
    
    // ///////////
    // utility methods
    $scope.resetData = function() {
        $scope.conceptData.data = undefined;
        $scope.locationData.data = undefined;
        $scope.categoryData.data = undefined;
        $scope.timelineData.data = undefined;
    };
    
    // save query params in the url
    $scope.updateUrlFromForm = function () {
        var query = [];
        for (var i = 0; i < 2; i++) {
            var params = {};
            var conditions = [];
            $.each($("#eventConditions" + i + "_tagList .tag"), function (i, element) {
                var el = $(element).data("options");
                conditions.push({ label: el.label, type: el.type, uri: el.uri });
            });
            if (conditions.length > 0)
                params.conditions = conditions;
            if ($scope.searchTabArray[i].keywords.trim() != "")
                params.keywords = $scope.searchTabArray[i].keywords.trim();
            if ($scope.searchTabArray[i].location && $scope.searchTabArray[i].location.uri) {
                params.locationUri = $scope.searchTabArray[i].location.uri;
                params.locationLabel = $scope.searchTabArray[i].location.label;
            }
            if ($scope.searchTabArray[i].publisher && $scope.searchTabArray[i].publisher.uri) {
                params.publisherUri = $scope.searchTabArray[i].publisher.uri;
                params.publisherLabel = $scope.searchTabArray[i].publisher.label;
            }
            if ($('#dp-start' + i).val() != "" && $('#dp-end' + i).val() != "") {
                var start = $('#dp-start' + i).data('datepicker').date.toString("yyyy-MM-dd");
                var end = $('#dp-end' + i).data('datepicker').date.toString("yyyy-MM-dd");
                params.date = { start: start, end: end };
            }
            if ($scope.searchTabArray[i].eventLang.val != "any")
                params.lang = $scope.searchTabArray[i].eventLang.val;
            var categoryData = $("#selected-category" + i).data("cat");
            if (categoryData) {
                params.categoryUri = categoryData.uri;
                params.categoryLabel = categoryData.label;
            }
            if ($scope.searchTabArray[i].minArticleCount > 0)
                params.minArticles = $scope.searchTabArray[i].minArticleCount;
            query.push(params);
        }
        
        $location.url("/?query=" + encodeURIComponent(JSON.stringify(query)), true);
        //window.history.replaceState(params, "", window.location.pathname + "?query=" + encodeURIComponent(JSON.stringify(params)));
    };
    
    // load the list of available categories
    $scope.loadEventCategories = function() {
        $.getJSON("/json/category?action=getCategories", function(catTree) {
            function createCats(catI, domEl, cats, skipPrefix) {
                $.each(cats, function(foo, cat) {
                    var catLi = $("<li>", { "class": cat.children ? "dropdown-submenu" : "" }).appendTo(domEl);
                    $("<a>", { "href": "#", "html": cat.label.replace(skipPrefix, "").trimLStr("/").replace(/_/g, " ").replace(/\//g, " &rarr; ") })
                        .data("cat", cat)
                        .click(function() {
                            var data = $(this).data("cat");
                            var label = data.label.replace(/_/g, " ").trimLStr("/").replace(/\//g, " &rarr; ");
                            $("#selected-category" + catI).data("cat", data);
                            $("#selected-category" + catI + " span.text").html(label);
                        })
                        .appendTo(catLi);
                    // set callback on click
                    if (cat.children) {
                        var submenu = $("<ul>", { "class": "dropdown-menu" }).appendTo(catLi);
                        createCats(catI, submenu, cat.children, cat.label);
                    }
                });
            }

            ;
            for (var _i = 0; _i < 2; _i++) {
                (function(i) {
                    var catsEl = $("#category-list" + i);
                    catsEl.html("");
                    var anyCat = $("<li>").appendTo(catsEl);
                    $("<a>", { "href": "#", "text": "Any category" })
                        .click(function() {
                            $("#selected-category" + i).data("cat", null);
                            $("#selected-category" + i + " span.text").html("Any category");
                        })
                        .appendTo(anyCat);
                    $("<li>", { "class": "divider" }).appendTo(catsEl);
                    createCats(i, catsEl, catTree.results);
                })(_i);
            }
        });
    };

    // ///////////////////////////////
    // inits on page loaded
    $().ready(function () {
        try {
            $('#navbar-compare-events').addClass("active");
        } catch (e) { }

        // setup graphs after the html is ready otherwise we cannot access the containerNm in the constructors of the graphs
        $scope.conceptData = {
            data: undefined,
            top0: new TopConceptList({ containerNm: "#top-concepts0", rectClsNm: "fill-blue2" }), top1: new TopConceptList({ containerNm: "#top-concepts1", rectClsNm: "fill-green" }),
            diff0: new DiffConceptList({ containerNm: "#diff-concepts0" }), diff1: new DiffConceptList({ containerNm: "#diff-concepts1" })
        };
        $scope.locationData = { data: undefined };
        $scope.categoryData = { data: undefined };
        $scope.timelineData = {
            data: undefined,
            graph: new DateBarCharts({ containerNm: "#events-timeline", tooltipTextCallback: $scope.getEventsTimelineTooltip })
        };
        
        for (var _i = 0; _i < 2; _i++) {
            (function(i) {
                $('#eventConditions' + i).tagsInput({ defaultText: "Topics of interest for {0} group".format(i == 0 ? "left":"right"), minInputWidth: 140 });
                // set up the autocomplete handlers for concepts
                $("#eventConditions" + i + "_tag").autocomplete({
                    source: function(request, response) {
                        var lang = $scope.langOptions.lang.val;
                        var query = "/json/suggestConcepts?prefix=" + request.term + "&lang=" + lang;
                        $.getJSON(query, function(suggList) {
                            response(suggList);
                        });
                    },

                    select: function(event, ui) {
                        $('#eventConditions' + i).addTag(ui.item.label, { focus: true, type: ui.item.type, uri: ui.item.uri, label: ui.item.label, classes: [ui.item.type, "white", "concept"] });
                        return false;
                    }
                }).data("uiAutocomplete")._renderItem = renderConceptItem;

                // set up the autocomplete handlers for location
                $("#location" + i).autocomplete({
                    source: function(request, response) {
                        var lang = $scope.langOptions.lang.val;
                        var query = "/json/suggestLocations?prefix=" + request.term + "&lang=" + lang;
                        $.getJSON(query, function(suggList) {
                            var arr = suggList.country[lang];
                            arr.push.apply(arr, suggList.city[lang]);
                            response(arr);
                        });
                    },

                    select: function(event, ui) {
                        $scope.searchTabArray[i].location = { uri: ui.item.wikiUri, label: ui.item.label };
                        $scope.$apply();
                        return false;
                    }
                }).data("uiAutocomplete")._renderItem = renderLocationItem;

                // set up the autocomplete handlers for location
                $("#publisher" + i).autocomplete({
                    source: function(request, response) {
                        var query = "/json/suggestSources?prefix=" + request.term;
                        $.getJSON(query, function(suggList) {
                            response(suggList);
                        });
                    },

                    select: function(event, ui) {
                        $scope.searchTabArray[i].publisher = { uri: ui.item.uri, label: ui.item.title };
                        $scope.$apply();
                        return false;
                    }
                }).data("uiAutocomplete")._renderItem = renderPublisherItem;

                // date picker for date range
                $('#dp-start' + i).datepicker({ format: 'yyyy-mm-dd' });
                $('#dp-end' + i).datepicker({ format: 'yyyy-mm-dd' });
            })(_i);
        }

        // load known categories
        $scope.loadEventCategories();

        // fix the overflow hidden due to accordion
        $scope.$watch("searchOpt1.isOpen", function () {
            if ($scope.searchOpt1.isOpen)
                setTimeout(function () { $("#group0 .accordion-body").css("overflow", "visible"); }, 250);
            else
                $("#group0 .accordion-body").css("overflow", "hidden");
        });
        $scope.$watch("searchOpt2.isOpen", function () {
            if ($scope.searchOpt2.isOpen)
                setTimeout(function () { $("#group1 .accordion-body").css("overflow", "visible"); }, 250);
            else
                $("#group1 .accordion-body").css("overflow", "hidden");
        });

        // changing the checkboxes for concepts
        var cbUpdateTimer;
        // refresh the entity graph if update button is clicked
        $(".entity-types .checkbox").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
            clearTimeout(cbUpdateTimer);
            cbUpdateTimer = setTimeout(function () {
                $scope.makeComparison();
                clearTimeout(cbUpdateTimer);
            }, 1000);
        });
        
        // load data from url if present
        var queryStr = $location.search().query;
        if (queryStr) {
            var query = queryStr ? JSON.parse(queryStr) : [{}, {}];
            for (var i = 0; i < 2; i++) {
                var params = query[i];
                if (params.conditions) {
                    $.each(params.conditions, function(foo, element) {
                        $('#eventConditions' + i).addTag(element.label, { focus: true, type: element.type, uri: element.uri, label: element.label, classes: [element.type, "white", "concept"] });
                    });
                }
                if (params.keywords)
                    $scope.searchTabArray[i].keywords = params.keywords;
                if (params.locationUri && params.locationLabel)
                    $scope.searchTabArray[i].location = { uri: params.locationUri, label: params.locationLabel };
                if (params.publisherUri && params.publisherLabel)
                    $scope.searchTabArray[i].publisher = { uri: params.publisherUri, label: params.publisherLabel };
                if (params.date) {
                    $('#dp-start' + i).data('datepicker').setValue(params.date.start);
                    $('#dp-end' + i).data('datepicker').setValue(params.date.end);
                }
                var eventLang = $.grep($scope.eventLangs, function (el, ind) { return el.val == params.lang; });
                if (eventLang.length > 0)
                    $scope.searchTabArray[i].eventLang = eventLang[0];
                else
                    $scope.searchTabArray[i].eventLang = $scope.eventLangs[0];
                if (params.categoryUri != undefined) {
                    var label = params.categoryLabel.replace(/_/g, " ").trimLStr("/").replace(/\//g, " &rarr; ");
                    $("#selected-category" + i).data("cat", { uri: params.categoryUri, label: params.categoryLabel });
                    $("#selected-category" + i + " span.text").html(label);
                }
                if (params.minArticles)
                    $scope.searchTabArray[i].minArticleCount = params.minArticles;
                
                if (params.date || params.keywords || params.categoryUri || params.minArticles || params.locationUri || params.publisherUri)
                    $scope["searchOpt" + (i + 1)].isOpen = true;
            }
            $scope.updateUrlFromForm();
            //$scope.prepareQueryArgs();
            $scope.makeComparison();
        }
        //$scope.tabs.concepts = true;
        //$scope.$apply();
        //if (Object.keys(query[0]).length > 0 && Object.keys(query[1]).length > 0)
        //    $scope.makeComparison();
    });
});