function showWait() {
    $("#wait").css("display", "block");
    $('.circle, .circle1').removeClass('wait-stop');
}

function hideWait() {
    $('.circle, .circle1').addClass('wait-stop');
    $("#wait").css("display", "none");
}

function getConceptType(item) {
    if (typeof item == "string")
        return item.substring(0, item.indexOf(":"));
    else {
        var uri = decodeURIComponent($(item).parents("tr").attr("id"));
        return getConceptType(uri);
    }
}

// remove from the concept uri the prefix that determines the type of the concept
function getConceptUrl(item) {
    if (typeof item == "string")
        return item;
    else {
        var uri = decodeURIComponent($(item).parents("tr").attr("id"));
        return getConceptUrl(uri);
    }
}

function saveFixes() {
    showWait();
    //$("#conceptTable").html("");
    $.getJSON('/jsonAdmin/editConcept?action=saveFixes', function (ret) {
        hideWait();
        setStatus(ret); 
    });
}

function loadFixes() {
    clearData();
    showWait();
    $.getJSON('/jsonAdmin/editConcept?action=loadFixes', function (ret) {
        hideWait();
        setStatus(ret);
        showConceptsFromStart();
    });
}

function applyChanges() {
    clearData();
    showWait();
    $(".btn").addClass("disabled");
    $("input").prop('disabled', true);
    $.getJSON('/jsonAdmin/editConcept?action=applyChanges', function (ret) {
        hideWait();
        $(".btn").removeClass("disabled");
        $("input").prop('disabled', false);
        setStatus(ret);
        $.get('/json/suggestConcepts?action=updatePrefixes');
    });
}

function callAction(methodName) {
    showWait();
    $.getJSON('/jsonAdmin/editConcept?action=' + methodName, function(ret) {
        hideWait();
        setStatus(ret);
    });
}

function wikiImportDesc() {
    callAction("importWikiConceptDescriptions");
}

function wikiImportImages() {
    callAction("importWikiConceptImages");
}

function geoDataImportConcepts() {
    callAction("importGeoConceptTypes");
}

function wikiImportTypes() {
    callAction("importWikiConceptTypes");
}

function wikiImportRedirects() {
    callAction("importWikiConceptRedirects");
}

function setDefaultLabelValues(lang) {
    $.getJSON('/json/concept?action=setDefaultLabels&lang=' + lang, function (ret) {
        alert(ret.info ? ret.info : ret.error);
    });
}

function clearData() {
    $("#status-line").html("&nbsp;");
    $("#conceptTable").html("");
    $("#prevNextPage").css("display", "none");
}

function setStatus(status) {
    $("#status-line").html(status.info ? status.info : status.error);
    if (status.info)
        $("#status-line").removeClass("error");
    else
        $("#status-line").addClass("error");
}

// toggle concept enabled/disabled
function toggleConcept(concept) {
    var url = getConceptUrl(concept);

    var buttonText = $(concept).html();
    if (buttonText == "Disable") {
        $(concept).html("Enable");
        $.getJSON('/jsonAdmin/editConcept?action=ignoreConcept&Uri=' + url, function(ret) {
            setStatus(ret);
            $(concept).parents("tr").find("#conceptTable .btn-primary").addClass("disabled");
            $(concept).parents("tr").find("select").prop("disabled", true);
            $(concept).parents("tr").css("background-color", "lightgray");
        });
    } else {
        $(concept).html("Disable");
        $.getJSON('/jsonAdmin/editConcept?action=unignoreConcept&Uri=' + url, function (ret) {
            setStatus(ret);
            $(concept).parents("tr").find("#conceptTable .btn-primary").removeClass("disabled");
            $(concept).parents("tr").find("select").prop("disabled", false);
            $(concept).parents("tr").css("background-color", "inherit");
        });
    }
}

function editConceptImages(concept) {
    var url = encodeURIComponent(getConceptUrl(concept));
    $.getJSON('/jsonAdmin/editConcept?action=getConceptInfo&Uri=' + url, function (ret) {
        if (ret.error) {
            alert(ret.error);
            return;
        }
        $('#edit-concept-image').modal("show");
        var out = "";
        $.each(ret.images, function(i, image) {
            out += '<div class="row">\n<div class="col-md-12">\n';
            out += '<label class="radio"><input name="img-options" type="radio" ' + (ret.primaryImage == image ? "checked" : "") + ' value="' + encodeURIComponent(image) + '" />\n';
            out += '<img class="concept-image" src="' + image + '" /></label></div>	</div>\n';
        });
        $('#edit-image-body').html(out);
        buildCheckBoxesAndRadios("#edit-image-modal .modal-body");
        setupLabel();
        $('#edit-image-modal').modal("show");
        $("#edit-image-save").unbind('click');
        $("#edit-image-save").bind('click', function () {
            var imageUrl = $("#edit-image-modal .modal-body label.checked").children("input").val();
            imageUrl = decodeURIComponent(imageUrl);
            if (imageUrl == "customUrl")
                imageUrl = $('#textboxCustom').val();
            $.getJSON('/jsonAdmin/editConcept?action=setConceptImage&SetAsPrimary=1&Uri=' + url + "&Image=" + encodeURIComponent(imageUrl), function (ret) {
                setStatus(ret);
            });
        });
    });
}

function editConceptDesc(concept) {
    var url = encodeURIComponent(getConceptUrl(concept));
    $.getJSON('/jsonAdmin/editConcept?action=getConceptInfo&Uri=' + url, function (ret) {
        if (ret.error) {
            alert(ret.error);
            return;
        }
        $("#edit-desc-textarea").val(ret.description);
        $('#edit-desc-modal').modal("show");
        
        $("#edit-desc-save").unbind('click');
        $("#edit-desc-save").bind('click', function() {
            var desc = $("#edit-desc-textarea").val();
            $.getJSON('/jsonAdmin/editConcept?action=setConceptDescription&Uri=' + url + "&Desc=" + encodeURIComponent(desc), function (ret) {
                setStatus(ret);
            });
        });
    });
}

var suggConcPage = 0;

function showPrevPage() {
    if (suggConcPage == 0)
        return;
    if ($("#conceptTable input.dirty").length == 0) {
        suggConcPage -= 1;
        showConcepts();
    } else {
        // ask the user if he really wishes to loose the unsaved changes
        $('#ignore-unsaved').unbind('click');
        $('#ignore-unsaved').bind('click', function () {
            suggConcPage -= 1;
            showConcepts();
        })
        $('#unsaved-changes').modal('show');
    }   
}

function showNextPage() {
    if ($("#conceptTable input.dirty").length == 0) {
        suggConcPage += 1;
        showConcepts();
    } else {
        // ask the user if he really wishes to loose the unsaved changes
        $('#ignore-unsaved').unbind('click');
        $('#ignore-unsaved').bind('click', function () {
            suggConcPage += 1;
            showConcepts();
        })
        $('#unsaved-changes').modal('show');
    }
}

// remove the markings that say that we have changed the element
function clearDirtyFlag(element) {
    $(element).parent().children("input.dirty").removeClass("dirty").prop("title", "");
}

function showConcepts() {
    showWait();
    var prefix = $('#conceptPrefix').val();
    var query = "/json/suggestConcepts?prefix=" + prefix + "&labelLang=LabelEng&labelLang=LabelSpa&labelLang=LabelDeu&labelLang=LabelSlv";
    query += "&lang=" + $("#lang").val();
    query += $("#entities").parent().hasClass("checked") ? "&source=entities" : "";
    query += $("#keyword").parent().hasClass("checked") ? "&source=keyword" : "";
    query += $("#wiki").parent().hasClass("checked") ? "&source=wiki" : "";
    query += "&count=20&page=" + suggConcPage;
    $("#pageNumLabel").html("Page " + (suggConcPage + 1));
    $.getJSON(query, function (suggList) {
        hideWait();
        $("#prevNextPage").css("display", "block");
        var out = "";
        out += '<table class="table table-condensed table-bordered">';
        out += "<thead><tr><th>Disable</th><th>Type</th><th>Concept url</th><th>Img</th><th>Desc</th><th>English label</th><th>Spanish label</th><th>German label</th><th>Slovene label</th></tr></thead>";

        var langs = ["LabelEng", "LabelSpa", "LabelDeu", "LabelSlv"];
        $("#conceptTable").html("");
        $.each(suggList, function (i, item) {
            out += '<tr id="' + encodeURIComponent(item.uri) + '">';
            out += "<td><button class='btn-link btn-small' href='' onclick='toggleConcept($(this));' >Disable</button></td>";
            out += "<td><select>";
            out += "<option value='org' " + (item.type != "org" ? "" : "selected") + ">Organization</option>";
            out += "<option value='person' " + (item.type != "person" ? "" : "selected") + ">Person</option>";
            out += "<option value='loc' " + (item.type != "loc" ? "" : "selected") + ">Location</option>";
            out += "<option value='keyword' " + (item.type != "keyword" ? "" : "selected") + ">Keyword</option>";
            out += "<option value='wiki' " + (item.type != "wiki" ? "" : "selected") + ">Wiki</option>";
            out += "</select></td>";

            var url = item.uri.substring(item.uri.indexOf(":") + 1);
            var urlLast = url.replace(/.*wikipedia.org\/wiki\//g, "");
            out += '<td><div class="input-prepend input-append margin5">' +
                '<button class="btn btn-primary btn-small openurl-btn" type="button" title="Open wikipedia page for concept"><span class="icon-share-alt"/></button>' +
                '<input style="width: 150px;" type="text" value="' + urlLast + '" title="' + url + '">' +
                '<button class="btn btn-primary btn-small reassign-btn" type="button" title="Reassign concept to a new url"><span class="icon-ok"/></button>' +
                '</div></td>';
            
            out += "<td><button class='btn-link btn-small' href='' onclick='editConceptImages($(this));' >Img</button></td>";
            out += "<td><button class='btn-link btn-small' href='' onclick='editConceptDesc($(this));' >Desc</button></td>";
            
            $.each(langs, function (i, lang) {
                //out += "<td><span class='text'>" + item[lang] + "</span><button class='btn pull-right edit-label " + lang + "' onclick='editLabel($(this))' ><span class='fui-new-16' /></button></td>";
                out += "<td><div class='input-append margin5'><input class='" + lang + "' type='text' value='" + item[lang] + "'><button class='btn btn-primary btn-small rename-btn' type='button' title='Update concept label' ><span class='icon-ok'/></button></div></td>";
            });
            out += "</tr>";
        });
        out += "</table>";
        $("#conceptTable").html(out);
		
        // open concept url
        $("#conceptTable .openurl-btn").bind("click", function () {
            var url = getConceptUrl(this);
            window.open(url, '_blank');
        });

        // reassign concept url to new url
        $("#conceptTable .reassign-btn").bind("click", function () {
            if ($(this).hasClass("disabled"))
                return;
            var htmlItem = $(this);
            var fromUrl = getConceptUrl(this);
            var toUrl = $(this).parents(".input-append").find("input").val();
            if (toUrl == "")
                return;
            clearDirtyFlag(this);
            // if the user specified the whole wikipedia url use it - otherwise prepend the base url address
            if (toUrl.indexOf("http://") != 0)
                toUrl = "http://en.wikipedia.org/wiki/" + toUrl;
            var args = { "action": "rewriteConcept", "FromUri": fromUrl, "ToUri": toUrl};
            $.getJSON('/jsonAdmin/editConcept', args, function (ret) {
                setStatus(ret);
                // if everything went ok then update the id
                if (ret.info)
                    htmlItem.parents("tr").attr("id", encodeURIComponent(toUrl));
            });
        });

        // change the type of the concept
        $('#conceptTable select').change(function () {
            var htmlItem = $(this);
            var url = getConceptUrl(this);
            var type = $(this).val();
            var args = { "action": "updateConcept", "Uri": url, "Type": type };
            $.getJSON('/jsonAdmin/editConcept', args, function (ret) {
                setStatus(ret);
                // if everything went ok then update the id
                if (ret.info) 
                    htmlItem.parents("tr").attr("id", encodeURIComponent(url));
            })
                .fail(function (jqxhr, textStatus, error) {
                    var err = textStatus + ', ' + error;
                    setStatus("Request Failed: " + err);
                });
        });
		
        // change the label of the concept
        $("#conceptTable .rename-btn").bind("click", function () {
            if ($(this).hasClass("disabled"))
                return;
            clearDirtyFlag(this);
            var url = getConceptUrl(this);
            var type = getConceptType(this);
            var newLabel = $(this).parents(".input-append").find("input").val();
            var args = { "action": "updateConcept", "Uri": url, "Type": type };
            var labelLang = getLabelLang(this);
            args[labelLang] = newLabel;
            $.getJSON('/jsonAdmin/editConcept', args, function (ret) {
                setStatus(ret);
            })
            .fail(function (jqxhr, textStatus, error) {
                var err = textStatus + ', ' + error;
                setStatus("Request Failed: " + err);
            });
        });
        
        // mark input boxes as changed whenever the user changes the content. used to alert the user of unsaved changes
        $('#conceptTable input').bind('input propertychange', function () {
            $(this).addClass("dirty");
            $(this).prop("title", "Note: The field contains unsaved changes. Press the icon next to it to save them.");
        });
    });
}

function getLabelLang(element) {
    var inputClass = $(element).parent().find("input").attr("class");
    inputClass = inputClass.replace("dirty", "");
    inputClass = inputClass.trim();
    return inputClass;
}

function showConceptsFromStart() {
    suggConcPage = 0;
    showConcepts();
}

$().ready(function () {
    var query = getURLParameterByName("q");
    if (query != "")
        $("#conceptPrefix").val(query);
    var lang = getURLParameterByName("lang");
    if (lang != "")
        $("#lang").val(lang);
    
    $('#conceptPrefix').bind('input propertychange', function () {
        var $this = $(this);
        var delay = 500; // 0.5 sec delay after last input

        clearTimeout($this.data('timer'));
        $this.data('timer', setTimeout(function() {
            $this.removeData('timer');
            //var prefix = $('#conceptPrefix').val();
            //if (prefix != "") {
            showConceptsFromStart();
            //}
        }, delay));
    });
    $("label.checkbox").bind("click", function () { showConceptsFromStart(); });
    
    // show most popular concepts
    showConcepts();
});