
eventRegApp.controller("ConceptClassEditorCtrl", function ($scope, $http, langOptions, utils) {
    $scope.utils = utils;
    $scope.langOptions = langOptions;
    
    $scope.classTypes = [{ val: "custom", name: "Custom" }, { val: "dbpedia", name: "DBpedia" }];
    $scope.classType = $scope.classTypes[0];
    
    $scope.classTree = undefined;

    $scope.error = undefined;
    $scope.info = undefined;
    
    $scope.classInfo = undefined;
    $scope.suggestedConcepts = undefined;
    
    // remove concept class prefixes
    $scope.updateConceptClassPrefixes = function(callback) {
        $http.get('/json/suggestConceptClasses?action=updatePrefixes')
            .success(function (data, status, headers, config) {
                callback && callback();
            });
    }

    // load the tree of classes
    $scope.loadClassTree = function () {
        $scope.classInfo = undefined;
        $http.get('/jsonAdmin/conceptClassAdmin?action=getClassTree&type=' + $scope.classType.val)
            .success(function (data, status, headers, config) {
                $scope.classTree = data.classes;
                $scope.error = data.error;
                $scope.info = data.info;
                $scope.loadClassInfo();
            });
    }
    
    // load info for the class
    $scope.loadClassInfo = function () {
        $scope.classInfo = undefined;
        if ($scope.currentNode == undefined)
            return;
        $http.get('/json/conceptClass?action=getClass&uri=' + encodeURIComponent($scope.currentNode.uri))
            .success(function (data, status, headers, config) {
                data.concepts.sort(function(a, b) {
                    return a.label.toUpperCase().localeCompare(b.label.toUpperCase());
                });
                $scope.suggestedConcepts = undefined;
                $scope.classInfo = data;
                $scope.error = data.error;
                $scope.info = data.info;
                $scope.buildTableUsingClassTree();
            });
    }
    
    // import all dbpedia classes
    $scope.importDbpediaClasses = function() {
        $http.get('/jsonAdmin/conceptClassAdmin?action=importClasses')
            .success(function (data, status, headers, config) {
                $scope.loadClassTree();
                $scope.error = data.error;
                $scope.info = data.info;
            });
    }
    
    // import dbpedia instances - concepts for each of the dbpedia class
    $scope.importDbpediaInstances = function () {
        $http.get('/jsonAdmin/conceptClassAdmin?action=importInstances')
            .success(function (data, status, headers, config) {
                $scope.loadClassTree();
                $scope.error = data.error;
                $scope.info = data.info;
            });
    }
    
    // create a new class
    $scope.addClass = function () {
        $scope.parentClass = undefined;
        $('#edit-class-dlg h3').val("Add New Class");
        $('#edit-class-dlg #label').val("");
        $('#edit-class-dlg #description').val("");
        $("#edit-class-dlg #parentClass_tag").val("");
        $('#edit-class-dlg').modal({});
        $('#edit-class-dlg .modal-footer button.btn-primary').click(function () {
            var vals = "";
            vals += '&label=' + encodeURIComponent($('#edit-class-dlg #label').val());
            vals += '&description=' + encodeURIComponent($('#edit-class-dlg #description').val());
            vals += '&parentUri=' + encodeURIComponent($scope.parentClass ? $scope.parentClass.uri : "");
            var req = '/jsonAdmin/conceptClassAdmin?action=createClass' + vals;
            $http.get(req).success(function (data, status, headers, config) {
                $scope.error = data.error;
                $scope.info = data.info;
                if (data.error == undefined) {
                    $scope.loadClassTree();
                    $scope.updateConceptClassPrefixes();
                }
            });
        });
    }
    
    // edit the currently selected class
    $scope.editClass = function () {
        $('#edit-class-dlg h3').val("Edit Class");
        $('#edit-class-dlg #label').val($scope.classInfo.label);
        $('#edit-class-dlg #description').val($scope.classInfo.description);
        $scope.parentClass = $scope.getParentClass($scope.classInfo.uri);
        if ($scope.parentClass) {
            $("#edit-class-dlg #parentClass_tag").val($scope.parentClass.label);
        }
        $('#edit-class-dlg').modal({});
        $('#edit-class-dlg .modal-footer button.btn-primary').click(function () {
            var changes = "";
            changes += '&uri=' + encodeURIComponent($scope.classInfo.uri);
            changes += '&label=' + encodeURIComponent($('#edit-class-dlg #label').val());
            changes += '&description=' + encodeURIComponent($('#edit-class-dlg #description').val());
            changes += '&parentUri=' + encodeURIComponent($scope.parentClass ? $scope.parentClass.uri : "");
            var req = '/jsonAdmin/conceptClassAdmin?action=updateClass' + changes;
            $http.get(req).success(function (data, status, headers, config) {
                $scope.error = data.error;
                $scope.info = data.info;
                if (data.error == undefined) {
                    $scope.loadClassInfo();
                    // update the prefixes if we've changed the label
                    if ($scope.parentClass.label != $('#edit-class-dlg #label').val())
                        $scope.updateConceptClassPrefixes();
                }
            });
        });
    }
    
    // delete currently selected class
    $scope.deleteClass = function() {
        if ($scope.currentNode == undefined)
            return;
        $http.get('/jsonAdmin/conceptClassAdmin?action=removeClass&uri=' + encodeURIComponent($scope.currentNode.uri))
            .success(function (data, status, headers, config) {
                $scope.loadClassTree();
            });
    }
   
    // add the concept to class
    $scope.addConceptToClass = function (concept) {
        if ($scope.currentNode == undefined)
            return;
        $http.get('/jsonAdmin/conceptClassAdmin?action=addInstanceToClass&uri=' + encodeURIComponent($scope.currentNode.uri) + "&conceptUri=" + encodeURIComponent(concept.uri))
            .success(function (data, status, headers, config) {
                $scope.error = data.error;
                $scope.info = data.info;
                if (data.error == undefined) {
                    $scope.classInfo.concepts.push(concept);
                    var html = "<tr><td><button class='remove btn btn-link' placeholder='remove concept from class'></button></td><td><a target='_blank' href='{0}'>{1}</a></td></tr>".format(concept.uri, concept.label);
                    $("#concepts-table table tbody").append(html);
                    $scope.bindRemoveClicks();
                }
            });
    }
    
    // add all instances from a particular class
    $scope.addInstancesFromClass = function(classUri) {
        if ($scope.currentNode == undefined)
            return;
        $http.get('/jsonAdmin/conceptClassAdmin?action=addInstancesFromClass&targetClassUri=' + encodeURIComponent($scope.currentNode.uri) + "&sourceClassUri=" + encodeURIComponent(classUri))
            .success(function (data, status, headers, config) {
                $scope.error = data.error;
                $scope.info = data.info;
                if (data.error == undefined)
                    $scope.loadClassInfo();
            });
    }
    
    // remove all concept instances from the class
    $scope.removeAllInstancesFromClass = function (classUri) {
        if ($scope.currentNode == undefined)
            return;
        $http.get('/jsonAdmin/conceptClassAdmin?action=removeAllInstancesFromClass&uri=' + encodeURIComponent($scope.currentNode.uri) )
            .success(function (data, status, headers, config) {
                $scope.error = data.error;
                $scope.info = data.info;
                if (data.error == undefined)
                    $scope.loadClassInfo();
            });
    }
    
    $scope.computeSuggestions = function() {
        if ($scope.classInfo == undefined)
            return;
        var uris = "";
        var ignoredUris = "";
        for (var n = 0; n < $scope.classInfo.concepts.length && n < 100; n++) {
            uris += "&uri=" + encodeURIComponent($scope.classInfo.concepts[n].uri);
            ignoredUris += "&ignoreUri=" + encodeURIComponent($scope.classInfo.concepts[n].uri);
        }
        var types = "";
        $.each($(".entity-types label.checked"), function (i, item) {
            types += "&conceptType=" + $(item).attr("for");
        });
        
        $http.get('/json/concept?action=getRelatedConcepts' + types + uris + ignoredUris)
            .success(function (data, status, headers, config) {
                $scope.error = data.error;
                $scope.info = data.info;
                if (data.error == undefined)
                    $scope.suggestedConcepts = data;
            });
    }
    
    $scope.addSuggestedConcept = function(concept) {
        $scope.suggestedConcepts = $.grep($scope.suggestedConcepts, function (conc) { return concept.uri != conc.uri });
        $scope.addConceptToClass(concept);
    }

    // ////////////////////////
    // HELPER methods
    $scope.buildTableUsingClassTree = function () {
        console.log("printing concept list:");
        var html = '<table class="table table-condensed table-striped"><col width="30px" /><col /><thead><tr><th></th><th>Concept</th></tr></thead>';
        for (var n = 0; n < $scope.classInfo.concepts.length; n++) {
            // ng-click="removeConceptFromClass(concept)"
            var item = $scope.classInfo.concepts[n];
            html += "<tr><td><button class='remove btn btn-link' placeholder='remove concept from class'></button></td><td><a class='concept black {2}' target='_blank' href='{0}'>{1}</a></td></tr>".format(item.uri, item.label, item.type);
            console.log(item.uri);
        }
        html += '</table>';
        $("#concepts-table").html(html);

        $scope.bindRemoveClicks();
    }
    
    $scope.bindRemoveClicks = function() {
        // remove the concept from the class
        $("#concepts-table button.remove").click(function (el) {
            var removeEl = $(this).parents("tr");
            var conceptUri = removeEl.find("a").attr("href");
            $http.get('/jsonAdmin/conceptClassAdmin?action=removeInstanceFromClass&uri=' + encodeURIComponent($scope.currentNode.uri) + "&conceptUri=" + encodeURIComponent(conceptUri))
            .success(function (data, status, headers, config) {
                $scope.error = data.error;
                $scope.info = data.info;
                if (data.error == undefined)
                    removeEl.remove();
            });
        });
    }
    
    $scope.getParentClass = function(uri) {
        function getParent(parent) {
            if (!parent.children)
                return undefined;
            for (var i = 0; i < parent.children.length; i++) {
                if (parent.children[i].uri == uri)
                    return parent;
            }
            for (var i = 0; i < parent.children.length; i++) {
                var ret = getParent(parent.children[i]);
                if (ret)
                    return ret;
            }
            return undefined;
        }
        
        for (var i = 0; i < $scope.classTree.length; i++) {
            var ret = getParent($scope.classTree[i]);
            if (ret)
                return ret;
        }
        return undefined;
    }

    $(document).ready(function () {
        $scope.loadClassTree();

        $scope.$watch("currentNode", function () {
            $scope.loadClassInfo();
        });
        
        $("#addConcept").autocomplete({
            source: function (request, response) {
                //var lang = $scope.langOptions.lang.val;
                var query = "/json/suggestConcepts?prefix=" + request.term; // + "&lang=" + lang;
                $.getJSON(query, function (suggList) {
                    response(suggList);
                });
            },

            select: function (event, ui) {
                $scope.addConceptToClass(ui.item);
                $("#addConcept").val("");
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderConceptItem;
        
        $("#addInstancesFromClass").autocomplete({
            source: function (request, response) {
                var query = "/json/suggestConceptClasses?prefix=" + request.term;
                $.getJSON(query, function (suggList) {
                    response(suggList);
                });
            },

            select: function (event, ui) {
                $scope.addInstancesFromClass(ui.item.uri);
                $("#addInstancesFromClass").val("");
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderConceptItem;
        
        
        $("#parentClass").autocomplete({
            source: function (request, response) {
                var query = "/json/suggestConceptClasses?prefix=" + request.term + "&source=" + $scope.classType.val;
                $.getJSON(query, function (suggList) {
                    response(suggList);
                });
            },

            select: function (event, ui) {
                $scope.parentClass = { uri: ui.item.uri, label: ui.item.label };
                $scope.$apply();
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderConceptItem;
        
        // refresh the entity graph if update button is clicked
        $(".entity-types .checkbox").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
        });

    });
});