
eventRegApp.controller("StatsCtrl", function ($scope, $http, langOptions, utils) {
    $scope.stats = undefined;
    $scope.clusterQ = undefined;

    $scope.error = undefined;
    $scope.langOptions = langOptions;

    $scope.utils = utils;
    $scope.isComputing = false;
    
    $scope.updateGraphs = function() {
        var graphs = $scope.graphs;
        if ($scope.accs.articlesOverTime) {
            graphs.articlesOverTime.clearData();
            graphs.articlesOverTime.addData({ dataArr: $scope.stats.articleCountDist, tooltipFormat: "Date: <strong>{0}</strong><br/>Article count: <strong>{1}</strong>" });
        }
        if ($scope.accs.storiesOverTime) {
            graphs.storiesOverTime.clearData();
            graphs.storiesOverTime.addData({ dataArr: $scope.stats.storyCountDist, tooltipFormat: "Date: <strong>{0}</strong><br/>Story count: <strong>{1}</strong>" });
        }
        if ($scope.accs.eventsOverTime) {
            graphs.eventsOverTime.clearData();
            var validItems = $.grep($scope.stats.eventCountDist, function (item, i) {
                return item.date.length > 4 && item.date.substr(0, 4) > 2000 && item.date.substr(0, 4) < 2050;
            });

            graphs.eventsOverTime.addData({ dataArr: validItems, tooltipFormat: "Date: <strong>{0}</strong><br/>Event count: <strong>{1}</strong>" });
            graphs.mergedEventsOverTime.clearData();
            var ratios = $.map(validItems, function (item, i) { return { date: item.date, val: item.val > 0 ? 100*item.mergedVal / item.val : 0 } });
            graphs.mergedEventsOverTime.addData({ dataArr: ratios, tooltipFormat: "Date: <strong>{0}</strong><br/>Percent of merged events: <strong>{1}</strong>" });
        }
        if ($scope.accs.spaningStories) {
            var data = $.map($scope.stats.storyDaySpanDist, function (d) { return { key: d.daySpan, val: d.val } });
            graphs.spaningStories.clearData();
            graphs.spaningStories.addData({ dataArr: data, tooltipFormat: "<strong>{1}</strong> stories<br/>span over <strong>{0}</strong> days" });
        }
        if ($scope.accs.storySizes) {
            var data = $.map($scope.stats.storySizeDist, function (d) { return { key: d.articleCount, val: d.val } });
            graphs.storySizes.clearData();
            graphs.storySizes.addData({ dataArr: data, tooltipFormat: "<strong>{1}</strong> stories<br/>have <strong>{0}</strong> articles" });
        }
    }

    $scope.computeEventStats = function () {
        $scope.isComputing = true;
        $http.get("/jsonAdmin/eventAdmin?action=computeEventStats")
            .success(function (retVal, status, headers, config) {
                $scope.isComputing = false;
                $scope.error = retVal.error;
                $scope.getEventStatList();
            })
            .error(function (data, status, headers, config) {
                $scope.isComputing = false;
                $scope.error = "Request timed out. Try again in a few moments.";
            })
    }

    $scope.getEventStatList = function () {
        $scope.isComputing = true;
        $http.get("/jsonAdmin/eventAdmin?action=getEventStatList")
            .success(function (retVal, status, headers, config) {
                $scope.error = retVal.error;
                $scope.eventStatList = retVal;
                $scope.isComputing = false;
            })
            .error(function (data, status, headers, config) {
                $scope.error = "Request timed out. Try again in a few moments.";
                $scope.isComputing = false; 
            })
    }

    $scope.getEventStats = function (name) {
        $scope.isComputing = true;
        $http.get("/jsonAdmin/eventAdmin?action=getEventStats&name=" + encodeURIComponent(name))
            .success(function (retVal, status, headers, config) {
                $scope.error = retVal.error;
                $scope.stats = retVal;
                $scope.isComputing = false;
            })
            .error(function (data, status, headers, config) {
                $scope.error = "Request timed out. Try again in a few moments.";
                $scope.isComputing = false;
            })
    };

    // evaluate the quality of clusters
    $scope.evaluateClusters = function () {
        $scope.isComputingClusters = true;
        $http.get("/jsonAdmin/eventAdmin?action=evaluateClusters")
           .success(function (retVal, status, headers, config) {
               $scope.error = retVal.error;
               $scope.getClusterAccuracy();
               $scope.isComputingClusters = false;
           })
           .error(function (data, status, headers, config) {
                $scope.error = "Request timed out. Try again in a few moments.";
                $scope.isComputingClusters = false;
            })
    }

    // display the quality of clusters
    $scope.getClusterAccuracy = function () {
        $http.get("/jsonAdmin/eventAdmin?action=getClusterAccuracy")
           .success(function (retVal, status, headers, config) {
               $scope.error = retVal.error;
               $scope.clusterQ = retVal;
               setTimeout(function () {
                   $scope.displayClusterQGraphs();
               }, 100);
           });
    }

    $scope.displayClusterQGraphs = function () {
        if (!$scope.clusterQ)
            return;
        var daily = $scope.clusterQ.daily;
        var precData = $.map(daily, function (d) { return { date: d.date, q1: d.precQuart1, q2: d.prec, q3: d.precQuart3, d1: d.precDec1, d9: d.precDec9 } });
        $scope.graphs.clusterQPrec.setData({ dataArr: precData });
        var recData = $.map(daily, function (d) { return { date: d.date, q1: d.recQuart1, q2: d.rec, q3: d.recQuart3, d1: d.recDec1, d9: d.recDec9 } });
        $scope.graphs.clusterQRec.setData({ dataArr: recData });
        var f1Data = $.map(daily, function (d) { return { date: d.date, q1: d.f1Quart1, q2: d.f1, q3: d.f1Quart3, d1: d.f1Dec1, d9: d.f1Dec9 } });
        $scope.graphs.clusterQF1.setData({ dataArr: f1Data });
    }

    $scope.countOverTimeTooltip = function (dataObj, dataPoint, pointIndex) {
        return dataObj.tooltipFormat.format(dataPoint.date, numberWithCommas(dataPoint.val));
    };
    
    $scope.storiesForTimeSpanTooltip = function (dataObj, dataPoint, pointIndex) {
        return dataObj.tooltipFormat.format(dataPoint.key, numberWithCommas(dataPoint.val));
    };
    
    $scope.storiesForStorySizeTooltip = function (dataObj, dataPoint, pointIndex) {
        return dataObj.tooltipFormat.format(numberWithCommas(dataPoint.key), numberWithCommas(dataPoint.val));
    };

    $scope.accs = {
        basicStats: true,
        articlesOverTime: false,
        storiesOverTime: false,
        eventsOverTime: false,
        spaningStories: false,
        storySizes: false,
        clusterQuality: false
    };
    $scope.tables = {
        articlesOverTime: false,
        storiesOverTime: false,
        eventsOverTime: false,
        spaningStories: false,
        storySizes: false,
        clusterQuality: false
    };
    $scope.$watch("accs", $scope.updateGraphs, true);
    
    $().ready(function() {
        $scope.graphs = {
            articlesOverTime: new DateLineChart({ containerNm: ".articles-over-time", minYVal: 0, margin: { left: 70 }, tooltipTextCallback: $scope.countOverTimeTooltip }),
            storiesOverTime: new DateLineChart({ containerNm: ".stories-over-time", minYVal: 0, margin: { left: 70 }, tooltipTextCallback: $scope.countOverTimeTooltip }),
            eventsOverTime: new DateLineChart({ containerNm: ".events-over-time", minYVal: 0, margin: { left: 70 }, tooltipTextCallback: $scope.countOverTimeTooltip }),
            mergedEventsOverTime: new DateLineChart({ containerNm: ".merged-events-over-time", minYVal: 0, margin: { left: 70 }, tooltipTextCallback: $scope.countOverTimeTooltip }),
            spaningStories: new LineChart({ containerNm: ".spaning-stories", minYVal: 0, margin: { left: 70 }, tooltipTextCallback: $scope.storiesForTimeSpanTooltip }),
            storySizes: new LineChart({ containerNm: ".story-sizes", minYVal: 0, margin: { left: 70 }, tooltipTextCallback: $scope.storiesForStorySizeTooltip }),
            clusterQPrec: new BoxPlot({ containerNm: ".cluster-prec", minYVal: 0, maxYVal: 1 }),
            clusterQRec: new BoxPlot({ containerNm: ".cluster-rec", minYVal: 0, maxYVal: 1 }),
            clusterQF1: new BoxPlot({ containerNm: ".cluster-f1", minYVal: 0, maxYVal: 1 }),
        };
        
        $scope.getEventStatList();
    });

});