
eventRegApp.controller("EventViewerCtrl", function ($scope, $http, langOptions, utils) {
    $scope.utils = utils;
    $scope.langOptions = langOptions;

    $scope.similarStoryTimeConstraint = {
        vals: [
            { name: "No time constraints", val: "-1" },
            { name: "Max 0 day apart", val: "0" },
            { name: "Max 1 day apart", val: "1" },
            { name: "Max 2 days apart", val: "2" },
            { name: "Max 3 days apart", val: "3" },
            { name: "Max 5 days apart", val: "5" },
            { name: "Max 10 days apart", val: "10" },
            { name: "Max 20 days apart", val: "20" },
            { name: "Max 30 days apart", val: "30" },
        ]
    }
    $scope.similarStoryTimeConstraint.selected = $scope.similarStoryTimeConstraint.vals[0];

    $scope.tabs = { articles: true, concepts: false, tagCloud: false, storyline: false, timeline: false, similarEvents: false, similarStories: false, editEvent: false }
    // concept graphs
    $scope.entityGraph = new TopConceptList({ containerNm: "#entities", rectClsNm: "fill-blue", maxScale: 100 });
    $scope.keywordGraph = new TopConceptList({ containerNm: "#keywords", rectClsNm: "fill-blue", maxScale: 100 });
    // tag cloud
    $scope.tagCloud = new TagCloud({ containerNm: "#keyword-tagcloud" });
    // event location map
    $scope.map = { center: { latitude: 46, longitude: 15 }, markers: [], zoom: 3, bounds: {} };
    // article trending
    $scope.articleTrendsGraph = undefined;

    $scope.eventUri = undefined;
    $scope.page = 0;
    $scope.count = 30;
    $scope.sortOpt = {
        sortBy: 'rel'
    }
    $scope.error = undefined;
    
    $scope.resetData = function () {
        // clear existing content
        $scope.allArticlesLoaded = false;
        $scope.articleList = [];   // articles in a single language about the event 
        $scope.tempArticles = [];  // article list. assigned in different situations (e.g. articles for similar story, clicking dot in article trends, ...)
        $scope.displayedArticleIndex = -1;
        $scope.hasData = false;
        
        $scope.error = undefined;
        $scope.eventData = {};
        $scope.tagCloudData = undefined;
        $scope.multiLingInfo = {};
        $scope.hasSimilarEvents = false;
        // the articles that are selected when the user clicks an article in the article timeline
        $scope.selectedArticleList = [];
        $scope.similarStories = [];
        $scope.hasStoryline = false;
        // clear displayed concepts
        $scope.redrawConcepts();
    };

    $scope.showTab = function (tabName) {
        $scope.tabs[tabName] = true;
    }

    // load info for the curr
    $scope.loadEventInfo = function (callback) {
        $scope.utils.showWait("", ".search-in-progress");
        $scope.resetData();
        $http.get('/json/event?action=getEventInfo&eventUri=' + encodeURIComponent($scope.eventUri) + "&includeStoryDetails=" + utils.isPowerUser())
            .success(function (eventData, status, headers, config) {
                $scope.utils.hideWait("", ".search-in-progress");
                $scope.hasData = (eventData.error == undefined);
                if (eventData.error)
                    $scope.error = eventData.error;
                else if (eventData.warning && eventData.newEventUri)
                    window.location.assign(window.location.origin + "/event/" + eventData.newEventUri);
                else {
                    $scope.eventData = eventData.eventInfo;
                    if ($scope.eventData.location) {
                        //$("#map-placeholder").css("display", "block");
                        $scope.map.center = { latitude: $scope.eventData.location.lat, longitude: $scope.eventData.location.long };
                        $scope.map.markers = [{ latitude: $scope.eventData.location.lat, longitude: $scope.eventData.location.long, title: $scope.eventData.location.labelEng }];
                        $scope.map.zoom = 5;
                        //setTimeout(function () {
                            $.each(window.googlemaps ? window.googlemaps : [], function (i, item) { google.maps.event.trigger(item, "resize"); });
                        //}, 100);
                    }
                    
                    $scope.displayEventInfoForLang();
                    callback && callback();
                }
            });
    };

    $scope.displayEventInfoForLang = function () {
        // set medoid in the currently selected language
        //var lang = $("#lang-combo").val();
        $scope.multiLingInfo = $scope.eventData.multiLingInfo[$scope.langOptions.lang.val];
        if (!$scope.multiLingInfo && Object.keys($scope.eventData.multiLingInfo).length > 0)
            $scope.multiLingInfo = $scope.eventData.multiLingInfo[Object.keys($scope.eventData.multiLingInfo)[0]];
        // apply new bindings and draw the concepts
        //$scope.$apply();
    };

    $scope.loadArticlesForLang = function () {
        $scope.page = 0;
        $scope.allArticlesLoaded = false;
        $scope.articleList = [];
        $scope.displayedArticleIndex = -1;
        $scope.loadNextArticlePage();
    };

    $scope.loadNextArticlePage = function () {
        if ($scope.allArticlesLoaded == true)
            return;
        
        $scope.utils.showWait("", "#tab-articles .wait-container");
        $http.get('/json/event?action=getEventArticles&eventUri=' + encodeURIComponent($scope.eventUri) + "&lang=" + langOptions.lang.val + "&page=" + $scope.page + "&count=" + $scope.count + "&sortBy=" + $scope.sortOpt.sortBy)
            .success(function (eventData, status, headers, config) {
                $scope.utils.hideWait("", "#tab-articles .wait-container");
                if (eventData.error) {
                    $scope.allArticlesLoaded = true;
                    return;
                }
                if (!eventData.articles || eventData.articles.length == 0)
                    $scope.allArticlesLoaded = true;
                if (eventData.articles) {
                    $.each(eventData.articles, function (ind, item) {
                        $scope.articleList.push(item);
                    });
                    if ($scope.articleList.length == eventData.totalCount)
                        $scope.allArticlesLoaded = true;
                }

                $scope.page++;
                // init lightbox after the dom has updated. not a nice hack but no other clear way
                setTimeout(function () {
                    initLightbox();
                }, 500);
            });
    };

    // clean the svg, restart drawing concepts from start
    $scope.redrawConcepts = function () {
        var entities = $scope.eventData.concepts ? $.grep($scope.eventData.concepts, function (concept, i) { return entityTypes[concept.type]; }) : [];
        var keywords = $scope.eventData.concepts ? $.grep($scope.eventData.concepts, function (concept, i) { return keywordTypes[concept.type]; }) : [];
        setTimeout(function () {
            $scope.entityGraph.setData(entities, $scope.langOptions.lang.val);
            $scope.keywordGraph.setData(keywords, $scope.langOptions.lang.val);
        }, 100);
    };

    // keyword tag cloud
    $scope.visualizeTagCloud = function () {
        if ($scope.tagCloudData) {
            $scope.redrawTagCloud();
            return;
        }

        $scope.utils.showWait("#tab-tagcloud .content", "#tab-tagcloud .wait-container");

        $http.get('/json/event?action=getEventKeywords&eventUri=' + encodeURIComponent($scope.eventUri) + '&lang=' + langOptions.lang.val)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-tagcloud .content", "#tab-tagcloud .wait-container");
                $scope.error = data.error;
                $scope.tagCloudData = data.keywordAggr;
                $scope.redrawTagCloud();
            });
    }

    $scope.redrawTagCloud = function () {
        $scope.tagCloud.setData($scope.tagCloudData);
    }

    // show similar events
    $scope.showSimilarEvents = function () {
        if ($scope.hasSimilarEvents)
            return;
        $scope.utils.showWait("#tab-similar-events .content", "#tab-similar-events .wait-container");
        $http.get('/json/event?action=getSimilarEvents&eventUri=' + encodeURIComponent($scope.eventUri))
            .success(function (eventData, status, headers, config) {
                $scope.utils.hideWait("#tab-similar-events .content", "#tab-similar-events .wait-container");
                displaySimilarEvents(eventData);
                $scope.hasSimilarEvents = true;
            });
    };

    // show the most similar stories
    $scope.showSimilarStories = function () {
        var langs = "";
        $.each($(".sim-story-langs label.checked"), function (i, item) {
            langs += "&lang=" + $(item).children("input").attr("value");
        });
        var types = "";
        $.each($(".sim-story-concept-types label.checked"), function (i, item) {
            types += "&conceptType=" + $(item).children("input").attr("value");
        });
        var dayDiff = $scope.similarStoryTimeConstraint.selected.val == '-1' ? "" : "&maxDayDiff=" + $scope.similarStoryTimeConstraint.selected.val;
        $scope.utils.showWait("#tab-similar-stories .content", "#tab-similar-stories .wait-container");
        $http.get('/json/event?action=getSimilarStories&addDetails=1&count=30&eventUri=' + encodeURIComponent($scope.eventUri) + langs + types + dayDiff).success(function (eventData, status, headers, config) {
            $scope.utils.hideWait("#tab-similar-stories .content", "#tab-similar-stories .wait-container");
            $scope.similarStories = eventData.similarStories;
        });
    };

    // show storyline of events
    $scope.showStoryline = function () {
        if ($scope.hasStoryline)
            return;
        $scope.utils.showWait("#tab-storyline .content", "#tab-storyline .wait-container");
        $http.get('/json/event?action=getSimilarEvents&eventUri=' + encodeURIComponent($scope.eventUri))
            .success(function (eventData, status, headers, config) {
                $scope.utils.hideWait("#tab-storyline .content", "#tab-storyline .wait-container");
                var timelineData = { type: "default", date: [] };

                //var timelineData = { headline: "Timeline of events", type: "default", text: "some text", date: [] };
                $.each(eventData.similarEvents, function (i, event) {
                    var item = {};
                    var eventDataPerLang = $scope.getEventInfoForCurrLang(event, false);
                    if (!eventDataPerLang)
                        return true;
                    item.headline = eventDataPerLang.title;
                    item.text = eventDataPerLang.summary.shorten(300);
                    item.startDate = event.eventDate.replace(/-/g, ',');
                    if (event.eventDateEnd != "")
                        item.endDate = event.eventDateEnd.replace(/-/g, ',');
                    if (event.images && event.images.length > 0)
                        item.asset = { media: event.images[0] };
                    timelineData.date.push(item);
                });

                // add the shown event
                var shownEvent = $scope.eventInfo;
                var shownMultiLingEvent = $scope.getEventInfoForCurrLang();
                if (shownEvent) {
                    var item = {};
                    item.headline = shownMultiLingEvent.title;
                    item.text = shownMultiLingEvent.summary.shorten(300);
                    item.startDate = shownEvent.eventDate.replace(/-/g, ',');
                    if (shownEvent.eventDateEnd != "")
                        item.endDate = shownEvent.eventDateEnd.replace(/-/g, ',');
                    if (shownEvent.images && shownEvent.images.length > 0)
                        item.asset = { media: shownEvent.images[0] };
                    timelineData.date.push(item);
                    timelineData.date.sort(function (a, b) {
                        if (a.startDate < b.startDate) return 1;
                        else if (a.startDate > b.startDate) return -1;
                        return 0;
                    });
                }

                createStoryJS({
                    type: 'timeline',
                    width: $("#timeline-embed").width(),
                    height: '600',
                    source: { timeline: timelineData },
                    start_at_slide: 0,
                    embed_id: 'timeline-embed'
                });
                $scope.hasStoryline = true;
            });
    };

    $scope.showArticleTrends = function () {
        if ($scope.articleTrends) {
            $scope._showArticleTrends();
            return;
        }
        $scope.utils.showWait("#tab-article-trends .content", "#tab-article-trends .wait-container");
        $http.get('/json/event?action=getEventTrend&eventUri=' + encodeURIComponent($scope.eventUri))
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-article-trends .content", "#tab-article-trends .wait-container");
                if (data && data.error) {
                    alert(data.error); return;
                }
                else if (data && data.articles) {
                    $scope.articleTrends = data.articles;
                    $scope._showArticleTrends();
                }
            });
    }

    $scope._showArticleTrends = function () {
        var langs = {};
        $.map($scope.articleTrends, function (val, i) { langs[val.lang] = 1; });
        var artsByLang = splitArticlesByLang($scope.articleTrends);
        var keys = Object.keys(artsByLang);
        var legend = $.map(keys, function (val) { return langToLabel[val] + " articles (#" + artsByLang[val].length + ")"; });
        var articlesList = $.map(keys, function (val) { return [artsByLang[val]]; });       // we need to double-array it otherwise it just creates one array - jquery pecularity
        if ($scope.articleTrendsGraph == undefined)
            $scope.articleTrendsGraph = new ArticleTrendsGraph({ containerNm: "#article-trends-container", defaultZoomEnabled: false });
        $scope.articleTrendsGraph.drawArticleTrends(articlesList, legend, 500, $scope.onDisplayArticlesDlgForUrls);
    }
    
    // //////////
    // event handlers

    // show the dialog with articles from the clicked similar story
    $scope.onShowArticlesForSimilarStory = function (story) {
        $scope.tempArticles = []; $scope.deletableArticles = false;
        $('#articles-dlg h3').html("Articles in the story");
        $('#articles-dlg').modal({}); 
        $http.get('/json/story?action=getArticles&count=20&uri=' + encodeURIComponent(story.uri)).success(function (ret, status, headers, config) {
            $scope.tempArticles = ret.results;
        });
    };

    // show the dialog with articles from the clicked similar story
    $scope.onShowRelatedArticlesToThisStory = function (story) {
        $scope.tempArticles = []; $scope.deletableArticles = false;
        $('#articles-dlg h3').html("Articles related to the story");
        $('#articles-dlg').modal({});
        $http.get('/json/story?action=getArticlesRelatedToStory&uri=' + encodeURIComponent(story.uri) + "&lang=" + $scope.langOptions.lang.val)
            .success(function (ret, status, headers, config) {
                $scope.tempArticles = ret.results;
            });
    };

    // the user decided to merge the similar story to the current event
    $scope.onMergeStoryToEvent = function (storyIndex) {
        var story = $scope.similarStories[storyIndex];
        $http.get('/jsonAdmin/eventAdmin?action=assignStoryToEvent&eventUri=' + encodeURIComponent($scope.eventUri) + '&storyUri=' + encodeURIComponent(story.uri))
            .success(function (ret, status, headers, config) {
                if (ret.error)
                    alert(ret.error);
                else
                    $scope.similarStories.splice(storyIndex, 1);
            })
        .error(function (data, status, headers, config) {
            alert(data);
        });
    };

    // show info why the particular story is not merged with the current event
    $scope.getWhyStoryNotMergedToEvent = function (story) {
        $scope.eventToStoryComparison = undefined;
        $http.get('/jsonAdmin/eventAdmin?action=getWhyStoryNotMergedToEvent&eventUri=' + encodeURIComponent($scope.eventUri) + '&storyUri=' + encodeURIComponent(story.uri))
            .success(function (ret, status, headers, config) {
                if (ret.error)
                    alert(ret.error);
                else {
                    $scope.eventToStoryComparison = ret.simInfo;
                    $('#not-merging-dlg').modal({});
                }
            });
    };

    $scope.onRemoveStoryFromEvent = function (storyIndex) {
        var story = $scope.eventData.stories[storyIndex];
        $http.get('/jsonAdmin/eventAdmin?action=removeStoryFromEvent&eventUri=' + encodeURIComponent($scope.eventUri) + '&storyUri=' + encodeURIComponent(story.uri))
            .success(function (ret, status, headers, config) {
                if (ret.error)
                    alert(ret.error);
                else {
                    $scope.eventData.stories.splice(storyIndex, 1);
                    // if the story was the primary story for the event then we had to push all other stories to a different event 
                    if (ret.redirectEventUri) {
                        window.location = window.location.origin + "/event/" + ret.redirectEventUri;
                    }
                }
            });
        var storyUri1 = story.uri;
        if ($scope.eventData.stories.length < 2) return;
        var storyUri2 = (storyIndex == 0) ? $scope.eventData.stories[1].uri : $scope.eventData.stories[0].uri;
        $http.get('/jsonAdmin/crossLingStoryLink?action=setClassForStoryPair&shouldMerge=false&storyUri1=' + encodeURIComponent(storyUri1) + '&storyUri2=' + encodeURIComponent(storyUri2))
            .success(function (ret, status, headers, config) {
                if (ret.error)
                    alert(ret.error);
            });
    };
    
    // display a dialog listing articles from articleUrls
    $scope.onDisplayArticlesDlgForUrls = function (articleUrls) {
        var urls = "";
        $scope.tempArticles = []; $scope.deletableArticles = true;
        $.each(articleUrls, function (i, uri) { urls += "articleUri=" + encodeURIComponent(uri) + "&"; });
        $http.get('/json/article?action=getArticlesForUrls&' + urls)
            .success(function (ret, status, headers, config) {
                if (ret.error)
                    alert(ret.error);
                else if (ret.articles) {
                    $('#articles-dlg h3').html("Articles");
                    $('#articles-dlg').modal({});
                    $scope.tempArticles = ret.articles;
                }
            });
    };

    // show the dialog with articles from the clicked similar story
    $scope.onEditEvent = function () {
        $('#edit-event-dlg #title').val($scope.multiLingInfo.title);
        $('#edit-event-dlg #summary').val($scope.multiLingInfo.summary);
        $('#edit-event-dlg #dp-start').data('datepicker').setValue($scope.eventData.eventDate);
        if ($scope.eventData.eventDateEnd)
            $('#edit-event-dlg #dp-end').data('datepicker').setValue($scope.eventData.eventDateEnd);
        else
            $('#edit-event-dlg #dp-end').val("");
        $('#edit-event-dlg #location').val($scope.getEventLocation());
        
        $('#edit-event-dlg').modal({});
        $('#edit-event-dlg .modal-footer button.btn-primary').click(function () {
            var changes = "";
            if ($scope.multiLingInfo.title != $('#edit-event-dlg #title').val())
                changes += "&title=" + encodeURIComponent($('#edit-event-dlg #title').val());
            if ($scope.multiLingInfo.summary != $('#edit-event-dlg #summary').val())
                changes += "&summary=" + encodeURIComponent($('#edit-event-dlg #summary').val());
            if ($scope.eventData.eventDate != $('#edit-event-dlg #dp-start').val())
                changes += "&eventDate=" + encodeURIComponent($('#edit-event-dlg #dp-start').val());
            if ($scope.eventData.eventDateEnd != $('#edit-event-dlg #dp-end').val())
                changes += "&eventDateEnd=" + encodeURIComponent($('#edit-event-dlg #dp-end').val());
            if ($scope.getEventLocation() != $('#edit-event-dlg #location').val()) {
                var uri = $('#edit-event-dlg #location').val() != "" ? $('#edit-event-dlg #location').data("locationUri") : "";
                changes += "&locationUri=" + encodeURIComponent(uri);
            }

            if (changes != "") {
                var req = '/jsonAdmin/eventAdmin?action=setEventInfo&lang=' + $scope.langOptions.lang.val + '&eventUri=' + encodeURIComponent($scope.eventUri) + changes;
                $http.get(req).success(function (data, status, headers, config) {
                    if (data.error)
                        $scope.error = data.error;
                    else
                        $scope.loadEventInfo();
                });
            }
        })
    };

    // ///////////////
    // power user's event actions
    // //////////////

    // user clicked the button to update event info
    $scope.updateEventInfo = function () {
        $http.get('/jsonAdmin/eventAdmin?action=updateEventInfo&eventUri=' + encodeURIComponent($scope.eventUri)).success(function (eventData, status, headers, config) {
            var text = "";
            if (eventData.info)
                text = eventData.info.replace(/\n/g, "<br/>");
            else if (eventData.error)
                text = eventData.error.replace(/\n/g, "<br/>");
            text = text.replace(/ /g, "&nbsp;");
            $("#event-actions-log").html(text);
        });
    };

    // remove the provided article from the story to which it is currently assigned to
    // if callback method is provided call it in case of success
    $scope.removeArticleFromEvent = function (article, callbackMethod) {
        $http.get('/jsonAdmin/eventAdmin?action=removeArticleFromStory&articleUri=' + encodeURIComponent(article.uri))
            .success(function (ret, status, headers, config) {
                if (ret.error)
                    alert(ret.error);
                else {
                    $scope.hideArticleFromEvent(article);
                    callbackMethod && callbackMethod();
                }
            });
    };

    $scope.hideArticleFromEvent = function (article) {
        $scope.tempArticles = $.grep($scope.tempArticles, function(art) { return art.uri != article.uri; });
        $scope.articleList = $.grep($scope.articleList, function (art) { return art.uri != article.uri; });
        $scope.articleTrends = $.grep($scope.articleTrends, function (art) { return art.uri != article.uri; });
    }

    $scope.getArticleDate = function (article) {
        var dateStr = article.date + " " + article.time;
        var date = d3.time.format("%Y-%m-%d %H:%M").parse(dateStr);
        return d3.time.format("%d %b, %H:%M")(date);
    };

    $scope.getEventTitle = function () {
        if (!$scope.multiLingInfo || !$scope.multiLingInfo.title)
            return "";
        return $('<div />').html($scope.multiLingInfo.title).text();
    };

    $scope.getEventSummary = function () {
        if (!$scope.multiLingInfo || !$scope.multiLingInfo.summary)
            return "";
        var summary = $scope.multiLingInfo.summary.shorten(700);
        return $('<div />').html(summary).text();
    };

    $scope.getEventLocation = function () {
        if (! $scope.eventData.location)
            return "";
        var lang = $scope.langOptions.lang.val;
        lang = lang.charAt(0).toUpperCase() + lang.slice(1); // capitalize first letter
        var locStr = "";
        var loc = $scope.eventData.location;
        locStr = loc["label" + lang];
        if (loc.country)
            locStr += ", " + loc.country["label" + lang];
        return locStr;
    };

    $scope.getEventDate = function () {
        if (!$scope.eventData)
            return "unknown";
        var dateStart = utils.getDateStr($scope.eventData.eventDate);
        var dateEnd = ($scope.eventData.eventDateEnd != "") ? utils.getDateStr($scope.eventData.eventDateEnd) : "";
        return dateStart + (dateEnd != "" ? " - " + dateEnd : "");
    };

    // /////////////////////
    // DISPLAYING ARTICLE CONTENT
    $scope.showArticle = function (articleIndex) {
        //initLightbox();
        $scope.displayedArticleIndex = articleIndex;
        displayArticle($scope.articleList[articleIndex]);
    };

    $scope.displayNextArticle = function () {
        var next = $scope.displayedArticleIndex + 1;
        next = next % $scope.articleList.length;
        $scope.displayedArticleIndex = next;
        if (next >= $scope.articleList.length || next < 0)
            return;
        displayArticle($scope.articleList[next]);
    };

    $scope.displayPrevArticle = function () {
        var prev = $scope.displayedArticleIndex - 1;
        if (prev < 0) prev = $scope.articleList.length - 1;
        $scope.displayedArticleIndex = prev;
        if (prev >= $scope.articleList.length || prev < 0)
            return;
        displayArticle($scope.articleList[prev]);
    };

    // get event info for the currently selected language
    // if without argument or emptyIfMissing then return undefined
    $scope.getEventInfoForCurrLang = function (eventData, emptyIfMissing) {
        emptyIfMissing = (emptyIfMissing == undefined ? 1 : emptyIfMissing);
        eventData = (eventData == undefined ? $scope.eventData : eventData);
        var info = eventData.multiLingInfo[$scope.langOptions.lang.val];
        if (info)
            return info;
        if (arguments.length == 0 || emptyIfMissing)
            return undefined;
        if (eventData.multiLingInfo["eng"])
            return eventData.multiLingInfo["eng"];
        if (Object.keys(eventData.multiLingInfo).length > 0)
            return eventData.multiLingInfo[Object.keys(eventData.multiLingInfo)[0]];
        return undefined;
    };
        
    $scope.onLangButtClicked = function (langIndex) {
        $scope.langOptions.lang = $scope.langOptions.langs[langIndex];
        $scope.displayEventInfoForLang();
        $scope.loadArticlesForLang();
    }

    // handler called when a different button for article languages is selected
    $scope.$on('langChangedBroadcast', function () {
        // display event info for the new language
        $scope.displayEventInfoForLang();
        // load articles for the language
        if ($scope.tabs.articles)
            $scope.loadArticlesForLang();
        else if ($scope.tabs.concepts)
            $scope.redrawConcepts();
    });

    // ///////////////
    // initialize variables and stuff
    $scope.resetData();

    $("#prevArticle").click(function () {
        $scope.displayPrevArticle();
    });
    $("#nextArticle").click(function () {
        $scope.displayNextArticle();
    });
        
    // update graphs if the width changed
    var lastWidth = $(window).width();
    $(window).resize(function () {
        if (lastWidth == $(window).width())
            return;
        lastWidth = $(window).width();
        if ($scope.tabs.concepts)
            $scope.redrawConcepts();
        else if ($scope.tabs.timeline)
            $scope._showArticleTrends();
    });

    $(document).ready(function () {
        var lang = getURLParameterByName("lang", "eng");
        var langs = $.grep($scope.langOptions.langs, function (el, ind) { return el.val == lang; });
        if (langs.length > 0)
            $scope.langOptions.lang = langs[0];
        $scope.sortByRel = getURLParameterByName("sort", "rel") == "rel";
        
        // load the event specified in the url parameters
        $scope.eventUri = getEventUri();
        if (isNaN(parseFloat($scope.eventUri.replace(/s/gi, "")))) {
            $scope.error = "Please enter a valid number of the event id";
        }
        else {
            $scope.loadEventInfo(function () {
                // call concept redraw after we repaint window and divs get some width
                //setTimeout(function () {
                //    $scope.redrawConcepts();
                //}, 10);
                $scope.loadArticlesForLang();
            });
        }
        
        //$('.make-switch').bootstrapSwitch('setSizeClass', '');

        // date picker for date range in edit event
        $('#edit-event-dlg #dp-start').datepicker({ format: 'yyyy-mm-dd' });
        $('#edit-event-dlg #dp-end').datepicker({ format: 'yyyy-mm-dd' });
        $(".datepicker.dropdown-menu").css("z-index", 1500);

        // set up the autocomplete handlers for location
        $("#edit-event-dlg #location").autocomplete({
            source: function (request, response) {
                $.getJSON("/json/suggestLocations?prefix=" + request.term + "&lang=" + $scope.langOptions.lang.val, function (suggList) {
                    var arr = suggList.country[lang];
                    arr.push.apply(arr, suggList.city[lang]);
                    response(arr);
                });
            },
            select: function (event, ui) {
                $('#edit-event-dlg #location').data("locationUri", ui.item.wikiUri);
                $('#edit-event-dlg #location').val(ui.item.label);
                return false;
            }
        }).data("uiAutocomplete")._renderItem = function (ul, element) {
            return $('<li id="' + encodeURIComponent(element.wikiUri) + '" class="' + element.type + ' black concept" title="' + element.wikiUri + '"><a title="' + element.wikiUri + '">' + element.label + '</a></li>').appendTo(ul);
        };
        $("ul.ui-autocomplete.ui-front").css("z-index", 1500);

        var simStoryUpdateTimer;
        // refresh the similar stories if any checkbox is changed
        $(".sim-story-checkboxes .checkbox").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
            clearTimeout(simStoryUpdateTimer);
            simStoryUpdateTimer = setTimeout(function () {
                $scope.showSimilarStories();
                clearTimeout(simStoryUpdateTimer);
            }, 1000);
        });
    });

    //// when we are 700px from the bottom of the page start loading new articles
    //$(window).scroll(function () {
    //    if ($(window).scrollTop() + $(window).height() + 700 > $(document).height())
    //        $scope.loadNextArticlePage();
    //});  
});

function parseUrl() {
    var params = {};
    params.eventUri = getEventUri();
    params.lang = getURLParameterByName("lang", "eng");
    params.sort = getURLParameterByName("sort", "rel");
    return params;
}

