
function ConceptMatrixGraph(_options) {
    // settings
    var options = $.extend({
        containerNm: undefined,  // name of the dom element hosting the svg
        graphChangeDuration: 600,   // how long does the animation take
        defaultZoomEnabled: false
    }, _options);

    var data = undefined;

    this.setData = function (_data) {
        function createRow(row, rowIndex) {
            d3.select(this).selectAll(".cell")
              .data(row)
                .enter()
                .append("rect")
                    .attr("class", "cell")
                    .attr("x", function (d, colInded) {
                        return x(order[colInded]);
                    })
                    .attr("width", x.rangeBand() - 1)
                    .attr("height", x.rangeBand() - 1)
                    .style("fill", function (d, colIndex) {
                        var val = row[order[colIndex]];
                        return c(val / maxVal);
                    })
                    .on("mouseover", function (val, colIndex) {
                        d3.selectAll(".selection").remove();
                        d3.selectAll(".row text").classed("active", function (d, i) { return i == rowIndex; });
                        d3.selectAll(".column text").classed("active", function (d, i) { return i == colIndex; });
                        svg.append("circle")
                            .attr("r", Math.sqrt(2 * Math.pow(x.rangeBand(), 2)) / 2)
                            .attr("class", "selection")
                            .attr("cx", x.rangeBand() / 2 + (x.rangeBand() * colIndex))
                            .attr("cy", x.rangeBand() / 2 + (x.rangeBand() * rowIndex))
                            .attr("fill", "#F0F0F0")
                            .on("mouseleave", function (val, colInd) {
                                //console.log(this);
                                d3.selectAll("text").classed("active", false);
                                d3.selectAll(".selection").remove();
                                //d3.selectAll(options.containerNm + " .selected").remove();
                            });
                        svg.append("text")
                            .attr("x", x.rangeBand() / 2 + (x.rangeBand() * colIndex))
                            .attr("y", x.rangeBand() / 2 + (x.rangeBand() * rowIndex))
                            .attr("dy", ".32em")
                            .attr("text-anchor", "middle")
                            .attr('class', function (val, ind) { return 'selection score'; })
                            .text(function () { return parseInt(row[order[colIndex]]) + "%"; });
                    });
        }
        
        data = _data;
        d3.select(options.containerNm + " svg").remove();
        if (data == undefined || data.concepts == undefined || data.matrix == undefined)
            return;
        // prepare the data - compute the .val values for each 
        var margin = { top: 220, right: 10, bottom: 10, left: 220 },
            width = Math.min(700, $(options.containerNm).width()),
            height = width,
            matrixWidth = width - margin.left - margin.right;
        var n = data.concepts.length,
            x = d3.scale.ordinal().rangeBands([0, matrixWidth]),
            c = d3.interpolate(d3.rgb(255, 255, 255), d3.rgb(0, 0, 0))
            //z = d3.scale.linear().domain([0, 4]).clamp(true),
            //c = d3.scale.category10().domain(d3.range(10));
        
        // Precompute the orders.
        var orders = {
            original: d3.range(n),
            name: d3.range(n).sort(function (a, b) { return d3.ascending(data.concepts[a].label, data.concepts[b].label); }),
            count: d3.range(n).sort(function (a, b) { return data.matrix[b][b] - data.matrix[a][a]; }),
            group: d3.range(n).sort(function (a, b) { return data.concepts[b].group - data.concepts[a].group; })
        };
        var order = orders.count;
        // The default sort order.
        x.domain(order);
        
        var matrix = $.map(order, function (matrixIndex, i) {
            return [data.matrix[matrixIndex]];
        });
        var concepts = $.map(order, function (matrixIndex, i) {
            return data.concepts[matrixIndex];
        });
        var maxVal = d3.max($.map(matrix, function (row) { return d3.max(row); }));
                
        var svg = d3.select(options.containerNm)
            .append("svg")
            .attr("class", "background")
            .attr("width", width)
            .attr("height", height)
                .append("g")
                .attr("transform", "translate(" + margin.left + "," + margin.top + ")");
        
        var row = svg.selectAll(".row")
            .data(matrix)
            .enter().append("g")
                .attr("class", "row")
                .attr("transform", function (d, i) {
                    return "translate(0," + x(order[i]) + ")";
                })
                .each(createRow);

        row.append("line")
            .attr("x2", width);

        row.append("text")
            .attr("x", -6)
            .attr("y", x.rangeBand() / 2)
            .attr("dy", ".32em")
            .attr("text-anchor", "end")
            .attr('class', function (val, ind) { return 'name concept ' + concepts[ind].type; })
            .text(function (d, i) { return concepts[i].label; });
                
        var column = svg.selectAll(".column")
            .data(data.matrix)
              .enter().append("g")
                .attr("class", "column")
                .attr("transform", function (d, i) { return "translate(" + x(order[i]) + ")rotate(-90)"; });

        column.append("line")
            .attr("x1", -width);

        column.append("text")
            .attr("x", 6)
            .attr("y", x.rangeBand() / 2)
            .attr("dy", ".32em")
            .attr("text-anchor", "start")
            .attr('class', function (val, ind) { return 'name concept ' + concepts[ind].type; })
            .text(function (d, i) { return concepts[i].label; });
    }
}