

function Dendrogram(_options) {
    var options = $.extend({
        containerNm: undefined,     // name of the dom element hosting the svg
        cloudWidth: 180,
        cloudHeight: 120,
        cloudMinFontSize: 10,
        cloudMaxFontSize: 18,
        cloudFontFamily: "Segoe UI",
        cloudFontWeight: "bold",
        cloudWordPadding: 1,
        onClickCallback: undefined
    }, _options);

    var data = undefined,
        tree = undefined,
        vis = undefined,
        eventCountRectHeight = 20,
        i = 0,
        margin = {},
        diagonal = d3.svg.diagonal().projection(function (d) { return [d.x, d.y]; });

    this.setData = function (_data) {
        data = _data;
        // remove old graph
        d3.select(options.containerNm + " svg").remove();

        if (!data.clusters || !data.concepts)
            return;

        var width = $(options.containerNm).width();
        var height = $(options.containerNm).height();
        margin = $.extend({ top: 20 + options.cloudHeight + eventCountRectHeight, right: 20 + options.cloudWidth / 2, bottom: 20 + options.cloudHeight / 2, left: 20 + options.cloudWidth / 2 }, options.margin);
        i = 0;
        data.clusters.x0 = 0;
        data.clusters.y0 = 0;

        tree = d3.layout.tree()
            .size([10000, 10000])
            .nodeSize([options.cloudWidth, options.cloudHeight + eventCountRectHeight])
            //.separation(function (a, b) {
            //    return a.parent == b.parent ? options.cloudWidth + 10 : options.cloudWidth * 1.5;
            //});

        vis = d3.select(options.containerNm)
            .append("svg:svg")
            //.attr("width", width)
            //.attr("height", height)
            .append("svg:g")
                .attr("transform", "translate(" + margin.left + "," + margin.top + ")");
               
        toggleAll(data.clusters, 100);   // first disable all nodes
        toggleAll(data.clusters, 2);   // enable nodes up to 3 levels deep
        update(data.clusters);
    }

    function update(source) {
        var duration = 500;
        // Compute the new tree layout.
        var nodes = tree.nodes(data.clusters).reverse();

        var minX = d3.min(nodes, function (node) { return node.x; });
        var maxX = d3.max(nodes, function (node) { return node.x; });
        var maxDepth = d3.max(nodes, function (node) { return node.depth; });
        
        // update the positions of the nodes + the size of the whole svg
        nodes.forEach(function (d) {
            d.y = d.depth * (100 + options.cloudHeight + eventCountRectHeight);
            d.x = d.x - minX;
        });

        d3.select(options.containerNm + " > svg")
            .transition()       // use the transition to change the size. otherwise we don't see the animation when closing a larger set of nodes
            .duration(duration)
            .attr("width", maxX - minX + margin.left + margin.right )
            .attr("height", (maxDepth +1) * (100 + options.cloudHeight + eventCountRectHeight))

        // Update the nodes
        var node = vis.selectAll("g.node")
            .data(nodes, function(d) { return d.id || (d.id = ++i); });

        // Enter any new nodes at the parent's previous position.
        var nodeEnter = node.enter()
            .append("svg:g")
                .attr("class", "node")
                .attr("opacity", 1)
                .attr("transform", function (d) { return "translate(" + source.x0 + "," + source.y0 + ")"; })
                .each(createTagCloud)
        
        // Transition nodes to their new position.
        var nodeUpdate = node.transition()
            .duration(duration)
            .attr("transform", function(d) { return "translate(" + d.x + "," + d.y + ")"; });

        nodeUpdate.select("circle")
            .attr("r", 6)
            .style("fill", function(d) { return d._children ? "lightsteelblue" : "#fff"; });

        // Transition exiting nodes to the parent's new position.
        var nodeExit = node.exit()
            .transition()
            .duration(duration)
            .attr("transform", function (d) { return "translate(" + source.x + "," + source.y + ")"; })
            .attr("opacity", 0)
            .remove();

        nodeExit.select("circle")
            .attr("r", 1e-6);
        
        var linkData = $.map(tree.links(nodes), function(link) { 
            return { source: link.source, target: { x: link.target.x, y: link.target.y - options.cloudHeight - eventCountRectHeight, id: link.target.id } };
        });
        // Update the links
        var link = vis.selectAll("path.link")
            //.data(tree.links(nodes), function(d) { return d.target.id; });
            .data(linkData, function (d) { return d.target.id; });

        // Enter any new links at the parent's previous position.
        link.enter()
            .insert("svg:path", "g")
            .attr("class", "link")
            .attr("d", function(d) {
                var o = {x: source.x0, y: source.y0};
                return diagonal({source: o, target: o});
            })
          .transition()
            .duration(duration)
            .attr("d", diagonal);

        // Transition links to their new position.
        link.transition()
            .duration(duration)
            .attr("d", diagonal);

        // Transition exiting nodes to the parent's new position.
        link.exit().transition()
            .duration(duration)
            .attr("d", function(d) {
                var o = {x: source.x, y: source.y};
                return diagonal({source: o, target: o});
            })
            .remove();

        // Stash the old positions for transition.
        nodes.forEach(function(d) {
            d.x0 = d.x;
            d.y0 = d.y;
        });
    }

    function createTagCloud(cluster) {
        var svgGroup = d3.select(this);
        // draw the dot on the bottom if we have children
        if ((cluster.children && cluster.children.length > 0) || (cluster._children && cluster._children.length > 0)) {
            svgGroup.append("svg:circle")
                .attr("x", 0)
                .attr("y", 0)
                .attr("r", 1e-6)
                .style("fill", function (d) { return d._children ? "lightsteelblue" : "#fff"; })
                .on("click", function (d) { toggle(d); update(d); });
        }
        var countGroup = svgGroup.append("svg:g")
            .attr("transform", "translate(-50,-" + (eventCountRectHeight + options.cloudHeight) + ")");
        
        countGroup.append("rect")
            .attr("x", 0)
            .attr("y", 0)
            .attr("width", 100)
            .attr("height", eventCountRectHeight - 2)
            .attr("class", "count-rect")
            .attr("rx", 4)
            .attr("ry", 4)
            .on("click", function (d) { options.onClickCallback && options.onClickCallback(d.events, d.topConcepts, data.concepts); });
            
        countGroup.append("text")
            .attr("x", 50)
            .attr("y", 9)
            .attr("class", "count-text")
            .text(cluster.eventCount + " events")
            .attr("dy", ".36em")
            .attr("text-anchor", "start")
            .attr("text-anchor", "middle")
            .on("click", function (d) { options.onClickCallback && options.onClickCallback(d.events, d.topConcepts, data.concepts); });

        var maxVal = d3.max(cluster.topConcepts, function (item) { return item.wgt; }),
                minVal = d3.min(cluster.topConcepts, function (item) { return item.wgt; }),
                diff = maxVal - minVal;
        var cloudData = $.map(cluster.topConcepts, function (item) {
            return { text: data.concepts[item.id], size: (options.cloudMinFontSize + (options.cloudMaxFontSize - options.cloudMinFontSize) * ((item.wgt - minVal) / diff)) };
        });
        d3.layout.cloud()
            .size([options.cloudWidth, options.cloudHeight])
            .timeInterval(10)
            .words(cloudData)
            .padding(options.cloudWordPadding)
            .rotate(function () { return 0; })
            .font(options.cloudFontFamily)
            .fontWeight(options.cloudFontWeight)
            .fontSize(function (d) { return d.size; })
            .on("end", function (words, bounds) {
                var scale = bounds ? Math.min(
                    options.cloudWidth / Math.abs(bounds[1].x - options.cloudWidth / 2),
                    options.cloudWidth / Math.abs(bounds[0].x - options.cloudWidth / 2),
                    options.cloudHeight / Math.abs(bounds[1].y - options.cloudHeight / 2),
                    options.cloudHeight / Math.abs(bounds[0].y - options.cloudHeight / 2)) / 2 : 1;
                var cloud = svgGroup.append("g")
                    .attr("transform", "translate(0,-" + (bounds[1].y / 2) + ")scale(" + scale + ")")
                    .attr("class", "events-" + cluster.eventCount);
                var text = cloud.selectAll("text")
                    .data(words)
                    .enter()
                    .append("text")
                    .style("font-size", function (d) { return d.size + "px"; })
                    .style("font-weight", function (d) { return d.weight; })
                    .style("font-family", function (d) { return options.cloudFontFamily; })
                    .style("fill", "black")
                    .attr("text-anchor", "middle")
                    .attr("transform", function (d) {
                        return "translate(" + [d.x, d.y] + ")rotate(" + d.rotate + ")";
                    })
                    .text(function (d) { return d.text; });                 
            })
            .start();
    }

    // Toggle children
    function toggle(d) {
        if (d.children) {
            d._children = d.children;
            d.children = null;
        } else {
            d.children = d._children;
            d._children = null;
        }
    }

    function toggleAll(d, depth) {
        if (depth != undefined && depth <= 0)
            return;
        if (d.children) {
            d.children.forEach(function (child) { toggleAll(child, depth != undefined ? depth - 1 : depth); });
        }
        if (d._children) {
            d._children.forEach(function (child) { toggleAll(child, depth != undefined ? depth - 1 : depth); });
        }
        toggle(d);
    }
}