
function ConceptTrends(containerName, tagInputName) {
    this.colors = d3.scale.category20();
    this.trendData = null;
    this.containerName = containerName;
    this.tagInputName = tagInputName;
    this.xScale = null;
    this.duration = 600;

    // return object with concept info for given concept id
    this.getConceptInfoForId = function(conceptId) {
        var matches = $.grep(this.trendData.conceptInfo, function(d) { return d.id == conceptId});
        return (matches.length == 1) ? matches[0] : undefined;
    }

    // set the list of concepts
    this.setTags = function (conceptList) {
        var self = this;
        $.each(conceptList, function (i, item) {
            item.valid = true;
            $("#" + self.tagInputName).addTag(item.label, { focus: true, type: "concept", uri: item.uri, id: item.id, label: item.label, classes: [item.type, "concept"] });
        });
    }

    // add one or more concepts to the graph
    this.addConcepts = function (data) {
        var self = this;
        // add concepts
        $.each(data.conceptInfo, function (i, concept) {
            concept.valid = true;
            self.trendData.conceptInfo.push(concept);
        });
        var dateToInd = {};
        for (var n = 0; n < data.trends.length; n++)
            dateToInd[data.trends[n].date] = n;
        // extend the trends for each day with data for new concepts
        $.each(this.trendData.trends, function (i, existingTrend) {
            if (dateToInd[existingTrend.date] == undefined)
                return;
            var newTrends = data.trends[dateToInd[existingTrend.date]];
            // add new trends to existing trends
            $.extend(existingTrend.conceptFreq, newTrends.conceptFreq);
        });
        // add path.area for new concepts
        var svg = d3.select("#" + this.containerName + " svg g");
        svg.selectAll(".area")
            .data(this.trendData.conceptInfo.map(function (concept) { return {} }))
            .enter()
            .append("path")
            .attr("class", "area");
        // update the graph with new concept data
        this._updateTrends();
    }
    
    // when the user removes a concept just set the freq for each day to 0. in this way we don't recolor remaining concepts
    this.onRemoveTag = function (concept) {
        var id = concept.data("options").id;
        $.each(this.trendData.trends, function(i, item) {
            item.conceptFreq[id] = 0;
        });
        var info = this.getConceptInfoForId(id);
        if (info)
            info.valid = false;
        this._updateTrends();
    }
    
    this.onUpdateNormalize = function() {
        this._updateTrends();
    }
    
    // remove all tags
    this.clearTags = function () {
        $.each($("#" + this.tagInputName + "_tagsinput .concept"), function (i, item) {
            $("#" + this.tagInputName).removeTag($(item));
        });
    }
    
    // choose an offset type (expand, wiggle, zero, silhouette)
    this.offsetType = "wiggle";

    this._updateTrends = function () {
        var self = this,
            totWidth = $("#" + this.containerName).width(),
            totHeight = $("#" + this.containerName).height(),
            margin = { top: 5, right: 25, bottom: 30, left: 25 },
            width = totWidth - margin.left - margin.right,
            height = totHeight - margin.top - margin.bottom,
            parse = d3.time.format("%Y-%m-%d").parse;

        var stack = d3.layout
            .stack()
            .values(function(d) { return d.values; })
            .offset(this.offsetType);
            //.order("inside-out");

        var layers = self.trendData.conceptInfo.map(function (concept) {
            return {
                "label": concept.label,
                "values": self.trendData.trends.map(function (dateTrend) {
                    var date = parse(dateTrend.date); date.setHours(12);
                    return { "dateStr": dateTrend.date, "x": date, "y": dateTrend.conceptFreq[concept.id] != undefined ? dateTrend.conceptFreq[concept.id] : 0 }
                })
            }
        });
        
        stack(layers);
        
        var yScale = d3.scale.linear()
            .range([height, 0])
            .domain([0, d3.max(layers.map(function (layer) { return d3.max(layer.values.map(function (val) { return val.y0 + val.y; })); }))]);

        //var yAxis = d3.svg.axis().scale(yScale).orient("left");

        // A area generator, for the dark stroke.
        var area = d3.svg.area()
            .interpolate("basis")
            .x(function (d) {
                return self.xScale(d.x);
            })
            .y0(function (d) {
                return yScale(d.y0);
            })
            .y1(function (d) {
                return yScale(d.y0 + d.y);
            });

        var svg = d3.select("#" + this.containerName + " svg g");
        svg.selectAll(".area")
            .data(layers)
            .transition()
            .duration(self.duration)
            .attr("d", function (d) { return area(d.values); })
            .style("fill", function (d, i) { return self.colors(i); });

        svg.selectAll(".area")
            .each(function (el, i) {
                if (self.trendData.conceptInfo[i].valid)
                    d3.select(this).attr("data-legend", self.trendData.conceptInfo[i].label)
                else
                    d3.select(this).attr("data-legend", null);
                d3.select(this).attr("data-legend-color", self.colors(i));
                d3.select(this).attr("data-legend-pos", 1000-i);
            });

        svg.selectAll(".area")
            .on("mousemove", function (d, i) {
                // remove the highlight rect
                $("#" + self.containerName + " .day-rect").remove();
                
                var coords = d3.mouse(this);
                var date = self.xScale.invert(coords[0]);
                date.setHours(12); date.setMinutes(0); date.setSeconds(0); date.setMilliseconds(0);
                var dateInt = date.valueOf();
                // find the data for the selected date
                var matchingVal = $.grep(d.values, function (data) { return data.x.valueOf() == dateInt; });
                if (matchingVal.length == 0)
                    return;
                // highlight the selected area with a darker color
                var origFill = $(this).attr("orig-fill") ? $(this).attr("orig-fill") : $(this).css("fill");
                $(this).attr("orig-fill", origFill);
                var darkerColor = d3.rgb(origFill).darker();
                $(this).css("fill", darkerColor.toString());

                // show the tooltip with the conctent related to selected concept on selected date
                $("#" + self.containerName + " .graph-tooltip")
                    .css("left", self.xScale(date) + "px")
                    .css("top", (10 + yScale(matchingVal[0].y0 + (matchingVal[0].y / 2))) + "px")
                    .html("On <b>{0}</b> the concept <br/><b>'{1}'</b><br/> appeared in articles <br/><b>{2}</b> times".format(matchingVal[0].dateStr, self.trendData.conceptInfo[i].label, matchingVal[0].y))
                    .removeClass("hidden");
                // show rect around the whole day
                date.setHours(0); date.setMinutes(0); date.setSeconds(0); date.setMilliseconds(0);
                var xDateStart = self.xScale(date);
                date.setHours(23); date.setMinutes(59);
                var xDateEnd = self.xScale(date);
                svg.selectAll(".day-rect")
                    .data([{ }])
                    .enter()
                    .append("rect")
                    .attr("class", "day-rect")
                    .attr("x", xDateStart)
                    .attr("y", margin.top)
                    .attr("width", xDateEnd - xDateStart)
                    .attr("height", height-5)
                    .attr("fill", "rgba(0,0,0,0.20")
                    .style("pointer-events", "none");
            })
            .on("mouseout", function (d, i) {
                // hide the rectangle
                $("#" + self.containerName + " .day-rect").remove();
                //Hide the tooltip
                $("#" + self.containerName + " .graph-tooltip").addClass("hidden");
                var origFill = $(this).attr("orig-fill");
                if (origFill)
                    $(this).css("fill", origFill);
            });

        // remove old legend
        d3.select("#" + this.containerName + " svg .legend").remove();
        // remove old legend-box
        d3.select("#" + this.containerName + " svg .legend-box").remove();
        var legendBox = d3.select("#" + this.containerName + " svg")
            .append("rect")
            .classed("legend-box", true)
            .on("mouseleave", function (d) {              
                self.hideLegendLabels(400);
            })
            .on("mouseenter", function (d) {
                self.showLegendLabels(400);
            });

        // create new legend
        var legend = d3.select("#" + this.containerName + " svg")
            .append("g")
            .attr("transform", "translate(" + 5 + "," + margin.top + ")")
            .classed("legend", true);

        var visibleItems = svg.selectAll(".area[data-legend]")[0].reverse();
        var legendItems = legend
            .selectAll("g")
            .data(visibleItems)
            .enter().append("g")
            .attr("transform", function (d, i) {
                return "translate(0," + (i * 20) + ")";
            })

        legendItems.append("rect")
            .classed("legend-rect", true)
            .attr("width", 18)
            .attr("height", 18)
            .style("fill", function (d, i) {
                return d3.select(d).attr("data-legend-color");
            });

        legendItems
            .append("svg:a")
            .classed("legend-label", true)
            //.attr("xlink:href", function (d, i) {
            //    return self.trendData.conceptInfo[i].uri;
            //})
            //.attr("target", "_blank")
            .attr("opacity", 0.0)
            .append("text")
                //.classed("legend-label", true)
                .attr("pointer-events", "none")
                .attr("x", 24)
                .attr("y", 9)
                .attr("dy", ".35em")
                .text(function (d, i) {
                    return d3.select(d).attr("data-legend");
                });

        var padding = 5;
        var itemsBBox = legend[0][0].getBBox();
        legendBox
            .attr("x", "0px")
            .attr("y", "0px")
            .attr("height", (itemsBBox.height + 2 * padding) + "px")
            .attr("width", "30px")
            .attr("data-width", (itemsBBox.width + 2 * padding) + "px");

        //d3.select("#" + this.containerName + " svg .legend").remove();
        //legend = svg.append("g")
        //    .attr("class", "legend")
        //    .attr("transform", "translate(5," + margin.top +")")
        //    .style("font-size", "14px")
        //    .call(d3.legend);
    }

    this.setConceptTrends = function (trends) {
        this.clearTags();
        this.trendData = trends;
        this.setTags(trends.conceptInfo);
    }

    this.displayConceptTrends = function () {
        var data = this.trendData;
        var totWidth = $("#" + this.containerName).width() - 20,
            totHeight = $("#" + this.containerName).height(),
            margin = { top: 5, right: 25, bottom: 30, left: 25 },
            width = totWidth - margin.left - margin.right,
            height = totHeight - margin.top - margin.bottom,
            parse = d3.time.format("%Y-%m-%d").parse;
        
        d3.select("#" + this.containerName + " svg").remove();
        var svg = d3.select("#" + this.containerName)
            .append("svg")
            .attr("width", totWidth)
            .attr("height", totHeight)
            .append("g")
            .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

        svg.selectAll(".area")
            .data(data.conceptInfo.map(function(concept) { return {} }))
            .enter()
            .append("path")
            .attr("class", "area");
        
        var maxDate = parse(data.trends[data.trends.length - 1].date); maxDate.setHours(12);
        this.xScale = d3.time.scale()
            .range([0, width])
            .domain([parse(data.trends[0].date), maxDate])
            .nice(d3.time.day);   // start and end the x scale on a day

        var xAxis = d3.svg.axis().scale(this.xScale).orient("bottom");
        svg.append("g")
            .attr("class", "x axis")
            .attr("transform", "translate(0," + height + ")")
            .call(xAxis);
        
        this._updateTrends();
        this.showLegendLabels(0);
        var self = this;
        setTimeout(function () { self.hideLegendLabels(); }, 5000);
    }

    // helper methods
    this.showLegendLabels = function (animationDuration) {
        if (animationDuration == undefined)
            animationDuration = 400;
        d3.select(".legend-box")
            .transition()
            .duration(animationDuration)
            .attr("width", d3.select(".legend-box").attr("data-width"));
        d3.selectAll(".legend-label")
            .transition()
            .delay(animationDuration /2)
            .duration(animationDuration)
            .attr("opacity", 1);
        
    }

    this.hideLegendLabels = function (animationDuration) {
        if (animationDuration == undefined)
            animationDuration = 400;
        d3.selectAll(".legend-label")
            .transition()
            .duration(animationDuration / 2)
            .attr("opacity", 0.0);
        d3.select(".legend-box")
            .transition()
            .delay(animationDuration / 2)
            .duration(animationDuration)
            .attr("width", "30px");
        
    }
}

//var conceptTrends = new ConceptTrends();