
var minValidYear = 2000;
var maxValidYear = 2015;

//var parseDate = d3.time.format("%b %Y").parse;
var formatDate = d3.time.format("%Y-%m-%d");
var parseDate = formatDate.parse;

function drawTimeline(rawData) {
    function draw() {
      svg.select("g.x.axis").call(xAxis);
      svg.select("g.y.axis").call(yAxis);
      svg.select("path.area").attr("d", area);
      svg.select("path.line").attr("d", line);
      svg.selectAll(".dot")
        .data(data)
        .attr("class", "dot")
        .attr("r", function (d) { return d.value > 0 ? dotScale(d.value): 0; })
        .attr("cx", function(d) { var mid = new Date(d.date); mid.setHours(12); return x(mid); })
        .attr("cy", function(d) { return y(0) + 35; });
    }

    // remember the cleaned data in case we call the redrawTimeline()
    d3.select("#events-timeline svg").remove();

    if (rawData == null)
        return;

    var data = [];
    var seenDates = {};
    rawData.forEach(function (d) {
        if (d.date.indexOf("-") == 0 || d.date.lastIndexOf("-") == d.date.length - 1)
            return;
        var date = parseDate(d.date);
        if (date.getFullYear() < minValidYear || date.getFullYear() > maxValidYear)
            return;
        seenDates[d.date] = date;
        data.push({ value: d.count, date: date});
    });
    for (dateStr in seenDates) {
        var dateNext = new Date(seenDates[dateStr].valueOf() + (24*1000*60*60));
        var dateNextStr = formatDate(dateNext);
        if (seenDates[dateNextStr] == undefined)
            data.push({ value: 0, date: dateNext });
    }
    // sort the dates - otherwise the graph is a mess
    data.sort(function(a, b) {
        if (a.date > b.date) return 1;
        return -1;
    });
    var margin = {top: 20, right: 10, bottom: 50, left: 50},
    width = $("#events-timeline").width() - margin.left - margin.right - 5,
    height = $("#events-timeline").height() - margin.top - margin.bottom - 5;

    var x = d3.time.scale()
        .range([0, width]);

    var y = d3.scale.linear()
        .range([height, 0]);

    var dotScale = d3.scale.log()
        .range([2, 10])
        .domain([1, d3.max(data.map(function (d) { return d.value; }))]);

    var xAxis = d3.svg.axis()
        .scale(x)
        .orient("bottom")
        .tickPadding(6);

    var yAxis = d3.svg.axis()
        .scale(y)
        .orient("left")
        .tickSize(-width)
        .tickPadding(6);

    var area = d3.svg.area()
        .interpolate("step-after")
        .x(function(d) { return x(d.date); })
        .y0(y(0))
        .y1(function(d) { return y(d.value); });

    var line = d3.svg.line()
        .interpolate("step-after")
        .x(function(d) { return x(d.date); })
        .y(function(d) { return y(d.value); });

    var svg = d3.select("#events-timeline").append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

    var zoom = d3.behavior.zoom()
        .on("zoom", draw);

    svg.append("clipPath")
        .attr("id", "clip")
        .append("rect")
        .attr("x", x(0))
        .attr("y", y(1))
        .attr("width", x(1) - x(0))
        .attr("height", height + margin.left + margin.bottom);

    svg.append("g")
        .attr("class", "y axis");       

    svg.append("g")
        .attr("class", "x axis")
        .attr("transform", "translate(0," + height + ")");

    svg.append("path")
        .attr("class", "area")
        .attr("clip-path", "url(#clip)");

    svg.append("path")
        .attr("class", "line")
        .attr("clip-path", "url(#clip)");

    svg.append("rect")
        .attr("class", "pane")
        .attr("width", width)
        .attr("height", height)
        .call(zoom);

    x.domain(d3.extent(data.map(function (d) { return d.date; })));
    y.domain([0, d3.max(data.map(function (d) { return d.value; }))+2]);
    zoom.x(x);

    svg.select("path.area").datum(data);
    svg.select("path.line").datum(data);
    svg.selectAll(".dot")
        .data(data)
        .enter().append("circle")
        .attr("clip-path", "url(#clip)")
        .attr("class", "dot");
    draw();
}
