
// parse article date into d3s date format
var parseDateTime = d3.time.format("%Y-%m-%d %H:%M").parse;
function getArtDateTime(art) {
    return parseDateTime(art.date + " " + art.time);
}

// count the number of articles that there are in the provided time window windowLenHrs
// assign a property aggrCount to each article that tells the number of articles in last windowLenHrs hours
function countArticlesInWindow(articles, windowLenHrs) {
    var artsInWindow = [];
    $.each(articles, function (i, article) {
        var curr = Date.parse(article.date + " " + article.time)
        artsInWindow.push(curr);
        while (artsInWindow.length > 0 && (curr - artsInWindow[0]) / (3600 * 1000) > windowLenHrs)
            artsInWindow.shift();
        // remember how many articles there were in the given window
        article.aggrCount = artsInWindow.length;
    });
}

// remove from the list articles that share the same date and time
function removeArticlesWithSameDateTime(articles) {
    $.each(articles, function (i, art) { art.articlesAtDate = [art.uri]; });
    var i = 0;
    while (true) {
        if (i >= articles.length)
            break;
        if (i > 0 && articles[i - 1].date == articles[i].date && articles[i - 1].time == articles[i].time) {
            // we remove the i-1-th item so copy what we need into the i-th article
            articles[i].articlesAtDate.push.apply(articles[i].articlesAtDate, articles[i - 1].articlesAtDate);
            articles.splice(i - 1, 1);
        }
        else
            i++;
    }
}

// put the text element into container containerNm, set it's class to classname
// set it's text to text and measure the bounding box
function getTextBBox(text, containerNm, className) {
    if (!text || text.length === 0) return 0;
    var container = d3.select('body').append('svg');
    container.append('text')
        //.attr({ x: -1000, y: -1000 })
        .attr({ x: 100, y: 100 })
        .attr('class', className)
        .text(text);
    var bbox = container.node().getBBox();
    container.remove();
    return bbox;
};

function measureTextWidth(text, containerNm, className) {
    var bbox = getTextBBox(text, containerNm, className);
    return bbox.width;
}

function measureTextHeight(text, containerNm, className) {
    var bbox = getTextBBox(text, containerNm, className);
    return bbox.height;
}

function makeNiceConceptLabels(concepts, labelNm) {
    return $.each(concepts, function (i, item) {
        var text = item[labelNm] ? item[labelNm] : item.label;
        item.fullLabel = text;
        item.shortLabel = text.shorten(35);
    });
};

function ZoomOnOffHandler(_containerNm, _checkBoxNm, _d3Zoom, _onZoomHandler, defaultZoomEnabled) {
    var containerNm = _containerNm;
    var checkBoxNm = _checkBoxNm;
    var d3Zoom = _d3Zoom;
    var onZoomHandler = _onZoomHandler;

    var zoomingEnabled = defaultZoomEnabled != undefined ? defaultZoomEnabled : true;
    var lastScale = undefined;
    var lastTranslate = undefined;
    
    this.setOnZoomHandler = function(handler) {
        onZoomHandler = handler;
    }

    this.onZoom = function () {
        if (!zoomingEnabled)
            return;

        onZoomHandler & onZoomHandler();
    }

    $(checkBoxNm).on('change', function () {
        zoomingEnabled = $(this).is(':checked');
        // if we've disabled zooming then remember current translate and scale params
        if (zoomingEnabled == false) {
            lastScale = d3Zoom.scale();
            lastTranslate = d3Zoom.translate();
            d3.select(containerNm + " svg g").on("wheel.zoom", null);
        }
        else {
            if (lastScale)
                d3Zoom.scale(lastScale);
            if (lastTranslate)
                d3Zoom.translate(lastTranslate);
            d3.select(containerNm + " svg g").call(d3Zoom);
        }
        // set or remove the resize class on the pane
        $(containerNm + " .pane").attr("class", "pane" + (zoomingEnabled ? " resize" : ""));
    });

    function initZoomControl () {
        // set default zooming state
        $(checkBoxNm).attr("checked", zoomingEnabled);
        // if we wish to start with zooming disabled then we also need to disable the d3's zoom event catching
        if (!zoomingEnabled)
            d3.select(containerNm + " svg g").on("wheel.zoom", null);
    }

    //$(document).ready(function () {
    //    initZoomControl();
    //});

    initZoomControl();
    
    d3Zoom.on("zoom", this.onZoom);
}