
function TagCloud(_options) {
    var options = $.extend({
        containerNm: undefined,     // name of the dom element hosting the svg
        fontFamily : "Segoe UI",
        onClickCallback: undefined,
        colorCount: 4,
        colorPalette: d3.scale.category20(),
        minFontSize: 15,
        maxFontSize: 80,
        fontWeight: "bold",
        wordPadding: 2
    }, _options);

    this.setData = function(kwArray) {
        var maxVal = d3.max(kwArray, function (item) { return item.weight; });
        var minVal = d3.min(kwArray, function (item) { return item.weight; });
        var diff = maxVal - minVal;
        var data = $.map(kwArray, function (item) {
            return { text: item.keyword.toLowerCase(), size: (options.minFontSize + (options.maxFontSize - options.minFontSize) * ((item.weight - minVal) / diff)) };
        });
        var width = $(options.containerNm).width() - 20,
            height = $(options.containerNm).height() - 20;

        d3.layout.cloud().size([width, height])
            .words(data)
            .padding(options.wordPadding)
            //.rotate(function () { return ~~(Math.random() * 2) * 90; })
            .rotate(function () { return 0; })
            .font(options.fontFamily)
            .fontWeight(options.fontWeight)
            .fontSize(function (d) { return d.size; })
            .on("end", this._draw)
.start();
    }

    this._draw = function (words) {
        
        d3.select(options.containerNm + " svg").remove();
        var width = $(options.containerNm).width() - 20,
            height = $(options.containerNm).height() - 20;

        var svg = d3.select(options.containerNm).append("svg")
            .attr("width", width)
            .attr("height", height);

        svg.append("g")
            .attr("transform", "translate(" + width/2 + "," + height/2 + ")")
            .selectAll("text")
            .data(words)
              .enter().append("text")
                .style("font-size", function (d) { return d.size + "px"; })
                .style("font-family", function (d) { return d.font; })
                .style("font-weight", function (d) { return d.weight; })
                .style("fill", function (d, i) {
                    return options.colorPalette(i % options.colorCount);
                })
                .attr("text-anchor", "middle")
                .attr("transform", function (d) {
                    return "translate(" + [d.x, d.y] + ")rotate(" + d.rotate + ")";
                })
                .text(function (d) { return d.text; });
    }
}