function ViewerArticleTrends() {
    
    this.drawArticleTrends = function (articlesList, legendItems, height, artClickCallback) {
        function zoomed() {
            //var svg = d3.select("#article-trends-container svg");
            svg.select("g.x.axis").call(xAxis);
            svg.select("g.y.axis").call(yAxis);
            svg.selectAll("path.line").attr("d", line);
            svg.selectAll("circle.dot")
                .attr("cx", function (d) {
                    return xScale(getArtDateTime(d));
                })
                .attr("cy", function (d) { return yScale(d.aggrCount); });
        }
        function drawArticleData(articles, index) {
            // draw area
            chartBody.append("path")
                .datum(articles)
                .attr("class", "line")
                .attr("d", line)
                .attr("stroke", colors(index))
                
            // draw points for articles
            chartBody.selectAll(".dot" + index)
                .data(articles)
                .enter().append("circle")
                .attr("class", "dot")
                .attr("r", function(d,i) { return 4 + d.articlesAtDate.length })
                .attr("cx", function (d) { return xScale(getArtDateTime(d)); })
                .attr("cy", function (d) { return yScale(d.aggrCount); })
                .style("stroke", function (d) { return colors(index); })
                .on("mouseover", function (d) {
                    //Get this bar's x/y values, then augment for the tooltip
                    var x = parseFloat(d3.select(this).attr("cx"));
                    var y = parseFloat(d3.select(this).attr("cy")) + 20;

                    //Update the tooltip position and value
                    d3.select("#trend-tooltip")
                        .style("left", x + "px")
                        .style("top", y + "px")
                        .select("#value");
                    $("#trend-tooltip .title").html(d.title);
                    $("#trend-tooltip .source").html(d.source_title);
                    $("#trend-tooltip .date").html("Article date: " + d.date + " " + d.time);
                    if (d.articlesAtDate && d.articlesAtDate.length > 1)
                        $("#trend-tooltip .count").html("+" + (d.articlesAtDate.length - 1) + " articles");
                    else
                        $("#trend-tooltip .count").html("");
                    d3.select("#trend-tooltip").classed("hidden", false);
                })
                .on("mouseout", function () {
                    //Hide the tooltip
                    d3.select("#trend-tooltip").classed("hidden", true);
                })
                .on("click", function (article) {
                    if (artClickCallback)
                        artClickCallback(article.articlesAtDate);
                    //scope.selectedArticleList.splice(0, scope.selectedArticleList.length);
                    //scope.selectedArticleList.push(article);
                    //scope.$apply();
                });
        }

        d3.select("#article-trends-container svg").remove();
        var interpolation = "linear";
        var colors = d3.scale.category10();
        var totWidth = $("#article-trends-container").width();  
        var margin = { top: 10, right: 40, bottom: 40, left: 40 },
            width = totWidth - margin.left - margin.right;
        var totHeight = height + margin.top + margin.bottom;
        if (legendItems && legendItems.length > 0)
            totHeight += legendItems.length * 30;

        var xScale = d3.time.scale().range([0, width]);
        var yScale = d3.scale.linear().range([height, 0]);

        xAxis = d3.svg.axis().scale(xScale).orient("bottom");
        yAxis = d3.svg.axis().scale(yScale).orient("left");

        var line = d3.svg.area()
            .interpolate(interpolation)
            .x(function (d) { return xScale(getArtDateTime(d)); })
            .y(function (d, i) { return yScale(d.aggrCount); });
                
        var zoom = d3.behavior.zoom()
            .on("zoom", zoomed);

        var svg = d3.select("#article-trends-container")
            .append("svg")
            .attr("width", width + margin.left + margin.right)
            .attr("height", totHeight)
            .append("g")
            .attr("transform", "translate(" + margin.left + "," + margin.top + ")")
            .call(zoom);
        
        svg.append("rect")
            .attr("class", "pane")
            .attr("width", width)
            .attr("height", height);

        svg.append("clipPath")
            .attr("id", "clip")
            .append("rect")
            .attr("x", 0)
            .attr("y", 0)
            .attr("width", width)
            .attr("height", height);

        svg.append("g")
            .attr("class", "x axis")
            .attr("transform", "translate(0," + height + ")")
            .call(xAxis);

        svg.append("g")
            .attr("class", "y axis")
            .call(yAxis);         

        var articles = d3.merge(articlesList);
        xScale.domain(d3.extent(articles.map(function (d) { return getArtDateTime(d); })));
        xScale.nice(d3.time.day);   // start and end the x scale on a day
        zoom.x(xScale);

        var chartBody = svg.append("g")
            .attr("clip-path", "url(#clip)");

        $.each(articlesList, function (i, arts) {
            countArticlesInWindow(arts, 8);  // add to each article the number of articles in the last X hours
            removeArticlesWithSameDateTime(arts); // remove articles with same datetime so that we don't have two y values for same x value
        });

        var maxYVal = d3.max(articlesList, function (arts) { return d3.max(arts, function (d) { return d.aggrCount; }) });
        yScale.domain([0, maxYVal + 0.02 * maxYVal]);
        $.each(articlesList, function (i, arts) {
            drawArticleData(arts, i);
        });

        if (legendItems && legendItems.length > 0) {
            // legend
            var legend = svg.selectAll('g.legend')
                .data(legendItems)
                .enter()
                .append('g')
                .attr('class', 'legend');

            legend.append('circle')
                .attr('cx', 80)
                .attr('cy', function (d, i) { return margin.top + height + 40 + 4 + (i * 30); })
                .attr('r', 7)
                //.attr('height', 10)
                .style('fill', function (d, i) {
                    return colors(i);
                });

            legend.append('text')
                .attr('x', 100)
                .attr('y', function (d, i) { return margin.top + height + 40 + 10 + (i * 30); })
                .text(function (d, i) { return legendItems[i] });
        }
        zoomed();
    };
};

var viewerArticleTrends = new ViewerArticleTrends();

