
function getEventInfo(event, lang) {
    var info = event.multiLingInfo[lang];
    if (!info && event.multiLingInfo && Object.keys(event.multiLingInfo).length > 0)
        info = event.multiLingInfo[Object.keys(event.multiLingInfo)[0]];
    return info;
}

function getEventTitle(event, lang) {
    var info = getEventInfo(event, lang);
    if (info) return info.title;
    return "";
}

function displaySimilarEvents(similarData) {
    var lang = $("#lang-combo").val();
    var bar_height = 20,
            gap = 2;
    var height = (bar_height + 2 * gap) * similarData.similarEvents.length,
        width = $("#similar-events-graph").width(),
        xChartOff = 20,
        chartWidth = 200;
    var xTextOff = xChartOff + chartWidth + 20;
    
    d3.select("#similar-events-graph svg").remove();
    var chart = d3.select("#similar-events-graph")
            .append('svg')
            .attr('class', 'chart')
            .attr('width', width)
            .attr('height', height);
    var barWidthScale = d3.scale.linear()
            .domain([0, 1])
            .range([0, chartWidth]);
    var y = d3.scale.ordinal()
            .domain(d3.range(0, similarData.similarEvents.length))
            .rangeBands([0, height]);
    // rect
    chart.selectAll("#similar-events-graph rect")
        .data(similarData.similarEvents)
        .enter().append("rect")
        .attr("x", xChartOff)
        .attr("y", function(val, ind) { return y(ind) + gap; })
        .attr("height", bar_height)
        .attr("width", function(val) { return 0; })
        .transition()
        .duration(1000)
        .attr("width", function(val) { return barWidthScale(val.sim); });
    
    // scores
    chart.selectAll("#similar-events-graph text.score")
        .data(similarData.similarEvents)
        .enter().append("text")
        .attr('class', 'score')
        .attr("y", function(val, ind) { return y(ind) + y.rangeBand() / 2; })
        .attr("dx", -5)
        .attr("dy", ".36em")
        .attr("text-anchor", "end")
        .text(function(val) { return Math.round(100 * val.sim) + "%"; })
        .attr("x", function (val, ind) { return xChartOff; })
        .transition()
        .duration(1000)
        .attr("x", function(val, ind) { return xChartOff + barWidthScale(val.sim); });
    
    // labels
    chart.selectAll("#similar-events-graph text.name")
        .data(similarData.similarEvents)
        .enter()
        .append("svg:a")
            .attr("xlink:href", function (val) {
                var validLang = "";
                if (val.eventInfo && val.eventInfo[lang]) validLang = lang;
                else if (val.eventInfo && Object.keys(val.eventInfo).length > 0) validLang = Object.keys(val.eventInfo)[0];
                return window.location.origin + "/event/" + val.uri + (validLang != "" ? "?lang=" + validLang : "");
            })
            .attr("target", "_blank")
            .append("text")
                .attr('class', 'name')
                .attr("x", xTextOff )
                .attr("y", function (val, ind) { return y(ind) + y.rangeBand() / 2; })
                .attr("id", function (val, ind) { return val.uri; })
                .attr("dy", ".36em")
                .attr("text-anchor", "start")
                .text(function (val) {
                    var title = getEventTitle(val, lang);
                    title = title.substring(0, 200) + (title.length > 200 ? "..." : "");
                    if (val.eventDate)
                        title += " (" + val.eventDate.replace(/-/g, "/") + ")";
                    return title;
                });

    $("#similar-events-graph text.name").on("mouseover", function() {
        var x = 0;
    });
}