function EventLocations() {

    this.map = null;
    this.markerCluster = null;
    this.infoBubble = null;

    this.bubbleTimer = null;

    this.setBubbleTimeout = function () {
        var self = this;
        clearTimeout(self.bubbleTimer);
        self.bubbleTimer = setTimeout(function () {
            if (self.infoBubble)
                self.infoBubble.close();
            clearTimeout(self.bubbleTimer);
        }, 2000);
    }

    this.resetData = function() {
        this.map = null;
        this.markerCluster = null;
    }

    this.hasMap = function () {
        return this.map != null;
    }

    this.createNewMap = function () {
        var self = this;
        // init map
        var mapOptions = {
            center: new google.maps.LatLng(0, 0),
            zoom: 1, mapTypeId: google.maps.MapTypeId.ROADMAP
        };
        this.map = new google.maps.Map(document.getElementById("events-map"), mapOptions);
        this.markerCluster = new MarkerClusterer(this.map, []);
        this.infoBubble = new InfoBubble({
            map: this.map,
            content: '<div class="phoneytext">Some </div>',
            position: new google.maps.LatLng(-35, 151),
            shadowStyle: 1,
            padding: 0,
            backgroundColor: 'rgb(57,57,57)',
            borderRadius: 10,
            arrowSize: 10,
            borderWidth: 0,
            disableAutoPan: true,
            hideCloseButton: true,
            disableAnimation: true,
            arrowPosition: 30,
            backgroundClassName: 'info-bubble-back',
            arrowStyle: 2
        });
        // show the location name in the tooltip
        google.maps.event.addListener(this.markerCluster, "mouseover", function (c) {
            var marker = c.getMarkers()[0];
            self.infoBubble.setPosition(marker.position);
            self.infoBubble.setContent('<div class="info-bubble-text">' + marker.title + '</div>');
            self.infoBubble.open();
            self.setBubbleTimeout();
        });

        //google.maps.event.addListener(this.markerCluster, "mouseout", function (c) {
        //    console.log("out");
        //    infoBubble.close();
        //});
        // we need to monitor the zoom event since the infobubble can otherwise stay open
        google.maps.event.addListener(this.map, 'zoom_changed', function () {
            self.infoBubble.close();
        });
    };

    this.triggerResizeEvent = function () {
        if (this.map)
            google.maps.event.trigger(this.map, 'resize');
    }
}