// d3.legend.js 
// (C) 2012 ziggy.jonsson.nyc@gmail.com
// MIT licence
 
(function() {
d3.legend = function(g) {
  g.each(function() {
    var g = d3.select(this),
        items = {},
        svg = d3.select(g.property("nearestViewportElement")),
        legendPadding = g.attr("data-style-padding") || 10,
        lb = g.selectAll(".legend-box").data([true]),
        li = g.selectAll(".legend-items").data([true]),
        lh = g.selectAll(".legend-head").data([true]),
        marginItemsTop = 30;
 
    lb.enter().append("rect").classed("legend-box", true);
    li.enter().append("g").classed("legend-items", true)
        .attr("transform", "translate(0," + marginItemsTop + ")");
    lh.enter().append("g").classed("legend-head", true)
      .attr("transform", "translate(0,0)");
 
    svg.selectAll("[data-legend]").each(function () {
        var self = d3.select(this);
        items[self.attr("data-legend")] = {
            pos: self.attr("data-legend-pos") || this.getBBox().y,
            color: self.attr("data-legend-color") != undefined ? self.attr("data-legend-color") : self.style("fill") != 'none' ? self.style("fill") : self.style("stroke")
        };
    });
 
    items = d3.entries(items).sort(function (a, b) { return a.value.pos - b.value.pos });
     
    li.selectAll("text")
        .data(items, function (d) { return d.key; })
        .call(function (d) { d.enter().append("text") })
        .call(function (d) { d.exit().remove() })
        .attr("y", function (d, i) { return (i * 1.2) + "em" })
        .attr("class", "legend-text")
        .attr("x", "1.2em")
        .text(function (d) { return d.key; });
    
    li.selectAll("rect")
        .data(items, function (d) { return d.key; })
        .call(function (d) { d.enter().append("rect") })
        .call(function (d) { d.exit().remove() })
        .attr("y", function (d, i) { return (i * 1.2) - 0.8 + "em"; })
        .attr("x", 0)
        .attr("width", "0.8em")
        .attr("height", "0.8em")
        .attr("class", "legend-symbol")
        .style("fill", function (d) { console.log(d.value.color); return d.value.color });
    
    var plusPath = "M 256.00-32.00C 114.625-32.00,0.00,82.609,0.00,224.00S 114.625,480.00, 256.00,480.00 c 141.391,0.00, 256.00-114.609, 256.00-256.00S 397.391-32.00, 256.00-32.00z M 256.00,416.00C 149.969,416.00, 64.00,330.031, 64.00,224.00s 85.969-192.00, 192.00-192.00 c 106.047,0.00, 192.00,85.969, 192.00,192.00S 362.047,416.00, 256.00,416.00z M 288.00,96.00l-64.00,0.00 l0.00,96.00 l-96.00,0.00 l0.00,64.00 l 96.00,0.00 l0.00,96.00 l 64.00,0.00 l0.00-96.00 l 96.00,0.00 l0.00-64.00 l-96.00,0.00 L 288.00,96.00 z";

    var buttRect = lh.append("rect")
        .attr("x", "-5px")
        .attr("y", "-5px")
        .attr("width", "20px")
        .attr("height", "20px")
        .attr("fill", "red");

    lh.append("g")
        .append("path")
        .attr("transform", "translate(-270,-270) scale(0.1,0.1)")
        .attr("d", plusPath);

    // add the "legend" text
    lh.append("text")
        .datum("Legend")
        .text(function (d) { return d; })
        .attr("font-weight", "bold")
        .attr("y", "0.7em")
        .attr("x", "1.2em");

    // Reposition and resize the box
    var lbbox = li[0][0].getBBox();
    
    lb.attr("x", (lbbox.x - legendPadding))
        .attr("y", (-lbbox.y - legendPadding - marginItemsTop))
        .attr("height", (lbbox.height + marginItemsTop + 2 * legendPadding))
        .attr("width", (lbbox.width + 2 * legendPadding));
  })
  return g;
}
})()