/*

	jQuery Tags Input Plugin 1.3.3
	
	Copyright (c) 2011 XOXCO, Inc
	
	Documentation for this plugin lives here:
	http://xoxco.com/clickable/jquery-tags-input
	
	Licensed under the MIT license:
	http://www.opensource.org/licenses/mit-license.php

	ben@xoxco.com

*/

(function($) {
	var delimiter = new Array();
	var tags_callbacks = new Array();
    $.fn.doAutosize = function(o) {
        var minWidth = $(this).data('minwidth'),
            maxWidth = $(this).data('maxwidth'),
            val = '',
            input = $(this),
            testSubject = $('#' + $(this).data('tester_id'));

        var val = input.val();
        if (val == "")
            val = o.defaultText;

        if (val == "")
            return;

        // Enter new content into testSubject
        var escaped = val.replace(/&/g, '&amp;').replace(/\s/g, ' ').replace(/</g, '&lt;').replace(/>/g, '&gt;');
        testSubject.html(escaped);
        // Calculate new width + whether to change
        var testerWidth = testSubject.width(),
            newWidth = (testerWidth + o.comfortZone) >= minWidth ? testerWidth + o.comfortZone : minWidth,
            currentWidth = input.width(),
            isValidWidthChange = (newWidth < currentWidth && newWidth >= minWidth)
                || (newWidth > minWidth && newWidth < maxWidth);

        // Animate width
        if (isValidWidthChange) {
            input.width(newWidth);
        }
    };

    $.fn.resetAutosize = function(options) {
        // alert(JSON.stringify(options));
        var minWidth = $(this).data('minwidth') || options.minInputWidth || $(this).width(),
            maxWidth = $(this).data('maxwidth') || options.maxInputWidth || ($(this).closest('.tagsinput').width() - options.inputPadding),
            val = '',
            input = $(this),
            testSubject = $('<tester/>').css({
                position: 'absolute',
                top: -9999,
                left: -9999,
                width: 'auto',
                fontSize: input.css('fontSize'),
                fontFamily: input.css('fontFamily'),
                fontWeight: input.css('fontWeight'),
                letterSpacing: input.css('letterSpacing'),
                whiteSpace: 'nowrap'
            }),
            testerId = $(this).attr('id') + '_autosize_tester';
        if (!$('#' + testerId).length > 0) {
            testSubject.attr('id', testerId);
            testSubject.appendTo('body');
        }

        input.data('minwidth', minWidth);
        input.data('maxwidth', maxWidth);
        input.data('tester_id', testerId);
        input.css('width', minWidth);
    };

    $.fn.addTag = function(value, options) {
        options = jQuery.extend({ focus: false, callback: true, classes : [] }, options);
        this.each(function() {
            var id = $(this).attr('id');
            value = jQuery.trim(value);

            if (value != '') {
                var newTag = $('<span>').addClass('tag');
                $.each(options.classes, function(i, val) {
                    newTag.addClass(val);
                });
                newTag.append($('<span>').text(value).append('&nbsp;&nbsp;'),
                    $('<a class="tagsinput-remove-link fui-cross-16">', {
                        href: '#',
                        title: 'Remove tag',
                        text: ''
                    }).click(function() {
                        return $('#' + id).removeTag(newTag);
                    })
                );
                $('#' + id + '_tagList').append(newTag);
                newTag.data("options", options);

                // clear the input box
                $('#' + id + '_tag').val('');
                if (options.focus) {
                    $('#' + id + '_tag').focus();
                } else {
                    $('#' + id + '_tag').blur();
                }

                if (options.callback && tags_callbacks[id] && tags_callbacks[id]['onAddTag']) {
                    var f = tags_callbacks[id]['onAddTag'];
                    f.call(this, newTag);
                }
                if (tags_callbacks[id] && tags_callbacks[id]['onChange']) {
                    var i = tagslist.length;
                    var f = tags_callbacks[id]['onChange'];
                    f.call(this, $(this), newTag);
                }
                // since we have at least one tag in the inputbox don't show the text if out of focus
                $('#' + id + '_tag').attr("placeholder", "");
            }

        });

        return false;
    };
		
    $.fn.removeTag = function (object) {

        $(object).detach();
        this.each(function() {
            var id = $(this).attr('id');
            if (tags_callbacks[id] && tags_callbacks[id]['onRemoveTag']) {
                var f = tags_callbacks[id]['onRemoveTag'];
                f.call(this, object);
            }
            if ($("#" + id + "_tagList .tag").length == 0)
                $('#' + id + '_tag').attr("placeholder", $('#' + id).data("settings").defaultText);
        });
        return false;
	};
	
	$.fn.tagExist = function(val) {
		var id = $(this).attr('id');
		var tagslist = $(this).val().split(delimiter[id]);
		return (jQuery.inArray(val, tagslist) >= 0); //true when tag exists, false when not
	};
	
	
    $.fn.tagsInput = function(options) {
        var settings = jQuery.extend({
            interactive: true,
            defaultText: '',
            minChars: 0,
            width: '',
            height: '',
            autocomplete: { selectFirst: false },
            'hide': true,
            'delimiter': ',',
            'unique': true,
            removeWithBackspace: true,
            placeholderColor: '#666666',
            autosize: true,
            comfortZone: 20,
            inputPadding: 6 * 2
        }, options);

        // GL: remember the settings
        this.data("settings", settings);
        this.data("nextId", 1);

        this.each(function() {
            if (settings.hide) {
                $(this).hide();
            }
            var id = $(this).attr('id');
            if (!id || delimiter[$(this).attr('id')]) {
                id = $(this).attr('id', 'tags' + new Date().getTime()).attr('id');
            }

            var data = jQuery.extend({
                pid: id,
                real_input: '#' + id,
                holder: '#' + id + '_tagsinput',
                input_wrapper: '#' + id + '_addTag',
                fake_input: '#' + id + '_tag'
            }, settings);

            delimiter[id] = data.delimiter;

            if (settings.onAddTag || settings.onRemoveTag || settings.onChange) {
                tags_callbacks[id] = new Array();
                tags_callbacks[id]['onAddTag'] = settings.onAddTag;
                tags_callbacks[id]['onRemoveTag'] = settings.onRemoveTag;
                tags_callbacks[id]['onChange'] = settings.onChange;
            }

            var markup = '<div id="' + id + '_tagsinput" class="tagsinput"><span id="' + id + '_tagList"></span><div class="tagsinput-add-container" id="' + id + '_addTag">';

            if (settings.interactive) {
                markup = markup + '<input id="' + id + '_tag" value="" placeholder="' + settings.defaultText + '" />';
            }
            markup = markup + '</div></div>';

            //var markup = '<div id="' + id + '_tagsinput" class="tagsinput"><div id="' + id + '_addTag">';

            //if (settings.interactive) {
            //    markup = markup + '<input id="' + id + '_tag" value="" placeholder="' + settings.defaultText + '" />';
            //}

            //markup = markup + '</div><div class="tags_clear"></div></div>';


            $(markup).insertAfter(this);

            $(data.holder).css('width', settings.width);
            $(data.holder).css('min-height', settings.height);
            $(data.holder).css('height', '100%');

            if (settings.interactive) {
                $(data.fake_input).val($(data.fake_input).attr('placeholder'));
                $(data.fake_input).css('color', settings.placeholderColor);
                $(data.fake_input).resetAutosize(settings);
                $(data.fake_input).doAutosize(settings);

                $(data.holder).bind('click', data, function(event) {
                    $(event.data.fake_input).focus();
                });

                $(data.fake_input).bind('focus', data, function(event) {
                    if ($(event.data.fake_input).val() == $(event.data.fake_input).attr('placeholder')) {
                        $(event.data.fake_input).val('');
                    }
                    $(event.data.fake_input).css('color', '#000000');
                });

                if (settings.autocomplete_url != undefined) {
                    autocomplete_options = { source: settings.autocomplete_url };
                    for (attrname in settings.autocomplete) {
                        autocomplete_options[attrname] = settings.autocomplete[attrname];
                    }

                    if (jQuery.Autocompleter !== undefined) {
                        $(data.fake_input).autocomplete(settings.autocomplete_url, settings.autocomplete);
                        $(data.fake_input).bind('result', data, function(event, data, formatted) {
                            if (data) {
                                $('#' + id).addTag(data[0] + "", { focus: true, unique: (settings.unique) });
                            }
                        });
                    } else if (jQuery.ui.autocomplete !== undefined) {
                        $(data.fake_input).autocomplete(autocomplete_options);
                        $(data.fake_input).bind('autocompleteselect', data, function(event, ui) {
                            $(event.data.real_input).addTag(ui.item.value, { focus: true, unique: (settings.unique) });
                            return false;
                        });
                    }


                } 
                // if user types a comma, create a new tag
                $(data.fake_input).bind('keypress', data, function(event) {
                    /*if (event.which == event.data.delimiter.charCodeAt(0) || event.which == 13) {
                        event.preventDefault();
                        if ((event.data.minChars <= $(event.data.fake_input).val().length) && (!event.data.maxChars || (event.data.maxChars >= $(event.data.fake_input).val().length)))
                            $(event.data.real_input).addTag($(event.data.fake_input).val(), { focus: true, unique: (settings.unique) });
                        $(event.data.fake_input).resetAutosize(settings);
                        return false;
                    } else */ if (event.data.autosize) {
                        $(event.data.fake_input).doAutosize(settings);

                    }
                });
                //Delete last tag on backspace
                data.removeWithBackspace && $(data.fake_input).bind('keydown', function(event) {
                    if (event.keyCode == 8 && $(this).val() == '') {
                        event.preventDefault();
                        var id = $(this).attr('id').replace(/_tag$/, '');
                        var tags = $("#" + id + "_tagList .tag");
                        if (tags.length > 0)
                            $('#' + id).removeTag(tags.last());
                        $(this).trigger('focus');
                    }
                });
                $(data.fake_input).blur();

                //Removes the not_valid class when user changes the value of the fake input
                if (data.unique) {
                    $(data.fake_input).keydown(function(event) {
                        if (event.keyCode == 8 || String.fromCharCode(event.which).match(/\w+|[áéíóúÁÉÍÓÚñÑ,/]+/)) {
                            $(this).removeClass('not_valid');
                        }
                    });
                }
            } // if settings.interactive
        });

        return this;

    };
	
	
})(jQuery);
