
function clearTime() {
    $('#dp-start').val("");
    $('#dp-end').val("");
}

function setTimeSpan(startSpan, endSpan) {
    var start = d3.time.day.offset(Date.today(), -startSpan);
    var end = d3.time.day.offset(Date.today(), -endSpan);
    $('#dp-start').data('datepicker').setValue(start);
    $('#dp-end').data('datepicker').setValue(end);
}

// handler for clicking a link for adding time condition
function setTime(start, end) {
    $('#dp-start').data('datepicker').setValue(start);
    $('#dp-end').data('datepicker').setValue(end);
};

// when a concept in an article is clicked it is added to the conditions
function addConceptConstraint(element) {
    var uri = $(element).attr("data-uri");
    var type = $(element).attr("data-type");
    var label = $(element).text();
    $('#articleConditions').addTag(label, { focus: true, type: type, uri: uri, label: label, classes: [type, "white", "concept"] });
};

eventRegApp.controller("SearchArticlesCtrl", function ($scope, $http, $browser, $location, langOptions, utils) {
    $scope.utils = utils;
    $scope.langOptions = langOptions;

    $scope.articlesPerPage = 25;
    $scope.articleConceptTrends = new ConceptTrendsGraph({ containerNm: "#concept-trends", tagInputNm: "#trending-concept-list" });
    // concept graphs
    $scope.entityGraph = new TopConceptList({ containerNm: "#concepts-entities" });
    $scope.keywordGraph = new TopConceptList({ containerNm: "#concepts-keywords" });
    // tag cloud
    $scope.tagCloud = new TagCloud({ containerNm: "#keyword-tagcloud" });
    // article timeline 
    $scope.articleTimelineGraph = new DateBarChart({ containerNm: "#article-timeline", tooltipTextCallback: $scope.getArticlesTimelineTooltip });
    
    $scope.searchOpt = {
        isOpen: false,
        articleLangs: [{ name: "Any language", val: "any" }, { name: "English", val: "eng" }, { name: "Spanish", val: "spa" }, { name: "German", val: "deu" }, { name: "Chinese", val: "zho" }, { name: "Slovene", val: "slv" }],
        articleLang: { name: "Any language", val: "any" },
        sortBy: "rel"
    };
    $scope.searchInProgress = true;
    
    $scope.vizTabs = { articles: true, concepts: false, tagCloud: false, time: false, entityGraph: false, conceptTrends: false };
    
    // //////////////////
    // handler for clicking the search button
    $scope.searchForArticles = function (skipIfNoConditions) {
        $scope.updateUrlFromForm();
        // clear existing content
        $scope.resetData();
        var args = "";
        var queryStr = $location.search().query;
        var query = queryStr ? JSON.parse(queryStr) : {};
        if (query.conditions) {
            $.each(query.conditions, function (i, element) {
                args += "conceptUri=" + encodeURIComponent(element.uri) + "&";
            });
        }
        if (query.publisherUri)
            args += "publisherUri=" + encodeURIComponent(query.publisherUri) + "&";
        if (query.date)
            args += "timeStart=" + encodeURIComponent(query.date.start) + "&timeEnd=" + encodeURIComponent(query.date.end) + "&";
        if (query.lang)
            args += "lang=" + encodeURIComponent(query.lang) + "&";
        // if not all languages are checked then add the condition for language
        if (skipIfNoConditions && args == "")
            return;
        
        args += "sortBy=" + query.sortBy;
        $scope.queryArgs = args;
        $scope.searchInProgress = true;
        $scope.utils.showWait("", ".search-in-progress");
        $scope.$apply();
        $scope.loadArticlePage(1, function() {
            $scope.searchInProgress = false;
        });
    };

    $scope.loadArticlePage = function (selectedPage, finishedCallback) {
        if ($scope.queryArgs == "")
            return;

        $scope.articles = [];
        $scope.utils.showWait("", "#tab-articles .wait-container");

        var args = $scope.queryArgs + "&page=" + (selectedPage-1) + "&count=" + $scope.articlesPerPage + "&resultType=list";
        console.log(utils.getTimeStamp() + " asking for article page " + selectedPage);
        $http.get("/json/article?action=getArticles&" + args)
            .success(function (articleData, status, headers, config) {
                console.log(utils.getTimeStamp() + " retrieved article page " + $scope.articlePage);
                $scope.utils.hideWait("", "#tab-articles .wait-container");
                $scope.utils.hideWait("", ".search-in-progress");
                $scope.error = articleData.error;

                if ($scope.articlePage == 1)
                    $scope.totalArticleCount = articleData.resultCount ? articleData.resultCount : 0;

                $scope.articles = articleData.results;
                finishedCallback && finishedCallback();
                
                // init lightbox after the dom has updated. not a nice hack but no other clear way
                setTimeout(function () {
                    initLightbox();
                }, 500);
            })
            .error(function(data, status, headers, config) {
                $scope.utils.hideWait("", "#tab-articles .wait-container");
                $scope.utils.hideWait("", ".search-in-progress");
                finishedCallback && finishedCallback();
            });
        
    };

    $scope.visualizeArticleConcepts = function () {
        if ($scope.conceptData) {
            $scope.redrawArticleConcepts();
            return;
        }
            
        $scope.utils.showWait("#tab-concepts .content", "#tab-concepts .wait-container");

        console.log(utils.getTimeStamp() + " asking for article concepts");
        $http.get("/json/article?action=getArticles&resultType=conceptAggr&conceptCount=40&" + $scope.queryArgs)
            .success(function (data, status, headers, config) {
                console.log(utils.getTimeStamp() + " retrieved article concepts");
                $scope.utils.hideWait("#tab-concepts .content", "#tab-concepts .wait-container");
                $scope.error = data.error;
                $scope.conceptData = data.conceptAggr;
                $scope.redrawArticleConcepts();
        });
    }

    $scope.redrawArticleConcepts = function () {
        var entities = $scope.conceptData ? $.grep($scope.conceptData, function (concept, i) { return entityTypes[concept.type]; }) : [];
        var keywords = $scope.conceptData ? $.grep($scope.conceptData, function (concept, i) { return keywordTypes[concept.type]; }) : [];
        entities = entities.slice(0, 40);
        keywords = keywords.slice(0, 40);
        $scope.entityGraph.setData(entities, $scope.langOptions.lang.val);
        $scope.keywordGraph.setData(keywords, $scope.langOptions.lang.val);
    }

    // keyword tag cloud
    $scope.visualizeTagCloud = function () {
        if ($scope.tagCloudData) {
            $scope.redrawTagCloud();
            return;
        }

        $scope.utils.showWait("#tab-tagcloud .content", "#tab-tagcloud .wait-container");

        $http.get("/json/article?action=getArticles&resultType=keywordAggr&" + $scope.queryArgs)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-tagcloud .content", "#tab-tagcloud .wait-container");
                $scope.error = data.error;
                $scope.tagCloudData = data.keywordAggr;
                $scope.redrawTagCloud();
            });
    }

    $scope.redrawTagCloud = function () {
        $scope.tagCloud.setData($scope.tagCloudData);
    }

    // // article timeline
    $scope.visualizeArticleTimes = function () {
        if ($scope.timelineData) {
            $scope.articleTimelineGraph.setData($scope.timelineData);
            return;
        }
        $scope.utils.showWait("#tab-timeline .content", "#tab-timeline .wait-container");
        
        console.log(utils.getTimeStamp() + " asking for article times");
        $http.get("/json/article?action=getArticles&resultType=timeAggr&" + $scope.queryArgs)
            .success(function (data, status, headers, config) {
                console.log(utils.getTimeStamp() + " retrieved article times");
                $scope.utils.hideWait("#tab-timeline .content", "#tab-timeline .wait-container");
                $scope.error = data.error;
                $scope.timelineData = data.timeAggr;
                $scope.articleTimelineGraph.setData($scope.timelineData);
            });
    };
    
    $scope.getArticlesTimelineTooltip = function() {
        console.log(arguments);
    }

    // // entity graph
    $scope.visualizeConceptGraph = function () {
        if (!$scope.entityGraphData)
            $scope.updateConceptGraph(true);
    };

    $scope.updateConceptGraph = function (showWait) {
        var types = "";
        $.each($(".concept-graph-checkboxes label.checked"), function (i, item) {
            types += "&conceptType=" + $(item).children("input").attr("value");
        });
        showWait && $scope.utils.showWait("#tab-concept-graph .content", "#tab-concept-graph .wait-container");
        
        console.log(utils.getTimeStamp() + " asking for concept graph");
        $http.get("/json/article?action=getArticles&resultType=conceptGraph&" + $scope.queryArgs + types)
            .success(function (data, status, headers, config) {
                console.log(utils.getTimeStamp() + " retrieved concept graph");
                $scope.utils.hideWait("#tab-concept-graph .content", "#tab-concept-graph .wait-container");
                $scope.error = data.error;
                $scope.entityGraphData = data.conceptGraph;
                showConceptGraph($scope.entityGraphData);
            });
    };

    $scope.visualizeConceptTrends = function() {
        if ($scope.articleConceptTrends.trendData)
            return;
        var types = "";
        $.each($(".trending-checkboxes label.checked"), function (i, item) {
            types += "&conceptType=" + $(item).children("input").attr("value")
        });
        $scope.utils.showWait("#tab-concept-trends .content", "#tab-concept-trends .wait-container");
        console.log(utils.getTimeStamp() + " asking for trending concepts");
        $http.get("/json/article?action=getArticles&resultType=trendingConcepts&" + $scope.queryArgs + types)
            .success(function (data, status, headers, config) {
                console.log(utils.getTimeStamp() + " retrieved trending concepts");
                $scope.utils.hideWait("#tab-concept-trends .content", "#tab-concept-trends .wait-container");
                $scope.error = data.error;
                $scope.ignoreOnAddTrendingConcept = true;
                $scope.articleConceptTrends.setConceptTrends(data);
                $scope.articleConceptTrends.displayConceptTrends();
                $scope.ignoreOnAddTrendingConcept = false;
            });
    };
    
    $scope.visualizeCategories = function () {
        if ($scope.categoryData) {
            $scope.redrawCategories();
            return;
        }
        $scope.utils.showWait("#tab-categories .content", "#tab-categories .wait-container");
        console.log(utils.getTimeStamp() + " asking for article categories");
        $http.get("/json/article?action=getArticles&resultType=categoryAggr&" + $scope.queryArgs)
            .success(function (data, status, headers, config) {
                console.log(utils.getTimeStamp() + " retrieved article categories");
                $scope.utils.hideWait("#tab-categories .content", "#tab-categories .wait-container");
                $scope.error = data.error;
                $scope.categoryData = data.categoryAggr;
                $scope.redrawCategories();
            });
    };
    
    $scope.redrawCategories = function () {
        $scope.sunburst.setData($scope.categoryData);
    };
    
    // //////////////
    // helper methods
    $scope.resetData = function () {
        $scope.articles = [];
        
        $scope.queryArgs = "";
        $scope.articlePage = 1;
        $scope.articlePages = 0;
        $scope.totalArticleCount = -1;
        $scope.error = null;

        $scope.entityGraphData = null;
        $scope.tagCloudData = null;
        $scope.articleConceptTrends.trendData = null;
        $scope.timelineData = null;
        $scope.conceptData = null;
        
        $scope.articleTimelineGraph.setData(null);

        $scope.vizTabs.articles = true;
    };

    $scope.updateUrlFromForm = function () {
        var params = {};
        var conditions = [];
        $.each($("#articleConditions" + "_tagList .tag"), function (i, element) {
            var el = $(element).data("options");
            conditions.push({ label: el.label, type: el.type, uri: el.uri });
        });
        if (conditions.length > 0)
            params.conditions = conditions;
        if ($scope.searchOpt.articleLang.val != "any")
            params.lang = $scope.searchOpt.articleLang.val;
        params.preferredLang = $scope.langOptions.lang.val;
        params.sortBy = $scope.searchOpt.sortBy;
        if ($('#dp-start').val() != "" && $('#dp-end').val() != "") {
            var start = $('#dp-start').data('datepicker').date.toString("yyyy-MM-dd");
            var end = $('#dp-end').data('datepicker').date.toString("yyyy-MM-dd");
            params.date = { start: start, end: end };
        }
        if ($scope.searchOpt.publisher && $scope.searchOpt.publisher.uri) {
            params.publisherUri = $scope.searchOpt.publisher.uri;
            params.publisherLabel = $scope.searchOpt.publisher.label;
        }
        $location.url("/?query=" + encodeURIComponent(JSON.stringify(params)), true);
        //window.history.replaceState(params, "", window.location.pathname + "?query=" + encodeURIComponent(JSON.stringify(params)));
    };

    $scope.ignoreOnAddTrendingConcept = false;
    $scope.onAddTrendingConcept = function (concepts) {
        if ($scope.ignoreOnAddTrendingConcept || concepts.length == 0)
            return;
        var conceptUri = $(concepts[0]).data("options").uri;
        $.getJSON("/json/article?action=getArticles&resultType=trendingConcepts&trendingConceptUri=" + encodeURIComponent(conceptUri) + "&" + $scope.queryArgs, function (data) {
            $scope.articleConceptTrends.addConcepts(data);
        });
    }
    
    // ///////////////
    // events
    $scope.$on('langChangedBroadcast', function () {
    });
    
    // /////////////////////
    // DISPLAYING ARTICLE CONTENT
    $scope.showArticle = function (articleIndex) {
        //initLightbox();
        $scope.displayedArticleIndex = articleIndex;
        displayArticle($scope.articles[articleIndex]);
    };

    $scope.displayNextArticle = function () {
        var next = $scope.displayedArticleIndex + 1;
        next = next % $scope.articles.length;
        $scope.displayedArticleIndex = next;
        if (next >= $scope.articles.length || next < 0)
            return;
        displayArticle($scope.articles[next]);
    };

    $scope.displayPrevArticle = function () {
        var prev = $scope.displayedArticleIndex - 1;
        if (prev < 0) prev = $scope.articles.length - 1;
        $scope.displayedArticleIndex = prev;
        if (prev >= $scope.articles.length || prev < 0)
            return;
        displayArticle($scope.articles[prev]);
    };

    $("#prevArticle").click(function () {
        $scope.displayPrevArticle();
    });
    $("#nextArticle").click(function () {
        $scope.displayNextArticle();
    });

    // ///////////////
    // initialize variables and stuff
    $scope.resetData();
    var lastWidth = $(window).width();
    $(window).resize(function () {
        if (lastWidth == $(window).width())
            return;
        if ($scope.conceptData && $scope.vizTabs.concepts)
            $scope.redrawArticleConcepts();
        else if ($scope.timelineData && $scope.vizTabs.time)
            drawTimeline($scope.timelineData);
        else if ($scope.tagCloudData && $scope.vizTabs.tagCloud)
            $scope.redrawTagCloud();
        else if ($scope.entityGraphData && $scope.vizTabs.entityGraph)
            showEntityGraph($scope.entityGraphData);
        else if ($scope.articleConceptTrends.trendData && $scope.vizTabs.conceptTrends)
            $scope.articleConceptTrends.displayConceptTrends();
        lastWidth = $(window).width();
    });
    
    $().ready(function () {
        try {
            $('#navbar-articles').addClass("active");
        } catch (e) { }

        // build the tags input box for entering concepts, ...
        $('#articleConditions').tagsInput({ defaultText: "What topics are you interested in?", minInputWidth: 140 });
        $("#articleConditions_tag").keypress(function (event) {
            if (event.which == 13)
                $scope.searchForArticles();
        });

        // set up the autocomplete handlers for concepts
        $("#articleConditions" + "_tag").autocomplete({
            source: function (request, response) {
                var lang = $scope.langOptions.lang.val;
                var query = "/json/suggestConcepts?prefix=" + request.term + "&lang=" + lang;
                $.getJSON(query, function (suggList) {
                    response(suggList);
                });
            },

            select: function (event, ui) {
                $('#articleConditions').addTag(ui.item.label, { focus: true, type: ui.item.type, uri: ui.item.uri, label: ui.item.label, classes: [ui.item.type, "white", "concept"] });
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderConceptItem;

        // set up the autocomplete handlers for location
        $("#publisher").autocomplete({
            source: function (request, response) {
                var query = "/json/suggestSources?prefix=" + request.term;
                $.getJSON(query, function (suggList) {
                    response(suggList);
                });
            },

            select: function (event, ui) {
                $scope.searchOpt.publisher = { uri: ui.item.uri, label: ui.item.title };
                $scope.$apply();
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderPublisherItem;

      
        // autocomplete input box in the concept trending tab
        $('#trending-concept-list').tagsInput({
            defaultText: "Topic trends to visualize", minInputWidth: 140,
            "onRemoveTag": function (concept) { $scope.articleConceptTrends.onRemoveTag(concept); },
            "onAddTag": function (concept) { $scope.onAddTrendingConcept(concept); }
        });

        $("#trending-concept-list" + "_tag").autocomplete({
            source: function (request, response) {
                var lang = $scope.langOptions.lang.val;
                var query = "/json/suggestConcepts?prefix=" + request.term + "&lang=" + lang;
                $.getJSON(query, function (suggList) { response(suggList); });
            },
            select: function (event, ui) {
                $('#trending-concept-list').addTag(ui.item.label, { focus: true, type: ui.item.type, uri: ui.item.uri, id: ui.item.id, label: ui.item.label, classes: [ui.item.type, "concept"] });
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderConceptItem;

        // date picker for date range
        $('#dp-start').datepicker({ format: 'yyyy-mm-dd' });
        $('#dp-end').datepicker({ format: 'yyyy-mm-dd' });

        var cbUpdateTimer;
        // refresh the entity graph if update button is clicked
        $(".concept-graph-checkboxes .checkbox").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
            clearTimeout(cbUpdateTimer);
            cbUpdateTimer = setTimeout(function () {
                $scope.updateConceptGraph(true);
                clearTimeout(cbUpdateTimer);
            }, 1000);
        });

        var conceptTrendingUpdateTimer;
        // refresh the trending concepts if different concept types are selected
        $(".trending-checkboxes .checkbox").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
            clearTimeout(conceptTrendingUpdateTimer);
            conceptTrendingUpdateTimer = setTimeout(function () {
                $scope.articleConceptTrends.trendData = null;
                $scope.visualizeConceptTrends();
                clearTimeout(conceptTrendingUpdateTimer);
            }, 1000);
        });

        var cbNormalizeTimer;
        // refresh the normalization of concept trends
        $("#concept-normalize").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
            clearTimeout(cbNormalizeTimer);
            cbNormalizeTimer = setTimeout(function () {
                $scope.articleConceptTrends.updateOptions({ offsetType : $("#normalize-label.checked").length > 0 ? "expand" : "wiggle"});
                $scope.articleConceptTrends.onUpdateNormalize();
                clearTimeout(cbNormalizeTimer);
            }, 100);
        });

        // //////////
        // if we have query arguments then parse them and do the search
        var paramsStr = $location.search().query;
        if (paramsStr) {
            var params = paramsStr ? JSON.parse(paramsStr) : {};
            if (params.conditions) {
                $.each(params.conditions, function (i, element) {
                    $('#articleConditions').addTag(element.label, { focus: true, type: element.type, uri: element.uri, label: element.label, classes: [element.type, "white", "concept"] });
                });
            }
            if (params.date)
                setTime(params.date.start, params.date.end);
            var articleLang = $.grep($scope.searchOpt.articleLangs, function (el, ind) { return el.val == params.lang; });
            if (articleLang.length > 0)
                $scope.searchOpt.articleLang = articleLang[0];
            else
                $scope.searchOpt.articleLang = $scope.searchOpt.articleLangs[0];
            var langs = $.grep($scope.langOptions.langs, function (el, ind) { return el.val == params.preferredLang; });
            if (langs.length > 0)
                $scope.langOptions.lang = langs[0];
            if (params.publisherUri && params.publisherLabel)
                $scope.searchOpt.publisher = { uri: params.publisherUri, label: params.publisherLabel };
            if (params.date || params.lang != undefined || params.publisherUri)
                $scope.searchOpt.isOpen = true;
            $scope.searchForArticles();
        }
    });
});
