
function clearTime() {
    $('#dp-start').val("");
    $('#dp-end').val("");
}

function setTimeSpan(startSpan, endSpan) {
    var start = d3.time.day.offset(Date.today(), -startSpan);
    var end = d3.time.day.offset(Date.today(), -endSpan);
    $('#dp-start').data('datepicker').setValue(start);
    $('#dp-end').data('datepicker').setValue(end);
}

// handler for clicking a link for adding time condition
function setTime(start, end) {
    $('#dp-start').data('datepicker').setValue(start);
    $('#dp-end').data('datepicker').setValue(end);
};

// when a concept in an event is clicked it is added to the conditions
function addConceptConstraint(element) {
    var uri = $(element).attr("data-uri");
    var type = $(element).attr("data-type");
    var label = $(element).text();
    $('#eventConditions').addTag(label, { focus: true, type: type, uri: uri, label: label, classes: [type, "white", "concept"] });
};

eventRegApp.controller("SearchEventsCtrl", function ($scope, $http, $browser, $location, langOptions, utils) {
    $scope.utils = utils;
    $scope.langOptions = langOptions;

    $scope.eventsPerPage = 25;
    $scope.vizTabs = { events: true, concepts: false, tagCloud: false, locations: false, time: false, conceptGraph: false, conceptMatrix: false, conceptTrends: false, eventClusters: false, categories: false };
    
    $scope.searchOpt = {
        isOpen: false,
        keywords: "",
        minArticleCount: 0,
        minArticleCountList: [0, 10, 20, 30, 40, 50, 70, 100, 150, 200],
        eventLangs: [{ name: "Any language", val: "any" }, { name: "English", val: "eng" }, { name: "Spanish", val: "spa" }, { name: "German", val: "deu" }, { name: "Chinese", val: "zho" }, { name: "Slovene", val: "slv" }],
        sortBy: "rel"
    };
    $scope.searchOpt.eventLang = $scope.searchOpt.eventLangs[0];
    $scope.searchInProgress = true;
    
    // //////////////////
    // event handler for clicking the search button
    $scope.searchForEvents = function (skipIfNoConditions) {
        $scope.updateUrlFromForm();
        // clear existing content
        $scope.resetData();
        var args = "";
        var queryStr = $location.search().query;
        var query = queryStr ? JSON.parse(queryStr) : {};
        if (query.conditions) {
            $.each(query.conditions, function (i, element) {
                args += "conceptUri=" + encodeURIComponent(element.uri) + "&";
            });
        }
        if (query.keywords)
            args += "keywords=" + encodeURIComponent(query.keywords) + "&";
        if (query.locationUri)
            args += "locationUri=" + encodeURIComponent(query.locationUri) + "&";
        if (query.publisherUri)
            args += "publisherUri=" + encodeURIComponent(query.publisherUri) + "&";
        if (query.date)
            args += "timeStart=" + encodeURIComponent(query.date.start) + "&timeEnd=" + encodeURIComponent(query.date.end) + "&";
        if (query.lang)
            args += "lang=" + encodeURIComponent(query.lang) + "&";
        if (query.categoryUri)
            args += "categoryUri=" + encodeURIComponent(query.categoryUri) + "&";
        // if not all languages are checked then add the condition for language
        if (query.minArticles && query.minArticles > 0)
            args += "minArticlesInEvent=" + query.minArticles + "&";
        
        if (skipIfNoConditions && args == "")
            return;

        args += "sortBy=" + query.sortBy;
        $scope.queryArgs = args;
        $scope.searchInProgress = true;
        $scope.utils.showWait("", ".search-in-progress");
        //$scope.$apply();
        $scope.loadEventPage(1, function() {
            $scope.searchInProgress = false;
        });
    };

    $scope.loadEventPage = function (selectedPage, finishedCallback) {
        if ($scope.queryArgs == "")
            return;

        $scope.events = [];
        $scope.utils.showWait("", "#tab-events .wait-container");
        var args = $scope.queryArgs + "&page=" + (selectedPage-1) + "&count=" + $scope.eventsPerPage + "&resultType=list";
        console.log(utils.getTimeStamp() + " event page: /json/event?action=getEvents&" + args);
        $http.get("/json/event?action=getEvents&" + args)
            .success(function (eventData, status, headers, config) {
                $scope.utils.hideWait("", "#tab-events .wait-container");
                $scope.utils.hideWait("", ".search-in-progress");
                $scope.error = eventData.error;

                // if this was request for the first page of events then also ask for event times and locations
                if ($scope.eventPage == 1)
                    $scope.totalEventCount = eventData.resultCount ? eventData.resultCount : 0;

                $scope.events = eventData.results;
                // compute top entities and keywords for each event. we don't want to have "ng-repeat="concept in utils.getEntities(event, 10)" in the html since it is called on each digest
                var lang = $scope.langOptions.lang.val;
                $.each($scope.events, function (i, event) {
                    // set the correct label for the concept (once, so we don't have to do it each time in the $apply)
                    $.each(event.concepts, function (j, concept) {
                        concept.label = utils.getConceptLabel(concept, lang);
                    });
                    event.keywords = utils.getKeywords(event, 10);
                    event.entities = utils.getEntities(event, 10);
                })
                finishedCallback && finishedCallback();
            })
            .error(function(data, status, headers, config) {
                $scope.utils.hideWait("", "#tab-events .wait-container");
                $scope.utils.hideWait("", ".search-in-progress");
                finishedCallback && finishedCallback();
            });
        
    };

    // concept list
    $scope.visualizeEventConcepts = function () {
        if ($scope.conceptData) {
            $scope.redrawEventConcepts();
            return;
        }
            
        $scope.utils.showWait("#tab-concepts .content", "#tab-concepts .wait-container");
        console.log(utils.getTimeStamp() + " event concepts: /json/event?action=getEvents&resultType=conceptAggr&conceptType=entities&conceptType=wiki&conceptCount=40&" + $scope.queryArgs);
        $http.get("/json/event?action=getEvents&resultType=conceptAggr&conceptType=entities&conceptType=wiki&conceptCount=40&" + $scope.queryArgs)
            .success(function (data, status, headers, config) {
                console.log(utils.getTimeStamp() + " retrieved event concepts");
                $scope.utils.hideWait("#tab-concepts .content", "#tab-concepts .wait-container");
                $scope.error = data.error;
                $scope.conceptData = data.conceptAggr;
                $scope.redrawEventConcepts();
        });
    }

    $scope.redrawEventConcepts = function () {
        var entities = $scope.conceptData ? $.grep($scope.conceptData, function (concept, i) { return entityTypes[concept.type]; }) : [];
        var keywords = $scope.conceptData ? $.grep($scope.conceptData, function (concept, i) { return keywordTypes[concept.type]; }) : [];
        entities = entities.slice(0, 40);
        keywords = keywords.slice(0, 40);
        $scope.entityList.setData(entities, $scope.langOptions.lang.val);
        $scope.keywordList.setData(keywords, $scope.langOptions.lang.val);
    }

    // keyword tag cloud
    $scope.visualizeTagCloud = function () {
        if ($scope.tagCloudData) {
            $scope.redrawTagCloud();
            return;
        }
        $scope.utils.showWait("#tab-tagcloud .content", "#tab-tagcloud .wait-container");
        console.log(utils.getTimeStamp() + " event tag cloud: /json/event?action=getEvents&resultType=keywordAggr&" + $scope.queryArgs + "&keywordLang=" + $scope.langOptions.lang.val);
        $http.get("/json/event?action=getEvents&resultType=keywordAggr&" + $scope.queryArgs + "&lang=" + $scope.langOptions.lang.val)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-tagcloud .content", "#tab-tagcloud .wait-container");
                $scope.error = data.error;
                $scope.tagCloudData = data.keywordAggr;
                $scope.redrawTagCloud();
            });
    }

    $scope.redrawTagCloud = function () {
        $scope.tagCloud.setData($scope.tagCloudData);
    }

    // locations
    $scope.visualizeEventLocations = function () {
        $("#events-map").css("width", $("#events-map").parent().width());
        $scope.eventLocations.triggerResizeEvent();
        if ($scope.eventLocations.hasMap()) {
            return;
        }
        $scope.utils.showWait("#tab-map .content", "#tab-map .wait-container");

        $scope.eventLocations.resetData();
        $("#tab-map h3").html("");
        console.log(utils.getTimeStamp() + " event locations: /json/event?action=getEvents&resultType=locAggr&" + $scope.queryArgs);
        $http.get("/json/event?action=getEvents&resultType=locAggr&" + $scope.queryArgs)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-map .content", "#tab-map .wait-container");
                $scope.error = data.error;
                var markers = [];
                var mybounds = new google.maps.LatLngBounds();
                $scope.eventLocations.createNewMap();
                if (data && data.locAggr) {
                    $.each(data.locAggr, function (foo, el) {
                        var latLng = new google.maps.LatLng(el.location.lat, el.location.long);
                        mybounds.extend(latLng);
                        for (var i = 0; i < el.count; i++) {
                            var marker = new google.maps.Marker({
                                position: latLng,
                                title: el.location.labelEng
                            });
                            markers.push(marker);
                        }
                    });
                };
                $("#tab-map h3").html(markers.length + " events have a known geographic location");
                if (markers.length > 0) {
                    $scope.eventLocations.markerCluster.addMarkers(markers);
                    if (data.locAggr.length >= 2)
                        $scope.eventLocations.map.fitBounds(mybounds);
                    else {
                        $scope.eventLocations.map.setCenter(markers[0].getPosition());
                        $scope.eventLocations.map.setZoom(7);
                    }
                }
            });
    };

    // // events timeline
    $scope.visualizeEventTimes = function () {
        if ($scope.timelineData) {
            $scope.eventsTimelineGraph.setData($scope.timelineData);
            return;
        }
        $scope.utils.showWait("#tab-timeline .content", "#tab-timeline .wait-container");
        console.log(utils.getTimeStamp() + " event times: /json/event?action=getEvents&resultType=timeAggr&" + $scope.queryArgs);
        $http.get("/json/event?action=getEvents&resultType=timeAggr&" + $scope.queryArgs)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-timeline .content", "#tab-timeline .wait-container");
                $scope.error = data.error;
                $scope.timelineData = data.timeAggr;
                $scope.eventsTimelineGraph.setData($scope.timelineData);
            });
    };
    
    $scope.getEventsTimelineTooltip = function(date, value) {
        return "<b>{0}</b> events occured on<br/><b>{1}</b>".format(numberWithCommas(value), d3.time.format("%e %B %Y")(date));
    }

    $scope.onShowEventsForDate = function (selection, selectedDate) {
        $scope.selectedEvents = [];
        if (!selectedDate)
            return;
        $scope.$apply();
        $('#selected-events-dlg h3').html("Top events for date " + selectedDate);
        $('#selected-events-wait').css("display", "block");
        $('#selected-events-dlg').modal({});
        var query = $scope.queryArgs;
        query = query.replace("timeStart=", "timeStartEx=").replace("timeEnd=", "timeEndEx=");  // in case we had previous time condition replace it with this one
        console.log(utils.getTimeStamp() + " events for date: /json/event?action=getEvents&page=0&count=30&resultType=list&" + query + "&timeStart=" + selectedDate + "&timeEnd=" + selectedDate);
        $http.get("/json/event?action=getEvents&page=0&count=30&resultType=list&" + query + "&timeStart=" + selectedDate + "&timeEnd=" + selectedDate)
            .success(function (ret, status, headers, config) {
                $('#selected-events-wait').css("display", "none");
                $scope.selectedEvents = ret.results;
            });
    }

    // // entity graph
    $scope.visualizeConceptGraph = function () {
        if (!$scope.conceptGraphData)
            $scope.updateConceptGraph(true);
    };

    $scope.updateConceptGraph = function (showWait) {
        var types = "";
        $.each($(".entity-graph-checkboxes label.checked"), function (i, item) {
            types += "&conceptType=" + $(item).children("input").attr("value");
        });
        showWait && $scope.utils.showWait("#tab-concept-graph .content", "#tab-concept-graph .wait-container");
        console.log(utils.getTimeStamp() + " event clusters : /json/event?action=getEvents&resultType=conceptGraph&" + $scope.queryArgs + types);
        $http.get("/json/event?action=getEvents&resultType=conceptGraph&" + $scope.queryArgs + types)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-concept-graph .content", "#tab-concept-graph .wait-container");
                $scope.error = data.error;
                $scope.conceptGraphData = data.conceptGraph;
                showConceptGraph($scope.conceptGraphData);
            });
    };

    // conceptMatrix
    $scope.visualizeConceptMatrix = function () {
        if ($scope.conceptMatrixData) {
            $scope.updateConceptMatrix(true);
            return;
        }
        var types = "";
        $.each($("#events-entity-matrix label.checked"), function (i, item) {
            types += "&conceptType=" + $(item).children("input").attr("value");
        });
        $scope.utils.showWait("#tab-concept-matrix .content", "#tab-concept-matrix .wait-container");
        console.log(utils.getTimeStamp() + " concept matrix: /json/event?action=getEvents&resultType=conceptMatrix&" + $scope.queryArgs + types);
        $http.get("/json/event?action=getEvents&resultType=conceptMatrix&" + $scope.queryArgs + types)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-concept-matrix .content", "#tab-concept-matrix .wait-container");
                $scope.error = data.error;
                $scope.conceptMatrixData = data.conceptMatrix;
                $scope.updateConceptMatrix();
            });
    };

    $scope.updateConceptMatrix = function (showWait) {
        $scope.eventsConceptMatrix.setData($scope.conceptMatrixData);
    };

    // concept trends
    $scope.visualizeConceptTrends = function() {
        if ($scope.eventConceptTrends.trendData)
            return;
        var types = "";
        $.each($(".trending-checkboxes label.checked"), function (i, item) {
            types += "&conceptType=" + $(item).children("input").attr("value")
        });
        $scope.utils.showWait("#tab-concept-trends .content", "#tab-concept-trends .wait-container");
        console.log(utils.getTimeStamp() + " concept trends: /json/event?action=getEvents&resultType=trendingConcepts&" + $scope.queryArgs + types);
        $http.get("/json/event?action=getEvents&resultType=trendingConcepts&" + $scope.queryArgs + types)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-concept-trends .content", "#tab-concept-trends .wait-container");
                $scope.error = data.error;
                $scope.ignoreOnAddEventTrendingConcept = true;
                $scope.eventConceptTrends.setConceptTrends(data);
                $scope.eventConceptTrends.displayConceptTrends();
                $scope.ignoreOnAddEventTrendingConcept = false;
            });
    };

    // event clusters
    $scope.visualizeEventClusters = function () {
        if ($scope.eventClustersData)
            return;
        $scope.utils.showWait("#tab-event-clusters .content", "#tab-event-clusters .wait-container");
        console.log(utils.getTimeStamp() + " event clusters : /json/event?action=getEvents&resultType=eventClusters&" + $scope.queryArgs);
        $http.get("/json/event?action=getEvents&resultType=eventClusters&" + $scope.queryArgs)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-event-clusters .content", "#tab-event-clusters .wait-container");
                $scope.error = data.error;
                $scope.eventClustersData = data;
                $scope.eventClusters.setData(data.eventClusters);
            });
    };

    $scope.onEventClustersClick = function (events, topConcepts, conceptIds) {
        $scope.selectedEvents = [];
        $scope.$apply();
        $('#selected-events-dlg h3').html("Events in the selected cluster");
        $('#selected-events-wait').css("display", "block");
        $('#selected-events-dlg').modal({});
        var eventUris = "";
        for (var n = 0; n < events.length && n < 30; n++)
            eventUris += "&eventUri=" + events[n];

        $http.get("/json/event?action=getEventInfo&includeConcepts=true&includeStories=false" + eventUris)
            .success(function (ret, status, headers, config) {
                $('#selected-events-wait').css("display", "none");
                $scope.selectedEvents = ret;
            });
    }

    // categories
    $scope.visualizeCategories = function() {
        if ($scope.categoryData) {
            $scope.redrawCategories();
            return;
        }
        $scope.utils.showWait("#tab-categories .content", "#tab-categories .wait-container");
        console.log(utils.getTimeStamp() + " event categories : /json/event?action=getEvents&resultType=categoryAggr&" + $scope.queryArgs);
        $http.get("/json/event?action=getEvents&resultType=categoryAggr&" + $scope.queryArgs)
            .success(function (data, status, headers, config) {
                $scope.utils.hideWait("#tab-categories .content", "#tab-categories .wait-container");
                $scope.error = data.error;
                $scope.categoryData = data.categoryAggr;
                $scope.redrawCategories();
            });
    };

    $scope.redrawCategories = function() {
        $scope.sunburst.setData($scope.categoryData);
    };

    $scope.onShowEventsForCategory = function (selection) {
        $scope.selectedEvents = [];
        if (!selection.uri)
            return;
        $scope.$apply();
        $('#selected-events-dlg h3').html("Top events for " + selection.uri.trimLStr("http://dmoz.com/").replace(/_/g, " ").trimLStr("/").replace(/\//g, " &rarr; "));
        $('#selected-events-wait').css("display", "block");
        $('#selected-events-dlg').modal({});
        
        $http.get("/json/event?action=getEvents&page=0&count=30&resultType=list&" + $scope.queryArgs + "&categoryUri=" + encodeURIComponent(selection.uri))
            .success(function (ret, status, headers, config) {
                $('#selected-events-wait').css("display", "none");
                $scope.selectedEvents = ret.results;
            });
    }

    // //////////////
    // helper methods
    $scope.loadEventCategories = function() {
        $.getJSON("/json/category?action=getCategories", function(catTree) {

            function createCats(domEl, cats, skipPrefix) {
                $.each(cats, function(i, cat) {
                    var catLi = $("<li>", { "class": cat.children ? "dropdown-submenu" : "" }).appendTo(domEl);
                    $("<a>", { "href": "#", "html": cat.label.replace(skipPrefix, "").trimLStr("/").replace(/_/g, " ").replace(/\//g, " &rarr; ") })
                        .data("cat", cat)
                        .click(function() {
                            var data = $(this).data("cat");
                            var label = data.label.replace(/_/g, " ").trimLStr("/").replace(/\//g, " &rarr; ");
                            $("#selected-category").data("cat", data);
                            $("#selected-category span.text").html(label);
                            $scope.updateUrlFromForm(); // update url
                        })
                        .appendTo(catLi);
                    // set callback on click
                    if (cat.children) {
                        var submenu = $("<ul>", { "class": "dropdown-menu" }).appendTo(catLi);
                        createCats(submenu, cat.children, cat.label);
                    }
                });
            };
            
            var catsEl = $("#category-list");
            catsEl.html("");
            var anyCat = $("<li>").appendTo(catsEl);
            $("<a>", { "href": "#", "text": "Any category" })
                .click(function() {
                    $("#selected-category").data("cat", null);
                    $("#selected-category span.text").html("Any category");
                    $scope.updateUrlFromForm(); // update url
                })
                .appendTo(anyCat);
            $("<li>", { "class": "divider" }).appendTo(catsEl);
            createCats(catsEl, catTree.results);
        });
    };

    $scope.resetData = function () {
        $scope.events = [];
        $scope.eventLocations.resetData();

        $scope.queryArgs = "";
        $scope.eventPage = 1;
        $scope.eventPages = 0;
        $scope.error = null;
        $scope.totalEventCount = -1;

        $scope.conceptData = null;
        $scope.tagCloudData = null;
        $scope.timelineData = null;
        $scope.conceptGraphData = null;
        $scope.conceptMatrixData = null;
        $scope.eventConceptTrends.trendData = null;
        $scope.eventClustersData = null;
        $scope.categoryData = null;
        
        $scope.eventsTimelineGraph.setData(null);

        $scope.vizTabs.events = true;
    };

    $scope.updateUrlFromForm = function () {
        var params = {};
        var conditions = [];
        $.each($("#eventConditions" + "_tagList .tag"), function (i, element) {
            var el = $(element).data("options");
            conditions.push({ label: el.label, type: el.type, uri: el.uri });
        });
        if (conditions.length > 0)
            params.conditions = conditions;
        if ($scope.searchOpt.keywords.trim() != "") {
            params.keywords = $scope.searchOpt.keywords.trim();
        }
        if ($scope.searchOpt.location && $scope.searchOpt.location.uri) {
            params.locationUri = $scope.searchOpt.location.uri;
            params.locationLabel = $scope.searchOpt.location.label;
        }
        if ($scope.searchOpt.publisher && $scope.searchOpt.publisher.uri) {
            params.publisherUri = $scope.searchOpt.publisher.uri;
            params.publisherLabel = $scope.searchOpt.publisher.label;
        }
        if ($('#dp-start').val() != "" && $('#dp-end').val() != "") {
            var format = d3.time.format("%Y-%m-%d");
            var start = format($('#dp-start').data('datepicker').date);
            var end = format($('#dp-end').data('datepicker').date);
            params.date = { start: start, end: end };
        }
        if ($scope.searchOpt.eventLang.val != "any")
            params.lang = $scope.searchOpt.eventLang.val;
        var categoryData = $("#selected-category").data("cat");
        if (categoryData) {
            params.categoryUri = categoryData.uri;
            params.categoryLabel = categoryData.label;
        }
        if ($scope.searchOpt.minArticleCount > 0)
            params.minArticles = $scope.searchOpt.minArticleCount;
        params.sortBy = $scope.searchOpt.sortBy;
        params.preferredLang = $scope.langOptions.lang.val;

        $location.url("/?query=" + encodeURIComponent(JSON.stringify(params)), true);
        //window.history.replaceState(params, "", window.location.pathname + "?query=" + encodeURIComponent(JSON.stringify(params)));
    };

    $scope.ignoreOnAddEventTrendingConcept = false;
    $scope.onAddEventTrendingConcept = function (concepts) {
        if ($scope.ignoreOnAddEventTrendingConcept || concepts.length == 0)
            return;
        var conceptUri = $(concepts[0]).data("options").uri;
        $.getJSON("/json/event?action=getEvents&resultType=trendingConcepts&trendingConceptUri=" + encodeURIComponent(conceptUri) + "&" + $scope.queryArgs, function (data) {
            $scope.eventConceptTrends.addConcepts(data);
        });
    }

    $scope.getMultiLingInfoForLang = function (event) {
        var eventInfo = event.multiLingInfo[$scope.langOptions.lang.val];
        if (eventInfo)
            return eventInfo;
        if (event.multiLingInfo["eng"])
            return event.multiLingInfo["eng"];
        if (Object.keys(event.multiLingInfo).length > 0)
            return event.multiLingInfo[Object.keys(event.multiLingInfo)[0]];
    }

    $scope.getEventLocation = function (event) {
        if (!event.location)
            return "";
        var lang = $scope.langOptions.lang.val;
        lang = lang.charAt(0).toUpperCase() + lang.slice(1); // capitalize first letter
        var locStr = "";
        locStr = event.location["label" + lang];
        if (event.location.country)
            locStr += ", " + event.location.country["label" + lang];
        return locStr;
    };

    // ///////////////
    // events
    $scope.$on('langChangedBroadcast', function () {
        // if we changed the display language then redo the search
        $scope.searchForEvents(true);
    });
    
    // ///////////////
    // initialize variables and stuff
    $scope.$watch("searchOpt.isOpen", function () {
        if ($scope.searchOpt.isOpen) {
            setTimeout(function () { $(".accordion-body").css("overflow", "visible"); }, 250);
        }
        else
            $(".accordion-body").css("overflow", "hidden");
    });
    var lastWidth = $(window).width();
    $(window).resize(function () {
        if (lastWidth == $(window).width())
            return;
        if ($scope.eventLocations.hasMap() && $scope.vizTabs.locations)
            $scope.visualizeEventLocations();
        else if ($scope.conceptData && $scope.vizTabs.concepts)
            $scope.redrawEventConcepts();
        else if ($scope.tagCloudData && $scope.vizTabs.tagCloud)
            $scope.redrawTagCloud();
        else if ($scope.timelineData && $scope.vizTabs.time)
            drawTimeline($scope.timelineData);
        else if ($scope.conceptGraphData && $scope.vizTabs.conceptGraph)
            showConceptGraph($scope.conceptGraphData);
        else if ($scope.conceptMatrixData && $scope.vizTabs.conceptMatrix)
            $scope.updateConceptMatrix();
        else if ($scope.eventConceptTrends.trendData && $scope.vizTabs.conceptTrends)
            $scope.eventConceptTrends.displayConceptTrends();
        else if ($scope.categoryData && $scope.vizTabs.categories)
            $scope.redrawCategories();
        lastWidth = $(window).width();
    });
    
    $().ready(function () {
        try {
            $('#navbar-events').addClass("active");
        } catch (e) { }

        // concept graphs
        $scope.entityList = new TopConceptList({ containerNm: "#concepts-entities" });
        $scope.keywordList = new TopConceptList({ containerNm: "#concepts-keywords" });
        $scope.tagCloud = new TagCloud({ containerNm: "#keyword-tagcloud" });
        
        // setup graphs after the html is ready otherwise we cannot access the containerNm in the constructors of the graphs
        $scope.eventLocations = new EventLocations();
        // events timeline 
        $scope.eventsTimelineGraph = new DateBarChart({ containerNm: "#events-timeline", tooltipTextCallback: $scope.getEventsTimelineTooltip, onClickCallback: $scope.onShowEventsForDate });
        // concept matrix
        $scope.eventsConceptMatrix = new ConceptMatrixGraph({ containerNm: "#concept-matrix"});
        // concept trends with riverflow
        $scope.eventConceptTrends = new ConceptTrendsGraph({ containerNm: "#concept-trends", tagInputNm: "#trending-concept-list", zoomToLastXDays: 30 });
        // clustering of events
        $scope.eventClusters = new Dendrogram({ containerNm: "#event-clusters", onClickCallback: $scope.onEventClustersClick });
        // sunburst for category visualization
        $scope.sunburst = new Sunburst({ containerNm: ".sunburst .chart", sequenceNm: ".sunburst .sequence", legendNm: ".sunburst .legend", onClickCallback: $scope.onShowEventsForCategory });
        
        $scope.resetData();
        
        // build the tags input box for entering concepts, ...
        $('#eventConditions').tagsInput({ defaultText: "What topics are you interested in?", minInputWidth: 140 });
        $("#eventConditions_tag").keypress(function (event) {
            if (event.which == 13)
                $scope.searchForEvents();
        });

        // Init tooltips
        //$("[data-toggle=tooltip]").tooltip();

        // set up the autocomplete handlers for concepts
        $("#eventConditions" + "_tag").autocomplete({
            source: function (request, response) {
                var lang = $scope.langOptions.lang.val;
                var query = "/json/suggestConcepts?prefix=" + request.term + "&lang=" + lang;
                $.getJSON(query, function (suggList) {
                    response(suggList);
                });
            },

            select: function (event, ui) {
                $('#eventConditions').addTag(ui.item.label, { focus: true, type: ui.item.type, uri: ui.item.uri, label: ui.item.label, classes: [ui.item.type, "white", "concept"] });
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderConceptItem;

        // set up the autocomplete handlers for location
        $("#location").autocomplete({
            source: function (request, response) {
                var lang = $scope.langOptions.lang.val;
                var query = "/json/suggestLocations?prefix=" + request.term + "&lang=" + lang;
                $.getJSON(query, function (suggList) {
                    var arr = suggList.country[lang];
                    arr.push.apply(arr, suggList.city[lang]);
                    response(arr);
                });
            },

            select: function (event, ui) {
                $scope.searchOpt.location = { uri: ui.item.wikiUri, label: ui.item.label };
                $scope.$apply();
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderLocationItem;

        // set up the autocomplete handlers for location
        $("#publisher").autocomplete({
            source: function (request, response) {
                var query = "/json/suggestSources?prefix=" + request.term;
                $.getJSON(query, function (suggList) {
                    response(suggList);
                });
            },

            select: function (event, ui) {
                $scope.searchOpt.publisher = { uri: ui.item.uri, label: ui.item.title };
                $scope.$apply();
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderPublisherItem;

      
        // autocomplete input box in the concept trending tab
        $('#trending-concept-list').tagsInput({
            defaultText: "Topic trends to visualize", minInputWidth: 140,
            "onRemoveTag": function (concept) { $scope.eventConceptTrends.onRemoveTag(concept); },
            "onAddTag": function (concept) { $scope.onAddEventTrendingConcept(concept); }
        });

        $("#trending-concept-list" + "_tag").autocomplete({
            source: function (request, response) {
                var lang = $scope.langOptions.lang.val;
                var query = "/json/suggestConcepts?prefix=" + request.term + "&lang=" + lang;
                $.getJSON(query, function (suggList) { response(suggList); });
            },
            select: function (event, ui) {
                $('#trending-concept-list').addTag(ui.item.label, { focus: true, type: ui.item.type, uri: ui.item.uri, id: ui.item.id, label: ui.item.label, classes: [ui.item.type, "concept"] });
                return false;
            }
        }).data("uiAutocomplete")._renderItem = renderConceptItem;

        // date picker for date range
        $('#dp-start').datepicker({ format: 'yyyy-mm-dd' });
        $('#dp-end').datepicker({ format: 'yyyy-mm-dd' });

        var conceptGraphUpdateTimer;
        // refresh the entity graph if update button is clicked
        $("#tab-concept-graph .checkbox").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
            clearTimeout(conceptGraphUpdateTimer);
            conceptGraphUpdateTimer = setTimeout(function () {
                $scope.updateConceptGraph();
                clearTimeout(conceptGraphUpdateTimer);
            }, 1000);
        });

        var conceptMatrixUpdateTimer;
        // refresh the entity graph if update button is clicked
        $("#tab-concept-matrix .checkbox").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
            clearTimeout(conceptMatrixUpdateTimer);
            conceptMatrixUpdateTimer = setTimeout(function () {
                $scope.conceptMatrixData = null;
                $scope.visualizeConceptMatrix();
                clearTimeout(conceptMatrixUpdateTimer);
            }, 1000);
        });
        
        var conceptTrendingUpdateTimer;
        // refresh the trending concepts if different concept types are selected
        $(".trending-checkboxes .checkbox").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
            clearTimeout(conceptTrendingUpdateTimer);
            conceptTrendingUpdateTimer = setTimeout(function () {
                $scope.eventConceptTrends.trendData = null;
                $scope.visualizeConceptTrends();
                clearTimeout(conceptTrendingUpdateTimer);
            }, 1000);
        });

        var cbNormalizeTimer;
        // refresh the normalization of concept trends
        $("#concept-normalize").click(function () {
            setupLabel();   // hack: cb and radioboxes don't get updated if inside accordions or tabs (using angular ui)
            clearTimeout(cbNormalizeTimer);
            cbNormalizeTimer = setTimeout(function () {
                $scope.eventConceptTrends.updateOptions({ offsetType : $("#normalize-label.checked").length > 0 ? "expand" : "wiggle"});
                $scope.eventConceptTrends.onUpdateNormalize();
                clearTimeout(cbNormalizeTimer);
            }, 100);
        });

        // //////////
        // if we have query arguments then parse them and do the search
        var paramsStr = $location.search().query;
        if (paramsStr) {
            var params = paramsStr ? JSON.parse(paramsStr) : {};
            if (params.conditions) {
                $.each(params.conditions, function (i, element) {
                    $('#eventConditions').addTag(element.label, { focus: true, type: element.type, uri: element.uri, label: element.label, classes: [element.type, "white", "concept"] });
                });
            }
            if (params.keywords)
                $scope.searchOpt.keywords = params.keywords;
            if (params.locationUri && params.locationLabel)
                $scope.searchOpt.location = { uri: params.locationUri, label: params.locationLabel };
            if (params.publisherUri && params.publisherLabel)
                $scope.searchOpt.publisher = { uri: params.publisherUri, label: params.publisherLabel };
            if (params.date)
                setTime(params.date.start, params.date.end);
            var eventLang = $.grep($scope.searchOpt.eventLangs, function (el, ind) { return el.val == params.lang; });
            if (eventLang.length > 0)
                $scope.searchOpt.eventLang = eventLang[0];
            else
                $scope.searchOpt.eventLang = $scope.searchOpt.eventLangs[0];
            if (params.categoryUri != undefined) {
                var label = params.categoryLabel.replace(/_/g, " ").trimLStr("/").replace(/\//g, " &rarr; ");
                $("#selected-category").data("cat", { uri: params.categoryUri, label: params.categoryLabel });
                $("#selected-category span.text").html(label);
            }
            var langs = $.grep($scope.langOptions.langs, function (el, ind) { return el.val == params.preferredLang; });
            if (langs.length > 0)
                $scope.langOptions.lang = langs[0];
            $scope.searchOpt.minArticleCount = params.minArticles != undefined ? params.minArticles : 0;
            
            if (params.date || params.keywords || params.categoryUri || params.minArticles != undefined || params.lang != undefined || params.publisherUri || params.locationUri)
                $scope.searchOpt.isOpen = true;
            $scope.searchForEvents();
        }
        
        // load categories and create submenus for them
        $scope.loadEventCategories();
    });
});
