/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "../NewsMiner/NewsLoad.h"
#include "../NewsMiner/NewsConst.h"
#include "re2/re2.h"

//TNewsProc* NewsProcPtr = NULL;
//BOOL WINAPI ConsoleHandlerRoutine(DWORD dwCtrlType)
//{
//	if (dwCtrlType == CTRL_CLOSE_EVENT) {
//		if (NewsProcPtr != NULL)
//			NewsProcPtr->Save();
//		return TRUE;
//	}
//	return FALSE;
//}


PNotify BigExceptionLogger;

bool BigExceptionHandler(char* MsgCStr)
{
	if (!BigExceptionLogger.Empty()) {
		TStr Output(MsgCStr);
#ifdef GLib_WIN
		Output += "Stack trace:\n";
		Output += TBufferStackWalker::GetStackTrace();
#endif
		BigExceptionLogger->OnStatus(Output);
	}
	TExcept::ThrowFull(MsgCStr, "");
	return false;
}

int main(int argc, char* argv[]){
	try {
		// set a hanlder for all exceptions that would otherwise throw a ExeStop
		TOnExeStop::PutOnExeStopF(BigExceptionHandler);

		//TQm::TEnv::Init();
		// create environment
		Env = ::TEnv(argc, argv, TNotify::StdNotify);
		// command line parameters
		Env.PrepArgs("News Build Process", 1);
		// common
		const TStr InXmlFPath = Env.GetIfArgPrefixStr("-inxml:", "./Raw/New/", "Input XML Folder");
		const TStr DoneXmlFPath = Env.GetIfArgPrefixStr("-outxml:", "./Raw/Done/", "Done XML Folder");
		const TStr InBinFPath = Env.GetIfArgPrefixStr("-inbin:", "./Raw/NewBin/", "Input processed binary files");
		const TStr DoneBinFPath = Env.GetIfArgPrefixStr("-outbin:", "./Raw/DoneBin/", "Done processed binary files");

		const TStr ModeStr = Env.GetIfArgPrefixStr("-mode:", "online", "Processing mode [online, parse, onlineBinary]");

		const TStr RawSource = Env.GetIfArgPrefixStr("-source:", "ijs-enrycher", "Raw Source [ijs, ijs-enrycher, ijs-xlike]");
		TStrV EmptyV;
		const TStr AddArticleUrl = Env.GetIfArgPrefixStr("-addurl:", "", "Forward-URL");
		const int SleepSec = Env.GetIfArgPrefixInt("-sleep:", 15, "Sleep-When-Waiting");
		const int SkipLast = Env.GetIfArgPrefixInt("-skipLast", 0, "Skip latest X files. Useful if using multiple processes to create Bin files");

		// if using onlineBinary mode then sleep some ms before individual calls - otherwise we can overload the serverarticles process
		const int SleepBinaryMSec = Env.GetIfArgPrefixInt("-sleepBetweenBinary:", 100, "Sleep time between sending individual binary files [ms]");

		const TStr DbsPath = Env.GetIfArgPrefixStr("-dbsPath:", "./dbs/Build/", "Folder with databases");

		// parse.filter
		TStrV SourceTypeV = Env.GetIfArgPrefixStrV("-stype:", EmptyV, "Allowed Source Types");
		TStrV LangV = Env.GetIfArgPrefixStrV("-lang:", EmptyV, "Allowed Languages");
		TStrV DomainV = Env.GetIfArgPrefixStrV("-domain:", EmptyV, "Allowed Domain List");
		const int MinBodyLen = Env.GetIfArgPrefixInt("-minBodyLen:", 0, "Minimum article body length to use the article");

		TStrV IgnoreSourceTypeV = Env.GetIfArgPrefixStrV("-istype:", EmptyV, "Ignored Source Types");
		TStrV IgnoreLangV = Env.GetIfArgPrefixStrV("-ilang:", EmptyV, "Ignored Languages");
		TStrV IgnoreDomainV = Env.GetIfArgPrefixStrV("-idomain:", EmptyV, "Ignored Domain List");

		const TStr DomainFNm = Env.GetIfArgPrefixStr("-domainf:", "", "FileName of Allowed Domain List");
		const TStr IgnoreDomainFNm = Env.GetIfArgPrefixStr("-idomainf:", "", "FileName of Ignored Domains");

		const TStr SettingsFNm = Env.GetIfArgPrefixStr("-settings:", "./settings/settingsBuild.json", "File with settings");
		const bool ExtractDatesP = Env.GetIfArgPrefixBool("-extractDates:", true, "Extract date occurences from articles");
		const TStr DateRegexFPath = Env.GetIfArgPrefixStr("-dateRegexPath:", "DateRegEx/", "Folder that contains regular expressions for extracting date mentions");
		const TStr CatListFNm = Env.GetIfArgPrefixStr("-catListFNm:", "DMoz.txt", "File containing the dmoz categories that we wish to consider.");

		const TStr CityWikiFNm = Env.GetIfArgPrefixStr("-cityWikiFNm:", "cityWikiList.txt", "File containing a list of wikipedia pages for cities.");
		const TStr CityMappingFNm = Env.GetIfArgPrefixStr("-cityMappingFNm:", "cityMapping.txt", "File containing a list of wikipedia pages for cities.");

		const bool IgnoreIfNoAnnotations = Env.GetIfArgPrefixBool("-ignoreIfNoAnnotations", false, "Ignore articles that have no annotations");

		if (Env.IsEndOfRun()) { return 0; }

		// prepare unicode base
		if (!TUnicodeDef::IsDef()) { TUnicodeDef::Load("./UnicodeDef.Bin"); }

		BigExceptionLogger = TFileNotify::New("./Logs/Log-Build-ExtremeExceptions.txt", true, true);

		TNewsProcSource Source = TNewsProcSource::undefined;
		if (RawSource == "ijs") Source = TNewsProcSource::NewsFeed;
		else if (RawSource == "ijs-enrycher") Source = TNewsProcSource::NewsFeedEnrycher;
		else if (RawSource == "ijs-xlike") Source = TNewsProcSource::NewsFeedXLike;

		TNewsProcMode Mode;
		if (ModeStr == "online") Mode = TNewsProcMode::online;
		else if (ModeStr == "parse") Mode = TNewsProcMode::parse;
		else if (ModeStr == "onlineBinary") Mode = TNewsProcMode::onlineBinary;
		else {
			printf("No valid value was provided for the mode parameter");
			return 0;
		}

		if (Mode == TNewsProcMode::online)
			TDir::GenDir(DoneXmlFPath);
		else if (Mode == TNewsProcMode::parse) {
			TDir::GenDir(DoneXmlFPath);
			TDir::GenDir(InBinFPath);
		}
		else if (Mode == TNewsProcMode::onlineBinary)
			TDir::GenDir(DoneBinFPath);

		TDir::GenDir("./Logs");

		// we might want to skip using fixes in build process since we cannot reload the data while the process is running. 
		// in the articles process, however we can reload the data and apply the changes
		// load fixes for concepts, ...
		//ConceptFix.LoadFixes();

		TStrSet DomainSet = TStrSet(DomainV);
		if (DomainFNm != "")
			DomainSet = TNewsProcFilter::ReadFile(DomainFNm);
		TStrSet IgnoreDomainSet = TStrSet(IgnoreDomainV);
		if (IgnoreDomainFNm != "")
			IgnoreDomainSet = TNewsProcFilter::ReadFile(IgnoreDomainFNm);

		// prepare filter
		PNewsProcFilter Filter = TNewsProcFilter::New(TStrSet(SourceTypeV), TStrSet(LangV), DomainSet, TStrSet(IgnoreSourceTypeV), TStrSet(IgnoreLangV), IgnoreDomainSet);
		Filter->MinBodyLen = MinBodyLen;

		const TStr SettingsStr = TFile::Exists(SettingsFNm) ? TStr::LoadTxt(SettingsFNm) : "{}";
		PJsonVal SettingsJson = TJsonVal::GetValFromStr(SettingsStr);

		// set the path to the databases
		PJsonVal NewsProcSettJson = SettingsJson->IsObjKey("newsProc") ? SettingsJson->GetObjKey("newsProc") : TJsonVal::NewObj();
		
		TNewsProc NewsProc(DbsPath, NewsProcSettJson, Filter, AddArticleUrl, Source);	
		NewsProc.SleepSec = SleepSec;
		NewsProc.SleepBinaryMSec = SleepBinaryMSec;
		NewsProc.SkipLast = SkipLast;
		NewsProc.IgnoreIfNoAnnotations = IgnoreIfNoAnnotations;
		
		//BOOL ret = SetConsoleCtrlHandler(ConsoleHandlerRoutine, TRUE);

		// if we want to extract dates from the articles then load regular expressions
		if (ExtractDatesP) {
			PJsonVal DateExtrJson = SettingsJson->IsObjKey("dateExtraction") ? SettingsJson->GetObjKey("dateExtraction") : TJsonVal::NewObj();
			PDateExtractor DateExtractor = TDateExtractor::New(new TFileNotify("./Logs/Log-DateExtractor.txt", true, true), DateExtrJson);
			for (int N = 0; N < NewsConst.LangV.Len(); N++)
				DateExtractor->LoadLangRegEx(DbsPath + DateRegexFPath, NewsConst.LangV[N]);
			NewsProc.SetDateExtractor(DateExtractor);
		}

		// load a list of prefered categories
		if (TFile::Exists(DbsPath + CatListFNm)) {
			TFIn CatListFIn(DbsPath + CatListFNm);
			TStrIntPrV PrefixV; TStrV IgnoreV; TStr LnStr;
			while (CatListFIn.GetNextLn(LnStr)) {
				// remove the "Top/" part from the beginning
				LnStr.ChangeStr("www.dmoz.org/Top/", "www.dmoz.org/");
				if (LnStr.IsWs()) { continue; }
				else if (LnStr.IsChIn('\t')) {
					TStr Prefix, MxLevelStr;
					LnStr.SplitOnCh(Prefix, '\t', MxLevelStr);
					PrefixV.Add(TStrIntPr(Prefix, MxLevelStr.GetInt()));
				}
				else if (LnStr[0] == '-') {
					printf("Ignore dmoz category: %s\n", LnStr.CStr());
					IgnoreV.Add(LnStr.GetSubStr(1));
				}
			}
			NewsProc.SetDMozCategories(PrefixV, IgnoreV);
		}
		
		if (Mode == TNewsProcMode::online) {
			NewsProc.LoadDbs();
			NewsProc.StartOnline(InXmlFPath, DoneXmlFPath);
		}
		else if (Mode == TNewsProcMode::parse) {
			NewsProc.LoadDbs();
			NewsProc.StartParsing(InXmlFPath, DoneXmlFPath, InBinFPath);
		}
		else if (Mode == TNewsProcMode::onlineBinary)
			NewsProc.StartOnlineBinary(InBinFPath, DoneBinFPath);
					
		return 0;
	}
	catch (PExcept Except)  {
		printf("Exception: %s\n", Except->GetMsgStr().CStr());
		return 1;
	}
}
