/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "LrnStoryLinking.h"
#include "../ServerArticles/EventRegBase.h"

//////////////////////////////////////
/// TLrnStoryLinkingExample
TLrnStoryLinkingExample::TLrnStoryLinkingExample() :
	Username(""), DateTime(""),
	ArtCount1(-1), ArtCount2(-1),
	StoryQuality1(-1), StoryQuality2(-1),
	AvgCosSimStory1(-1), AvgCosSimStory2(-1),
	VarianceStory1(-1), VarianceStory2(-1),
	RatioAnnotatedArticlesStory1(0), RatioAnnotatedArticlesStory2(0),
	CCAArticleMatchCount(-1), CCAArticleMatchRatio(-1), CCAAvgMatchWgt(-1),
	TimeStDev1(-1), TimeStDev2(-2), TimeAvg1(-1), TimeAvg2(-1),
	CoreTimeStDev1(-1), CoreTimeStDev2(-2), CoreTimeAvg1(-1), CoreTimeAvg2(-1),
	ExtractedDatesStory1(-1), ExtractedDatesStory2(-1), ExtractedDateJaccardSim(-1),
	ConceptCosSim(-1), ConceptJaccardSim(-1), 
	EntityCosSim(-1), EntityJaccardSim(-1),
	KeywordCosSim(-1), KeywordJaccardSim(-1),
	CategorySim(-1), HasSameLocation(-1), ShouldMerge(false) 
{

}

TLrnStoryLinkingExample::TLrnStoryLinkingExample(PJsonVal ExampleJson)
{
	Username = ExampleJson->GetObjStr("_username", "");
	DateTime = ExampleJson->GetObjStr("_datetime", "");

	StoryUri1 = ExampleJson->GetObjStr("_storyUri1", "");
	StoryUri2 = ExampleJson->GetObjStr("_storyUri2", "");

	StoryLang1 = ExampleJson->GetObjStr("lang1", "");
	StoryLang2 = ExampleJson->GetObjStr("lang2", "");
	
	ArtCount1 = ExampleJson->GetObjNum("artCount1", -1);
	ArtCount2 = ExampleJson->GetObjNum("artCount2", -1);

	StoryQuality1 = ExampleJson->GetObjNum("storyQuality1", -1);
	StoryQuality2 = ExampleJson->GetObjNum("storyQuality2", -1);

	AvgCosSimStory1 = ExampleJson->GetObjNum("avgCosSimStory1", -1);
	AvgCosSimStory2 = ExampleJson->GetObjNum("avgCosSimStory2", -1);
	VarianceStory1 = ExampleJson->GetObjNum("varianceStory1", -1);
	VarianceStory2 = ExampleJson->GetObjNum("varianceStory2", -1);

	RatioAnnotatedArticlesStory1 = ExampleJson->GetObjNum("ratioAnnotatedArticlesStory1", 0);
	RatioAnnotatedArticlesStory2 = ExampleJson->GetObjNum("ratioAnnotatedArticlesStory2", 0);

	CCAArticleMatchCount = ExampleJson->GetObjNum("CCAArticleMatchCount", -1);
	CCAArticleMatchRatio = ExampleJson->GetObjNum("CCAArticleMatchRatio", -1);
	CCAAvgMatchWgt = ExampleJson->GetObjNum("CCAAvgMatchWgt", -1);
		
	TimeStDev1 = ExampleJson->GetObjNum("timeStDev1", -1);
	TimeStDev2 = ExampleJson->GetObjNum("timeStDev2", -1);
	TimeAvg1 = ExampleJson->GetObjNum("timeAvg1", -1);
	TimeAvg2 = ExampleJson->GetObjNum("timeAvg2", -1);
	CoreTimeStDev1 = ExampleJson->GetObjNum("coreTimeStDev1", -1);
	CoreTimeStDev2 = ExampleJson->GetObjNum("coreTimeStDev2", -1);
	CoreTimeAvg1 = ExampleJson->GetObjNum("coreTimeAvg1", -1);
	CoreTimeAvg2 = ExampleJson->GetObjNum("coreTimeAvg2", -1);

	ExtractedDatesStory1 = ExampleJson->GetObjNum("extractedDatesStory1", -1);;
	ExtractedDatesStory2 = ExampleJson->GetObjNum("extractedDatesStory2", -1);;
	ExtractedDateJaccardSim = ExampleJson->GetObjNum("extractedDateJaccardSim", -1);

	ConceptCosSim = ExampleJson->GetObjNum("conceptCosSim", -1);
	EntityCosSim = ExampleJson->GetObjNum("entityCosSim", -1);
	KeywordCosSim = ExampleJson->GetObjNum("keywordCosSim", -1);
	ConceptJaccardSim = ExampleJson->GetObjNum("conceptJaccardSim", -1);
	EntityJaccardSim = ExampleJson->GetObjNum("entityJaccardSim", -1);
	KeywordJaccardSim = ExampleJson->GetObjNum("keywordJaccardSim", -1);

	CategorySim = ExampleJson->GetObjNum("categorySim", -1);
	HasSameLocation = ExampleJson->GetObjNum("hasSameLocation", -1);
	
	ShouldMerge = ExampleJson->GetObjBool("shouldMerge", false);
}

PJsonVal TLrnStoryLinkingExample::GetJson() const
{
	PJsonVal ExampleJson = TJsonVal::NewObj();
	ExampleJson->AddToObj("_username", Username);
	ExampleJson->AddToObj("_datetime", DateTime);

	ExampleJson->AddToObj("_storyUri1", StoryUri1);
	ExampleJson->AddToObj("_storyUri2", StoryUri2);

	ExampleJson->AddToObj("lang1", StoryLang1);
	ExampleJson->AddToObj("lang2", StoryLang2);

	ExampleJson->AddToObj("artCount1", ArtCount1);
	ExampleJson->AddToObj("artCount2", ArtCount2);

	ExampleJson->AddToObj("storyQuality1", StoryQuality1);
	ExampleJson->AddToObj("storyQuality2", StoryQuality2);

	ExampleJson->AddToObj("avgCosSimStory1", AvgCosSimStory1);
	ExampleJson->AddToObj("avgCosSimStory2", AvgCosSimStory2);
	ExampleJson->AddToObj("varianceStory1", VarianceStory1);
	ExampleJson->AddToObj("varianceStory2", VarianceStory2);

	ExampleJson->AddToObj("ratioAnnotatedArticlesStory1", RatioAnnotatedArticlesStory1);
	ExampleJson->AddToObj("ratioAnnotatedArticlesStory2", RatioAnnotatedArticlesStory2);

	ExampleJson->AddToObj("CCAArticleMatchCount", CCAArticleMatchCount);
	ExampleJson->AddToObj("CCAArticleMatchRatio", CCAArticleMatchRatio);
	ExampleJson->AddToObj("CCAAvgMatchWgt", CCAAvgMatchWgt);

	ExampleJson->AddToObj("timeStDev1", TimeStDev1);
	ExampleJson->AddToObj("timeStDev2", TimeStDev2);
	ExampleJson->AddToObj("timeAvg1", TimeAvg1);
	ExampleJson->AddToObj("timeAvg2", TimeAvg2);
	ExampleJson->AddToObj("coreTimeStDev1", CoreTimeStDev1);
	ExampleJson->AddToObj("coreTimeStDev2", CoreTimeStDev2);
	ExampleJson->AddToObj("coreTimeAvg1", CoreTimeAvg1);
	ExampleJson->AddToObj("coreTimeAvg2", CoreTimeAvg2);

	ExampleJson->AddToObj("extractedDatesStory1", ExtractedDatesStory1);
	ExampleJson->AddToObj("extractedDatesStory2", ExtractedDatesStory2);
	ExampleJson->AddToObj("extractedDateJaccardSim", ExtractedDateJaccardSim);

	ExampleJson->AddToObj("conceptCosSim", ConceptCosSim);
	ExampleJson->AddToObj("entityCosSim", EntityCosSim);
	ExampleJson->AddToObj("keywordCosSim", KeywordCosSim);
	ExampleJson->AddToObj("conceptJaccardSim", ConceptJaccardSim);
	ExampleJson->AddToObj("entityJaccardSim", EntityJaccardSim);
	ExampleJson->AddToObj("keywordJaccardSim", KeywordJaccardSim);

	ExampleJson->AddToObj("categorySim", CategorySim);
	ExampleJson->AddToObj("hasSameLocation", HasSameLocation);

	ExampleJson->AddToObj("shouldMerge", ShouldMerge);
	
	return ExampleJson;
}


//////////////////////////////////////
/// TLrnStoryLinkingExampleTable

// load example table from disk
bool TLrnStoryLinkingExampleTable::LoadTable(const TStr& FileNm)
{
	if (TFile::Exists(FileNm)) {
		TFIn FIn(FileNm);
		TStr LnStr;
		while (FIn.GetNextLn(LnStr)) {
			try {
				PJsonVal ExampleJson = TJsonVal::GetValFromStr(LnStr);
				ExampleV.Add(TLrnStoryLinkingExample(ExampleJson));
			}
			catch (PExcept E) {
				TNotify::StdNotify->OnStatus("TLrnStoryLinking::LoadTable. Error while loading a learning example. Error: " + E->GetMsgStr());
				NewsBase->ErrorLogFile->NotifyErr("TLrnStoryLinking::LoadTable. Error while loading a learning example", E);
			}
			catch (...) { }
		}
		return true;
	}
	else
		return false;
}

void TLrnStoryLinkingExampleTable::SaveTable(const TStr& FileNm) const
{
	TFOut FOut(FileNm);
	for (int N = 0; N < ExampleV.Len(); N++) {
		//const TLrnStoryLinkingExample& Ex = ExampleV[N];
		PJsonVal ExampleJson = ExampleV[N].GetJson();
		const TStr LnStr = TJsonVal::GetStrFromVal(ExampleJson);
		FOut.PutStrLn(LnStr);
	}
}

void TLrnStoryLinkingExampleTable::AddExample(const TLrnStoryLinkingExample& Example)
{
	ExampleV.Add(Example);
	Changed = true;
}

//////////////////////////////////////
/// TLrnStoryLinking

TLrnStoryLinking::TLrnStoryLinking(const PNewsBase& _NewsBase, const TStr& _Path) : 
	NewsBase(_NewsBase), Path(_Path), ExampleTable(NULL)
{
	ModelCrossLing = NULL;
	// load data on startup
	bool ok = LoadLearningData();
	TStr FName = NewsBase->CustomDataPath + "LrnStoryLinking.bin";
	if (TFile::Exists(FName)) {
		try {
			TFIn FIn(FName);
			ModelCrossLing = new TSvm::TLinModel(FIn);
		}
		catch (...) {
			ModelCrossLing = NULL;
			TNotify::StdNotify->OnStatus("TLrnStoryLinking. Error while loading model from LrnStoryLinking.bin.");
		}
	}
}

TLrnStoryLinking::~TLrnStoryLinking()
{
	// save data on exit
	Save();
	if (!ExampleTable.Empty() && ExampleTable->HasTableChanged())
		SaveLearningData();
	if (!ExampleTable.Empty())
		TCrossLingModel::SaveFtrVecData(Path, ExampleTable, NewsBase);
	if (ModelCrossLing != NULL)
		delete ModelCrossLing;
	ModelCrossLing = NULL;
}


void TLrnStoryLinking::Save()
{
	try {
		TFOut FOut(NewsBase->CustomDataPath + "LrnStoryLinking.bin");
		if (ModelCrossLing != NULL) {
			ModelCrossLing->Save(FOut);
		}
	}
	catch (...) {
		TNotify::StdNotify->OnStatus("TLrnStoryLinking::Save. Error while saving LrnStoryLinking.bin.");
	}
}

// load example table from file
bool TLrnStoryLinking::LoadLearningData()
{
	TStr FileNm = Path + "storyLinking.json";
	try {
		ExampleTable = TLrnStoryLinkingExampleTable::New(NewsBase);
		return ExampleTable->LoadTable(FileNm);
	}
	catch (PExcept E) {
		TNotify::StdNotify->OnStatus("TLrnStoryLinking::LoadLearningData. Error while trying to load a learning data table. Error: " + E->GetMsgStr());
		NewsBase->ErrorLogFile->NotifyErr("TLrnStoryLinking::LoadLearningData. Error while trying to load a learning data table", E);
		return false;
	}
	catch (...) { 
		TNotify::StdNotify->OnStatus("TLrnStoryLinking::LoadLearningData. Error while loading example table from storyLinking.json.");
		return false;
	}
}

// save the example table to a file
void TLrnStoryLinking::SaveLearningData()
{
	try {
		TStr FileNm = Path + "storyLinking.json";
		if (!ExampleTable.Empty()) {
			ExampleTable->SaveTable(FileNm);
			TCrossLingModel::SaveFtrVecData(Path, ExampleTable, NewsBase);
		}
	}
	catch (...) {
		TNotify::StdNotify->OnStatus("Exception while saving learning data.");
		NewsBase->ErrorLogFile->NotifyErr("TLrnStoryLinking::SaveLearningData. Exception while saving learning data.");
	}
}

// build a prediction model using the current example table
// until a model is built we cannot call ShouldStoriesMerge()
PJsonVal TLrnStoryLinking::BuildModel(const bool& GenerateReport)
{
	try {
		if (ModelCrossLing != NULL)
			delete ModelCrossLing;
		ModelCrossLing = NULL;
		 
		// prepare training set
		if (ExampleTable.Empty() || ExampleTable->Len() == 0) {
			TNotify::StdNotify->OnStatus("TLrnStoryLinking::BuildModel. Unable to build a story-linking model. Example table is empty.");
			NewsBase->ErrorLogFile->NotifyErr("TLrnStoryLinking::BuildModel. Unable to build a story-linking model. Example table is empty.");
			return TJsonVal::NewObj("error", TStr("Unable to build a model. Example table is empty."));
		}

		TVec<TFltV> ExamplesV; TFltV LabelsV;
		TCrossLingModel::GetSVMExampleTable(ExampleTable, ExamplesV, LabelsV);
		TSvm::TLinModel Model = TSvm::SolveClassify<TVec<TFltV>>(ExamplesV, ExamplesV[0].Len(), ExamplesV.Len(), LabelsV, 1.0, 1.0, 10 * 1000, 10000, 1e-6, 10000);
		ModelCrossLing = new TSvm::TLinModel(Model);

		if (GenerateReport)
			return TCrossLingModel::EvaluateSVMAccuracy(ExamplesV, LabelsV);
		else
			return TJsonVal::NewObj("info", TStr("Model was successfully built."));
	}
	catch (PExcept E) {
		TNotify::StdNotify->OnStatus("TLrnStoryLinking::BuildModel. Error while building a story-linking model. Error: " + E->GetMsgStr());
		NewsBase->ErrorLogFile->NotifyErr("TLrnStoryLinking::BuildModel. Error while building a story-linking model", E);
		return TJsonVal::NewObj("error", TJsonVal::NewStr(E->GetMsgStr()));
	}
	catch (...) {
		TNotify::StdNotify->OnStatus("Unknown exception happened while building a crosslingual linking model.");
		NewsBase->ErrorLogFile->NotifyErr("TLrnStoryLinking::BuildModel. Unknown exception happened while building a crosslingual linking model.");
		return TJsonVal::NewObj("error", TStr("Unknown exception happened while building a crosslingual linking model."));
	}
}

void TLrnStoryLinking::RebuildLearningTable(const PEventRegBase& EventRegBase)
{
	PStoryLinkingExampleTable OldExampleTable = ExampleTable;
	ExampleTable = TLrnStoryLinkingExampleTable::New(NewsBase);
	if (!ExampleTable.Empty()) {
		for (int ExN = 0; ExN < OldExampleTable->Len(); ExN++) {
			const TLrnStoryLinkingExample& Example = OldExampleTable->GetExample(ExN);
			AddLearningExample(EventRegBase, Example.StoryUri1, Example.StoryUri2, Example.ShouldMerge, Example.Username);
		}
	}
}

// make a learning example from two stories
void TLrnStoryLinking::MakeLearningExample(const PEventRegBase& EventRegBase, const TStr& StoryUri1, const TStr& StoryUri2, TLrnStoryLinkingExample& Example) const
{
	Example.StoryUri1 = StoryUri1;
	Example.StoryUri2 = StoryUri2;
	
	const uint64 StoryId1 = EventRegBase->StoryStore->GetRecId(StoryUri1);
	const uint64 StoryId2 = EventRegBase->StoryStore->GetRecId(StoryUri2);

	Example.ArtCount1 = EventRegBase->GetStoryArticleCount(StoryId1);
	Example.ArtCount2 = EventRegBase->GetStoryArticleCount(StoryId2);
	Example.StoryLang1 = EventRegBase->GetStoryLang(StoryId1);
	Example.StoryLang2 = EventRegBase->GetStoryLang(StoryId2);

	Example.StoryQuality1 = NewsBase->GetStoryQuality(StoryId1);
	Example.StoryQuality2 = NewsBase->GetStoryQuality(StoryId2);

	// avg cos similarity of articles in the story
	Example.AvgCosSimStory1 = EventRegBase->StoryStore->GetFieldNmFlt(StoryId1, "AvgCosToCentroid");
	Example.AvgCosSimStory2 = EventRegBase->StoryStore->GetFieldNmFlt(StoryId2, "AvgCosToCentroid");
	Example.VarianceStory1 = EventRegBase->StoryStore->GetFieldNmFlt(StoryId1, "Variance");
	Example.VarianceStory2 = EventRegBase->StoryStore->GetFieldNmFlt(StoryId2, "Variance");
	
	// ratio of articles that are annotated
	int Annotated1, Total1, Annotated2, Total2;
	EventRegBase->GetStoryAnnArticles(StoryId1, Annotated1, Total1);
	EventRegBase->GetStoryAnnArticles(StoryId2, Annotated2, Total2);
	if (Total1 > 0)
		Example.RatioAnnotatedArticlesStory1 = Annotated1 / (double) Total1;
	if (Total2 > 0)
		Example.RatioAnnotatedArticlesStory2 = Annotated2 / (double) Total2;

	// how many articles from one story point to another story
	TSimStoryInfo SimStoryInfo;
	EventRegBase->ComputeCCASimilarityBetweenStories(StoryId1, StoryId2, SimStoryInfo);
	
	if (SimStoryInfo.ArtMatchCount > 0)
		SimStoryInfo.SumWgts /= SimStoryInfo.ArtMatchCount;
	Example.CCAArticleMatchCount = SimStoryInfo.ArtMatchCount;
	Example.CCAArticleMatchRatio = SimStoryInfo.MaxArtCount > 0  ? (SimStoryInfo.ArtMatchCount / (double) (SimStoryInfo.MaxArtCount)) : 0;
	Example.CCAAvgMatchWgt = SimStoryInfo.SumWgts;

	Example.ConceptCosSim = EventRegBase->GetConceptSimStories(StoryId1, StoryId2, TEventRegBase::stCos);
	Example.EntityCosSim = EventRegBase->GetConceptSimStories(StoryId1, StoryId2, TEventRegBase::stCos, ctEntities);
	Example.KeywordCosSim = EventRegBase->GetConceptSimStories(StoryId1, StoryId2, TEventRegBase::stCos, ctWiki);
	Example.ConceptJaccardSim = EventRegBase->GetConceptSimStories(StoryId1, StoryId2, TEventRegBase::stJaccard);
	Example.EntityJaccardSim = EventRegBase->GetConceptSimStories(StoryId1, StoryId2, TEventRegBase::stJaccard, ctEntities);
	Example.KeywordJaccardSim = EventRegBase->GetConceptSimStories(StoryId1, StoryId2, TEventRegBase::stJaccard, ctWiki);

	// compute average dates and variances inside each story
	double AvgTime1, AvgTime2, StDevTime1, StDevTime2;
	double CoreAvgTime1, CoreAvgTime2, CoreStDevTime1, CoreStDevTime2;
	if (EventRegBase->GetStoryTimes(StoryId1, AvgTime1, StDevTime1, CoreAvgTime1, CoreStDevTime1)) {
		Example.TimeAvg1 = AvgTime1;
		Example.TimeStDev1 = StDevTime1;
		Example.CoreTimeAvg1 = CoreAvgTime1;
		Example.CoreTimeStDev1 = CoreStDevTime1;
	}
	if (EventRegBase->GetStoryTimes(StoryId2, AvgTime2, StDevTime2, CoreAvgTime2, CoreStDevTime2)) {
		Example.TimeAvg2 = AvgTime2;
		Example.TimeStDev2 = StDevTime2;
		Example.CoreTimeAvg2 = CoreAvgTime2;
		Example.CoreTimeStDev2 = CoreStDevTime2;
	}

	// compute the ExtractedDateJaccardSim
	TStr CommonDates1 = EventRegBase->StoryStore->GetFieldNmStr(StoryId1, "CommonDates");
	TStr CommonDates2 = EventRegBase->StoryStore->GetFieldNmStr(StoryId2, "CommonDates");
	TExtractedDateColl DateColl1 = TExtractedDateColl::DecodeExtractedDates(CommonDates1);
	TExtractedDateColl DateColl2 = TExtractedDateColl::DecodeExtractedDates(CommonDates2);
	int MatchingDateCount = 0;	// count the number of dates that co-occur in both stories
	for (int N = 0; N < DateColl1.Len(); N++) {
		TExtractedDate ExDate1 = DateColl1.ExtractedDateV[N];
		TExtractedDate ExDate2;
		if (DateColl2.GetExtractedDate(ExDate1.GetDate(), ExDate2))
			MatchingDateCount += ExDate1.GetFreq() + ExDate2.GetFreq();
	}
	Example.ExtractedDatesStory1 = 0; Example.ExtractedDatesStory2 = 0;
	for (int N = 0; N < DateColl1.Len(); N++)
		Example.ExtractedDatesStory1 += DateColl1.ExtractedDateV[N].GetFreq();
	for (int N = 0; N < DateColl2.Len(); N++)
		Example.ExtractedDatesStory2 += DateColl2.ExtractedDateV[N].GetFreq();
	if (Example.ExtractedDatesStory1 + Example.ExtractedDatesStory2 > 0)
		Example.ExtractedDateJaccardSim = MatchingDateCount / (double) (Example.ExtractedDatesStory1 + Example.ExtractedDatesStory2);

	// same location?
	TRec LocRec1 = NewsBase->StoryStore->GetRec(StoryId1).DoSingleJoin(NewsBase->Base, "hasLocation");
	TRec LocRec2 = NewsBase->StoryStore->GetRec(StoryId2).DoSingleJoin(NewsBase->Base, "hasLocation");
	Example.HasSameLocation = 0;
	if (!LocRec1.IsDef() || !LocRec2.IsDef()) Example.HasSameLocation = -1;
	else if (LocRec1.GetRecId() == LocRec2.GetRecId()) Example.HasSameLocation = 1;

	// compute category similarity	
	PRecSet Cats1 = NewsBase->StoryStore->GetRec(StoryId1).DoJoin(NewsBase->Base, "hasCategory");
	PRecSet Cats2 = NewsBase->StoryStore->GetRec(StoryId2).DoJoin(NewsBase->Base, "hasCategory");
	double MaxSim = 0;
	for (int N1 = 0; N1 < Cats1->GetRecs(); N1++) {
		for (int N2 = 0; N2 < Cats2->GetRecs(); N2++) {
			double Sim = NewsBase->CategoryBase->GetCategorySimilarity(Cats1->GetRecId(N1), Cats2->GetRecId(N2));
			MaxSim = MAX(Sim, MaxSim);
		}
	}
	
	Example.CategorySim = MaxSim;
}

// make a learning example for the provided story and the correct class value (provided by the user)
void TLrnStoryLinking::AddLearningExample(const PEventRegBase& EventRegBase, const TStr& StoryUri1, const TStr& StoryUri2, const bool& ShouldStoriesMerge, const TStr& Username)
{
	if (!EventRegBase->StoryStore->IsRecNm(StoryUri1) || !EventRegBase->StoryStore->IsRecNm(StoryUri2))
		return;
	
	// if we already have this story pair as an example remove it and then re-add it. it's possible that we've made a mistake and merged two stories and later decided to split them
	RemoveExample(StoryUri1, StoryUri2);
		
	TLrnStoryLinkingExample Example;
	// populate example features
	MakeLearningExample(EventRegBase, StoryUri1, StoryUri2, Example);
	Example.Username = Username;
	TTm Tm = TTm::GetCurLocTm();
	Example.DateTime = Tm.GetStr(false);
	// set the correct class value
	Example.ShouldMerge = ShouldStoriesMerge;
	// add learning example
	ExampleTable->AddExample(Example);
}

// predict the class value for the given story pair
// returns false if the model hasn't been built yet. Use the Probability parameter to decide to merge or not
bool TLrnStoryLinking::ShouldStoriesMerge(const PEventRegBase& EventRegBase, const TStr& StoryUri1, const TStr& StoryUri2, double& Probability)
{
	if (ModelCrossLing == NULL) {
		TNotify::StdNotify->OnStatus("Model hasn't been built yet. Unable to classify a new story pair");
		return false;
	}
	TLrnStoryLinkingExample Example;
	// populate example features
	MakeLearningExample(EventRegBase, StoryUri1, StoryUri2, Example);
	// make feature vector from example
	TFltV FtrV; TStrV FtrNmV;
	TCrossLingModel::MakeFtrV(Example, FtrV, FtrNmV);
	
	Probability = ModelCrossLing->Predict(FtrV);

	return true;
}

// do we already have information about a story par StoryId1, StoryId2 ?
bool TLrnStoryLinking::IsKnownPair(const TStr& StoryUri1, const TStr& StoryUri2) const
{
	if (ExampleTable.Empty())
		return false;
	int Examples = ExampleTable->Len();
	for (int N = 0; N < Examples; N++) {
		const TLrnStoryLinkingExample& Example = ExampleTable->GetExample(N);
		if (Example.StoryUri1 == StoryUri1 && Example.StoryUri2 == StoryUri2)
			return true;
		if (Example.StoryUri1 == StoryUri2 && Example.StoryUri2 == StoryUri1)
			return true;
	}
	return false;
}

void TLrnStoryLinking::RemoveExample(const TStr& StoryUri1, const TStr& StoryUri2) 
{
	if (ExampleTable.Empty())
		return;
	int Examples = ExampleTable->Len();
	for (int N = Examples-1; N >= 0; N--) {
		const TLrnStoryLinkingExample& Example = ExampleTable->GetExample(N);
		if (Example.StoryUri1 == StoryUri1 && Example.StoryUri2 == StoryUri2)
			ExampleTable->RemoveExample(N);
		else if (Example.StoryUri1 == StoryUri2 && Example.StoryUri2 == StoryUri1)
			ExampleTable->RemoveExample(N);
	}
}

// //////////////////////////////////////////
// creating SVM example table, normalization methods, testing svm accuracy, ...

// creates an example table where each row is an example and columns are features
// normalizes each feature in ExampleV to have mean = 0 and stdev = 1
// assumptions: the last item in ExampleV represents a dummy feature - it's value will not be normalized
void TCrossLingModel::GetSVMExampleTable(const PStoryLinkingExampleTable& ExampleTable, TVec<TFltV>& ExampleV, TFltV& LabelV)
{
	int Examples = ExampleTable->Len();
	ExampleV.Clr(); LabelV.Clr();
	if (Examples == 0)
		return;

	for (int N = 0; N < Examples; N++) {
		const TLrnStoryLinkingExample& Example = ExampleTable->GetExample(N);
		TFltV FtrV; TStrV FtrNmV;
		MakeFtrV(Example, FtrV, FtrNmV);
		ExampleV.Add(FtrV);
		LabelV.Add(Example.ShouldMerge ? 1 : -1, true);
	}
	int Features = ExampleV[0].Len();
	// normalize each feature to mean = 0, dev = 1
	// NOTE: don't normalize the last (dummy) feature since it serves as the free coeficient
	for (int FtrN = 0; FtrN < Features - 1; FtrN++)
		TPredictionModel::NormalizeColumn(ExampleV, FtrN);
	// change -1s to 0s to make them like redundant
	for (int N = 0; N < Examples; N++) {
		for (int FtrN = 0; FtrN < Features - 1; FtrN++) {
			if (TPredictionModel::IsMissing(ExampleV[N][FtrN]))
				ExampleV[N][FtrN] = 0;
		}
	}
}

// make learning features from the given example
void TCrossLingModel::MakeFtrV(const TLrnStoryLinkingExample& Example, TFltV& FtrV, TStrV& FtrNmV)
{
	FtrV.Clr(); FtrNmV.Clr();

	FtrV.Add(Example.ArtCount1 + Example.ArtCount2);	 FtrNmV.Add("TotArts");		// total article count
	FtrV.Add(MIN(Example.ArtCount1, Example.ArtCount2)); FtrNmV.Add("MinArts");	// min article count
	FtrV.Add(MAX(Example.ArtCount1, Example.ArtCount2)); FtrNmV.Add("MaxArts");	// max article count

	FtrV.Add((Example.StoryQuality1 + Example.StoryQuality2) / 2.0);	FtrNmV.Add("AvgStoryQ");
	FtrV.Add(MIN(Example.StoryQuality1, Example.StoryQuality2));		FtrNmV.Add("MinStoryQ");	// min quality

	FtrV.Add((Example.AvgCosSimStory1 + Example.AvgCosSimStory2) / 2.0);	FtrNmV.Add("AvgCosSim");
	FtrV.Add(MIN(Example.AvgCosSimStory1, Example.AvgCosSimStory2));		FtrNmV.Add("MinCosSim");	// min cos similarity
	FtrV.Add(MAX(Example.AvgCosSimStory1, Example.AvgCosSimStory2));		FtrNmV.Add("MaxCosSim");	// max cos similarity
	FtrV.Add((Example.VarianceStory1 + Example.VarianceStory2) / 2.0);		FtrNmV.Add("AvgVariance");
	FtrV.Add(MIN(Example.VarianceStory1, Example.VarianceStory2));			FtrNmV.Add("MinVariance");	// min variance
	FtrV.Add(MAX(Example.VarianceStory1, Example.VarianceStory2));			FtrNmV.Add("MaxVariance");	// min variance

	FtrV.Add(Example.CCAArticleMatchCount);	FtrNmV.Add("CCAMatchCount");
	FtrV.Add(Example.CCAArticleMatchRatio); FtrNmV.Add("CCAArtMatchRatio");
	FtrV.Add(Example.CCAAvgMatchWgt);		FtrNmV.Add("CCAAvgMatchWgt");

	FtrV.Add(Example.ConceptCosSim);		FtrNmV.Add("ConceptCosSim");
	FtrV.Add(Example.EntityCosSim);			FtrNmV.Add("EntityCosSim");
	FtrV.Add(Example.KeywordCosSim);		FtrNmV.Add("KeywordCosSim");
	FtrV.Add(Example.ConceptJaccardSim);	FtrNmV.Add("ConceptJaccardSim");
	FtrV.Add(Example.EntityJaccardSim);		FtrNmV.Add("EntityJaccardSim");
	FtrV.Add(Example.KeywordJaccardSim);	FtrNmV.Add("KeywordJaccardSim");
	FtrV.Add(Example.ConceptCosSim * Example.RatioAnnotatedArticlesStory1 *  Example.RatioAnnotatedArticlesStory2);		FtrNmV.Add("WgtCosineSim");
	FtrV.Add(Example.ConceptJaccardSim * Example.RatioAnnotatedArticlesStory1 *  Example.RatioAnnotatedArticlesStory2);	FtrNmV.Add("WgtJaccardSim");

	FtrV.Add(Example.RatioAnnotatedArticlesStory1);		FtrNmV.Add("RatioAnnArts1");
	FtrV.Add(Example.RatioAnnotatedArticlesStory2);		FtrNmV.Add("RatioAnnArts2");
	FtrV.Add(MIN(Example.RatioAnnotatedArticlesStory1, Example.RatioAnnotatedArticlesStory2));		FtrNmV.Add("MinRatioAnnArts");

	FtrV.Add((Example.TimeStDev1 + Example.TimeStDev2) / 2.0);	FtrNmV.Add("AvgTimeStDev");		// avg stdev
	FtrV.Add(MAX(Example.TimeStDev1, Example.TimeStDev2));		FtrNmV.Add("MaxTimeStDev");		// max stdev
	FtrV.Add(abs(Example.TimeAvg1 - Example.TimeAvg2));			FtrNmV.Add("TimeDiff");			// diff in avg time

	FtrV.Add((Example.CoreTimeStDev1 + Example.CoreTimeStDev2) / 2.0);	FtrNmV.Add("CoreAvgTimeStDev");		// avg stdev
	FtrV.Add(MAX(Example.CoreTimeStDev1, Example.CoreTimeStDev2));		FtrNmV.Add("CoreMaxTimeStDev");		// max stdev
	FtrV.Add(abs(Example.CoreTimeAvg1 - Example.CoreTimeAvg2));			FtrNmV.Add("CoreTimeDiff");			// diff in avg time

	FtrV.Add(Example.ExtractedDatesStory1);	FtrNmV.Add("ExtractedDates1");
	FtrV.Add(Example.ExtractedDatesStory2);	FtrNmV.Add("ExtractedDates2");
	FtrV.Add(Example.ExtractedDateJaccardSim);	FtrNmV.Add("ExtractedDateJaccardSim");

	FtrV.Add(Example.CategorySim);		FtrNmV.Add("CategorySim");
	FtrV.Add(Example.HasSameLocation);	FtrNmV.Add("HasSameLocation");

	if (Example.StoryLang1 != "" && Example.StoryLang2 != "")
		FtrV.Add((Example.StoryLang1 == Example.StoryLang2) ? 1 : 0);
	else
		FtrV.Add(-1);
	FtrNmV.Add("SameLang");
	FtrV.Add(1); FtrNmV.Add("DummyVar");
}


void TCrossLingModel::SaveFtrVecData(const TStr& Path, const PStoryLinkingExampleTable& ExampleTable, const PNewsBase& NewsBase)
{
	try {
		TStr FileNm = Path + "storyLinkingFtrVec.tab";
		TFOut FOut(FileNm);
		if (!ExampleTable.Empty()) {
			for (int ExN = 0; ExN < ExampleTable->Len(); ExN++) {
				const TLrnStoryLinkingExample& Example = ExampleTable->GetExample(ExN);
				TFltV FtrV; TStrV FtrNmV;
				MakeFtrV(Example, FtrV, FtrNmV);
				if (ExN == 0) {
					// add feature names
					for (int N = 0; N < FtrNmV.Len(); N++)
						FOut.PutStr(FtrNmV[N] + "\t");
					// add class name
					FOut.PutStrLn("ShouldMerge");
					// line specifying continuous variables and discrete class
					for (int N = 0; N < FtrV.Len(); N++)
						FOut.PutStr("c\t");	// attrs
					FOut.PutStrLn("d");		// class
					// the line saying which attr is the class
					for (int N = 0; N < FtrNmV.Len(); N++)
						FOut.PutStr("\t");
					FOut.PutStrLn("c");
				}
				for (int N = 0; N < FtrV.Len(); N++)
					FOut.PutStrFmt("%.3f\t", FtrV[N]);
				FOut.PutStrLn(Example.ShouldMerge ? "yes" : "no");
			}
		}
	}
	catch (...) {
		TNotify::StdNotify->OnStatus("Exception while saving learning data with features.");
		NewsBase->ErrorLogFile->NotifyErr("TLrnStoryLinking::SaveFtrVecData. Exception while saving learning data with features.");
	}
}

PJsonVal TCrossLingModel::EvaluateSVMAccuracy(const TVec<TFltV>& ExampleV, const TFltV& LabelV)
{
	TFltV AccV;
	int Folds = 5;
	for (int FoldN = 0; FoldN < Folds; FoldN++) {
		TVec<TFltV> TrainV; TFltV TrainLabelV;
		TVec<TFltV> TestV; TFltV TestLabelV;
		for (int N = 0; N < ExampleV.Len(); N++) {
			if ((N + FoldN) % Folds == 0) { TestV.Add(ExampleV[N]); TestLabelV.Add(LabelV[N]); }
			else { TrainV.Add(ExampleV[N]); TrainLabelV.Add(LabelV[N]); }
		}
		TSvm::TLinModel Model = TSvm::SolveClassify<TVec<TFltV>>(TrainV, TrainV[0].Len(), TrainV.Len(), TrainLabelV, 1.0, 1.0, 10 * 1000, 10000, 1e-6, 10000);
		int Correct = 0;
		for (int N = 0; N < TestV.Len(); N++) {
			double Val = Model.Predict(TestV[N]);
			double Expected = TestLabelV[N];
			if (Val * Expected > 0) Correct++;		// if both Val and Expected have the same sign then it means that the prediction is correct
		}
		TFlt Acc = 100 * Correct / float(TestV.Len());
		AccV.Add(Acc);
		TNotify::StdNotify->OnStatusFmt("Accuracy on fold %d: %.3f%%", FoldN+1, Acc.Val);
	}
	
	TFlt AvgAcc = TLinAlg::SumVec(AccV) / float(AccV.Len());
	TNotify::StdNotify->OnStatusFmt("Average accuracy: %.3f%%", AvgAcc.Val);
	return TJsonVal::NewObj("acc", AvgAcc);
}

