#ifndef CROSSLINGSTORYLINK_H
#define CROSSLINGSTORYLINK_H

#include <base.h>
#include <net.h>
#include "../NewsMiner/NewsBase.h"
#include "PredictionModel.h"

// forward declaration
class TEventRegBase;
typedef TPt<TEventRegBase> PEventRegBase;

class TLrnStoryLinking;
typedef TPt<TLrnStoryLinking> PStoryLinking;

class TLrnStoryLinkingExample {
public:
	TStr Username;
	TStr DateTime;

	TStr StoryUri1;
	TStr StoryUri2;
	int ArtCount1;
	int ArtCount2;
	TStr StoryLang1;
	TStr StoryLang2;

	int StoryQuality1;
	int StoryQuality2;

	double AvgCosSimStory1;
	double AvgCosSimStory2;
	double VarianceStory1;
	double VarianceStory2;

	double RatioAnnotatedArticlesStory1;	// ratio of articles in story 1 that have annotations
	double RatioAnnotatedArticlesStory2;	// ratio of articles in story 2 that have annotations

	int CCAArticleMatchCount;		// number of similarity links between the stories
	double CCAArticleMatchRatio;	// ratio ([0-1]) of articles that point between the stories. it would be 1 if all articles in the larger story would point only to articles in smaller story
	double CCAAvgMatchWgt;			// of those articles that are matched between the stories, what is the average sim wgt
		
	double ConceptCosSim;			// cos sim of the two concept vectors
	double EntityCosSim;			// cos sim of the two entity vectors
	double KeywordCosSim;			// cos sim of the two keywords vectors
	double ConceptJaccardSim;		// jaccard similarity ([0-1]) of concepts that appear in both stories
	double EntityJaccardSim;		// jaccard similarity ([0-1]) of entities that appear in both stories
	double KeywordJaccardSim;		// jaccard similarity ([0-1]) of keywords that appear in both stories
	
	double TimeStDev1;				// st. dev. in time in articles from story 1
	double TimeStDev2;				// st. dev. in time in articles from story 2
	double TimeAvg1;				// average time of articles in story 1
	double TimeAvg2;				// average time of articles in story 1

	double CoreTimeStDev1;			// st. dev. in time in core articles from story 1
	double CoreTimeStDev2;			// st. dev. in time in core articles from story 2
	double CoreTimeAvg1;			// average time of core articles in story 1
	double CoreTimeAvg2;			// average time of core articles in story 1
	
	int ExtractedDatesStory1;		// number of extracted dates found in story1
	int ExtractedDatesStory2;		// number of extracted dates found in story2
	double ExtractedDateJaccardSim;	// jaccard similarity of extracted dates
	
	double CategorySim;				// similarity of categories of both stories
	int HasSameLocation;			// do both stories have the same location (0 if not, -1 if we don't know, 1 if yes)

	TBool ShouldMerge;				// class value

	TLrnStoryLinkingExample();
	TLrnStoryLinkingExample(PJsonVal ExampleJson);
	PJsonVal GetJson() const;
};

class TLrnStoryLinkingExampleTable;
typedef TPt<TLrnStoryLinkingExampleTable> PStoryLinkingExampleTable;
class TLrnStoryLinkingExampleTable {
public:
	friend class TPt<TLrnStoryLinkingExampleTable>;
private:
	TCRef CRef;
	PNewsBase NewsBase;
	TVec<TLrnStoryLinkingExample> ExampleV;
	
	bool Changed;
	int FtrCount; 
	TLrnStoryLinkingExampleTable(const PNewsBase& _NewsBase) : NewsBase(_NewsBase) { 
		FtrCount = 16;
		Changed = false;
	}
public:
	static PStoryLinkingExampleTable New(const PNewsBase& NewsBase) {
		return new TLrnStoryLinkingExampleTable(NewsBase);
	}
	
	bool HasTableChanged() { return Changed; }
	bool LoadTable(const TStr& FileNm);
	void SaveTable(const TStr& FileNm) const;

	void AddExample(const TLrnStoryLinkingExample& Example);
	void RemoveExample(const int& ValN) { EAssertR(ValN < Len() && ValN >= 0, "Example table: Index outside of valid range.");  ExampleV.Del(ValN); }
	int Len() { return ExampleV.Len(); }
	const TLrnStoryLinkingExample& GetExample(const int& ValN) { EAssertR(ValN < Len() && ValN >= 0, "Example table: Index outside of valid range.");  return ExampleV[ValN]; }
	
};

//////////////////////////////////////
class TLrnStoryLinking {
private:
	TCRef CRef; 
public:
	friend class TPt<TLrnStoryLinking>;

private:	
	PNewsBase NewsBase;
	PStoryLinkingExampleTable ExampleTable; 
	TStr Path;

	TSvm::TLinModel* ModelCrossLing;

	TLrnStoryLinking(const PNewsBase& _NewsBase, const TStr& _Path);	
	~TLrnStoryLinking();
	void Save();

	void MakeLearningExample(const PEventRegBase& EventRegBase, const TStr& StoryUri1, const TStr& StoryUri2, TLrnStoryLinkingExample& Example) const;

public:
	static PStoryLinking New(const PNewsBase& NewsBase, const TStr& Path) { return new TLrnStoryLinking(NewsBase, Path); }

	//
	bool LoadLearningData();
	void SaveLearningData();
	
	PJsonVal BuildModel(const bool& GenerateReport = false);
	void AddLearningExample(const PEventRegBase& EventRegBase, const TStr& StoryUri1, const TStr& StoryUri2, const bool& ShouldStoriesMerge, const TStr& Username);
	void RebuildLearningTable(const PEventRegBase& EventRegBase);

	// use the model to predict if the two stories should be merged or not
	bool ShouldStoriesMerge(const PEventRegBase& EventRegBase, const TStr& StoryUri1, const TStr& StoryUri2, double& Probability);

	bool IsKnownPair(const TStr& StoryUri1, const TStr& StoryUri2) const;
	void RemoveExample(const TStr& StoryUri1, const TStr& StoryUri2);
	TStr GetExampleTablePath() { return TStr(Path); }

	int GetExamplesCount() const { return ExampleTable.Empty() ? 0 : ExampleTable->Len(); }
};

class TCrossLingModel : TPredictionModel
{
public:
	static void GetSVMExampleTable(const PStoryLinkingExampleTable& ExampleTable, TVec<TFltV>& ExampleV, TFltV& LabelV);
	static void MakeFtrV(const TLrnStoryLinkingExample& Example, TFltV& FtrV, TStrV& FtrNmV);
	static void SaveFtrVecData(const TStr& Path, const PStoryLinkingExampleTable& ExampleTable, const PNewsBase& NewsBase);
	static PJsonVal EvaluateSVMAccuracy(const TVec<TFltV>& ExampleV, const TFltV& LabelV);
};

#endif