/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "PredictionModel.h"

double TPredictionModel::Eps = 0.00001;


// substract mean and divide by dev - take into account missing values (-1) by skipping them
void TPredictionModel::NormalizeColumn(TVec<TFltV>& ExampleV, const int& ColInd)
{
	const double Mean = GetMean(ExampleV, ColInd);
	for (int N = 0; N < ExampleV.Len(); N++) {
		const double Val = ExampleV[N][ColInd];
		if (Val == -1) continue;
		ExampleV[N][ColInd] -= Mean;
	}
	const double Dev = GetStdDev(ExampleV, ColInd);
	if (Dev < TPredictionModel::Eps)
		return;
	for (int N = 0; N < ExampleV.Len(); N++) {
		const double Val = ExampleV[N][ColInd];
		if (TPredictionModel::IsMissing(Val)) continue;
		ExampleV[N][ColInd] /= Dev;
	}
}

// compute mean value of a column. ignore missing values
double TPredictionModel::GetMean(TVec<TFltV>& ExampleV, const int& ColInd)
{
	int Count = 0; double Sum = 0;
	for (int N = 0; N < ExampleV.Len(); N++) {
		const double Val = ExampleV[N][ColInd];
		if (TPredictionModel::IsMissing(Val)) continue;
		Sum += Val; Count++;
	}
	return Count > 0 ? Sum / (double) Count : 0;
}

// compute std. dev. of a column
double TPredictionModel::GetStdDev(TVec<TFltV>& ExampleV, const int& ColInd)
{
	double Mean = GetMean(ExampleV, ColInd);
	int Count = 0; double Sum = 0;
	for (int N = 0; N < ExampleV.Len(); N++) {
		const double Val = ExampleV[N][ColInd];
		if (Val == -1) continue;
		Sum += (Val - Mean)*(Val - Mean); Count++;
	}
	return Count > 0 ? sqrt(Sum / (double) Count) : 0;
}

bool TPredictionModel::IsMissing(const double& Val)
{
	if (Val > -1 - TPredictionModel::Eps && Val < -1 + TPredictionModel::Eps)
		return true;
	return false;
}