#ifndef CATEGORYBASE_H
#define CATEGORYBASE_H

#include <base.h>
#include <qminer_core.h>
//#include <qminer_srv.h>

using namespace TQm;

class TCategoryBase;
typedef TPt<TCategoryBase> PCategoryBase;
class TCategoryBase {
public:
	friend class TPt<TCategoryBase>;

private:
	TCRef CRef;

	// index to base
	PBase Base;
	TStr DmozPrefix;
	PStore CategoryStore;
	TCategoryBase(const PBase& _Base);

public:
	~TCategoryBase() {}
	static PCategoryBase New(const PBase& _Base) {
		return new TCategoryBase(_Base);
	}

	const TStr& GetDMozUriPrefix() const { return DmozPrefix; }
	// craate missing dmoz categories for label and all super-labels ("Arts\Movies" -> "Arts", "Arts\Movies")
	void CreateDMozCats(const TStr& Label);
	// Arts -> http:\\dmoz.com\Arts
	TStr GetDMozCatUriForLabel(const TStr& Label) { return (Label.IsPrefix(DmozPrefix)) ? Label : DmozPrefix + Label; }
	// http:\\dmoz.com\Arts -> Arts
	TStr GetLabelForDMozCatUri(const TStr& Uri) { return Uri.TrimLeft(DmozPrefix); }

	// get direct and indirect children categories of CategoryRecId
	TUInt64Set GetAllSubCategories(const uint64& CategoryRecId);
	TUInt64Set GetAllSubCategories(const TUInt64V& CategoryRecV);

	PRecSet GetChildCategories(const TStr& CategoryNm);
	PRecSet GetChildCategories(const uint64& CategoryRecId);

	// compute similarity between the two provided categories
	double GetCategorySimilarity(const uint64& CatRecId1, const uint64& CatRecId2);

	PJsonVal Category(const uint64& CategoryRecId);
	PJsonVal Categories(const PRecSet& CategoryRecSet);
};

#endif