/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "ConceptFolder.h"

void TConceptFolder::InitVals() {
	LastUpdatedArticleId = 0;
	TimeOfLastArticleUpdate = TTm::GetTmFromMSecs(0);
	TimeOfLastEventUpdate = TTm::GetTmFromMSecs(0);
	AutoAddArticles = true;
	RestrictToSetConcepts = false;
	RestrictToSetSources = false;
	RestrictToSetCategories = false;
	RestrictToSetLocations = false;;
	RestrictToSetKeywords = false;

	IncludeSubCategories = false;
	UseLearnedInterests = false;
	LearnInterests = false;

	FolderOwnerId = TUInt64::Mx;
	Visibility = PrivateFolder;
}

TConceptFolder::TConceptFolder(const PJsonVal& Json)
{
	InitVals();
	Uri = Json->GetObjStr("uri", "");
	
	/*LastUpdatedArticleId = (int) Json->GetObjNum("lastUpdatedArticleId", 0);
	TimeOfLastArticleUpdate = Json->GetObjNum("timeOfLastArticleUpdate", 0);
	TimeOfLastEventUpdate = Json->GetObjNum("timeOfLastEventUpdate", 0);*/

	AutoAddArticles = Json->GetObjBool("autoAddArticles", true);
	RestrictToSetConcepts = Json->GetObjBool("restrictToSetConcepts", false);
	RestrictToSetSources = Json->GetObjBool("restrictToSetSources", false);
	RestrictToSetLocations = Json->GetObjBool("restrictToSetLocations", false);
	RestrictToSetKeywords = Json->GetObjBool("restrictToSetKeywords", false);
	RestrictToSetCategories = Json->GetObjBool("restrictToSetCategories", false);

	IncludeSubCategories = Json->GetObjBool("includeSubCategories", false);
	UseLearnedInterests = Json->GetObjBool("useLearnedInterests", true);
	LearnInterests = Json->GetObjBool("learnInterests", false);
	
	FolderOwnerId = (uint64) Json->GetObjNum("folderOwnerId", 0);
	Visibility = (int) Json->GetObjNum("visibility", 0);

	if (Json->IsObjKey("conceptIdV"))
		LoadUInt64Arr(Json->GetObjKey("conceptIdV"), ConceptIdV);
	if (Json->IsObjKey("sourceIdV"))
		LoadUInt64Arr(Json->GetObjKey("sourceIdV"), SourceIdV);
	if (Json->IsObjKey("categoryIdV"))
		LoadUInt64Arr(Json->GetObjKey("categoryIdV"), CategoryIdV);
	if (Json->IsObjKey("locationIdV"))
		LoadUInt64Arr(Json->GetObjKey("locationIdV"), LocationIdV);
	if (Json->IsObjKey("keywordV"))
		LoadStrArr(Json->GetObjKey("keywordV"), KeywordV);
	if (Json->IsObjKey("langV"))
		LoadStrArr(Json->GetObjKey("langV"), LangV);
	
	if (Json->IsObjKey("conceptWgtV"))
		LoadFltArr(Json->GetObjKey("conceptWgtV"), ConceptWgtV);
	if (Json->IsObjKey("sourceWgtV"))
		LoadFltArr(Json->GetObjKey("sourceWgtV"), SourceWgtV);
	if (Json->IsObjKey("categoryWgtV"))
		LoadFltArr(Json->GetObjKey("categoryWgtV"), CategoryWgtV);
	if (Json->IsObjKey("locationWgtV"))
		LoadFltArr(Json->GetObjKey("locationWgtV"), LocationWgtV);
	if (Json->IsObjKey("keywordWgtV"))
		LoadFltArr(Json->GetObjKey("keywordWgtV"), KeywordWgtV);
	
	if (Json->IsObjKey("fixedConceptIdV")) {
		TUInt64V FixedConceptIdV; LoadUInt64Arr(Json->GetObjKey("fixedConceptIdV"), FixedConceptIdV);
		FixedConceptIdsH = TUInt64Set(FixedConceptIdV);
	}

	ArticleTreshWgt = (int) Json->GetObjNum("articleTreshWgt", 40);
	EventTreshWgt = (int) Json->GetObjNum("eventTreshWgt", 40);
	ArticlesInFolder = (int) Json->GetObjNum("articlesInFolder", 0);
}

TConceptFolder::TConceptFolder(const uint64& ConceptFolderId, const PStore& ConceptFolderStore)
{
	InitVals();
	Uri = ConceptFolderStore->GetRecNm(ConceptFolderId);

	LastUpdatedArticleId = ConceptFolderStore->GetFieldNmUInt64(ConceptFolderId, "LastUpdatedArticleId");
	ConceptFolderStore->GetFieldNmTm(ConceptFolderId, "TimeOfLastArticleUpdate", TimeOfLastArticleUpdate);
	ConceptFolderStore->GetFieldNmTm(ConceptFolderId, "TimeOfLastEventUpdate", TimeOfLastEventUpdate);
		
	AutoAddArticles = ConceptFolderStore->GetFieldNmBool(ConceptFolderId, "AutoAddArticles");
	RestrictToSetConcepts = ConceptFolderStore->GetFieldNmBool(ConceptFolderId, "RestrictToSetConcepts");
	RestrictToSetSources = ConceptFolderStore->GetFieldNmBool(ConceptFolderId, "RestrictToSetSources");
	RestrictToSetCategories = ConceptFolderStore->GetFieldNmBool(ConceptFolderId, "RestrictToSetCategories");
	RestrictToSetLocations = ConceptFolderStore->GetFieldNmBool(ConceptFolderId, "RestrictToSetLocations");
	RestrictToSetKeywords = ConceptFolderStore->GetFieldNmBool(ConceptFolderId, "RestrictToSetKeywords");

	IncludeSubCategories = ConceptFolderStore->GetFieldNmBool(ConceptFolderId, "IncludeSubCategories");
	UseLearnedInterests = ConceptFolderStore->GetFieldNmBool(ConceptFolderId, "UseLearnedInterests");
	LearnInterests = ConceptFolderStore->GetFieldNmBool(ConceptFolderId, "LearnInterests");

	FolderOwnerId = ConceptFolderStore->GetFieldNmUInt64(ConceptFolderId, "FolderOwnerId");
	Visibility = ConceptFolderStore->GetFieldNmInt(ConceptFolderId, "Visibility");

	ConceptIdV = GetIntArrAsUInt64Arr(ConceptFolderStore, ConceptFolderId, "ConceptIdV");
	SourceIdV = GetIntArrAsUInt64Arr(ConceptFolderStore, ConceptFolderId, "SourceIdV");
	CategoryIdV = GetIntArrAsUInt64Arr(ConceptFolderStore, ConceptFolderId, "CategoryIdV");
	LocationIdV = GetIntArrAsUInt64Arr(ConceptFolderStore, ConceptFolderId, "LocationIdV");
	ConceptFolderStore->GetFieldNmStrV(ConceptFolderId, "KeywordV", KeywordV);
	ConceptFolderStore->GetFieldNmStrV(ConceptFolderId, "LangV", LangV);
	
	ConceptFolderStore->GetFieldNmFltV(ConceptFolderId, "ConceptWgtV", ConceptWgtV);
	ConceptFolderStore->GetFieldNmFltV(ConceptFolderId, "SourceWgtV", SourceWgtV);
	ConceptFolderStore->GetFieldNmFltV(ConceptFolderId, "CategoryWgtV", CategoryWgtV);
	ConceptFolderStore->GetFieldNmFltV(ConceptFolderId, "KeywordWgtV", KeywordWgtV);

	TUInt64V FixedConceptIdV = GetIntArrAsUInt64Arr(ConceptFolderStore, ConceptFolderId, "FixedConceptIdV");
	FixedConceptIdsH = TUInt64Set(FixedConceptIdV);

	ArticleTreshWgt = ConceptFolderStore->GetFieldNmInt(ConceptFolderId, "ArticleTreshWgt");
	EventTreshWgt = ConceptFolderStore->GetFieldNmInt(ConceptFolderId, "EventTreshWgt");
	ArticlesInFolder = ConceptFolderStore->GetFieldNmInt(ConceptFolderId, "ArticlesInFolder");
}

// call the default json constructor and then additionally parse the conceptUriV, sourceUriV, ... and read the counts if the uri exists in conceptfolderstore
TConceptFolder::TConceptFolder(const PJsonVal& Json, const PNewsBase& NewsBase)
{
	InitVals();
	// have to do copy/paste here from the json constructor since c++ doesn't allow calling another constructor
	Uri = Json->GetObjStr("uri", "");
	
	// counts are not serialized to json so we read them from the store itself
	if (NewsBase->ConceptFolderStore->IsRecNm(Uri)) {
		const uint64 ConceptFolderId = NewsBase->ConceptFolderStore->GetRecId(Uri);
		LastUpdatedArticleId = NewsBase->ConceptFolderStore->GetFieldNmUInt64(ConceptFolderId, "LastUpdatedArticleId");
		NewsBase->ConceptFolderStore->GetFieldNmTm(ConceptFolderId, "TimeOfLastArticleUpdate", TimeOfLastArticleUpdate);
		NewsBase->ConceptFolderStore->GetFieldNmTm(ConceptFolderId, "TimeOfLastEventUpdate", TimeOfLastEventUpdate);
	}

	AutoAddArticles = Json->GetObjBool("autoAddArticles", true);
	RestrictToSetConcepts = Json->GetObjBool("restrictToSetConcepts", false);
	RestrictToSetSources = Json->GetObjBool("restrictToSetSources", false);
	RestrictToSetCategories = Json->GetObjBool("restrictToSetCategories", false);
	RestrictToSetLocations = Json->GetObjBool("restrictToSetLocations", false);
	RestrictToSetKeywords = Json->GetObjBool("restrictToSetKeywords", false);
	
	IncludeSubCategories = Json->GetObjBool("includeSubCategories", false);
	UseLearnedInterests = Json->GetObjBool("useLearnedInterests", true);
	LearnInterests = Json->GetObjBool("learnInterests", false);

	FolderOwnerId = (uint64) Json->GetObjNum("folderOwnerId", 0);
	Visibility = (int) Json->GetObjNum("visibility", 0);

	if (Json->IsObjKey("conceptWgtV"))
		LoadFltArr(Json->GetObjKey("conceptWgtV"), ConceptWgtV);
	if (Json->IsObjKey("sourceWgtV"))
		LoadFltArr(Json->GetObjKey("sourceWgtV"), SourceWgtV);
	if (Json->IsObjKey("categoryWgtV"))
		LoadFltArr(Json->GetObjKey("categoryWgtV"), CategoryWgtV);
	if (Json->IsObjKey("locationWgtV"))
		LoadFltArr(Json->GetObjKey("locationWgtV"), LocationWgtV);
	if (Json->IsObjKey("keywordWgtV"))
		LoadFltArr(Json->GetObjKey("keywordWgtV"), KeywordWgtV);
	
	ArticleTreshWgt = (int) Json->GetObjNum("articleTreshWgt", 40);
	EventTreshWgt = (int) Json->GetObjNum("eventTreshWgt", 40);
	ArticlesInFolder = (int) Json->GetObjNum("articlesInFolder", 0);

	// parse the uris
	if (Json->IsObjKey("conceptUriV")) {
		PJsonVal Arr = Json->GetObjKey("conceptUriV");
		for (int N = 0; N < Arr->GetArrVals(); N++) {
			TStr Uri = Arr->GetArrVal(N)->GetStr();
			if (NewsBase->ConceptStore->IsRecNm(Uri))
				ConceptIdV.Add(NewsBase->ConceptStore->GetRecId(Uri));
		}
	}
	if (Json->IsObjKey("sourceUriV")) {
		PJsonVal Arr = Json->GetObjKey("sourceUriV");
		for (int N = 0; N < Arr->GetArrVals(); N++) {
			TStr Uri = Arr->GetArrVal(N)->GetStr();
			if (NewsBase->SourceStore->IsRecNm(Uri))
				SourceIdV.Add(NewsBase->SourceStore->GetRecId(Uri));
		}
	}
	if (Json->IsObjKey("categoryUriV")) {
		PJsonVal Arr = Json->GetObjKey("categoryUriV");
		for (int N = 0; N < Arr->GetArrVals(); N++) {
			TStr Uri = Arr->GetArrVal(N)->GetStr();
			if (NewsBase->CategoryStore->IsRecNm(Uri))
				CategoryIdV.Add(NewsBase->CategoryStore->GetRecId(Uri));
		}
	}
	if (Json->IsObjKey("locationUriV")) {
		int CityCountryKeyId = NewsBase->IndexVoc->GetKeyId(NewsBase->SourceStoreId, "CityCountry");
		PJsonVal Arr = Json->GetObjKey("locationUriV");
		for (int N = 0; N < Arr->GetArrVals(); N++) {
			TStr Uri = Arr->GetArrVal(N)->GetStr();
			uint64 Id = NewsBase->IndexVoc->AddWordStr(CityCountryKeyId, Uri);
			LocationIdV.Add(Id);
		}
	}
	if (Json->IsObjKey("keywordV")) {
		LoadStrArr(Json->GetObjKey("keywordV"), KeywordV);
	}
	if (Json->IsObjKey("langV")) {
		LoadStrArr(Json->GetObjKey("langV"), LangV);
	}
		
	if (Json->IsObjKey("fixedConceptUriV")) {
		PJsonVal Arr = Json->GetObjKey("fixedConceptUriV");
		for (int N = 0; N < Arr->GetArrVals(); N++) {
			TStr Uri = Arr->GetArrVal(N)->GetStr();
			if (NewsBase->ConceptStore->IsRecNm(Uri))
				FixedConceptIdsH.AddKey(NewsBase->ConceptStore->GetRecId(Uri));
		}
	}
}

PJsonVal TConceptFolder::GetJson() 
{
	PJsonVal Json = TJsonVal::NewObj();
	Json->AddToObj("uri", Uri);

	// Json->AddToObj("lastUpdatedArticleId", (int) LastUpdatedArticleId);
	// don't serialize TimeOfLastEventUpdate and TimeOfLastArticleUpdate

	Json->AddToObj("autoAddArticles", TJsonVal::NewBool(AutoAddArticles));
	Json->AddToObj("restrictToSetConcepts", TJsonVal::NewBool(RestrictToSetConcepts));
	Json->AddToObj("restrictToSetSources", TJsonVal::NewBool(RestrictToSetSources));
	Json->AddToObj("restrictToSetCategories", TJsonVal::NewBool(RestrictToSetCategories));
	Json->AddToObj("restrictToSetLocations", TJsonVal::NewBool(RestrictToSetLocations));
	Json->AddToObj("restrictToSetKeywords", TJsonVal::NewBool(RestrictToSetKeywords));

	Json->AddToObj("includeSubCategories", TJsonVal::NewBool(IncludeSubCategories));
	Json->AddToObj("useLearnedInterests", TJsonVal::NewBool(UseLearnedInterests));
	Json->AddToObj("learnInterests", TJsonVal::NewBool(LearnInterests));

	Json->AddToObj("folderOwnerId", (int) FolderOwnerId);
	Json->AddToObj("visibility", Visibility);

	Json->AddToObj("conceptIdV", TJsonVal::NewArr(GetIntArr(ConceptIdV)));
	Json->AddToObj("sourceIdV", TJsonVal::NewArr(GetIntArr(SourceIdV)));
	Json->AddToObj("categoryIdV", TJsonVal::NewArr(GetIntArr(CategoryIdV)));
	Json->AddToObj("locationIdV", TJsonVal::NewArr(GetIntArr(LocationIdV)));
	Json->AddToObj("keywordV", TJsonVal::NewArr(KeywordV));
	Json->AddToObj("langV", TJsonVal::NewArr(LangV));

	Json->AddToObj("conceptWgtV", TJsonVal::NewArr(ConceptWgtV));
	Json->AddToObj("sourceWgtV", TJsonVal::NewArr(SourceWgtV));
	Json->AddToObj("categoryWgtV", TJsonVal::NewArr(CategoryWgtV));
	Json->AddToObj("locationWgtV", TJsonVal::NewArr(LocationWgtV));
	Json->AddToObj("keywordWgtV", TJsonVal::NewArr(KeywordWgtV));
	
	TUInt64V FixedConceptIdV;  FixedConceptIdsH.GetKeyV(FixedConceptIdV);
	Json->AddToObj("fixedConceptIdV", TJsonVal::NewArr(GetIntArr(FixedConceptIdV)));

	Json->AddToObj("articleTreshWgt", ArticleTreshWgt);
	Json->AddToObj("eventTreshWgt", EventTreshWgt);
	Json->AddToObj("articlesInFolder", ArticlesInFolder);

	return Json;
}

PJsonVal TConceptFolder::GetJsonForStore()
{
	PJsonVal Json = TJsonVal::NewObj();
	Json->AddToObj("URI", Uri);

	Json->AddToObj("LastUpdatedArticleId", TJsonVal::NewNum(LastUpdatedArticleId));
	Json->AddToObj("TimeOfLastArticleUpdate", TimeOfLastArticleUpdate.GetWebLogDateTimeStr(true, "T", false));
	Json->AddToObj("TimeOfLastEventUpdate", TimeOfLastEventUpdate.GetWebLogDateTimeStr(true, "T", false));

	Json->AddToObj("AutoAddArticles", TJsonVal::NewBool(AutoAddArticles));
	
	Json->AddToObj("RestrictToSetConcepts", TJsonVal::NewBool(RestrictToSetConcepts));
	Json->AddToObj("RestrictToSetSources", TJsonVal::NewBool(RestrictToSetSources));
	Json->AddToObj("RestrictToSetCategories", TJsonVal::NewBool(RestrictToSetCategories));
	Json->AddToObj("RestrictToSetLocations", TJsonVal::NewBool(RestrictToSetLocations));
	Json->AddToObj("RestrictToSetKeywords", TJsonVal::NewBool(RestrictToSetKeywords));
	
	Json->AddToObj("IncludeSubCategories", TJsonVal::NewBool(IncludeSubCategories));
	Json->AddToObj("UseLearnedInterests", TJsonVal::NewBool(UseLearnedInterests));
	Json->AddToObj("LearnInterests", TJsonVal::NewBool(LearnInterests));

	Json->AddToObj("FolderOwnerId", (int) FolderOwnerId);
	Json->AddToObj("Visibility", Visibility);

	Json->AddToObj("ConceptIdV", TJsonVal::NewArr(GetIntArr(ConceptIdV)));
	Json->AddToObj("SourceIdV", TJsonVal::NewArr(GetIntArr(SourceIdV)));
	Json->AddToObj("CategoryIdV", TJsonVal::NewArr(GetIntArr(CategoryIdV)));
	Json->AddToObj("LocationIdV", TJsonVal::NewArr(GetIntArr(LocationIdV)));
	Json->AddToObj("KeywordV", TJsonVal::NewArr(KeywordV));
	Json->AddToObj("LangV", TJsonVal::NewArr(LangV));

	Json->AddToObj("ConceptWgtV", TJsonVal::NewArr(ConceptWgtV));
	Json->AddToObj("SourceWgtV", TJsonVal::NewArr(SourceWgtV));
	Json->AddToObj("CategoryWgtV", TJsonVal::NewArr(CategoryWgtV));
	Json->AddToObj("LocationWgtV", TJsonVal::NewArr(LocationWgtV));
	Json->AddToObj("KeywordWgtV", TJsonVal::NewArr(KeywordWgtV));

	TUInt64V FixedConceptIdV;  FixedConceptIdsH.GetKeyV(FixedConceptIdV);
	Json->AddToObj("FixedConceptIdV", TJsonVal::NewArr(GetIntArr(FixedConceptIdV)));

	Json->AddToObj("ArticleTreshWgt", ArticleTreshWgt);
	Json->AddToObj("EventTreshWgt", EventTreshWgt);
	Json->AddToObj("ArticlesInFolder", ArticlesInFolder);

	return Json;
}

void TConceptFolder::LoadStrArr(const PJsonVal& JsonArr, TStrV& Arr)
{
	Arr.Clr();
	if (JsonArr.Empty() || !JsonArr->IsArr() || JsonArr->GetArrVals() == 0)
		return;
	int Len = JsonArr->GetArrVals();
	Arr.Gen(Len);

	for (int N = 0; N < Len; N++)
		Arr[N] = JsonArr->GetArrVal(N)->GetStr();
}

void TConceptFolder::LoadUInt64Arr(const PJsonVal& JsonArr, TUInt64V& Arr)
{
	Arr.Clr();
	if (JsonArr.Empty() || !JsonArr->IsArr() || JsonArr->GetArrVals() == 0)
		return;
	int Len = JsonArr->GetArrVals();
	Arr.Gen(Len);

	for (int N = 0; N < Len; N++)
		Arr[N] = (uint64) JsonArr->GetArrVal(N)->GetNum();
}


void TConceptFolder::LoadFltArr(const PJsonVal& JsonArr, TFltV& Arr)
{
	Arr.Clr();
	if (JsonArr.Empty() || !JsonArr->IsArr() || JsonArr->GetArrVals() == 0)
		return;
	int Len = JsonArr->GetArrVals();
	Arr.Gen(Len);

	for (int N = 0; N < Len; N++)
		Arr[N] = JsonArr->GetArrVal(N)->GetNum();
}

TIntV TConceptFolder::GetIntArr(const TUInt64V& Arr)
{
	int Len = Arr.Len();
	TIntV Ret(Len);
	for (int N = 0; N < Len; N++)
		Ret[N] = (int) Arr[N];
	return Ret;
}

TUInt64V TConceptFolder::GetIntArrAsUInt64Arr(const PStore& Store, const uint64& RecId, const TStr& FieldNm)
{
	TIntV DataV;
	Store->GetFieldNmIntV(RecId, FieldNm, DataV);
	TUInt64V RetV (DataV.Len());
	for (int N = 0; N < DataV.Len(); N++)
		RetV[N] = (uint64) DataV[N];
	return RetV;
}

int TConceptFolder::AddConcept(const TUInt64& ConceptId, const double& Wgt)
{
	RemoveConcept(ConceptId);
	ConceptIdV.Add(ConceptId);
	ConceptWgtV.Add(Wgt);
	return ConceptIdV.Len() - 1;
}

void TConceptFolder::RemoveConcept(const TUInt64& ConceptId)
{
	int Pos = ConceptIdV.SearchForw(ConceptId);
	if (Pos >= 0) {
		RemoveFixedConcept(ConceptId);
		ConceptIdV.Del(Pos);
		ConceptWgtV.Del(Pos);
	}
}

void TConceptFolder::AddSource(const TUInt64& SourceId, const double& Wgt)
{
	RemoveSource(SourceId);
	SourceIdV.Add(SourceId);
	SourceWgtV.Add(Wgt);
}

void TConceptFolder::RemoveSource(const TUInt64& SourceId)
{
	int Pos = SourceIdV.SearchForw(SourceId);
	if (Pos >= 0) {
		SourceIdV.Del(Pos);
		SourceWgtV.Del(Pos);
	}
}

void TConceptFolder::AddCategory(const TUInt64& CategoryId, const double& Wgt)
{
	RemoveCategory(CategoryId);
	CategoryIdV.Add(CategoryId);
	CategoryWgtV.Add(Wgt);
}

void TConceptFolder::RemoveCategory(const TUInt64& CategoryId)
{
	int Pos = CategoryIdV.SearchForw(CategoryId);
	if (Pos >= 0) {
		CategoryIdV.Del(Pos);
		CategoryWgtV.Del(Pos);
	}
}

void TConceptFolder::AddFixedConcept(const TUInt64& ConceptId)
{
	FixedConceptIdsH.AddKey(ConceptId);
}

void TConceptFolder::RemoveFixedConcept(const TUInt64& ConceptId)
{
	FixedConceptIdsH.DelIfKey(ConceptId);
}

void TConceptFolder::AddLocation(const TUInt64& LocationId, const double& Wgt)
{
	RemoveLocation(LocationId);
	LocationIdV.Add(LocationId);
	LocationWgtV.Add(Wgt);
}

void TConceptFolder::RemoveLocation(const TUInt64& LocationId)
{
	int Pos = LocationIdV.SearchForw(LocationId);
	if (Pos >= 0) {
		LocationIdV.Del(Pos);
		LocationWgtV.Del(Pos);
	}
}

void TConceptFolder::GetConceptKdV(TUInt64FltKdV& ConceptKdV, const bool& SortByWgt) const
{
	ConceptKdV.Clr();
	EAssertR(ConceptIdV.Len() == ConceptWgtV.Len(), "Concept ids and wgts are not of the same length");
	for (int N = 0; N < ConceptIdV.Len(); N++)
		ConceptKdV.Add(TUInt64FltKd(ConceptIdV[N], ConceptWgtV[N]));
	if (SortByWgt)
		ConceptKdV.SortCmp(TCmpKeyDatByDat<TUInt64, TFlt>(false));
}