#ifndef CONCEPTFOLDER_H
#define CONCEPTFOLDER_H

#include <base.h>
#include "NewsConst.h"
#include "NewsBase.h"

class TConceptFolder;
typedef TPt<TConceptFolder> PConceptFolder;
class TConceptFolder {
public:
	friend class TPt<TConceptFolder>;

private:
	TCRef CRef;
	void InitVals();

public:
	TStr Uri;

	uint64 LastUpdatedArticleId;
	TTm TimeOfLastArticleUpdate;
	TTm TimeOfLastEventUpdate;
	
	bool AutoAddArticles;
	bool RestrictToSetConcepts;
	bool RestrictToSetSources;
	bool RestrictToSetCategories;
	bool RestrictToSetLocations;
	bool RestrictToSetKeywords;
		
	bool IncludeSubCategories;
	bool UseLearnedInterests;
	bool LearnInterests;

	uint64 FolderOwnerId;
	int Visibility;

	TUInt64V ConceptIdV;
	TUInt64V SourceIdV;
	TUInt64V CategoryIdV;
	TUInt64V LocationIdV;
	TStrV KeywordV;
	TStrV LangV;

	TFltV ConceptWgtV;
	TFltV SourceWgtV;
	TFltV CategoryWgtV;
	TFltV LocationWgtV;
	TFltV KeywordWgtV;
	
	// concept ids from HasConceptSpV whose weights should not be updated automatically
	// based on added articles. this is used to represent user specified interests
	TUInt64Set FixedConceptIdsH;

	int ArticleTreshWgt;
	int EventTreshWgt;
	int ArticlesInFolder;
	
	TConceptFolder(const PJsonVal& Json);
	TConceptFolder(const uint64& ConceptFolderId, const PStore& ConceptFolderStore);
	
	TConceptFolder(const PJsonVal& Json, const PNewsBase& NewsBase);

public:
	static PConceptFolder New() { return new TConceptFolder(TJsonVal::NewObj()); }
	static PConceptFolder New(const uint64& ConceptFolderId, const PStore& ConceptFolderStore) {
		if (!ConceptFolderStore->IsRecId(ConceptFolderId))
			return NULL;
		return new TConceptFolder(ConceptFolderId, ConceptFolderStore);
	}
	static PConceptFolder New(const TStr& ConceptFolderUri, const PStore& ConceptFolderStore) {
		if (!ConceptFolderStore->IsRecNm(ConceptFolderUri))
			return NULL;
		uint64 ConceptFolderId = ConceptFolderStore->GetRecId(ConceptFolderUri);
		return new TConceptFolder(ConceptFolderId, ConceptFolderStore);
	}
	// use when json contains conceptIdV, sourceIdV, ...
	static PConceptFolder New(const PJsonVal& Json) { return new TConceptFolder(Json); }
	// use when json contains conceptUriV, sourceUriV, ...
	static PConceptFolder New(const PJsonVal& Json, const PNewsBase& NewsBase) {
		return new TConceptFolder(Json, NewsBase);
	}

	PJsonVal GetJson();
	PJsonVal GetJsonForStore();
	
	void LoadStrArr(const PJsonVal& JsonArr, TStrV& Arr);
	void LoadUInt64Arr(const PJsonVal& JsonArr, TUInt64V& Arr);
	void LoadFltArr(const PJsonVal& JsonArr, TFltV& Arr);
	TIntV GetIntArr(const TUInt64V& Arr);
	TUInt64V GetIntArrAsUInt64Arr(const PStore& Store, const uint64& RecId, const TStr& FieldNm);
	
	int AddConcept(const TUInt64& ConceptId, const double& Wgt = DefaultConceptWgt);
	void RemoveConcept(const TUInt64& ConceptId);

	void AddSource(const TUInt64& SourceId, const double& Wgt);
	void RemoveSource(const TUInt64& SourceId);
	bool IsSource(const TUInt64& SourceId) const { return SourceIdV.IsIn(SourceId); }

	void AddCategory(const TUInt64& CategoryId, const double& Wgt);
	void RemoveCategory(const TUInt64& CategoryId);
	bool IsCategory(const TUInt64& CategoryId) const { return CategoryIdV.IsIn(CategoryId); }

	void AddFixedConcept(const TUInt64& ConceptId);
	void RemoveFixedConcept(const TUInt64& ConceptId);
	bool IsConceptFixed(const TUInt64& ConceptId) const { return FixedConceptIdsH.IsKey(ConceptId); }
	bool IsConceptFixed(const int& ConceptId) const { return FixedConceptIdsH.IsKey((TUInt64) ConceptId); }

	void AddLocation(const TUInt64& LocationId, const double& Wgt);
	void RemoveLocation(const TUInt64& LocationId);
	
	//void UpdateFromFolder(const TFolderRec& Folder, bool AlsoUpdateMetaInfo = false);
	void GetConceptKdV(TUInt64FltKdV& ConceptKdV, const bool& SortByWgt = true) const;
};


#endif