#ifndef GEOBASE_H
#define GEOBASE_H

#include <base.h>
#include <qminer.h>
#include <qminer_srv.h>

#include "NewsConst.h"
#include "NewsFixDb.h"
#include "ConceptLabel.h"

using namespace TQm;
using namespace TQm::TStorage;

class TNewsBase;

class TGeoBase;
typedef TPt<TGeoBase> PGeoBase;
class TGeoBase{
public:
  friend class TPt<TGeoBase>;

private:
	TCRef CRef;

	TStr FPath;
	// ids for faster access
	uchar CountryStoreId;
	uchar CityStoreId;
	uchar LanguageStoreId;

	int CountryWikiKeyId;
	int CityWikiKeyId;
	THash<TUInt64, TInt> CityWgtH;
	THash<TUInt64, TInt> CountryWgtH;

	typedef THash<TStr, TVec<TUInt64IntKd> > PrefToSugg;
	THash<TStr, PrefToSugg> CountryPrefixToIdWgtHH;
	THash<TStr, PrefToSugg > CityPrefixToIdWgtHH;

	TGeoBase(const TStr& FPath, const TFAccess& FAccess, const PConceptLabel ConceptLabel);
	TGeoBase(const TStr& FPath, const TStr& StoreDefFNm, const PConceptLabel ConceptLabel);

	void Init();

	// clean on exit
	~TGeoBase();

	// autosuggest data
	TInt NextLangInd;
	TBool SortByPopularity;
	THash<TUInt64, TInt> CountryData;
	THash<TUInt64, TInt> CityData;
	// load prefixes from disk
	bool LoadPrefixes(const bool& SortByPopularity);
	// update data frequencies
	void UpdateDataCounts();

public:
	// index to base
	PBase Base;
	// pointers to the index and stores
	PIndexVoc IndexVoc;
	PIndex Index;
	PStore CountryStore;
	PStore CityStore;
	PStore LanguageStore;

	

	PConceptLabel ConceptLabel;

	static PGeoBase Load(const TStr& FPath, const TFAccess& FAccess, const PConceptLabel ConceptLabel) {
		return new TGeoBase(FPath, FAccess, ConceptLabel);
	}
	static PGeoBase New(const TStr& FPath, const TStr& StoreDefFNm, const PConceptLabel ConceptLabel) {
		return new TGeoBase(FPath, StoreDefFNm, ConceptLabel);
	}
	
	// import data from json files
	void ImportData(const PNewsFixDb& NewsFixDb, const TStr& CountryInfoJsonFNm, const TStr& CityInfoJsonFNm);

	// do we have any data in the stores
	bool HasImportedData() { return CountryStore->GetRecs() > 0 || CityStore->GetRecs() > 0; }
	
	void ClearWeights() { CountryWgtH.Clr(); CityWgtH.Clr(); }

	// auto suggestions
	void LoadOrUpdatePrefixes(const bool& SortByPopularity = true);
	void SetSorting(const bool& _SortByPopularity = true);
	void UpdatePrefixes(const bool& Report = true);
	void SuggestCountries(const TStr& Prefix, const TStr& Lang, const int& MaxCount, TUInt64IntKdV& Suggestions);
	void SuggestCities(const TStr& Prefix, const TStr& Lang, const int& MaxCount, TUInt64IntKdV& Suggestions);

	TStr GetCityForGeoId(const TStr& GeoId) const;
	TStr GetCountryForGeoId(const TStr& GeoId) const;
	TStr GetCountryForCity(const TStr& WikiUri) const;
	TStr GetGeoIdForCity(const TStr& WikiUri) const;
	TStr GetGeoIdForCountry(const TStr& WikiUri) const;
	bool GetCitiesForCountry(const TStr& WikiUri, TStrV& CityWikiUriV) const;
	bool IsCity(const TStr& WikiUri) const;
	bool IsCountry(const TStr& WikiUri) const;

	bool IncreaseWgt(const TStr& WikiUri, const int Wgt = 1);
	bool DecreaseWgt(const TStr& WikiUri, const int Wgt = 1);
	int GetWgt(const TStr& WikiUri);
	
	PJsonVal GetInfoForCountry(const TStr& WikiUri, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const bool& IncludeUri = false, const bool& IncludeLocation = false, const bool& IncludeDetailedInfo = false);
	PJsonVal GetInfoForCity(const TStr& WikiUri, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const bool& IncludeUri = false, const bool& IncludeLocation = false, const bool& IncludePopulation = false, const bool& IncludeCountryInfo = false);

	void ImportGeoConcepts(const TNewsBase* NewsBase);
};


// PlusMachines-Server-Search
class TGeoAutoSuggestSrv : public TSAppSrvFun {
private:
	PGeoBase GeoBase;

public:
	TGeoAutoSuggestSrv(const PGeoBase& _GeoBase) : TSAppSrvFun("suggestLocations", saotJSon) {
		GeoBase = _GeoBase;
	}
	static PSAppSrvFun New(const PGeoBase& GeoBase) { 
		return new TGeoAutoSuggestSrv(GeoBase); 
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);
};

#endif