/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "NewsActivity.h"


TActivityItem::TActivityItem(TSIn& SIn)
{
	ItemId.Load(SIn);
	ActivityId.Load(SIn);
	TSecTm SecTm(SIn);
	Tm = TTm(SecTm);
}

void TActivityItem::Save(TSOut& SOut) const
{
	ItemId.Save(SOut);
	ActivityId.Save(SOut);
	Tm.GetSecTm().Save(SOut);
}

// load activity history
TNewsActivity::TNewsActivity(const TStr& _FName, const TStr& _SettingsFileName) : FName(_FName), SettingsFileName(_SettingsFileName)
{
	ArticleCount = 10000;
	StoryCount = 10000;
}

bool TNewsActivity::Load()
{
	const TStr SettingsStr = TFile::Exists(SettingsFileName) ? TStr::LoadTxt(SettingsFileName) : "{}";
	const PJsonVal AllSettings = TJsonVal::GetValFromStr(SettingsStr);
	const PJsonVal ActivitySettings = AllSettings->IsObjKey("newsActivity") ? AllSettings->GetObjKey("newsActivity") : TJsonVal::NewObj();
		
	TNotify::StdNotify->OnStatusFmt("Loading news activity data...");

	ArticleCount = (int) ActivitySettings->GetObjNum("ArticleCount", 10000);
	StoryCount = (int) ActivitySettings->GetObjNum("StoryCount", 10000);
	TNotify::StdNotify->OnStatusFmt("News activity: ArticleCount: %d", ArticleCount);
	TNotify::StdNotify->OnStatusFmt("News activity: StoryCount: %d", StoryCount);
	
	if (TFile::Exists(FName)) {
		try {
			
			TFIn FIn(FName);
			LastArticleActivityId.Load(FIn);
			LastStoryActivityId.Load(FIn);
			ArticleActivityL = TLst<TActivityItem>(FIn);
			StoryActivityL = TLst<TActivityItem>(FIn);
			TNotify::StdNotify->OnStatusFmt("Loaded.");
		}
		catch (...) {
			TNotify::StdNotify->OnStatusFmt("Exception while loading news activity data.");
			return false;
		}
	}
	else {
		LastArticleActivityId = 0;
		LastStoryActivityId = 0;
	}
	return true;
}

TNewsActivity::~TNewsActivity()
{
	Save();
}

void TNewsActivity::Save()
{
	TFOut FOut(FName);
	LastArticleActivityId.Save(FOut);
	LastStoryActivityId.Save(FOut);
	ArticleActivityL.Save(FOut);
	StoryActivityL.Save(FOut);
}

// add activity regarding articles
void TNewsActivity::AddArticleActivity(const uint64& ArticleId, const TTm Tm)
{
	ArticleActivityL.AddBack(TActivityItem(ArticleId, LastArticleActivityId, Tm));
	LastArticleActivityId++;
	if (ArticleActivityL.Len() > ArticleCount)
		ArticleActivityL.DelFirst();
}

// add activity regarding story
void TNewsActivity::AddStoryActivity(const uint64& StoryId, const TTm Tm)
{
	StoryActivityL.AddBack(TActivityItem(StoryId, LastStoryActivityId, Tm));
	LastStoryActivityId++;
	if (StoryActivityL.Len() > StoryCount)
		StoryActivityL.DelFirst();
}