#ifndef NEWSARTICLE_H
#define NEWSARTICLE_H

#include "base.h"
#include "mine.h"

#include "NewsFixDb.h"
#include "ConceptLabel.h"

// forward declarations
class TNewsFixDb;
typedef TPt<TNewsFixDb> PNewsFixDb;

class TNewsMemEncoding {
public:
	static void EncodeMem(const TMem& InMem, TMem& OutMem) {
		const int Len = InMem.Len();
		OutMem.Reserve(2 * Len);
		for (int ChN = 0; ChN < Len; ChN++) {
			const uchar Ch = uchar(InMem[ChN]);
			OutMem += TCh::GetHexCh((Ch / 16) % 16);
			OutMem += TCh::GetHexCh(Ch % 16);
		}
	}

	static void DecodeMem(const TMem& InMem, TMem& OutMem) {
		const int Len = InMem.Len(); EAssert(TInt::IsEven(Len));
		OutMem.Reserve(Len / 2); int ChN = 0;
		while (ChN < Len) {
			const char MshCh = InMem[ChN]; ChN++;
			const char LshCh = InMem[ChN]; ChN++;
			const uchar Ch = uchar(TCh::GetHex(MshCh) * 16 + TCh::GetHex(LshCh));
			OutMem += Ch;
		}
	}
};

///////////////////////////////
// News-Concept
ClassTV(TNewsConcept, TNewsConceptV)//{
private:
	TStr Uri;
	TStr Label; // short label
	TStr Type; // type of concept (E.g. keyword or topic)
	TStr Desc; // long description
	// weight
	TBool WgtP;
	TFlt Wgt;

public:
	TNewsConcept() { }
	TNewsConcept(const TStr& _Uri, const TStr& _Label, const TStr& _Type);
	TNewsConcept(const TStr& _Uri, const TStr& _Label, const TStr& _Type, const double& _Wgt);
	// binary serialization
	TNewsConcept(TSIn& SIn);
	void Save(TSOut& SOut) const;
	// XML serialization
	TNewsConcept(const PXmlTok& ConceptTok);
	PXmlTok GetXmlTok() const;

	uint64 GetMemUsed() const;

	// concept basic information
	const TStr& GetUri() const { return Uri; }
	const TStr& GetLabel() const { return Label; }
	const TStr& GetType() const { return Type; }
	bool IsDesc() const { return !Desc.Empty(); }
	const TStr& GetDesc() const { return Desc; }
	bool IsWgt() const { return WgtP; }
	double GetWgt() const { return Wgt; }
	// changing basic info
	void PutLabel(const TStr& _Label) { Label = _Label; }
	void PutDesc(const TStr& _Desc) { Desc = _Desc; }
};

///////////////////////////////
// News-Article
class TNewsArticle {
public:
	// source
	TStr SourceUri;
	TStr SourceTitle;
	TStr SourceType;
	TBool SourceLocP;
	TFltPr SourceLoc;
	TStr SourceCountry;
	TStr SourceCity;
	// article
	TStr ArticleLang;
	TStr ArticleUri;
	TTm ArticlePubTm;
	TTm ArticleCrawlTm;
	TStr ArticleTitle;
	TStr ArticleBody;
	TStrV ArticleKeyWd;
	TStr ArticleClustId;
	TStr ArticleGoogleClustId;
	// auxilary
	TStrKdV AuxKdV;
	// images
	TStrV ImageUriV;
	// concept
	TNewsConceptV ConceptV;
	// similar articles (id, lang, wgt)
	TStrStrIntTrV SimArticleV;
	// id in the ConceptV that represents the location concept where the article content happened
	// value is set if concept of type location is mentioned as one of the first 3 words in the article
	// set to TInt::Mx if the location was not found
	TInt LocationConceptN;
	// json encoded info about detected dates
	TStr DatesJson;
	TStrFltPrV CategoryV;

public:
	TNewsArticle() { LocationConceptN = TInt::Mx; }
	// binary serialization
	TNewsArticle(TSIn& SIn);
	void Save(TSOut& SOut) const;
	// XML serialization
	TNewsArticle(const PXmlTok& ArticleTok,
		const PTokenizer& SearchTok, const PTokenizer& FtrTok);
	PXmlTok GetXmlTok() const;
	// Json for asserting into generic base
	PJsonVal GetJson(const PNewsFixDb NewsFixDb) const;

	uint64 GetMemUsed() const;

	void AddCategory(const TStr& CatName, const double& Wgt);
};

///////////////////////////////
// News-Source-Types
class TNewsSourceType {
public:
	static TStr MainstreamNews;
	static TStr Weblog;
	static TStr MicroBlog;
};


#endif