#ifndef NEWS_H
#define NEWS_H

#include <base.h>
#include <qminer.h>
#include "NewsCounts.h"
#include "NewsConst.h"
#include "NewsActivity.h"
#include "CategoryBase.h"
#include "NewsFixDb.h"
#include "logger.h"
#include "NewsArticle.h"
#include "ConceptLabel.h"
#include "../Trends/ConceptTrendsManager.h"
#include "../Trends/ConceptDailyTrendInfo.h"
#include "GeoBase.h"
#include "../Utils/ValidatedStoryLinks.h"
#include "../Utils/ValidatedArticleLinks.h"

using namespace TQm;
using namespace TQm::TStorage;

// forward declarations
class TNewsBase;
typedef TPt<TNewsBase> PNewsBase;
class TNewsFixDb;
typedef TPt<TNewsFixDb> PNewsFixDb;
class TNewsBaseFix;
typedef TPt<TNewsBaseFix> PNewsBaseFix;
class TNewStoryTrigger;

/////////////////////////////////
//// News-Notifier
//class TNewsNotify {
//public:
//	static PNotify Notify;
//};

enum EnumStoryFlag : int { sfUserSetDate = 1 << 0, sfUserSetTitle = 1 << 1, sfUserSetSummary = 1 << 2, sfUserSetLocation = 1 << 3, sfModelSetDate = 1 << 4, sfModelSetTitle = 1 << 5, sfModelSetSummary = 1 << 6 };

typedef THash<TUInt64, TInt> TRecDict;

///////////////////////////////
// News-Base
class TNewsBase
{
public:
	friend class TPt<TNewsBase>;
private:
	TCRef CRef;

public:
	// index to base
	PBase Base;
	// pointers to the index and stores
	PIndexVoc IndexVoc;
	PIndex Index;
	PStore ArticleStore;
	PStore SourceStore;
	PStore ConceptStore;
	PStore ConceptClassStore;
	PStore ConceptFolderStore;
	PStore UserStore;
	
	PStore StoryStore;
	PStore EventStore;
	PStore ImageStore;
	PStore DateStore;
	PStore TagStore;
	PStore CategoryStore;

	// ids for faster access
	uint ArticleStoreId;
	uint SourceStoreId;
	uint ConceptStoreId;
	uint ConceptClassStoreId;
	uint ConceptFolderStoreId;
	uint UserStoreId;

	uint StoryStoreId;
	uint EventStoreId;
	uint ImageStoreId;
	uint DateStoreId;
	uint TagStoreId;
	uint CategoryStoreId;

	// timer for garbage collection
	TTmTimer* Timer;
	TInt NewArticles;
	// links to newsfeed ids
	TStrUInt64H NewsfeedIdH;
	
	TStr SettingsFNm;
	TStr DbsPath;
	TStr CustomDataPath;		// = ServerArticles/CustomData/

	PNewsCounts NewsCounts;
	PNewsActivity NewsActivity;
	PNewsFixDb NewsFixDb;
	PNewsBaseFix NewsBaseFix;
	PGeoBase GeoBase;
	PValidatedStoryLinks ValidatedStoryLinks;
	PValidatedArticleLinks ValidatedArticleLinks;
	PConceptLabel ConceptLabel;
	
	PCategoryBase CategoryBase;
	PConceptTrendsManager TrendsManager;
	PDailyTrendInfos DailyTrendInfos;
	PStoreTrigger NewArticleTrigger;
	TNewStoryTrigger* NewStoryTrigger;

	PLogger InfoLogFile;
	PLogger ActivityLogFile;
	PLogger ErrorLogFile;

private:
	
	bool ClearInfoForEmptyStoriesAndEvents;
	int GarbageCollectionTimerMins;


	// for creating a new index and store
	TNewsBase(const PGeoBase GeoBase, const PNewsFixDb NewsFixDb, const PConceptLabel ConceptLabel, const TStr& StoreDefFNm, const TStr& FPath, const int64& IndexCacheSize, const int64& ArticleCacheSize, const TStr& SettingsFNm, const TStr& DbsPath, const TStr& TempFPath);
	// for loading existing index and stores for update or read-only
	TNewsBase(const PGeoBase GeoBase, const PNewsFixDb NewsFixDb, const PConceptLabel ConceptLabel, const TStr& FPath, const TFAccess& FAccess, const int64& IndexCacheSize, const int64& ArticleCacheSize, const TStr& SettingsFNm, const TStr& DbsPath, const TStr& TempFPath);

	// initialize shortcuts
	void Init();
	// save newsbase
	void Save();
	// clean on exit
	~TNewsBase() { Save(); }
public:
	// for creating new index and stores or updating existing
	static PNewsBase New(const PGeoBase GeoBase, const PNewsFixDb NewsFixDb, const PConceptLabel ConceptLabel, const TStr& StoreDefFNm, const TStr& FPath, const int64& IndexCacheSize, const int64& ArticleCacheSize, const TStr& SettingsFNm, const TStr& DbsPath, const TStr& TempFPath);
	// for loading existing index and stores for update or read-only
	static PNewsBase Load(const PGeoBase GeoBase, const PNewsFixDb NewsFixDb, const PConceptLabel ConceptLabel, const TStr& FPath, const TFAccess& FAccess, const int64& IndexCacheSize, const int64& ArticleCacheSize, const TStr& SettingsFNm, const TStr& DbsPath, const TStr& TempFPath);

	void ReloadSettings();

	/// article methods
	uint64 AddArticle(const TNewsArticle& Article);
	uint64 AddArticle(const PJsonVal& ArticleVal);	
	void DeleteArticleInfo(const uint64& ArticleId);
	PRecSet GetSimilarArticles(const uint64& ArticleId, const TStr& Lang) const;
	/*void DeleteStoryInfo(const uint64& StoryId);
	void DeleteEventInfo(const uint64& EventId);*/

	/// concept methods
	// get concept label in desired lang or default if ""
	TStr GetConceptLabel(const uint64& ConceptId, const TStr& Lang) const;
	TStr GetConceptLabel(const TStr& ConceptUri, const TStr& Lang) const;

	///  source methods
	// prettifying titles and URIs
	static TStr ParseSourceUri(const TStr& InUri, const bool& KeepWwwP = true);
	static TStr ParseSourceTitle(const TStr& InTitle);
	bool IsSourceEnabled(const TStr& SourceUri);
	bool HasLocationForArticleSource(const uint64& ArticleId) const;

	/// story methods
	uint64 AddStory(const TStr& Story, const TStr& Lang);
	uint64 AddStory(const uint64& ArticleId, const TStr& Story);
	void PutMedoid(const uint64& MedoidId, const TStr& Story);
	bool GetStoryFlag(const uint64& StoryId, const EnumStoryFlag& Flag) const;
	void SetStoryFlag(const uint64& StoryId, const EnumStoryFlag& Flag, const bool& Val);
	void ClearFlagForStories(const PRecSet& StoryRecSet, const EnumStoryFlag& StoryFlag, const TStr& Lang = "");
	bool IsStoryFlagSet(const PRecSet& StoryRecSet, const EnumStoryFlag& StoryFlag, const TStr& Lang = "");
	int GetStoryQuality(const uint64& StoryId);
	void ClearStoryInformation(const uint64& StoryId);
	uint64 GetCentralArticle(const uint64& StoryId) const;

	uint64 GetBestStory(const uint64& EventId, const TStr& Lang = "") const;
	
	// event methods
	void ClearEventInformation(const uint64& EventId);

	// newsfeed ids
	void AddNewsfeedId(const uint64& ArticleId, const TStr& NewsfeedId);
	void RemoveNewsfeedId(const TStr& NewsfeedId);
	uint64 GetArticleIdForNewsfeedId(const TStr& NewsfeedId) const;
	
	void RebuildNewsCountsAndTrends();

	// check if index already exists
	static bool Exists(const TStr& IndexFPath) { return TIndex::Exists(IndexFPath) && TFile::Exists(IndexFPath + "Source.BaseStore"); }

	/// recset methods
	void IntersectRecSets(PRecSet& Dest, const PRecSet& Source, const bool& AddSourceFqs = true);
	void RemoveRecSets(PRecSet& Dest, const PRecSet& RecsToRemove);

	/// logging methods
	PLogger GetInfoLogger() const { return InfoLogFile; }
	PLogger GetActivityLogger() const { return ActivityLogFile; }
	PLogger GetErrorLogger() const { return ErrorLogFile; }
	void SetInfoLogger(PLogger Logger) { InfoLogFile = Logger; }
	void SetActivityLogger(PLogger Logger) { ErrorLogFile = Logger; }
	void SetErrorLogger(PLogger Logger) { ErrorLogFile = Logger; }

	
};

///////////////////////////////
// NewsBase fixes
class TNewsBaseFix {
public:
	friend class TPt<TNewsBaseFix>;
private:
	TCRef CRef;

	PNotify Notify;
	TNewsBase* NewsBase;
	PNewsFixDb NewsFixDb;

	TNewsBaseFix(TNewsBase* _NewsBase) {
		NewsBase = _NewsBase;
		NewsFixDb = NewsBase->NewsFixDb;
		Notify = ConsoleLog;
	}

	void RemoveInvalidConcept(const TStr& ConceptUri);
	void RewriteConceptUri(const TStr& BadUri, const TStr& GoodUri);

public:
	static PNewsBaseFix New( TNewsBase* NewsBase) { return new TNewsBaseFix(NewsBase); }

	// apply fixes from conceptInfo.txt on newsbase (concept rewrites, concept types, concept labels)
	void ApplyConceptFixesOnNewsBase();
	// apply fixes from sourceInfo.txt on newsbase (source info, removal of disabled sources, ...)
	void ApplySourceFixesOnNewsBase();
	// check ConceptFix GetConceptType and if the type is not set there check the current type in NewsBase->ConceptStore
	bool GetConceptType(const TStr& Url, TStr& Type) const;
	//bool GetConceptUri(const TStr& Url, TStr& Uri) const;

	PJsonVal SetConceptImage(const TStr& ConceptUri, const TStr& ImageUrl, const bool& SetAsPrimary = false, const bool& SetPrimaryIfFirst = true);
	PJsonVal SetConceptDescription(const TStr& ConceptUri, const TStr& Description);
	void SetConceptType(const TStr& ConceptUri, const TStr& Type);
	void SetConceptLabel(const TStr& ConceptUri, const TStr& Lang, const TStr& NewLabel);
	void SetMissingLabelsFromWikiUri(const TStr& Lang);
};

///////////////////////////////
// News-NewsfeedId-Trigger
class TNewsArticleTrigger : public TStoreTrigger {
	// link to newsbase
	TNewsBase* NewsBase;
	
	TNewsArticleTrigger(TNewsBase* _NewsBase) : NewsBase(_NewsBase) { }
public:
	static PStoreTrigger New(TNewsBase* NewsBase) { return new TNewsArticleTrigger(NewsBase); }
	
	void OnAdd(const TRec& ArticleRec);
	void OnUpdate(const TRec& ArticleRec) {}
	void OnDelete(const TRec& ArticleRec);
};

class TNewStoryTrigger : public TStoreTrigger {
private:
	PLogger InfoLogFile;
	PLogger ErrorLogFile;
	TNewsBase* NewsBase;
	
	TNewStoryTrigger(TNewsBase* _NewsBase, const PLogger& _InfoLogFile, const PLogger& _ErrorLogFile) : NewsBase(_NewsBase), InfoLogFile(_InfoLogFile), ErrorLogFile(_ErrorLogFile) {
	}

public:
	static TNewStoryTrigger* New(TNewsBase* NewsBase, const PLogger& InfoLogFile, const PLogger& ErrorLogFile) { return new TNewStoryTrigger(NewsBase, InfoLogFile, ErrorLogFile); }

	void OnAdd(const TRec& Rec);
	void OnUpdate(const TRec& Rec) { }
	void OnDelete(const TRec& Rec) { }
};

#endif