#ifndef NEWSCONST_H
#define NEWSCONST_H

#include <base.h>

enum FolderVisibility { PrivateFolder = 0, PublicFolder, CommunityFolder };

const double DefaultConceptWgt = 10.0;
const TStr HomeFolderName("::Home::");
const TStr AdminUserUri("::ADMIN::");
const TStr DefLang("eng");  // default language - used for fallback

// default number of articles returned in a query
const TInt DefaultNumberOfReturnedArticlesPerPage = 20;
const int ShortArticleBodyLen = 100;
const int DefaultArticleBodyLen = 300;
const int LongerArticleBodyLen = 500;

enum EnumConceptType {
	ctNone = 0, 
	ctPerson = 0x01,
	ctLoc = 0x02,
	ctOrg = 0x4,
	ctEntities = 0x1 | 0x2 | 0x4, 
	//ctCategory = 0x8, 
	ctKeyword = 0x10, 
	ctWiki = 0x20, 
	ctConceptFolder = 0x40, 
	ctConcepts = 0x1 | 0x2 | 0x4 | 0x10 | 0x20,
	ctAll = 0xFF
};

enum EnumConceptClassType { cctNone = 0, cctDbpedia = 0x1, cctCustom = 0x2, cctAll = 0xFF };

class TNewsConst
{
public:
	TNewsConst();

	TFlt Eps;
	
	TStr TypeLocation;
	TStr TypePerson;
	TStr TypeOrganization;
	TStr TypeConceptFolder;
	TStr TypeKeyword;
	TStr TypeWiki;

	TStr ConceptFolderUriPrefix;
	
	TStr ClassTypeDBPedia;
	TStr ClassTypeCustom;

	// all concept types (person, org, loc, keyword, wiki)
	TStrSet ConceptSet;
	// set of entity types (person, org, loc)
	TStrSet EntitySet;
	// set of keyword types (keyword)
	TStrSet KeywordSet;
	// set of wiki types (wiki)
	TStrSet WikiSet;
	
	TStrSet ConceptFolderSet;

	TStrV ConceptTypeV;
	
	// set of languages we care about (when doing clustering, autosuggest, ...)
	//TStrSet LangH;
	//TStrV LangV;
	TStrV LangCoreV;			// core languages - eng, spa, deu, zho
	TStrV LangSimilarV;			// languages for which we have computed similar articles
	TStrV LangAllV;				// full list of languages that we manage - needed to compute autosuggestions, etc.

	bool IsEntity(const TStr& Type) const;
	bool IsKeywordOrWiki(const TStr& Type) const;

	bool IsClassDbpedia(const TStr& Type) const { return Type == ClassTypeDBPedia; }
	bool IsClassCustom(const TStr& Type) const { return Type == ClassTypeCustom; }

	void SetLangInfo(const PJsonVal& LangInfo);
	
	EnumConceptType GetValidTypes(const TStrV& TypeV, const EnumConceptType& TypeForEmptyTypeV = ctAll) const;
	bool IsValidType(const TStr& Type, const EnumConceptType& ValidTypes) const;
	TStr GetTypeLabel(const TStr& Type) const;
	TStr GetPluralTypeLabel(const TStr& Type) const;
	TStr GetJsonNmForLang(const TStr& Lang) const;		// "eng" -> "labelEng"
	TStr GetLangForJsonNm(const TStr& JsonNm) const;	// "labelEng" -> "eng"
	bool IsJsonNmForLang(const TStr& JsonNm) const;
};

// global instance of NewsConst
extern TNewsConst NewsConst;
extern PNotify ConsoleLog;
#endif


