/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "NewsCounts.h"

bool TNewsCounts::Load()
{
	if (TFile::Exists(FName)) {
		try {
			TNotify::StdNotify->OnStatus("Loading news counts...");
			TFIn FIn(FName);
			ConceptIdFqH.Load(FIn);
			SourceIdFqH.Load(FIn);
			ArticleCount.Load(FIn);
			AnnArticleCount.Load(FIn);
			DateToArticleCountH.Load(FIn);
			DateToAnnArticleCountH.Load(FIn);
			DateToStoryCountH.Load(FIn);
			TNotify::StdNotify->OnStatus("Loaded.");
			return true;
		}
		catch (...) {
			TNotify::StdNotify->OnStatus("There was exception while loaded news counts from the file");
			return false;
		}
	}
	else {
		TNotify::StdNotify->OnStatus("Failed to load news counts.");
		return false;
	}
}

void TNewsCounts::Save()
{
	TFOut FOut(FName);
	ConceptIdFqH.Save(FOut);
	SourceIdFqH.Save(FOut);
	ArticleCount.Save(FOut);
	AnnArticleCount.Save(FOut);
	DateToArticleCountH.Save(FOut);
	DateToAnnArticleCountH.Save(FOut);
	DateToStoryCountH.Save(FOut);
}

void TNewsCounts::GetTopConcepts(const int Count, TUInt64IntKdV& ConcIdFqV)
{
	ConceptIdFqH.Defrag();
	ConceptIdFqH.SortByDat(false);
	for (int KeyId = ConceptIdFqH.FFirstKeyId(), N = 0; ConceptIdFqH.FNextKeyId(KeyId) && N < Count; N++)
		ConcIdFqV.Add(TUInt64IntKd(ConceptIdFqH.GetKey(KeyId), (int) ConceptIdFqH[KeyId]));
}

void TNewsCounts::GetTopSources(const int Count, TUInt64IntKdV& SourceIdFqV)
{
	SourceIdFqH.Defrag();
	SourceIdFqH.SortByDat(false);
	for (int KeyId = SourceIdFqH.FFirstKeyId(), N = 0; SourceIdFqH.FNextKeyId(KeyId) && N < Count; N++)
		SourceIdFqV.Add(TUInt64IntKd(SourceIdFqH.GetKey(KeyId), (int) SourceIdFqH[KeyId]));
}
