#ifndef NEWSCOUNTS_H
#define NEWSCOUNTS_H

#include <base.h>

class TNewsCounts;
typedef TPt<TNewsCounts> PNewsCounts;
class TNewsCounts {
public:
	friend class TPt<TNewsCounts>;

private:
	TCRef CRef;

	TStr FName;
	THash<TUInt64, TUInt64> ConceptIdFqH;
	THash<TUInt64, TUInt64> SourceIdFqH;
	TUInt64 ArticleCount;
	TUInt64 AnnArticleCount;
	THash<TStr, TUInt64> DateToArticleCountH;
	THash<TStr, TUInt64> DateToStoryCountH;
	THash<TStr, TUInt64> DateToAnnArticleCountH;

	TNewsCounts(const TStr& _FName) : FName(_FName) { };
	~TNewsCounts() { Save(); }

public:
	static PNewsCounts New(const TStr& FName) {
		return new TNewsCounts(FName);
	}

	bool Load();
	void Save();

	// total article counts
	uint64 GetArticleCount() const { return ArticleCount; }
	void AddArticleCount(const uint64& Fq = 1) { ArticleCount += Fq; }
	void SetArticleCount(const uint64& Fq) { ArticleCount = Fq; }

	// total annotated article counts
	uint64 GetAnnArticleCount() const { return AnnArticleCount; }
	void AddAnnArticleCount(const uint64& Fq = 1) { AnnArticleCount += Fq; }
	void SetAnnArticleCount(const uint64& Fq) { AnnArticleCount = Fq; }

	// article counts for a date
	uint64 GetArticleCount(const TStr& Date) { return DateToArticleCountH.IsKey(Date) ? (uint64) DateToArticleCountH.GetDat(Date) : (uint64) 0; }
	void AddArticleCount(const TStr& Date, const uint64& Fq = 1) { DateToArticleCountH.AddDat(Date) += Fq; }
	void SetArticleCount(const TStr& Date, const uint64& Fq) { DateToArticleCountH.AddDat(Date, Fq); }
	void ClearDailyArticleCounts() { DateToArticleCountH.Clr(); }

	// annotated article counts for a date
	uint64 GetAnnArticleCount(const TStr& Date) { return DateToAnnArticleCountH.IsKey(Date) ? (uint64) DateToAnnArticleCountH.GetDat(Date) : (uint64) 0; }
	void AddAnnArticleCount(const TStr& Date, const uint64& Fq = 1) { DateToAnnArticleCountH.AddDat(Date) += Fq; }
	void SetAnnArticleCount(const TStr& Date, const uint64& Fq) { DateToAnnArticleCountH.AddDat(Date, Fq); }
	void ClearDailyAnnArticleCounts() { DateToAnnArticleCountH.Clr(); }

	// counts per source
	uint64 GetSourceFq(const uint64& SourceRecId) { return SourceIdFqH.IsKey(SourceRecId) ? (uint64) SourceIdFqH.GetDat(SourceRecId) : (uint64) 0; }
	void SetSourceFq(const uint64& SourceRecId, const uint64& Fq) { SourceIdFqH.AddDat(SourceRecId, Fq); }
	void AddSourceFq(const uint64& SourceRecId, const uint64& Fq = 1) { SourceIdFqH.AddDat(SourceRecId) += Fq; }
	bool IsSource(const uint64& SourceRecId) { return SourceIdFqH.IsKey(SourceRecId); }

	// stories created on date
	uint64 GetStoryCount(const TStr& Date) { return DateToStoryCountH.IsKey(Date) ? (uint64) DateToStoryCountH.GetDat(Date) : (uint64) 0; }
	void AddStoryCount(const TStr& Date, const uint64& Fq = 1) { DateToStoryCountH.AddDat(Date) += Fq; }
	void SetStoryCount(const TStr& Date, const uint64& Fq) { DateToStoryCountH.AddDat(Date, Fq); }
	void ClearDailyStoryCounts() { DateToStoryCountH.Clr(); }

	// counts per concept
	uint64 GetConceptFq(const uint64& ConceptRecId) { return ConceptIdFqH.IsKey(ConceptRecId) ? (uint64) ConceptIdFqH.GetDat(ConceptRecId) : (uint64) 0; }
	void SetConceptFq(const uint64& ConceptRecId, const uint64& Fq) { ConceptIdFqH.AddDat(ConceptRecId, Fq); }
	void AddConceptFq(const uint64& ConceptRecId, const uint64& Fq = 1) { ConceptIdFqH.AddDat(ConceptRecId) += Fq; }
	void RemoveConcept(const uint64& ConceptRecId) { ConceptIdFqH.DelIfKey(ConceptRecId); }
	bool IsConcept(const uint64& ConceptRecId) { return ConceptIdFqH.IsKey(ConceptRecId); }

	void GetTopConcepts(const int Count, TUInt64IntKdV& ConcIdFqV);
	void GetTopSources(const int Count, TUInt64IntKdV& SourceIdFqV);

	void ClearCounts() {
		ArticleCount = 0;
		AnnArticleCount = 0;
		SourceIdFqH.Clr();
		ConceptIdFqH.Clr();
		DateToArticleCountH.Clr();
		DateToStoryCountH.Clr();
		DateToAnnArticleCountH.Clr();
	}
};


#endif