#ifndef NEWSFIX_H
#define NEWSFIX_H

#include <base.h>
//#include "news.h"
#include "NewsCounts.h"

// forward declaration
class TNewsBaseFix;

class TNewsFixDb;
typedef TPt<TNewsFixDb> PNewsFixDb;
class TNewsFixDb {
private:
	TCRef CRef;

	TStr Path;
	bool EditedConceptData;
	bool EditedSourceData;

	//
	// concept related stuff

	THash<TStr, THash<TStr, TStr> > ConceptUriToInfoHH;
	// concept uri to new uri
	//THash<TStr, TStr> ConceptUriRewriteH;
	// concept uri (without type, e.g. http:\/\/en.wikipedia.org\/wiki\/Associated_Press) to type (e.g. org)
	//THash<TStr, TStr> ConceptUriToTypeH;
	// (concept uri, lang) to label (e.g. (http://wiki.../Associated_press, eng) -> "Associated Press")
	//THash<TStr, THash<TStr, TStr> > ConceptUriToLabelHH;	// uri -> { eng: label1, deu: label2, .. }
	// which concept uris should be ignored
	//THashSet<TStr> ConceptIgnoreH;
	// concept image
	//THash<TStr, TStr> ConceptUriToImageH;

	//
	// source related stuff
	THash<TStr, PJsonVal> SourceUriToJsonH;

	bool LoadConceptInfoBin();

public:
	friend class TPt <TNewsFixDb>;
	friend class TNewsBaseFix;

	TNewsFixDb();
	~TNewsFixDb();

	static PNewsFixDb New(const TStr& _Path, const bool& LoadFixes) {
		PNewsFixDb Fix = new TNewsFixDb();
		Fix->SetDbsPath(_Path);
		if (LoadFixes) {
			Fix->LoadConceptFixes();
			Fix->LoadSourceFixes();
		}
		return Fix;
	}

	// set the path where the fixes should be loaded from and saved to
	void SetDbsPath(const TStr& _Path) { Path = _Path; }
	// load fixes from the disk
	void LoadConceptFixes();
	void LoadSourceFixes();
	// save fixes to the disk
	void SaveConceptFixes();
	void SaveSourceFixes();
	
	// one method that can fix all the concept properties at once. 
	// it can find a new url if the concept has a rewrite rule
	// it fixes the concept type if needed and it's label for language Lang
	void CorrectConceptInfo(TStr& ConceptUri, TStr& ConceptType, TStr& ConceptLabel, const TStr& Lang) const;
	
	bool HasConceptIgnore(const TStr& ConceptUri) const;
	void SetConceptIgnore(const TStr& ConceptUri);
	void RemoveConceptIgnore(const TStr& ConceptUri);

	void SetConceptRewrite(const TStr& FromUri, const TStr& ToUri);
	// replace the url with the correct url
	bool GetConceptRewrite(TStr& Uri) const;
	bool GetConceptRewrite(const TStr& FromUri, TStr& ToUri) const;
	bool HasConceptRewrite(const TStr& Uri) const;
		
	void SetConceptType(const TStr& Uri, const TStr& Type);
	bool HasConceptType(const TStr& Uri) const;
	bool GetConceptType(const TStr& Uri, TStr& Type) const;

	void SetConceptLabel(const TStr& Uri, const TStr& Lang, const TStr& Label);
	bool GetConceptLabel(const TStr& Uri, const TStr& Lang, TStr& Label) const ;
	void RemoveConceptLabel(const TStr& Uri, const TStr& Lang);
	const THash<TStr, TStr> GetConceptLabels(const TStr& Uri) const;

	void SetConceptImage(const TStr& Uri, const TStr& Img);
	bool GetConceptImage(const TStr& Uri, TStr& Img) const;

	//static TStr GetConceptUrl(const TStr& ConceptUri);
	void SetSourceTitle(const TStr& Uri, const TStr& Title);
	void SetSourceDescription(const TStr& Uri, const TStr& Desc);
	void SetSourceEnabled(const TStr& Uri, const bool& Enabled);
	void SetSourceImportance(const TStr& Uri, const int& Importance);
	void SetSourceCity(const TStr& Uri, const TStr& City);
	void SetSourceCountry(const TStr& Uri, const TStr& Country);
	void SetSourceLatLong(const TStr& Uri, const double& Lat, const double& Long);
	void SetSourceTags(const TStr& Uri, const TStrV& TagV);
	void SetSourceImage(const TStr& Uri, const TStr& ImageUrl);
	void SetSourceThumbImage(const TStr& Uri, const TStr& ImageUrl);

	int GetSourceInfos() const { return SourceUriToJsonH.Len(); }
	bool HasSourceInfo(const TStr& Uri) const { return SourceUriToJsonH.IsKey(Uri); }
	PJsonVal GetSourceInfo(const TStr& Uri) const { return SourceUriToJsonH.GetDat(Uri); }
	PJsonVal GetSourceInfo(const int& KeyId) const { return SourceUriToJsonH.IsKeyId(KeyId) ? SourceUriToJsonH[KeyId] : NULL; }

private:
	// get source json from SourceUriToJsonH. If it doesn't exist create new json, set it's uri value, set it to SourceUriToJsonH and return it
	PJsonVal GetSourceJson(const TStr& Uri);
};

#endif