#ifndef NEWS_LOAD_H
#define NEWS_LOAD_H

//#include <qminer_srv.h>
#include <net.h>
#include <logger.h>
#include "../../src/third_party/libuv/include/uv.h"
//#include <thread.h>
//#include "news.h"
#include "NewsArticle.h"
#include "NewsConst.h"
#include "enrycherdoc.h"
#include "enrycherutil.h"
//#include <webpgfetchpersist.h>
#include "../Utils/DateExtractor.h"

///////////////////////////////
// News-Notifier
class TNewsNotify {
public:
	static PNotify Notify;
};

enum TTaskStatus { waitingForData, readyToParse, outputReady };
class TNewsProc;
class TNewsProcFilter;
typedef TPt<TNewsProcFilter> PNewsProcFilter;

class TTask {
private:
	uv_mutex_t StatusMutex;
	uv_mutex_t DataMutex;
	uv_mutex_t BufferMutex;
	TTaskStatus Status;
	TStr FileName;
	uint64 RequestId;
	TMem Buffer;

public:
	TNewsProc* NewsProc;

	TTask() {};
	TTask(TNewsProc* NewsProc);
	TTaskStatus GetStatus();
	void SetStatus(const TTaskStatus& _Status);

	void SetNewTaskData(const TStr& _FileName, const uint64& _RequestId);
	void GetTaskData(TStr& _FileName, uint64& _RequestId);

	void SetBuffer(const TMem& _Buffer);
	void LockBufferMutex();
	TMem GetBuffer();
	void UnlockBufferMutex();
};

///////////////////////////////
// News-Processing
enum TNewsProcSource { undefined, NewsFeed, NewsFeedEnrycher, NewsFeedXLike };
enum TNewsProcMode { online, parse, onlineBinary };

class TNewsProc {
private:
	TStr DbsPath;
	TStr AddArticleUrl;
	TStr RedirectDynamicFNm;

	uv_rwlock_t HashLock;
	TStrHash<TStr> WikiStaticRedirectH;
	TStrHash<TStr> WikiDynamicRedirectH;
	THash<TStr, TStr> WikiArticleLocationMappingH;

	TStr GetWikiRedirect(const TStr& InStr);
	void SetWikiRedirect(const TStr& InStr, const TStr& OutStr);
	bool HasWikiRedirect(const TStr& InStr);
	void SaveWikiRedirectFile();
	TStr GetClearTextBody(const PXmlTok& ArticleTok);

	TStrHash<TStr> EntityBlackListH;
	bool IsEntityBlackList(const TStr& EntityStr);

	TStrHash<TStr> LocationH;
	// return true if the EntityStr is a known location
	bool IsLocationUri(const TStr& EntityStr);
	PDateExtractor DateExtractor;

	// dmoz categories
	TStrIntPrV CatPrefixV;
	TStrV CatIgnoreV;

	PLogger InfoLogFile;
	PNewsProcFilter Filter;

	TNewsProcSource Source;
		
	TVec<uv_thread_t> ThreadV;
	TVec<TTask* > TaskV;
	
	StringPiece Matches[10];
	RE2 ReStartsWithLoc;

public:
	bool IgnoreIfNoAnnotations;
	int  SleepSec;
	int SleepBinaryMSec;
	int SkipLast;

	TNewsProc(const TStr& DbsPath, const PJsonVal& NewsProcSett, const PNewsProcFilter& Filter, const TStr& AddArticleUrl, const TNewsProcSource& _Source);
	
	void LoadDbs();
	void LoadCityWikiFile();

	void SetDateExtractor(const PDateExtractor& _DateExtractor) { DateExtractor = _DateExtractor; }
	void SetDMozCategories(const TStrIntPrV& PrefixV, const TStrV& IgnoreV) { CatPrefixV = TStrIntPrV(PrefixV); CatIgnoreV = TStrV(IgnoreV); }
	
	bool IsIgnoredCategory(const TStr& CategoryNm);
	TStr GetValidCategoryDepth(const TStr& CategoryNm);

	// parse enrycher document
	void ParseEnrycherDoc(TNewsArticle& Article, const PEnrycherDoc& Doc);
	void ParseXLikeDoc(TNewsArticle& Article, const PXmlTok& Doc);
	
	// newsfeed -- parse
	int ParseNewsFeedFile(const TStr& InXmlFNm, TMem& OutBuffer);
		
	// send memory buffer to the IndexForwardUrlStr url
	void SendBuffer(const TMem& Buffer);
	
	// watch folder for new files and parse them
	void ParseOnline(const TNewsProcSource& Source, const TStr& InXmlFPath, const PTokenizer& FtrTok, const TStrV& IndexForwardUrlStrV, const TStr& OutXmlFPath, const int& SleepSec, const TStr& FtrTokFNm);

	void ExtractArticleDates(TNewsArticle& Article, const PEnrycherDoc& Doc);

	// process articles and send them to the server
	void StartOnline(const TStr& InXmlFPath, const TStr& OutXmlFPath);
	// process articles and only save the binary files
	void StartParsing(const TStr& InXmlFPath, const TStr& OutXmlFPath, const TStr& OutBinFPath);
	// load binary files and send them to the server
	void StartOnlineBinary(const TStr& InBinFPath, const TStr& OutBinFPath);

	static void ProcessFile(void *arg);

	bool EnumerateFiles(const TStr FPath, const TStrV& FileExV, TVec<TPair<TUInt, TStr> >& FTmFNmV);
};


///////////////////////////////
// News-Processing-Filter
ClassTP(TNewsProcFilter, PNewsProcFilter)//{
protected:
	TStrSet SourceTypeSet;
	TStrSet LangSet;
	TStrSet DomainSet;

	TStrSet IgnoreDomainSet;
	TStrSet IgnoreSourceTypeSet;
	TStrSet IgnoreLangSet;

	TStr GetDomain(const TStr& UrlStr) const;

	TNewsProcFilter();
	TNewsProcFilter(const PNewsProcFilter& Filter);
	TNewsProcFilter(const TStrSet& _SourceTypeSet, const TStrSet& _LangSet, const TStrSet& _DomainSet,
		const TStrSet& _IgnoreSourceTypeSet, const TStrSet& _IgnoreLangSet, const TStrSet& _IgnoreDomainSet);
	TNewsProcFilter(const TStr& SourceTypeFN, const TStr& LangFN, const TStr& DomainFN,
		const TStr& IgnoreSourceTypeFN, const TStr& IgnoreLangFN, const TStr& IgnoreDomainFN);

public:
	TInt MinBodyLen;

	static PNewsProcFilter New() { return new TNewsProcFilter(); }
	static PNewsProcFilter New(const PNewsProcFilter& Filter) { return new TNewsProcFilter(Filter); }

	static PNewsProcFilter New(const TStrSet& SourceTypeSet, const TStrSet& LangSet, const TStrSet& DomainSet, const TStrSet& IgnoreSourceTypeSet,
		const TStrSet& IgnoreLangSet, const TStrSet& IgnoreDomainSet) {
		return new TNewsProcFilter(SourceTypeSet, LangSet, DomainSet, IgnoreSourceTypeSet, IgnoreLangSet, IgnoreDomainSet);
	}

	static TStrSet ReadFile(const TStr& FileName);

	void SetSourceTypes(const TStrSet& _SourceTypeSet) { SourceTypeSet = TStrSet(_SourceTypeSet); }
	void SetLangs(const TStrSet& _LangSet) { LangSet = TStrSet(_LangSet); }
	void SetDomains(const TStrSet& _DomainSet) { DomainSet = TStrSet(_DomainSet); }

	void SetIgnoreSourceTypes(const TStrSet& _SourceTypeSet) { IgnoreSourceTypeSet = TStrSet(_SourceTypeSet); }
	void SetIgnoreLangs(const TStrSet& _LangSet) { IgnoreLangSet = TStrSet(_LangSet); }
	void SetIgnoreDomains(const TStrSet& _DomainSet) { IgnoreDomainSet = TStrSet(_DomainSet); }

	virtual bool Pass(const TNewsArticle& Article) const;
	virtual bool Ignore(const TNewsArticle& Article) const;
};

#endif