#ifndef NEWSSEARCH_H
#define NEWSSEARCH_H

#include "NewsBase.h"
#include "CategoryBase.h"
#include "ConceptFolderBase.h"
#include <qminer_srv.h>

class TEventQuery {
public:
	TStr Keywords;
	TStrV ConceptUriV;
	
	TStr DateStart;
	TStr DateEnd;
	TStrV LangV;
	TStrV IgnoreLangV;
	TStrV CategoryV;
	bool CategoryIncludeSub;
	TIntPr MinMaxArtsInEvent;
	TStrV LocationUriV;
	TStrV PublisherUriV;
	TStrV DateMentionV;

	TStr IgnoreKeywords;
	TStrV IgnoreConceptUriV;
	TStrV IgnoreCategoryV;
	bool IgnoreCategoryIncludeSub;
	TStrV IgnoreLocationUriV;
	TStrV IgnorePublisherUriV;

	void SetDateStart(const TTm& Tm) {
		DateStart = Tm.GetWebLogDateStr();
	}
	void SetDateEnd(const TTm& Tm) {
		DateEnd = Tm.GetWebLogDateStr();
	}

	// return the time in the format that can be used when querying for events
	static TStr GetQueryDate(const TTm& Tm) {
		return Tm.GetWebLogDateStr();
	}

	TEventQuery() { MinMaxArtsInEvent.Val1 = 0; MinMaxArtsInEvent.Val2 = TInt::Mx; }
};

class TArticleQuery {
public:
	TStr Keywords;			// results should have all specified keywords
	TStrV ConceptUriV;		// results should have all specified concepts
	TStr DateStart;			// results should have date >= DateStart
	TStr DateEnd;			// results should have date <= DateEnd
	TStrV LangV;			// results can be in any of the specified languages
	TStrV PublisherUriV;	// results can be by any of the specified publishers
	TStrV CategoryV;			// results can be about any of the specified categories
	bool CategoryIncludeSub;	// consider also subcategories as valid categories
	TStrV LocationUriV;			// results can be from any of the specified locations
	TStrV DateMentionV;			// results can mention any of the specified dates

	TStr IgnoreKeywords;
	TStrV IgnoreConceptUriV;
	TStrV IgnoreLangV;
	TStrV IgnoreLocationUriV;
	TStrV IgnorePublisherUriV;
	TStrV IgnoreCategoryV;
	bool IgnoreCategoryIncludeSub;

	void SetDateStart(const TTm& Tm) {
		DateStart = Tm.GetWebLogDateStr();
	}
	void SetDateEnd(const TTm& Tm) {
		DateEnd = Tm.GetWebLogDateStr();
	}

	// return the time in the format that can be used when querying for articles
	static TStr GetQueryDate(const TTm& Tm) {
		return Tm.GetWebLogDateStr();
	}

	TArticleQuery() {  }
};



class TNewsSearch;
typedef TPt<TNewsSearch> PNewsSearch;
class TNewsSearch {
public:
	friend class TPt<TNewsSearch>;

private:
	PBase Base;
	PIndex Index;
	PIndexVoc IndexVoc;
	PStore ArticleStore;
	PStore ConceptStore;
	PStore CategoryStore;
	PStore SourceStore;
	PStore StoryStore;
	PStore EventStore;
	uchar ArticleStoreId;
	uchar ConceptStoreId;
	uchar CategoryStoreId;
	uchar SourceStoreId;
	uchar StoryStoreId;
	uchar EventStoreId;

	TCRef CRef;
	TNewsSearch(const PNewsBase& _NewsBase, const PConceptFolderBase& _ConceptFolderBase) :
		NewsBase(_NewsBase), ConceptFolderBase(_ConceptFolderBase)
	{
		Base = NewsBase->Base;
		Index = NewsBase->Index;
		IndexVoc = NewsBase->IndexVoc;
		ArticleStore = NewsBase->ArticleStore;
		ConceptStore = NewsBase->ConceptStore;
		CategoryStore = NewsBase->CategoryStore;
		SourceStore = NewsBase->SourceStore;
		StoryStore = NewsBase->StoryStore;
		EventStore = NewsBase->EventStore;
		ArticleStoreId = ArticleStore->GetStoreId();
		ConceptStoreId = ConceptStore->GetStoreId();
		CategoryStoreId = CategoryStore->GetStoreId();
		SourceStoreId = SourceStore->GetStoreId();
		StoryStoreId = StoryStore->GetStoreId();
		EventStoreId = EventStore->GetStoreId();
	}

public:
	PNewsBase NewsBase;
	
	PConceptFolderBase ConceptFolderBase;

	static PNewsSearch New(const PNewsBase& NewsBase, const PConceptFolderBase& ConceptFolderBase) {
		return new TNewsSearch(NewsBase, ConceptFolderBase);
	}

	bool SearchEvents(const TEventQuery& Query, PRecSet& RecSet, PJsonVal& ErrorJson) const;
	bool SearchArticles(const TArticleQuery& Query, PRecSet& RecSet, PJsonVal& ErrorJson) const;

	bool FindArticlesForKeywords(const TStr& Keywords, PRecSet& RecSet, TStr& Error) const;
	bool FindArticlesForConcept(const TStr& ConceptUri, PRecSet& RecSet, TStr& Error) const;
	bool FindArticlesForConcepts(const TStrV& ConceptUriV, PRecSet& RecSet, TStr& Error) const;
	bool FindArticlesForLocations(const TStrV& PublisherUriV, PRecSet& RecSet, TStr& Error) const;
	bool FindArticlesForPublisher(const TStr& PublisherUri, PRecSet& RecSet, TStr& Error) const;
	bool FindArticlesForPublishers(const TStrV& PublisherUriV, PRecSet& RecSet, TStr& Error) const;
	bool FindArticlesForDates(const TStr& StartDate, const TStr& EndDate, PRecSet& RecSet, TStr& Error) const;
	bool FindArticlesForLangs(const TStrV& LangV, PRecSet& RecSet, TStr& Error) const;
	bool FindArticlesForCategories(const TStrV& CategoryUriV, const bool& IncludeSubCategories, PRecSet& RecSet, TStr& Error) const;
	bool FindArticlesForDateMentions(const TStrV& DateMentionV, PRecSet& RecSet, TStr& Error) const;

	bool FindEventsForKeywords(const TStr& Keywords, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForConcept(const TStr& ConceptUri, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForConcepts(const TStrV& ConceptUriV, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForDates(const TStr& StartDate, const TStr& EndDate, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForLangs(const TStrV& LangV, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForLocations(const TStrV& LocationUriV, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForPublisher(const TStr& PublisherUri, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForPublishers(const TStrV& PublisherUriV, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForCategories(const TStrV& CategoryUriV, const bool& IncludeSubCategories, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForDateMentions(const TStrV& DateMentionV, PRecSet& RecSet, TStr& Error) const;
	bool FindEventsForCoverage(const int& MinArtCount, const int& MaxArtCount, PRecSet& RecSet, TStr& Error) const;

	bool FindStoriesForKeywords(const TStr& Keywords, PRecSet& RecSet, TStr& Error) const;
	bool FindStoriesForConcept(const TStr& ConceptUri, PRecSet& RecSet, TStr& Error) const;
	bool FindStoriesForConcepts(const TStrV& ConceptUriV, PRecSet& RecSet, TStr& Error) const;
	bool FindStoriesForDates(const TStr& StartDate, const TStr& EndDate, PRecSet& RecSet, TStr& Error) const;
	
	TStrV ParseDateMentions(const TStrV& DateMentionV, const TStr& DateMentionStart = "", const TStr& DateMentionEnd = "") const;
private:
	PRecSet SetEventScoresBasedOnStoryQuality(const PRecSet& RecSet) const;
	PRecSet SetStoryScoresBasedOnStoryQuality(const PRecSet& RecSet) const;
};

#endif