#ifndef NEWSSRVUTIL_H
#define NEWSSRVUTIL_H

#include "NewsBase.h"
#include "ConceptFolder.h"
#include "NewsConst.h"

enum EnumArticleInfoFlags : int { aifBasicInfo = 1 << 0, aifIncludeBody = 1 << 1, aifIncludeTitle = 1 << 2, aifIncludeConcepts = 1 << 3, aifIncludeSourceInfo = 1 << 4, aifIncludeStoryUri = 1 << 5, aifIncludeEventUri = 1 << 6, aifIncludeDuplicateList = 1 << 7, aifIncludeOriginalArticleInfo = 1 << 8, aifIncludeCategories = 1 << 9, aifIncludeLocation = 1 << 10, aifIncludeImage = 1 << 11, aifIncludeExtractedDates = 1 << 12, aifIncludeNewsfeedId = 1 << 13 };
enum EnumStoryInfoFlags : int { sifIncludeConcepts = 1 << 0, sifIncludeBasicStats = 1 << 1, sifIncludeTitle = 1 << 2, sifIncludeSummary = 1 << 3, sifIncludeLocation = 1 << 4, sifIncludeCategory = 1 << 5, sifIncludeMedoidArticle = 1 << 6, sifIncludeStoryDate = 1 << 7, sifIncludeExtractedDates = 1 << 8 };
enum EnumEventInfoFlags : int { eifIncludeConcepts = 1 << 0, eifIncludeStories = 1 << 1, eifIncludeMultiLingInfo = 1 << 2, eifIncludeCategories = 1 << 3, eifIncludeLocation = 1 << 4, eifIncludeImages = 1 << 5, eifIncludeArticleCounts = 1 << 6 };
enum EnumPublisherInfoFlags : int { pifIncludeTitle = 1 << 0, pifIncludeDescription = 1 << 1, pifIncludeTags = 1 << 2, pifIncludeLocation = 1 << 3, pifIncludeImportance = 1 << 4, pifIncludeDetails = 1 << 5, pifIncludeArticleCount = 1 << 6 };
enum EnumConceptInfoFlags : int { picIncludePrimImage = 1 << 0, picIncludeAllImages = 1 << 1, picIncludeDescription = 1 << 2, picIncludeTrends = 1 << 3  };

extern EnumArticleInfoFlags DefArticleFlags;
extern EnumStoryInfoFlags DefStoryFlags;
extern EnumEventInfoFlags DefEventFlags;
extern EnumPublisherInfoFlags DefPublisherFlags;
extern EnumConceptInfoFlags DefConceptFlags;
extern EnumConceptInfoFlags ConceptFlagsFull;

enum EnumTimeWindow { twTotal, twMonth, twDay, twHour, twUndef };

class TNewsSrvUtil;
typedef TPt<TNewsSrvUtil> PNewsSrvUtil;
class TNewsSrvUtil {
private:
	TCRef CRef;
public:
	friend class TPt<TNewsSrvUtil>;
	PNewsBase NewsBase;
	PGeoBase GeoBase;

	PStore ArticleStore;
	PStore ConceptStore;
	PStore ConceptClassStore;
	PStore ConceptFolderStore;
	PStore UserStore;
	PStore CategoryStore;
	PStore StoryStore;
	PStore SourceStore;
	PStore EventStore;
	PStore ImageStore;
	PStore TagStore;
	PBase Base;

	uchar ArticleStoreId;
	uchar ConceptStoreId;
	uchar ConceptClassStoreId;
	uchar ConceptFolderStoreId;
	uchar UserStoreId;
	uchar CategoryStoreId;
	uchar StoryStoreId;
	uchar SourceStoreId;
	uchar EventStoreId;
	uchar ImageStoreId;
	uchar TagStoreId;
	
public:
	
	TNewsSrvUtil(const PNewsBase& _NewsBase);
	static PNewsSrvUtil New(const PNewsBase& NewsBase) { return new TNewsSrvUtil(NewsBase); }

	static EnumTimeWindow GetTimeWindow(const TStr& Text);
	TStr GetKeyForTimeWindow(const uint64& ArticleId, const EnumTimeWindow& TimeWindow) const;

	PJsonVal Article(const uint64& ArticleId, const int& MaxBodyLen = -1, const EnumConceptType& ConceptTypes = ctNone, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumArticleInfoFlags& ArticleFlags = DefArticleFlags) const;
	PJsonVal Concept(const TUInt64& ConceptId, const int ConceptFreq = -1, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumConceptInfoFlags ConceptFlags = DefConceptFlags) const;
	PJsonVal Event(const uint64& EventRecId, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumConceptType& ConceptTypes = ctAll, const EnumEventInfoFlags& EventFlags = DefEventFlags, const EnumStoryInfoFlags& StoryFlags = DefStoryFlags, const int& EventImageCount = 3, const int& StoryImageCount = 3) const;
	PJsonVal Story(const uint64& StoryRecId, const int& Score, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumConceptType& ConceptTypes = ctAll, const EnumStoryInfoFlags& StoryFlags = DefStoryFlags, const int& StoryImageCount = 3) const;
	PJsonVal Source(const uint64& SourceRecId, const EnumPublisherInfoFlags& SourceFlags = DefPublisherFlags) const;
	PJsonVal ConceptFolder(const TStr& ConceptFolderUri, const bool& IncludeDetails = true, const bool& IncludeConceptInfo = true, const TStrV& LabelLanguageV = NewsConst.LangCoreV) const {
		uint64 ConceptFolderId = ConceptFolderStore->IsRecNm(ConceptFolderUri) ? ConceptFolderStore->GetRecId(ConceptFolderUri) : TUInt64::Mx.Val;
		return ConceptFolder(ConceptFolderId, IncludeDetails, IncludeConceptInfo, LabelLanguageV);
	}
	PJsonVal ConceptFolder(const uint64& ConceptFolderId, const bool& IncludeDetails = true, const bool& IncludeConceptInfo = true, const TStrV& LabelLanguageV = NewsConst.LangCoreV) const;
	PJsonVal User(const uint64& UserId) const;
	PJsonVal Folder(const uint64 FolderRecId) const;
	PJsonVal Location(const uint64& ConceptRecId, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const bool& IncludeUri = true, const bool& IncludeLocation = true, const bool& IncludeDetails = false) const;
	PJsonVal Category(const uint64& CategoryRecId) const;
	PJsonVal Tag(const uint64& TagRecId) const;

	PJsonVal Articles(const PRecSet& ArticleSet, const int& MaxBodyLen = DefaultArticleBodyLen, const TUInt64Set& IgnoreArticleSet = TUInt64Set(), const bool& IncludeWgt = false, const EnumConceptType& IncludeConceptTypes = ctNone, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumArticleInfoFlags& ArticleFlags = DefArticleFlags) const;
	PJsonVal Concepts(const PRecSet& ConceptSet, const TUInt64Set& IgnoreConceptSet = TUInt64Set(), const int MaxCountPerType = TInt::Mx, const EnumConceptType& IncludeConceptTypes = ctAll, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumConceptInfoFlags ConceptFlags = DefConceptFlags) const;
	PJsonVal Concepts(const TUInt64V& ConceptV, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumConceptInfoFlags ConceptFlags = DefConceptFlags) const;
	PJsonVal FolderConceptsAndKeywords(const PRecSet& ConceptAndKeywordSet) const;
	PJsonVal FolderKeywords(const PRecSet& ConceptSet) const;
	PJsonVal DailyTopics(const PRecSet& ArticleSet, const TUInt64Set& IgnoreConceptSet = TUInt64Set(), const TStrV& LabelLanguageV = NewsConst.LangCoreV, const int& Page = 0, const int& Count = 10) const;
	
	PJsonVal Events(const PRecSet& EventRecSet, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumConceptType& ConceptTypes = ctAll, const EnumEventInfoFlags& EventFlags = DefEventFlags, const EnumStoryInfoFlags& StoryFlags = DefStoryFlags, const int& EventImageCount = 3, const int& StoryImageCount = 3) const;
	PJsonVal Stories(const PRecSet& StoryRecSet, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumConceptType& ConceptTypes = ctAll, const EnumStoryInfoFlags& StoryFlags = DefStoryFlags, const int& StoryImageCount = 3) const;
	PJsonVal Sources(const PRecSet& SourceSet, const EnumPublisherInfoFlags& SourceFlags = DefPublisherFlags) const;
	PJsonVal Folders(const PRecSet& FolderSet) const;
	PJsonVal Categories(const PRecSet& CategorySet, const int& MaxCount = TInt::Mx) const;
	PJsonVal Tags(const PRecSet& TagSet, const bool& AddWgts = false) const;

	PJsonVal ConceptFolders(const PRecSet& FolderSet, const bool& IncludeDetails = true, const bool& IncludeConceptInfo = true, const TStrV& LabelLanguageV = NewsConst.LangCoreV) const;
	PJsonVal Users(const PRecSet& UserSet) const;

	PJsonVal ArticleSourceAggr(const TStrKdV& FldNmValPrV, const PRecSet& ArticleSet, const TStr& FldNmPrefix = "sourceAggr") const;
	PJsonVal ArticleDateMentionAggr(const PRecSet& ArticleSet) const;
	PJsonVal EventsRecentActivity(const PRecSet& EventRecSet, const int& MaxCount = 100, const TTm& OnlyUpdatesAfterTm = TTm::GetTmFromMSecs(0), const uint64& LastActivityId = 0, const TStr& Lang = "eng", const bool& EventsWithLocationOnly = true, const double& MinAvgCosSim = 0.0, const bool& EventsWithLangOnly = false) const;

	PJsonVal ConceptFolderFull(const uint64& ConceptRecId, const bool& AddLabelsForConcepts = false, const TStrV& LabelLanguageV = NewsConst.LangCoreV) const;

	PJsonVal ArticleUris(const PRecSet& ArticleRecSet) const;
	PJsonVal ArticleIds(const PRecSet& ArticleRecSet) const;
	PJsonVal ArticleUris(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet, const TStr& FldNmPrefix = "articleUris") const;
	PJsonVal ArticleIds(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet, const TStr& FldNmPrefix = "articleIds") const;
	PJsonVal Articles(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet, const TStr& FldNmPrefix = "articles") const;
	PJsonVal RelatedConcepts(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet, const TUInt64Set& IgnoreConceptSet = TUInt64Set(), const TStr& FldNmPrefix = "relatedConcepts") const;
	PJsonVal ArticleRecentActivity(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet) const;
	PJsonVal DailyTopics(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet, const TUInt64Set& IgnoreConceptSet = TUInt64Set()) const;

	PJsonVal Events(const TStrKdV& FldNmValPrV, const PRecSet& EventRecSet, const TStr& FldNmPrefix = "events");
	PJsonVal EventsRecentActivity(const TStrKdV& FldNmValPrV, const PRecSet& EventRecSet) const;
	
	EnumArticleInfoFlags GetArticleFlags(const TStrKdV& FldNmValPrV, const TStr& FldNmPrefix = "article", const EnumArticleInfoFlags& DefFlags = DefArticleFlags) const;
	EnumStoryInfoFlags GetStoryFlags(const TStrKdV& FldNmValPrV, const TStr& FldNmPrefix = "story", const EnumStoryInfoFlags& DefFlags = DefStoryFlags) const;
	EnumEventInfoFlags GetEventFlags(const TStrKdV& FldNmValPrV, const TStr& FldNmPrefix = "event", const EnumEventInfoFlags& DefFlags = DefEventFlags) const;
	EnumConceptInfoFlags GetConceptFlags(const TStrKdV& FldNmValPrV, const TStr& FldNmPrefix = "concept", const EnumConceptInfoFlags& DefFlags = DefConceptFlags) const;
	

	int GetStoryArticleCount(const uint64& StoryId) const;
	int GetEventArticleCount(const uint64& EventId) const;

	uint64 GetValidEventId(const uint64& EventId) const;
	void GetConceptCountsByType(const PRecSet& ConceptRecSet, int& Persons, int& Locations, int& Organizations, int& Wikis) const;
};

#endif