#ifndef ENRYCHERDOC_H
#define ENRYCHERDOC_H

#include <base.h>

///////////////////////////////
// Forward-Declarations
ClassHdTP(TEnrycherDoc, PEnrycherDoc);

///////////////////////////////
// Enrycher-Token
ClassTV(TEnrycherToken, TEnrycherTokenV)//{
private:
	TInt TokenId;
	TStr TokenStr;
	TStrStrH KeyValH;

private:
	// required by vector
	TEnrycherToken(): TokenId(-1) { }
	// function exposing members to TEnrycherDoc
	void SetId(int _TokenId) { TokenId = _TokenId; }

public:
	// for creation of new tokens, id is assigned by TEnrycherDoc
	TEnrycherToken(const TStr& _TokenStr):
	  TokenId(-1), TokenStr(_TokenStr) { }

	// deserilization from XML
	TEnrycherToken(const PXmlTok& TokenTok, int SentId);
	// serilization to XML
	PXmlTok GetXmlTok(int SentId) const;

	// elements
	bool Empty() const { return TokenStr.Empty(); }
	int GetId() const { return TokenId; }
	const TStr& GetStr() const { return TokenStr; }
	// key-values (e.g. POS tags)
	bool IsKey(const TStr& Key) const { return KeyValH.IsKey(Key); }
	const TStr& GetKeyVal(const TStr& Key) const { return KeyValH.GetDat(Key); }
	void AddKeyVal(const TStr& Key, const TStr& Val) { KeyValH.AddDat(Key, Val); }

	friend class TVec<TEnrycherToken>;
	friend class TEnrycherDoc;
};

///////////////////////////////
// Enrycher-Sentence
ClassTV(TEnrycherSent, TEnrycherSentV)//{
private:
    TInt SentId;
    TStr SentStr;
    TEnrycherTokenV TokenV;

private:
	// required by vector
    TEnrycherSent(): SentId(-1) { }
	// function exposing members to TEnrycherDoc
	void SetId(int _SentId) { SentId = _SentId; }
	TEnrycherToken& GetToken(int TokenN) { return TokenV[TokenN]; }

public:
	// for creation of new tokens, id is assigned by TEnrycherDoc
    TEnrycherSent(const TStr& _SentStr):
	  SentId(-1), SentStr(_SentStr) { }

	// deserilization from XML
    TEnrycherSent(const PXmlTok& SentTok);
    // serilization to XML
    PXmlTok GetXmlTok() const;

    // sentence level
    bool Empty() const { return SentStr.Empty(); }
    int GetId() const { return SentId; }
    const TStr& GetStr() const { return SentStr; }
    // token level
    bool IsTokenized() const { return !TokenV.Empty(); }
    int GetTokens() const { return TokenV.Len(); }
	const TEnrycherToken& GetToken(int TokenN) const { return TokenV[TokenN]; }
	void AddToken(const TEnrycherToken& Token) { TokenV.Add(Token); }

    friend class TVec<TEnrycherSent>;
	friend class TEnrycherDoc;
};

///////////////////////////////
// Enrycher-Paragraph
ClassTV(TEnrycherPara, TEnrycherParaV)//{
private:
    TStr TagNm;
    TEnrycherSentV SentV;

private:
	// required by vector
    TEnrycherPara() { }
	// function exposing members to TEnrycherDoc
    TEnrycherSent& GetSent(int SentN) { return SentV[SentN]; }

public:
	// for creation of new tokens, id is assigned by TEnrycherDoc
    TEnrycherPara(const TStr& _TagNm): TagNm(_TagNm) { }

	// deserilization from XML
    TEnrycherPara(const PXmlTok& ParaTok);
    // serilization to XML
    PXmlTok GetXmlTok() const;

    // paragraph
	bool Empty() const { return SentV.Empty() || (SentV.Len() == 1 && SentV[0].Empty()); }
    const TStr& GetTagNm() const { return TagNm; }
	TStr GetStr() const;
    // sentences
    int GetSents() const { return SentV.Len(); }
    const TEnrycherSent& GetSent(int SentN) const { return SentV[SentN]; }
	void AddSent(const TEnrycherSent& Sent) { SentV.Add(Sent); }

    friend class TVec<TEnrycherPara>;
	friend class TEnrycherDoc;
};

///////////////////////////////
// Enrycher-Annotation-Instance
ClassTV(TEnrycherAnnInst, TEnrycherAnnInstV)//{
private:
    TInt AnnInstId;
    TStr AnnInstStr;
    TIntPrV SentIdTokenIdV;

private:
	// required by vector
    TEnrycherAnnInst() { }

	// function exposing members to TEnrycherDoc
	void SetId(int _AnnInstId) { AnnInstId = _AnnInstId; }

public:

	// for creation of new annotation instances, id is assigned by TEnrycherDoc
    TEnrycherAnnInst(const TStr& _AnnInstStr, const TIntPrV& _SentIdTokenIdV):
	  AnnInstId(-1), AnnInstStr(_AnnInstStr), SentIdTokenIdV(_SentIdTokenIdV) { }

	// deserilization from XML
    TEnrycherAnnInst(const PXmlTok& AnnInstTok);
    // serilization
    PXmlTok GetXmlTok() const;

	// equvivalence operator
	bool operator==(const TEnrycherAnnInst& AnnInst) const {
		return SentIdTokenIdV == AnnInst.SentIdTokenIdV; }

    // instance
    int GetId() const { return AnnInstId; }
    const TStr& GetStr() const { return AnnInstStr; }
	// tokens
	int GetTokens() const { return SentIdTokenIdV.Len(); }
	int GetSentId(int TokenN) const { return SentIdTokenIdV[TokenN].Val1; }
	int GetTokenId(int TokenN) const { return SentIdTokenIdV[TokenN].Val2; }

    friend class TVec<TEnrycherAnnInst>;
	friend class TEnrycherDoc;
};

///////////////////////////////
// Enrycher-Semantic-Attribute
ClassTV(TEnrycherSemAttr, TEnrycherSemAttrV)//{
private:
	TStr AboutStr;
	TStr TypeStr;
	TStr ResourceStr;
	TStr LiteralStr;
	TStr DisplayNameStr;

private:
	// required by vector
	TEnrycherSemAttr() { }

public:
    TEnrycherSemAttr(const TStr& _AboutStr, const TStr& _TypeStr, const TStr& _ResourceStr, const TStr& _LiteralStr, const TStr& _DisplayNameStr):
	  AboutStr(_AboutStr), TypeStr(_TypeStr), ResourceStr(_ResourceStr), LiteralStr(_LiteralStr), DisplayNameStr(_DisplayNameStr) {
	  if (TypeStr.Empty() && (ResourceStr.Empty() || LiteralStr.Empty())) TExcept::Throw("Not a valid Enrycher Semantic Attribute");
	}

	// deserilization from XML
	TEnrycherSemAttr(const PXmlTok& SemAttrTok);
    // serilization to XML
    PXmlTok GetXmlTok() const;

	// elements
	const TStr& GetType() const { return TypeStr; }
	bool IsResource() const { return !ResourceStr.Empty(); }
	const TStr& GetResource() const { return ResourceStr; }
	bool IsLiteral() const { return !LiteralStr.Empty(); }
	const TStr& GetLiteral() const { return LiteralStr; }
	bool IsDisplayName() const { return !DisplayNameStr.Empty(); }
	const TStr& GetDisplayName() const { return DisplayNameStr; }
	bool IsAbout() const { return !AboutStr.Empty(); }
	const TStr& GetAbout() const { return AboutStr; }
	friend class TVec<TEnrycherSemAttr>;
};

///////////////////////////////
// Enrycher-Annotation
ClassTV(TEnrycherAnn, TEnrycherAnnV)//{
private:
    // meta-data
    TInt AnnId;
    TStr AnnStr;
    TStr AnnType;
    // instances
    TEnrycherAnnInstV AnnInstV;
    // semantics
	TEnrycherSemAttrV SemAttrV;

private:
	// required by vector
    TEnrycherAnn() { }

	// functions exposing members to TEnrycherDoc
	void SetId(int _AnnId) { AnnId = _AnnId; }
	TEnrycherAnnInst& GetAnnInst(int AnnInstN) { return AnnInstV[AnnInstN]; }
	int AddAnnInst(const TEnrycherAnnInst& AnnInst) { int AnnInstN = AnnInstV.Add(AnnInst); return AnnInstN; }
	int AddSemAttr(const TEnrycherSemAttr& SemAttr) { int SemAttrN = SemAttrV.Add(SemAttr); return SemAttrN; }

public:
	// for creation of new annotations, id is assigned by TEnrycherDoc
    TEnrycherAnn(const TStr& _AnnStr, const TStr& _AnnType):
	  AnnId(-1), AnnStr(_AnnStr), AnnType(_AnnType) { }

	// deserilization from XML
    TEnrycherAnn(const PXmlTok& AnnTok);
    // serilization to XML
    PXmlTok GetXmlTok() const;

	bool Empty() const { return AnnStr.Empty(); }
    // meta-data
    int GetId() const { return AnnId; }
    const TStr& GetStr() const { return AnnStr; }
    const TStr& GetType() const { return AnnType; }
    // instances
    int GetAnnInsts() const { return AnnInstV.Len(); }
    const TEnrycherAnnInst& GetAnnInst(int AnnInstN) const { return AnnInstV[AnnInstN]; }
	// semantics
	int GetSemAttrs() const { return SemAttrV.Len(); }
	const TEnrycherSemAttr& GetSemAttr(int SemAttrN) const { return SemAttrV[SemAttrN]; }
	
    friend class TVec<TEnrycherAnn>;
    friend class THashKeyDat<TInt, TEnrycherAnn>;
	friend class TEnrycherDoc;
};

///////////////////////////////
// Enrycher-Assertion
ClassTV(TEnrycherAssert, TEnrycherAssertV)//{
private:
	// assertion level
	TInt AssertId;
	// subject (assertion, instance)
	TIntPr SubjAnnId;
	TIntPrV SubjModAnnIdV;
	// verb (assertion, instance)
	TIntPr VerbAnnId;
	TIntPrV VerbModAnnIdV;
	// object (assertion, instance)
	TIntPr ObjAnnId;
	TIntPrV ObjModAnnIdV;

private:
	// [de]serialization helping functions
	TIntPr ParseAnnTok(const PXmlTok& Tok, const TStr& TagNm) const;
	void ParseAnnTokV(const PXmlTok& Tok, const TStr& TagNm,TIntPrV& AnnIdV) const;
	PXmlTok GetAnnTok(const TIntPr& AnnId, const TStr& TagNm,
		const TEnrycherDoc* EnrycherDoc) const;
	PXmlTok GetAnnTok(const TIntPr& AnnId, const TIntPrV& AnnIdV, const TStr& TagNm,
		const TStr& SubTagNm, const TEnrycherDoc* EnrycherDoc) const;

	// required by vector
	TEnrycherAssert(): SubjAnnId(TIntPr(-1, -1)),
		VerbAnnId(TIntPr(-1, -1)), ObjAnnId(TIntPr(-1, -1)) { }
public:
	// deserilization from XML
	TEnrycherAssert(const PXmlTok& AssertTok);
    // serilization to XML
	PXmlTok GetXmlTok(const TEnrycherDoc* EnrycherDoc) const;

	// assertion ids
	int GetAssertId() const { return AssertId; }
	int GetSubjAnnId() const { return SubjAnnId.Val1; }
	int GetVerbAnnId() const { return VerbAnnId.Val1; }
	int GetObjAnnId() const { return ObjAnnId.Val1; }

	const TIntPr &GetSubjAnnInstId() const { return SubjAnnId; }
	const TIntPr &GetVerbAnnInstId() const { return VerbAnnId; }
	const TIntPr &GetObjAnnInstId() const { return ObjAnnId; }

	friend class TVec<TEnrycherAssert>;
};

///////////////////////////////
// Enrycher-Pipeline-Trace
ClassTP(TEnrycherPipeline, PEnrycherPipeline)//{
private:
	TStrV ToolNmV;
	TTmV ToolTmV;
	TIntV ToolDurationV;

private:
	// required by vector
	TEnrycherPipeline() { }
	// required by static New
	TEnrycherPipeline(const PXmlTok& PipelineTok);

public:
	static PEnrycherPipeline New() { return new TEnrycherPipeline; }

	// deserilization from XML
	static PEnrycherPipeline LoadXml(const PXmlTok& PipelineTok) {
		return new TEnrycherPipeline(PipelineTok); }
    // serilization to XML
    PXmlTok GetXmlTok() const;

	// tools
	bool Empty() const { return ToolNmV.Empty(); }
	int GetTools() const { return ToolNmV.Len(); }
	const TStr& GetToolNm(int ToolN) const { return ToolNmV[ToolN]; }
	const TTm& GetToolTm(int ToolN) const { return ToolTmV[ToolN]; }
	int GetToolDuration(int ToolN) const { return ToolDurationV[ToolN]; }
	void AddTool(const TStr& Nm, const TTm& Tm, int Duration) {
		ToolNmV.Add(Nm); ToolTmV.Add(Tm); ToolDurationV.Add(Duration); }
};

///////////////////////////////
// Enrycher-Document
ClassTP(TEnrycherDoc, PEnrycherDoc)//{
private:
    // meta-data
    TStr DocId;
	TEnrycherSemAttrV SemAttrV;
    PEnrycherPipeline Pipeline;
    // paragraphs
    TEnrycherParaV ParaV;
	THash<TInt, TIntPr> SentIdToParaNSentNH;
    // annotations
    THash<TInt, TEnrycherAnn> AnnH;
    // assertions
    TEnrycherAssertV AssertV;

	// for uniquenes of IDs
	TInt NextSentId; // for sentences
	TInt NextId; // and for anything else

private:
	// required by static New
    TEnrycherDoc();
	// required by static LoadXml
	TEnrycherDoc(const PXmlTok& TopTok);
	// update next available ID
	void UpdateNextId(int Id, TInt& NextId) { 
		if (Id >= NextId) { NextId = Id + 1; } }
	// reset next available IDs
	void ResetNextIds();

public:
	// for creating an empty document
	static PEnrycherDoc New() {
		return new TEnrycherDoc; }

	// deserilization from XML
    static PEnrycherDoc LoadXml(const PXmlTok& TopTok) {
        return new TEnrycherDoc(TopTok); }
    static PEnrycherDoc LoadXml(const PXmlDoc& XmlDoc) {
		if (XmlDoc->IsOk()) { return new TEnrycherDoc(XmlDoc->GetTok()); }
		else { TExcept::Throw(XmlDoc->GetMsgStr()); }
        return new TEnrycherDoc(); 
	}
    static PEnrycherDoc LoadXml(const PSIn& SIn) {
		return TEnrycherDoc::LoadXml(TXmlDoc::LoadTxt(SIn));
	}
    static PEnrycherDoc LoadXml(const TStr& XmlFNm) {
		return TEnrycherDoc::LoadXml(TFIn::New(XmlFNm)); 
	}
    // serilization to XML
    PXmlDoc GetXmlDoc() const;
    void SaveXml(const PSOut& SOut) const { GetXmlDoc()->SaveTxt(SOut); }
    void SaveXml(const TStr& XmlFNm) const { GetXmlDoc()->SaveTxt(XmlFNm); }

	// meta-data
	TStr GetDocId() const { return DocId; }
	int GetSemAttrs() const { return SemAttrV.Len(); }
	const TEnrycherSemAttr& GetSemAttr(int SemAttrN) const { return SemAttrV[SemAttrN]; }
	int AddSemAttr(const TEnrycherSemAttr& NewSemAttr);
	const PEnrycherPipeline& GetPipeline() const { return Pipeline; }
	double GetSentiment() const;

	static void AddAsciiCharsToUni(const char *Str, TIntV& XmlChA);
	static void AddStrToXmlUni(const TStr& PlainChA, TIntV& XmlChA);
	static void AddUniToXmlUni(const TIntV& PlainChA, TIntV& XmlChA);
	static void AddStrToUni(const TStr& Str, TIntV& ChA);
	static void AddTokToUni(const PXmlTok& Tok, const bool& XmlP, TIntV& ChA);
	static void SaveTxt(const PXmlDoc& Doc, const PSOut& Out);
	static void SaveUni(const PXmlDoc& Doc, TIntV& Out);
	void SaveXmlUtf8(const PSOut& Out) { SaveTxt(GetXmlDoc(), Out); }
	
	// paragraphs
	bool IsParas() const { return !ParaV.Empty(); }
	int GetParas() const { return ParaV.Len(); }
	const TEnrycherPara& GetPara(int ParaN) const { return ParaV[ParaN]; }
	const TEnrycherSent& GetSent(int SentId) const;
	bool IsSentId(const int SentId) const;
	int AddPara(const TEnrycherPara& NewPara);
	void GetAllText(TChA& ChA) const;

    // annotations
    bool IsAnns() const { return !AnnH.Empty(); }
    int GetAnns() const { return AnnH.Len(); }
    const TEnrycherAnn& GetAnnById(int AnnId) const { return AnnH.GetDat(AnnId); }
	int AddAnn(const TEnrycherAnn& NewAnn);
	int AddAnnInst(int AnnId, const TEnrycherAnnInst& NewAnnInst);
	int AddAnnSemAttr(int AnnId, const TEnrycherSemAttr& NewSemAttr);

	// iteration over annotations
	int GetFirstAnn() const { return AnnH.FFirstKeyId(); }
	bool GetNextAnn(int& KeyId) const { return AnnH.FNextKeyId(KeyId); }
    const TEnrycherAnn& GetAnnByKeyId(int KeyId) const { return AnnH[KeyId]; }

	// assertions
	bool IsAssert() const { return !AssertV.Empty(); }
	int GetAsserts() const { return AssertV.Len(); }
	const TEnrycherAssert& GetAssert(int AssertN) { return AssertV[AssertN]; }
};

#endif
