/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "enrycherutil.h"

///////////////////////////////
// Enrycher-Executer
PBowDocBs TEnrycherUtil::GetSentBow(const PEnrycherDoc& Doc) {
	// load sentences into bag-of-words
	PSwSet SwSet = TSwSet::New(swstEn523);
	PStemmer Stemmer = TStemmer::New(stmtPorter, true);
	PBowDocBs BowDocBs = TBowDocBs::New(SwSet, Stemmer, NULL);
	for (int ParaN = 0; ParaN < Doc->GetParas(); ParaN++) {
		const TEnrycherPara& Para = Doc->GetPara(ParaN);
		for (int SentN = 0; SentN < Para.GetSents(); SentN++) {
			const TStr SentId = TStr::Fmt("%d-%d", ParaN, SentN);
			const TStr SentStr = Para.GetSent(SentN).GetStr();
			BowDocBs->AddHtmlDoc(SentId, TStrV(), SentStr);
		}
	}
	return BowDocBs;
}

PBowSpV TEnrycherUtil::GetSentCentroid(const PBowDocBs& BowDocBs, const PBowDocWgtBs& BowDocWgtBs) {
	TIntV AllDIdV; BowDocBs->GetAllDIdV(AllDIdV);
	return TBowClust::GetConceptSpV(BowDocWgtBs, TBowSim::New(bstCos), AllDIdV);
}

void TEnrycherUtil::GetAnnWgt(const PEnrycherDoc& Doc, TIntFltKdV& AnnIdWgtV) {
	// get sentence documents and centroid
	PBowDocBs BowDocBs = GetSentBow(Doc);
	PBowSim BowSim = TBowSim::New(bstCos);
	PBowDocWgtBs BowDocWgtBs = TBowDocWgtBs::New(BowDocBs, 
		BowDocBs->GetDocs() > 1 ? bwwtNrmTFIDF : bwwtNrmEq);
	// get article centorid
	PBowSpV AllSpV = GetSentCentroid(BowDocBs, BowDocWgtBs);
	// go over all the annotations
	AnnIdWgtV.Gen(Doc->GetAnns(), 0);
	int AnnKeyId = Doc->GetFirstAnn();
	while (Doc->GetNextAnn(AnnKeyId)) {
		const TEnrycherAnn& Ann = Doc->GetAnnByKeyId(AnnKeyId);
		// concatinate all sentences, where instances occur
		const int Insts = Ann.GetAnnInsts();
		//if (Insts == 0) { continue; }
		EAssertR(Insts > 0, TStr::Fmt("We didn't get any annotation instances for annotation %s", Ann.GetStr().CStr()));
		TChA AnnContext;
		double PredefinedWgt = 0;
		for (int InstN = 0; InstN < Insts; InstN++) {
			const TEnrycherAnnInst& Inst = Ann.GetAnnInst(InstN);
			if (Inst.GetTokens() == 0) { continue; }
			const int SentId = Inst.GetSentId(0);
			/*if (SentId == -1) {
				PredefinedWgt = ((double) Inst.GetTokenId(0)) / 1000;
				break;
			}*/
			if (Doc->IsSentId(SentId))
				AnnContext += Doc->GetSent(SentId).GetStr();
			AnnContext += '\n';
		}
		PBowSpV AnnSpV = BowDocBs->GetSpVFromHtmlStr(AnnContext, BowDocWgtBs);
		// compute similarity of sentences with with the article centorid
		const double AnnWgt = PredefinedWgt > 1e-7 ? PredefinedWgt : BowSim->GetSim(AnnSpV, AllSpV);
		// remember
		AnnIdWgtV.Add(TIntFltKd(Ann.GetId(), AnnWgt));
	}
}
