#ifndef ACTIVITYBASE_H
#define ACTIVITYBASE_H

#include <qminer.h>
#include "../NewsMiner/news.h"

//const uint64 InvalidUInt64(TUInt64::Mx);
//enum FolderVisibility { PrivateFolder = 0, PublicFolder, CommunityFolder };

// the minimum number of articles that had to be added since we last updated the concept folder
//const int MinNewArticlesToUpdateConceptFolder = 5000;

//const TStr ConceptFolderPrefix("conceptFolder");

//////////////////////////////////////
// User-Base
ClassTP(TActivityBase, PActivityBase)//{
public:
	TFAccess FAccess;

	// index to base
	PBase Base;
	PGenericBase GenericBase;
	
	uchar ArticleStoreId;
	uchar ConceptStoreId;
	
	uchar UserStoreId;
	uchar FolderStoreId;
	uchar ArtActStoreId;
	uchar ActStoreId;

	PStore UserStore;
	PStore FolderStore;
	PStore ActStore;
	PStore ArtActStore;
	// pointers to the index and stores
	PIndexVoc IndexVoc;
	PIndex Index;
	
	int JoinFolderHasStoredArticlesId;
	int JoinFolderHasLikedArticlesId;
	int JoinFolderHasLikedArticlesInPastId;		// these are the articles that the user used to like but then unliked. we need tho remember them so that we don't update user's folder concepts for these articles
	int JoinArticleLikedById;
	int JoinArticleStoredBy;
	int JoinFolderHasActivityId;		// (general) activity join
	int JoinFolderHasArtActStoredId;
	int JoinFolderHasArtActLikedId;

	PNewsBase NewsBase;
	
private:
	// for loading existing index and stores for update or read-only
	TActivityBase(const PNewsBase _NewsBase, const TStr& IndexFPath, const TFAccess& FAccess, const int64& IndexCacheSize, const int64& StoreCacheSize);
	// for creating a new index and store
	TActivityBase(const TStr& StoreDefFNm, const PNewsBase _NewsBase, const TStr& IndexFPath, const int64& IndexCacheSize, const int64& StoreCacheSize);
	void CommonInit();
public:
	~TActivityBase();
	// for creating new index and stores or updating existing
	static PActivityBase New(const TStr& CuratedNewsDef, const PNewsBase _NewsBase, const TStr& IndexFPath, const int64& IndexCacheSize, const int64& StoreCacheSize);
	
	static TStr CreateKey(const TStr& UserSqlId, const TStr& FolderId) { return UserSqlId + "-" + FolderId; }
	static TStr GetUserSqlId(const TStr& Key) { TStr L, R; Key.SplitOnCh(L, '-', R); return L; }
	static TStr GetFolderId(const TStr& Key) { TStr L, R; Key.SplitOnCh(L, '-', R); return R; }

	//// folder methods
	uint64 AddFolder(const TStr& UserSqlId, const TStr& FolderId);
	bool IsFolder(const TStr& UserSqlId, const TStr& FolderId);
	//void DeleteFolder(const TStr& UserSqlId, const TStr& FolderId);
	uint64 GetFolderRecId(const TStr& UserSqlId, const TStr& FolderId);
	PRecSet GetFoldersForUser(const TStr& UserSqlId);
	void DeleteFolder(const uint64& FolderRecId);

	// storing of articles into folders
	void AddStoredArticle(const uint64& FolderRecId, const uint64& ArticleRecId);
	void RemoveStoredArticle(const uint64& FolderRecId, const uint64& ArticleRecId);
	
	// liking articles
	bool IsLikedArticle(const uint64& FolderRecId, const uint64& ArticleRecId);
	bool IsLikedArticleInPast(const uint64& FolderRecId, const uint64& ArticleRecId);
	void AddLikedArticle(const uint64& FolderRecId, const uint64& ArticleRecId);
	void RemoveLikedArticle(const uint64& FolderRecId, const uint64& ArticleRecId);

	PRecSet GetLikedArtAct(const uint64& FolderRecId);
	PRecSet GetStoredArtAct(const uint64& FolderRecId);
	PRecSet GetLikedArticles(const uint64& FolderRecId);
	PRecSet GetLikedArticlesInPast(const uint64& FolderRecId);
	PRecSet GetStoredArticles(const uint64& FolderRecId);

	// loading/storing data to files
	void SaveData(const TStr& FName, const PStore Store);
	void LoadData(const TStr& FName, const PStore Store);
	void SaveJoinsData();
	void LoadJoinsData(const TStr& FName, const int JoinId);
	TStr GetListOfRecIds(const PRecSet& RecSet);
};

#endif


