#ifndef EVENTREGSRVUTIL_H
#define EVENTREGSRVUTIL_H

#include <qminer_srv.h>
#include "../Utils/IterativeKMeans.h"
#include "../NewsMiner/NewsBase.h"
#include "EventRegBase.h"

using namespace TQm;

class TEventRegSrvUtil;
typedef TPt<TEventRegSrvUtil> PEventRegSrvUtil;
class TEventRegSrvUtil{
public:
  friend class TPt<TEventRegSrvUtil>;

private:
	PStore StoryStore;
	PStore EventStore;
	uchar StoryStoreId;
	uchar EventStoreId;

	THash<TStr, TBool> ValidDateH;	// date (YYYY-MM-DD) -> true/false

	TCRef CRef;
	TEventRegSrvUtil(const PNewsBase& _NewsBase, const PEventRegBase& _EventRegBase, const PNewsSrvUtil& _NewsSrvUtil ) : 
		NewsBase(_NewsBase), EventRegBase(_EventRegBase), NewsSrvUtil(_NewsSrvUtil) 
		{
			StoryStore = NewsBase->StoryStore;
			EventStore = EventRegBase->EventStore;
			StoryStoreId = StoryStore->GetStoreId();
			EventStoreId = EventStore->GetStoreId();
		}
	PJsonVal GetClusterDetails(const TClusterInfo& ClusterInfo, const PBowDocBs& BowDocBs);

public:
	PNewsBase NewsBase;
	PEventRegBase EventRegBase;
	PNewsSrvUtil NewsSrvUtil;

	enum MatrixScoringMethod { pairTfIdf, chiSquare, pmi };

	static PEventRegSrvUtil New(const PNewsBase& NewsBase, const PEventRegBase& EventRegBase, const PNewsSrvUtil& NewsSrvUtil ) {
		return new TEventRegSrvUtil(NewsBase, EventRegBase, NewsSrvUtil); }
	
	PJsonVal EventProps(const uint64& EventRecId) const;
	PJsonVal EventsProps(const PRecSet& EventRecSet) const;

	PJsonVal EventsTimeAggr(const PRecSet& EventRecSet);
	PJsonVal EventsLocAggr(const PRecSet& EventRecSet, const TStrV& LabelLanguageV = NewsConst.LangCoreV);
	PJsonVal EventsLocTimeAggr(const PRecSet& EventRecSet, const TStrV& LabelLanguageV = NewsConst.LangCoreV);
	PJsonVal EventsTopPublisherAggr(const PRecSet& EventRecSet, const int& Count = 20, const bool& IncludePublisherDetails = true);
	PJsonVal EventsDateMentionAggr(const PRecSet& EventRecSet, const int& MinDaysApart, const int& MinDateMentionCount);
	PJsonVal EventsCategoryAggr(const PRecSet& EventRecSet);
	PJsonVal EventsConceptAggr(const PRecSet& EventRecSet, const EnumConceptType& ValidTypes = ctEntities, const int& ConceptCount = 30, const TStrV& LabelLanguageV = NewsConst.LangCoreV);
	PJsonVal EventsKeywordAggr(const PRecSet& EventRecSet, const TStr& Lang, const int& SampleSize = 500);
	PJsonVal EventsConceptMatrix(const PRecSet& EventRecSet, const EnumConceptType& ValidTypes = ctEntities, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const int& ConceptCount = 20, const MatrixScoringMethod& ScoringMethod = pmi, const int& SampleSize = 500);
	PJsonVal EventsConceptGraph(const PRecSet& EventRecSet, const EnumConceptType& ValidTypes = ctEntities, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const int& ConceptCount = 25, const int& LinkCount = 50, const int& SampleSize = 500);
	PJsonVal EventsTrendingConcepts(const PRecSet& EventRecSet, const TStrV& TrendingConceptV, const int& ConceptCount = 25, const TStrV& LabelLanguageV = NewsConst.LangCoreV);

	PJsonVal EventKeywordAggr(const uint64& EventId, const TStr& Lang);
	PJsonVal EventSourceAggr(const TStrKdV& FldNmValPrV, const uint64& EventId);
	PJsonVal EventArticleTrend(const TStrKdV& FldNmValPrV, const uint64& EventRecId, const TStr& FldNmPrefix = "articleTrend") const;
	PJsonVal EventDateMentionAggr(const uint64& EventId);
	
	// use k-means iteratively to find subgroups of EventRecSet
	PJsonVal EventsGetEventClusters(const TStrKdV& FldNmValPrV, const PRecSet& EventRecSet);
	
	void GetConceptPairs(const PRecSet& EventRecSet, const EnumConceptType& ValidTypes, THash<TPair<TUInt64, TUInt64>, TInt>& ConceptPairToCntH, THash<TUInt64, TInt>& ConceptToCntH, const int& SampleSize = 500, const int& MinConceptWgt = 5);
	inline double Sqr(const double& Val) { return Val*Val; }

	PRecSet GetEventArticles(const uint64& EventId, const TStrV& LangV) const;
};

#endif