/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "EventStats.h"

TEventStat::~TEventStat()
{
	Save();
}

void TEventStat::Load(const TStr& _FPath)
{
	FPath = _FPath;
	const TStr FName = FPath + "CustomData/EventStats.bin";
	try {
		if (TFile::Exists(FName)) {
			TFIn FIn(FName);
			EventStatV.Load(FIn);
		}
	}
	catch (const PExcept& E) {
		TNotify::StdErrNotify->OnStatusFmt("Problem loading event stats: %s", E->GetMsgStr().CStr());
	}
}

void TEventStat::Save() {
	if (FPath == "")
		return;
	if (EventStatV.Len() > 5)
		EventStatV.Trunc(5);
	try {
		TFOut FOut(FPath + "CustomData/EventStats.bin");
		EventStatV.Save(FOut);
	}
	catch (const PExcept& E) {
		TNotify::StdErrNotify->OnStatusFmt("Problem saving event stats: %s", E->GetMsgStr().CStr());
	}
}

// find event stat instance for the given date
bool TEventStat::GetStat(const TStr& Date, TEventStatInfo& StatInfo)
{
	for (int N = 0; N < EventStatV.Len(); N++) {
		if (EventStatV[N].Date == Date) {
			StatInfo = EventStatV[N];
			return true;
		}
	}
	return false;
}