#ifndef EVENTSTATS
#define EVENTSTATS

#include <base.h>

class TEventStatInfo
{
public:
	TStr Date;

	TUInt64 ArticleCount;
	TUInt64 StoryCount;
	TUInt64 EventCount;
	TUInt64 ConceptCount;
	TUInt64 SourceCount;
	TUInt64 CategoryCount;
	TUInt64 ValidStories;		// stories with some articles
	TUInt64 ValidEvents;
	TUInt64 EventsWithCrossLingualStories;
		
	TUInt64 ArticleDuplicates;
	TUInt64 ArticlesWithStory;
		
	THash<TStr, TInt> EventsPerDay;
	THash<TInt, TInt> DaySpanToCountH;
	THash<TStr, TInt> DayToStoryCountH;
	THash<TInt, TInt> StorySizeToCountH;
		
	THash<TStr, TInt> DayToEventCountH;
	THash<TStr, TInt> DayToEventStoryCountH;	// number of merged stories for events on that day. DayToMergedStoriesCountH / DayToEventCountH gives the average number of merged stories for events on that day
	THash<TStr, TInt> DayToMergedEventCountH;	// remember for each day how many events have more than one story in them
	
	THash<TStr, TInt> DayToArticleCountH;				// number of articles on a particular day
	THash<TStr, TInt> DayToArticlesWithStoryCountH;		// number of articles that have an assigned story
	THash<TStr, TInt> DayToAnnotatedArticleCountH;		// number of articles that have annotated concepts in them
	THash<TPair<TStr, TStr>, TUInt64> DayTypeToAnnotationsCountH;	//(date, [person,loc,org,wiki]) -> number  -- the number of annotations of this type that we've found in articles on that date

	THash<TStr, TInt> LangPairToEventMergeCountH;		// lang1-lang2 -> count. number of events that have clusters in lang1 and lang2

	TEventStatInfo() {
		Date = TTm::GetCurLocTm().GetWebLogDateTimeStr();
		ArticleCount = 0;
		StoryCount = 0;
		EventCount = 0;
		ConceptCount = 0;
		SourceCount = 0;
		CategoryCount = 0;

		ValidStories = 0;
		ValidEvents = 0;
		ArticleDuplicates = 0;
		ArticlesWithStory = 0;
	}

	TEventStatInfo(TSIn& SIn)
	{
		Date.Load(SIn);

		ArticleCount.Load(SIn);
		StoryCount.Load(SIn);
		EventCount.Load(SIn);
		ConceptCount.Load(SIn);
		SourceCount.Load(SIn);
		CategoryCount.Load(SIn);
		ValidStories.Load(SIn);
		ValidEvents.Load(SIn);
		EventsWithCrossLingualStories.Load(SIn);

		ArticleDuplicates.Load(SIn);
		ArticlesWithStory.Load(SIn);

		EventsPerDay.Load(SIn);
		DaySpanToCountH.Load(SIn);
		DayToStoryCountH.Load(SIn);
		StorySizeToCountH.Load(SIn);

		DayToEventCountH.Load(SIn);
		DayToEventStoryCountH.Load(SIn);
		DayToMergedEventCountH.Load(SIn);
		DayToArticleCountH.Load(SIn);
		DayToArticlesWithStoryCountH.Load(SIn);
		DayToAnnotatedArticleCountH.Load(SIn);

		LangPairToEventMergeCountH.Load(SIn);
	}

	void Save(TSOut& SOut) const
	{
		Date.Save(SOut);
		
		ArticleCount.Save(SOut);
		StoryCount.Save(SOut);
		EventCount.Save(SOut);
		ConceptCount.Save(SOut);
		SourceCount.Save(SOut);
		CategoryCount.Save(SOut);
		ValidStories.Save(SOut);
		ValidEvents.Save(SOut);
		EventsWithCrossLingualStories.Save(SOut);

		ArticleDuplicates.Save(SOut);
		ArticlesWithStory.Save(SOut);
		
		EventsPerDay.Save(SOut);
		DaySpanToCountH.Save(SOut);
		DayToStoryCountH.Save(SOut);
		StorySizeToCountH.Save(SOut);

		DayToEventCountH.Save(SOut);
		DayToEventStoryCountH.Save(SOut);
		DayToMergedEventCountH.Save(SOut);
		DayToArticleCountH.Save(SOut);
		DayToArticlesWithStoryCountH.Save(SOut);
		DayToAnnotatedArticleCountH.Save(SOut);

		LangPairToEventMergeCountH.Save(SOut);
	}
};

class TEventStat
{
	TStr FPath;
public:
	TVec<TEventStatInfo> EventStatV;

	TEventStat() { }
	~TEventStat();
	
	void AddStat(const TEventStatInfo& EventStat) { EventStatV.Ins(0, EventStat); EventStatV.Trunc(5); }
	void Load(const TStr& _FPath);
	void Save();
	// find event stat instance for the given date
	bool GetStat(const TStr& Date, TEventStatInfo& StatInfo);
};

#endif