/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "net.h"
#include "CategorySrv.h"
#include "../Utils/GroupNotify.h"


int main(int argc, char* argv[]){

	try {
		// create environment
		Env = ::TEnv(argc, argv, TNotify::StdNotify);
		// command line parameters
		Env.PrepArgs("News categories server", 1);

		// port
		const int PortN = Env.GetIfArgPrefixInt("-port:", 8097, "server port");
		// paths
		const TStr CategoryDbPath = Env.GetIfArgPrefixStr("-catDataFolder:", "./dbs/Categories/", "Folder with category data");
				
		if (Env.IsEndOfRun()) { return 0; }

		// prepare unicode base
		if (!TUnicodeDef::IsDef()) { TUnicodeDef::Load("./UnicodeDef.Bin"); }
				
		// array of server functions
		TSAppSrvFunV SrvFunV;
		SrvFunV.Add(TCategorySrv::New(CategoryDbPath));
		
		// prepare the web-server logger
		//PNotify Notify = new TFileNotify(IndexFPath + "Log-ArticlesWebRequests.txt");
		//PNotify Notify = TNotify::StdNotify;
		TVec<PNotify> NotifierV;
		NotifierV.Add(TNotify::StdNotify());
		NotifierV.Add(TFileNotify::New("./Logs/Log-CategoryServer.txt", true, true));
		PNotify Notify = TGroupNotify::New(NotifierV);
	
		// start server
		PWebSrv WebSrv = TSAppSrv::New(PortN, SrvFunV, Notify, true);
		TLoop::Run();
		return 0;
	}
	catch (const PExcept& E)
	{
		ErrNotify(E->GetMsgStr());
		return 1;
	}
	catch (...){
		ErrNotify("Unrecognized exception occured.");
		return 1;
	}
}
