/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "User.h"

uint64 TUser::TimeStartMS = TTm::GetMSecsFromTm(TTm(1970, 1, 1));

TUser::TUser(const PJsonVal& Json)
{
	Uri = Json->GetObjStr("uri", "");
	Name = Json->GetObjStr("name", "");
	LastAccessTime = Json->GetObjNum("lastAccessTime", 0);
}

TUser::TUser(const uint64& UserId, const PStore& UserStore)
{
	Uri = UserStore->GetRecNm(UserId);
	Name = UserStore->GetFieldNmStr(UserId, "Name");
	LastAccessTime = UserStore->GetFieldNmFlt(UserId, "LastAccessTime");
}

PUser TUser::New(const uint64& UserId, const PStore& UserStore)
{
	if (!UserStore->IsRecId(UserId))
		return NULL;
	return new TUser(UserId, UserStore);
}

PUser TUser::New(const TStr& UserNm, const PStore& UserStore)
{
	if (!UserStore->IsRecNm(UserNm))
		return NULL;
	uint64 UserId = UserStore->GetRecId(UserNm);
	return new TUser(UserId, UserStore);
}


PJsonVal TUser::GetJson()
{
	PJsonVal Json = TJsonVal::NewObj("uri", Uri);
	Json->AddToObj("name", Name);
	Json->AddToObj("lastAccessTime", TJsonVal::NewNum(LastAccessTime));
	return Json;
}

PJsonVal TUser::GetJsonForStore()
{
	PJsonVal Json = TJsonVal::NewObj("URI", Uri);
	Json->AddToObj("Name", Name);
	Json->AddToObj("LastAccessTime", TJsonVal::NewNum(LastAccessTime));
	return Json;
}

double TUser::GetTime()
{
	uint64 TimeMS = TSysTm::GetMSecsFromTm(TTm::GetCurLocTm());
	// get seconds since 1970/1/1
	double DiffS = (TimeMS - TimeStartMS) / 1000;
	double DayTimeStamp = DiffS / 86400.0;
	return DayTimeStamp;
}