#ifndef USERBASE_H
#define USERBASE_H

#include <base.h>
#include <qminer.h>
#include <qminer_gs.h>
#include "User.h"
#include "../NewsMiner/ConceptFolder.h"
#include "logger.h"

// the minimum number of articles that had to be added since we last updated the concept folder
//const int MinNewArticlesToUpdateConceptFolder = 5000;

using namespace TQm;

class TUserBase;
typedef TPt<TUserBase> PUserBase;
class TUserBase {
public:
	friend class TPt<TUserBase>;
private:
	TCRef CRef;

public:
	// index to base
	PBase Base;
	// pointers to the index and stores
	PIndexVoc IndexVoc;
	PIndex Index;
	
	PStore ConceptStore;
	PStore ConceptFolderStore;
	PStore UserStore;

	uchar ConceptStoreId;
	uchar ConceptFolderStoreId;
	uchar UserStoreId;

	TStrHash<TInt, TBigStrPool> ConceptsH;
	TStrHash<TInt, TBigStrPool> SourcesH;
	TStrHash<TInt, TBigStrPool> LocationsH;
	TStrV AvailableFolderUriV;

private:
	// for creating a new index and store
	TUserBase(const TStr& StoreDefFNm, const TStr& FPath, const int64& IndexCacheSize,
		const int64& DataCacheSize, const TStr& TempFPath);
	// for loading existing index and stores for update or read-only
	TUserBase(const TStr& FPath, const TFAccess& FAccess,
		const int64& IndexCacheSize, const int64& DataCacheSize, const TStr& TempFPath);
	// clean on exit
	~TUserBase() { Save(); }
	// initialize shortcuts
	void Init();
	// save newsbase
	void Save();
	
	TStr GetNewFolderUri();
	void AddAvailableFolderUri(const TStr& ConceptFolderUri);
	TStr GetAvailableFolderUri();
	void GetFolderConcepts(const uint64& FolderId, const bool& SortByWgts, TUInt64FltKdV& ConceptV);
	void RemoveJsonStores(const PJsonVal& JsonScheme, const TStrSet& KeepStores);
	
public:
	// settings
	int VerbosityLevel;
	PLogger Logger;
	char NotifyBuf[100 * 1024];

	// for creating new index and stores or updating existing
	static PUserBase New(const TStr& StoreDefFNm, const TStr& FPath,
		const int64& IndexCacheSize, const int64& DataCacheSize, const TStr& TempFPath);
	// for loading existing index and stores for update or read-only
	static PUserBase Load(const TStr& FPath, const TFAccess& FAccess,
		const int64& IndexCacheSize, const int64& DataCacheSize, const TStr& TempFPath);
		
	// check if index already exists
	static bool Exists(const TStr& IndexFPath) {
		return TIndex::Exists(IndexFPath) && TFile::Exists(IndexFPath + "Source.BaseStore");
	}

	// user methods
	uint64 AddUser(const TStr& UserUri, const TStr& UserName);
	PUser GetUser(const uint64& UserId);
	PUser GetUser(const TStr& UserUri);
	void SetUserName(const TStr& UserUri, const TStr& UserName);

	// folder methods
	uint64 AddFolder(const uint64& UserId, const TStr& FolderName, const FolderVisibility& Visibility);
	PJsonVal GetFolderJson(const uint64& FolderId);
	void DeleteFolder(const uint64& FolderId);
	void UpdateFolder(const PConceptFolder& Folder);
	void UpdateConcept(const TStr& ConceptUri, const TStr& Label);

	PConceptFolder GetFolder(const uint64& FolderId);
	uint64 GetFolderOwner(const TStr& FolderUri);
	PRecSet GetFoldersForUser(const uint64& UserId, const bool& IncludeHomeFolder = false);
	uint64 GetHomeFolderId(const uint64& UserId);
	void SetAsHomeFolder(const uint64& UserId, const uint64& FolderId);

	void UpdateFolderConceptsWithArticleConcepts(const uint64 FolderId, const TUInt64FltKdV& ImportantArticleConceptsV);

	void AddIdsForFolderJson(const PJsonVal& JsonData, const bool& RemoveUris = true);
	void AddUrisForFolderJson(const PJsonVal& JsonData, const bool& RemoveIds = true);
	void IncreaseArticleCount(const uint64& FolderId);

	//// functions related to following friends
	PRecSet GetFollowingFriends(const uint64& UserId);
	PRecSet GetFollowedByFriends(const uint64& UserId);
	void FollowAdd(const uint64& UserId, const uint64& FriendUserId);
	void FollowRemove(const uint64& UserId, const uint64& FriendUserId);
	//PRecSet FindSimilarFolders(const uint64& FolderId, int MaxFolderCount);
	//PRecSet FindSimilarUsers(const uint64 UserId, int MaxUserCount);

	void PrintFolderDefinition(const uint64& FolderId, const int& NumOfConcepts = 5);
	void PrintRecSet(const PRecSet& RecSet, const int& PrintCount);

	//// folder and user seach functions
	/*PRecSet FindUsers(const TStr& Prefix, int MaxCount = 20);
	PRecSet FindFolders(const TStr& Prefix, int MaxCount = 20);*/

	int AddConcept(const TStr& Uri) { return ConceptsH.AddKey(Uri); }
	int AddSource(const TStr& SourceUri) { return SourcesH.AddKey(SourceUri); }
	int AddLocation(const TStr& Location) { return LocationsH.AddKey(Location); }
	TStr GetConcept(const int& ConceptId) { return ConceptsH.GetKey(ConceptId); }
	TStr GetSource(const int& SourceId) { return SourcesH.GetKey(SourceId); }
	TStr GetLocation(const int& LocationId) { return LocationsH.GetKey(LocationId); }

	void NotifyVerbose(const int VerbosityLevel, const char *Str);
	void NotifyVerboseFmt(const int VerbosityLevel, const char *FmtStr, ...);

};

/*
//////////////////////////////////////
// User-Base
ClassTP(TUserBase, POgUserBase)//{
public:
	// index to base
	PBase Base;
	
	// pointers to the index and stores
	PIndexVoc IndexVoc;
	PIndex Index;
	
	TUserStore* UserStore;
	TFolderStore* FolderStore;
	uchar UserStoreId;
	uchar FolderStoreId;
			
	int FolderNameKWdsKeyId;	// id of the key used for indexing words from folder names
	//int FolderVisibilityId;	// id of the key used to index the folder visiblity
	int UserNameKWdsKeyId;		// id of the key used for indexing words from user names
	int LikedArticlesKeyId;		// id of the key used for storing liked articles for a person

	// settings
	int VerbosityLevel;

	TStrHash<TInt, TBigStrPool> ConceptsH;
	TStrHash<TInt, TBigStrPool> SourcesH;
	TStrHash<TInt, TBigStrPool> LocationsH;

private:
	PLogger Logger;
	char NotifyBuf[100*1024];
	TFAccess FAccess;
	int64 IndexCacheSize;
	int64 UserCacheSize;
	int64 FolderCacheSize;

	// for loading existing index and stores for update or read-only
	TUserBase(const TStr& IndexFPath, const TFAccess& FAccess, 
		const int64& _IndexCacheSize, const int64& _UserCacheSize, const int64& _FolderCacheSize);
	// for creating a new index and store
	TUserBase(const TStr& IndexFPath, const int64& _IndexCacheSize, 
		const int64& _UserCacheSize, const int64& _FolderCacheSize);
	void CommonInit();
	
	void Save();
	void GetFolderConcepts(const uint64& FolderId, const bool& SortByWgts, TUInt64FltKdV& ConceptV);

public:
	~TUserBase();
	// for creating new index and stores or updating existing
	static POgUserBase New(const TStr& IndexFPath, const int64& IndexCacheSize, const int64& UserCacheSize, const int64& FolderCacheSize);

	// user management and methods
	void AddUser(const TStr& UserSQLId, const TStr& UserName);
	void SetUserName(const uint64& UserId, const TStr& UserName);
	uint64 GetFolderOwner(const uint64& FolderId) { return FolderStore->GetFolderOwner(FolderId); }
	uint64 GetHomeFolderId(const uint64& UserId);
	uint64 GetFolderIdByName(const uint64& UserId, const TStr& FolderName);
	uint64 GetFolderId(const uint64& UserId, const TUInt64& FolderId);
	PRecSet GetFoldersForUser(const uint64& UserId, const bool& IncludeHomeFolder = false);
	PJsonVal GetJsonForFolder(const uint64& FolderId);
	TFolderRec GetFolderForJson(const PJsonVal& JsonData);

	//// folder methods
	uint64 AddFolder(const uint64& UserId, const TStr& FolderName, const FolderVisibility& Visibility);
	void RemoveFolder(const uint64& UserId, const uint64& FolderId);
	void ModifyFolder(const uint64& FolderId, const TStr& FolderName, const FolderVisibility& Visibility);
	void SetFolderVisibility(const uint64& FolderId, const FolderVisibility& Visibility);
	//void RemoveSuggestedArticlesFromFolder(const uint64& FolderId);
	void UpdateFolderConceptsWithArticleConcepts(const uint64 FolderId, const TUInt64FltKdV& ImportantArticleConceptsV);
	
	//// functions related to following friends
	PRecSet GetFollowedFriends(const uint64& UserId);
	void FollowAdd(const uint64& UserId, const uint64&FriendRecId);
	void FollowRemove(const uint64& UserId, const uint64&FriendRecId);
	PRecSet FindSimilarFolders(const uint64& FolderId, int MaxFolderCount);
	PRecSet FindSimilarUsers(const uint64 UserId, int MaxUserCount);

	void PrintFolderDefinition(const uint64& FolderId, const int& NumOfConcepts = 5);
	void PrintRecSet(const PRecSet& RecSet, const int& PrintCount);
		
	//// folder and user seach functions
	PRecSet FindUsers(const TStr& Prefix, int MaxCount = 20);
	PRecSet FindFolders(const TStr& Prefix, int MaxCount = 20);
	
	int AddConcept(const TStr& Uri) { return ConceptsH.AddKey(Uri); }
	int AddSource(const TStr& SourceUri) { return SourcesH.AddKey(SourceUri); }
	int AddLocation(const TStr& Location) { return LocationsH.AddKey(Location); }
	TStr GetConcept(const int& ConceptId) { return ConceptsH.GetKey(ConceptId); }
	TStr GetSource(const int& SourceId) { return SourcesH.GetKey(SourceId); }
	TStr GetLocation(const int& LocationId) { return LocationsH.GetKey(LocationId); }

	void NotifyVerbose(const int VerbosityLevel, const char *Str);
	void NotifyVerboseFmt(const int VerbosityLevel, const char *FmtStr, ...);	
};
*/

#endif


