#include <base.h>
#include <qminer_srv.h>

#include "../NewsMiner/NewsBase.h"
#include "../NewsMiner/NewsCounts.h"
#include "../NewsMiner/NewsSearch.h"
//#include "../UserStore.h"
#include "../NewsMiner/NewsSrvUtil.h"

#ifndef ARTICLESRV_H
#define ARTICLESRV_H

class TArticleSrvUtil;

///////////////////////////////////////////////
// article server
class TArticleSrv : public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PNewsSrvUtil NewsSrvUtil;
	PNewsSearch NewsSearch;
	PStore ArticleStore;

	enum MatrixScoringMethod { pairTfIdf, chiSquare, pmi };

public:
	TArticleSrv(const PNewsBase& _NewsBase, const PNewsSearch& _NewsSearch, const PNewsSrvUtil& _NewsSrvUtil);
	static PSAppSrvFun New(const PNewsBase& NewsBase, const PNewsSearch& NewsSearch, const PNewsSrvUtil& NewsSrvUtil) {
		return new TArticleSrv(NewsBase, NewsSearch, NewsSrvUtil);
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);

	PJsonVal GetArticle(const TStrKdV& FldNmValPrV);
	PJsonVal GetArticles(const TStrKdV& FldNmValPrV);
	PJsonVal GetLatestArticles(const TStrKdV& FldNmValPrV);
	PJsonVal GetArticlesForUrls(const TStrKdV& FldNmValPrV);
	PJsonVal GetOriginalArticle(const TStrKdV& FldNmValPrV);

private:
	PJsonVal GetSingleArticle(const TStr& Uri, const TStrKdV& FldNmValPrV);

	PJsonVal ArticleTimeAggr(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet, const TStr& FldNmPrefix = "timeAggr");
	PJsonVal ArticleCategoryAggr(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet);
	PJsonVal ArticleKeywordAggr(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet, const TStr& Lang, const int& KeywordCount);
	PJsonVal ArticleConceptAggr(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet);
	PJsonVal ArticleConceptMatrix(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet);
	PJsonVal ArticleConceptGraph(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet);
	PJsonVal ArticleTrendingConcepts(const TStrKdV& FldNmValPrV, const PRecSet& ArticleRecSet);

	void GetConceptPairs(const PRecSet& ArticleRecSet, const EnumConceptType& ValidTypes, THash<TPair<TUInt64, TUInt64>, TInt>& ConceptPairToCntH, THash<TUInt64, TInt>& ConceptToCntH, const int SampleSize = 1000);
	inline double Sqr(const double& Val) { return Val*Val; }
};



#endif