#ifndef CONCEPTCLASSSRV_H
#define CONCEPTCLASSSRV_H

#include <qminer_srv.h>
#include "../NewsMiner/NewsBase.h"
#include "../NewsMiner/NewsSrvUtil.h"

class TConceptClassBase;
typedef TPt<TConceptClassBase> PConceptClassBase;
class TConceptClassBase {
public:
	friend class TPt<TConceptClassBase>;

private:
	TCRef CRef;

	PNewsBase NewsBase;
	// index to base
	PBase Base;
	PIndex Index;
	PStore ConceptStore;
	PStore ConceptClassStore;
	uchar ConceptStoreId;
	uchar ConceptClassStoreId;

	TInt NextUriId;
	TStrV AvailableClassUriV;

	TStr CustomClassPrefix;

	TConceptClassBase(const PNewsBase& _NewsBase);
	~TConceptClassBase() { Save(); }

	bool GetAvailableClassUri(TStr& Uri);

	// create next valid uri for the concept class
	TStr GetNextUri();

public:

	static PConceptClassBase New(const PNewsBase& _NewsBase) {
		return new TConceptClassBase(_NewsBase);
	}

	void Load();
	void Save();

	bool ImportClasses(const TStr& FName, TStr& ErrMsg);
	bool ImportInstances(const TStr& FName, TStr& ErrMsg);

	uint64 AddClass(const TStr& Label, const TStr& Description, const TStr& Type, const TStrV& ConceptUriV, const TStr& SuperClassUri);

	void SetLabel(const TStr& Uri, const TStr& Label, const TStr& Lang = DefLang);
	TStr GetLabel(const TStr& Uri, const TStr& Lang) const;
	bool HasLabel(const TStr& Uri, const TStr& Lang) const;
	PJsonVal GetLabels(const TStr& Uri) const;

	void SetSuperClass(const TStr& Uri, const TStr& SuperUri);

	int AddInstancesToClass(const TStr& Uri, const TUInt64V& ConceptIdV);
	void RemoveInstancesFromClass(const TStr& Uri, const TUInt64V& ConceptIdV);
	void DeleteClass(const TStr& Uri);

	PRecSet GetInstances(const TStr& Uri) const;

	void ConceptUriToV(const TStrV& ConceptUriV, TUInt64V& ConceptIdV) const;

	// get direct and indirect children categories of a class
	TUInt64Set GetAllSubClasses(const TStr& ClassUri) const;
	TUInt64Set GetAllSubClasses(const uint64& ClassId) const;

	TUInt64Set GetSubClasses(const TStr& ClassUri) const;
	TUInt64Set GetSubClasses(const uint64& ClassId) const;

	void GetValidConceptClasses(const TStrSet& TypeH, TStrV& ConceptClassUriV);
};

///////////////////////////////////////////////
// Concept Class base server
class TConceptClassSrvBase : public TSAppSrvFun {
protected:
	PNewsBase NewsBase;
	PNewsSrvUtil NewsSrvUtil;
	PConceptClassBase ConceptClassBase;

	TConceptClassSrvBase(const TStr& FunNm, const PNewsBase& _NewsBase, const PConceptClassBase& _ConceptClassBase, const PNewsSrvUtil& _NewsSrvUtil, const bool& _LogRqToFile);

public:
	PJsonVal ConceptClass(const uint64& ClassId, const bool& IncludeConceptsAndDescription = false, const TStrV& LabelLanguageV = NewsConst.LangCoreV) const;
};


///////////////////////////////////////////////
// Concept Class server
class TConceptClassSrv : public TConceptClassSrvBase {
protected:
	TConceptClassSrv(const PNewsBase& _NewsBase, const PConceptClassBase& _ConceptClassBase, const PNewsSrvUtil& _NewsSrvUtil, const bool& _LogRqToFile):
		TConceptClassSrvBase("conceptClass", _NewsBase, _ConceptClassBase, _NewsSrvUtil, _LogRqToFile) { }
public:
	static PSAppSrvFun New(const PNewsBase& NewsBase, const PConceptClassBase& ConceptClassBase, const PNewsSrvUtil& NewsSrvUtil, const bool& LogRqToFile) {
		return new TConceptClassSrv(NewsBase, ConceptClassBase, NewsSrvUtil, LogRqToFile);
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);

	PJsonVal GetClass(const TStrKdV& FldNmValPrV);
};

#endif
