/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "ConceptDailyTrendsSrv.h"



///////////////////////////////////////////////
// Daily concept trends
///////////////////////////////////////////////
TStr TConceptDailyTrendsExSrv::ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv)
{
	PJsonVal ResJson = NULL;
	try {
		TStr Action = GetFldVal(FldNmValPrV, "action", "");
		if (Action == "getTopTrends")
			ResJson = GetTopTrends(FldNmValPrV);
		else if (Action == "getCustomTopTrends")
			ResJson = GetCustomTopTrends(FldNmValPrV);
		else if (Action == "getConceptDailyFreq")
			ResJson = GetConceptDailyFreq(FldNmValPrV);
		else if (Action == "getArticlesForTopTrends")
			ResJson = GetArticlesForTopTrends(FldNmValPrV);
		else if (Action == "getTrendGroups")
			ResJson = GetTrendGroups(FldNmValPrV);
		else if (Action == "getTopConcepts")
			ResJson = GetTopConcepts(FldNmValPrV);
		else if (Action == "getTrendClusters")
			ResJson = GetTrendClusters(FldNmValPrV);
		else if (Action == "getConceptPage")
			ResJson = GetConceptPage(FldNmValPrV);
		else if (Action == "getConceptStories")
			ResJson = GetConceptStories(FldNmValPrV);
		else if (Action == "getConceptEvents")
			ResJson = GetConceptEvents(FldNmValPrV);
		else if (Action == "getStory")
			ResJson = GetStory(FldNmValPrV);
		else if (Action == "getEvent")
			ResJson = GetEvent(FldNmValPrV);
		else if (Action == "getArticle")
			ResJson = GetArticle(FldNmValPrV);
		else if (Action == "getSourcePageInfo")
			ResJson = SourcePageInfo(FldNmValPrV);
		else if (Action == "getFeaturedEventsAndConcepts")
			ResJson = GetFeaturedEventsAndConcepts(FldNmValPrV);
		else
			ResJson = TJsonVal::NewObj("error", TStr::Fmt("Unknown action '%s' provided. Known actions are getTopTrends, getCustomTopTrends, getArticlesForTopTrends, getConceptPage, getConceptStories, getStory, getSourcePageInfo, getFeaturedEventsAndConcepts.", Action.CStr()));

		// some method calls return null so we have to create an empty json object
		if (ResJson.Empty())
			ResJson = TJsonVal::NewObj();
	}
	catch (PExcept E) {
		ResJson = TJsonVal::NewObj();
		NewsBase->ErrorLogFile->NotifyErr("TConceptDailyTrendsExSrv::ExecJSon.", E);
		TNotify::StdNotify->OnStatus("TConceptDailyTrendsExSrv::ExecJSon. Error: " + E->GetMsgStr());
	}
	catch (...) {
		ResJson = TJsonVal::NewObj("error", TJsonVal::NewStr("General exception."));
		NewsBase->ErrorLogFile->NotifyErr("TConceptDailyTrendsExSrv::ExecJSon. General exception.");
	}

	// handle jsonP requests
	if (IsFldNm(FldNmValPrV, "callback")) {
		TStr Callback = GetFldVal(FldNmValPrV, "callback", "");
		return Callback + "(" + TJsonVal::GetStrFromVal(ResJson) + ")";
	}
	else
		return TJsonVal::GetStrFromVal(ResJson);
}

// return top concepts for the default arguments (used when calling SetParams)
PJsonVal TConceptDailyTrendsExSrv::GetTopTrends(const TStrKdV& FldNmValPrV)
{
	if (!ConceptDailyTrends->HasParams())
		return TJsonVal::NewObj("error", TStr("Parameters have not been set yet. Unable to provide trending articles. Call first the setParams action."));

	if (!ConceptDailyTrends->HasTopTrends())
		ConceptDailyTrends->UpdateTrends();

	if (ConceptDailyTrends->HasTopTrends())
		return ConceptDailyTrends->GetTopTrends();
	else
		return TJsonVal::NewObj("error", TStr("Unknown error. Unable to obtain top trends."));
}

// return top concept for custom settings
PJsonVal TConceptDailyTrendsExSrv::GetCustomTopTrends(const TStrKdV& FldNmValPrV)
{
	if (!ConceptDailyTrends->HasParams())
		return TJsonVal::NewObj("error", TStr("Parameters have not been set yet. Unable to provide trending articles. Call first the setParams action."));

	// the width of one day, two or three
	TInt Width = GetFldInt(FldNmValPrV, "width", 1);
	const int RetCount = GetFldInt(FldNmValPrV, "retCount", 30);
	TStr ConceptTypes = GetFldVal(FldNmValPrV, "conceptType", "person,org,loc");	// which concept types should be included
	TStrV ConceptTypeV; ConceptTypes.SplitOnAllCh(',', ConceptTypeV);
	EnumConceptType ValidTypes = NewsConst.GetValidTypes(ConceptTypeV);
	TUInt64V TopConceptV; ConceptDailyTrends->GetTopTrends(ValidTypes, RetCount, TopConceptV);
	return ConceptDailyTrends->ConceptTrends(TopConceptV);
}


// provided a set of conceptUri=... return [{concept info + }]
PJsonVal TConceptDailyTrendsExSrv::GetConceptDailyFreq(const TStrKdV& FldNmValPrV)
{
	TStrV ConceptUriV; GetFldValV(FldNmValPrV, "conceptUri", ConceptUriV);
	TUInt64V ConceptIdV;
	for (int N = 0; N < ConceptUriV.Len(); N++) {
		if (NewsBase->ConceptStore->IsRecNm(ConceptUriV[N]))
			ConceptIdV.Add(NewsBase->ConceptStore->GetRecId(ConceptUriV[N]));
	}

	return ConceptDailyTrends->GetConceptDailyFreq(ConceptIdV);
}

PJsonVal TConceptDailyTrendsExSrv::GetArticlesForTopTrends(const TStrKdV& FldNmValPrV)
{
	if (!ConceptDailyTrends->HasParams())
		return TJsonVal::NewObj("error", TStr("Parameters have not been set yet. Unable to provide trending articles. Call first the setParams action."));
	TStrV ConceptLangV; GetFldValV(FldNmValPrV, "conceptLang", ConceptLangV);
	if (ConceptLangV.Empty()) ConceptLangV.Add("");
	TInt Page = GetFldInt(FldNmValPrV, "page", 0);
	TInt Count = GetFldInt(FldNmValPrV, "count", 20);

	TUInt64V TopConceptIdV;
	ConceptDailyTrends->GetTopConceptsByType(ctEntities, 30, TopConceptIdV);
	PRecSet ArticleSet = ConceptDailyTrends->GetArticlesForConcepts(TopConceptIdV);
	return ConceptDailyTrends->GetArticles(ArticleSet, ConceptLangV, Page, Count);
}

// get concept info (name, desc, img) + trends for last month
// include also the latest storyCount stories
PJsonVal TConceptDailyTrendsExSrv::GetConceptPage(const TStrKdV& FldNmValPrV)
{
	const TStr ConceptUri = GetConceptUri(FldNmValPrV);
	const int StoryCount = GetFldInt(FldNmValPrV, "storyCount", 20);
	const int EventCount = GetFldInt(FldNmValPrV, "eventCount", 20);
	// the concept should have at least MinConceptScoreInStory importance in the story (value range 0-100)
	const int MinConceptScoreInStory = GetFldInt(FldNmValPrV, "minConceptScoreInStory", 20);

	const int ArticleCount = GetFldInt(FldNmValPrV, "articleCount", 20);
	// concepts in the returned articles should have their importance higher than MinConceptScoreInArticle (range 0-5)
	const int MinConceptScoreInArticle = GetFldInt(FldNmValPrV, "minConceptScoreInArticle", 3);
	// return only articles less than articleTimeWindow days old
	const int ArticleTimeWindow = GetFldInt(FldNmValPrV, "articleTimeWindow", 2);

	const int RelatedConceptCount = GetFldInt(FldNmValPrV, "relatedConceptCount", 20);
	if (!NewsBase->ConceptStore->IsRecNm(ConceptUri))
		return TJsonVal::NewObj("error", TJsonVal::NewStr(TStr::Fmt("Unknown concept uri %s", ConceptUri.CStr())));
	uint64 ConceptRecId = NewsBase->ConceptStore->GetRecId(ConceptUri);


	PJsonVal ResJson = TJsonVal::NewObj();
	ResJson->AddToObj("conceptInfo", ConceptDailyTrends->GetConceptInfo(ConceptRecId));
	ResJson->AddToObj("conceptMonthlyTrend", ConceptDailyTrends->GetConceptTrendArray(ConceptRecId, ConceptDailyTrends->GetMonthlyTrendV()));
	if (RelatedConceptCount > 0)
		ResJson->AddToObj("relatedConcepts", ConceptDailyTrends->GetRelatedConceptsFromLatestArticles(ConceptRecId, RelatedConceptCount));
	if (StoryCount > 0)
		ResJson->AddToObj("stories", ConceptDailyTrends->GetConceptStories(ConceptRecId, 0, StoryCount, MinConceptScoreInStory));
	if (EventCount > 0) {
		const int EventsMaxDaysBack = GetFldInt(FldNmValPrV, "eventsMaxDaysBack", TInt::Mx);
		const int EventsMaxDaysAhead = GetFldInt(FldNmValPrV, "eventsMaxDaysAhead", 1);
		const TStr EventLang = GetFldVal(FldNmValPrV, "eventLang", "eng");
		ResJson->AddToObj("events", ConceptDailyTrends->GetConceptEvents(ConceptRecId, 0, EventCount, EventLang, MinConceptScoreInStory, EventsMaxDaysBack, EventsMaxDaysAhead));
	}
	ResJson->AddToObj("articles", ConceptDailyTrends->GetConceptArticles(ConceptRecId, 0, ArticleCount, MinConceptScoreInArticle));
	return ResJson;
}

// get story summaries for the concept
PJsonVal TConceptDailyTrendsExSrv::GetConceptStories(const TStrKdV& FldNmValPrV)
{
	const TStr ConceptUri = GetConceptUri(FldNmValPrV);
	const int Page = GetFldInt(FldNmValPrV, "page", 0);
	const int Count = GetFldInt(FldNmValPrV, "count", 20);
	const int MinConceptScoreInStory = GetFldInt(FldNmValPrV, "minConceptScoreInStory", 20);

	if (!NewsBase->ConceptStore->IsRecNm(ConceptUri))
		return TJsonVal::NewObj("error", TJsonVal::NewStr(TStr::Fmt("Unknown concept uri %s", ConceptUri.CStr())));
	uint64 ConceptRecId = NewsBase->ConceptStore->GetRecId(ConceptUri);
	PJsonVal RetJson = TJsonVal::NewObj("stories", ConceptDailyTrends->GetConceptStories(ConceptRecId, Page, Count, MinConceptScoreInStory));
	return RetJson;
}

// get story summaries for the concept
PJsonVal TConceptDailyTrendsExSrv::GetConceptEvents(const TStrKdV& FldNmValPrV)
{
	const TStr ConceptUri = GetConceptUri(FldNmValPrV);
	const int Page = GetFldInt(FldNmValPrV, "page", 0);
	const int Count = GetFldInt(FldNmValPrV, "count", 20);
	const int EventsMaxDaysBack = GetFldInt(FldNmValPrV, "eventsMaxDaysBack", TInt::Mx);
	const int EventsMaxDaysAhead = GetFldInt(FldNmValPrV, "eventsMaxDaysAhead", 1);
	const TStr EventLang = GetFldVal(FldNmValPrV, "eventLang", "eng");
	const int MinConceptScoreInStory = GetFldInt(FldNmValPrV, "minConceptScoreInEvent", 20);

	if (!NewsBase->ConceptStore->IsRecNm(ConceptUri))
		return TJsonVal::NewObj("error", TJsonVal::NewStr(TStr::Fmt("Unknown concept uri %s", ConceptUri.CStr())));
	uint64 ConceptRecId = NewsBase->ConceptStore->GetRecId(ConceptUri);
	PJsonVal RetJson = TJsonVal::NewObj("events", ConceptDailyTrends->GetConceptEvents(ConceptRecId, Page, Count, EventLang, MinConceptScoreInStory, EventsMaxDaysBack, EventsMaxDaysAhead));
	return RetJson;
}

// return the articles assigned to the story
// if page == 0 also include the story information
PJsonVal TConceptDailyTrendsExSrv::GetEvent(const TStrKdV& FldNmValPrV)
{
	const int Page = GetFldInt(FldNmValPrV, "page", 0);
	const int Count = GetFldInt(FldNmValPrV, "count", 20);
	const TStr EventUri = GetFldVal(FldNmValPrV, "uri", "");
	const int ConceptCount = GetFldInt(FldNmValPrV, "conceptCount", 20);
	const int RelatedConceptCount = GetFldInt(FldNmValPrV, "relatedConceptCount", 20);
	TStrV ConceptTypeV; GetFldValV(FldNmValPrV, "conceptType", ConceptTypeV);
	EnumConceptType ValidTypes = NewsConst.GetValidTypes(ConceptTypeV, ctEntities);
	const TStr EventLang = GetFldVal(FldNmValPrV, "eventLang", "eng");
	const TStr SortBy = GetFldVal(FldNmValPrV, "sortBy", "rel").GetLc();	// date | rel

	if (!NewsBase->EventStore->IsRecNm(EventUri))
		return TJsonVal::NewObj("error", TJsonVal::NewStr(TStr::Fmt("Unknown event uri %s", EventUri.CStr())));
	const uint64 EventRecId = NewsBase->EventStore->GetRecId(EventUri);

	PJsonVal ResJson = TJsonVal::NewObj();
	// if first page then include story info and related concepts
	if (Page == 0) {
		ResJson->AddToObj("eventInfo", ConceptDailyTrends->GetEvent(EventRecId, ConceptCount, EventLang, NewsConst.LangCoreV, ValidTypes));
		PRecSet RelatedConceptSet = NewsBase->EventStore->GetRec(EventRecId).DoJoin(NewsBase->Base, "hasConcept");
		ResJson->AddToObj("relatedConcepts", ConceptDailyTrends->GetConceptTrends(RelatedConceptSet, RelatedConceptCount, TIntSet(), ctEntities));
	}

	// add location
	TRec LocRec = NewsBase->EventStore->GetRec(EventRecId).DoSingleJoin(NewsBase->Base, "hasLocation");
	if (LocRec.IsDef())
		ResJson->AddToObj("location", ConceptDailyTrends->GetNewsSrvUtil()->Location(LocRec.GetRecId()));

	// add articles
	PRecSet StorySet = NewsBase->EventStore->GetRec(EventRecId).DoJoin(NewsBase->Base, "hasStory");
	PRecSet ArticleRecSet = StorySet->DoJoin(NewsBase->Base, "hasArticle");

	// sort by decreasing time
	if (SortBy == "date") ArticleRecSet->SortByField(false, NewsBase->ArticleStore->GetFieldId("DateTime"));
	else				  ArticleRecSet->SortByField(false, NewsBase->ArticleStore->GetFieldId("CosToCentroid"));

	ResJson->AddToObj("articles", ConceptDailyTrends->GetStoryArticles(ArticleRecSet, Page, Count));
	return ResJson;
}

// return the articles assigned to the story
// if page == 0 also include the story information
PJsonVal TConceptDailyTrendsExSrv::GetStory(const TStrKdV& FldNmValPrV)
{
	const int Page = GetFldInt(FldNmValPrV, "page", 0);
	const int Count = GetFldInt(FldNmValPrV, "count", 20);
	const TStr StoryUri = GetFldVal(FldNmValPrV, "uri", "");
	const int RelatedConceptCount = GetFldInt(FldNmValPrV, "relatedConceptCount", 20);
	TStrV ConceptTypeV; GetFldValV(FldNmValPrV, "conceptType", ConceptTypeV);
	EnumConceptType ValidTypes = NewsConst.GetValidTypes(ConceptTypeV, ctEntities);
	const TStr SortBy = GetFldVal(FldNmValPrV, "sortBy", "rel").GetLc();	// date | rel

	if (!NewsBase->StoryStore->IsRecNm(StoryUri))
		return TJsonVal::NewObj("error", TJsonVal::NewStr(TStr::Fmt("Unknown story uri %s", StoryUri.CStr())));
	const uint64 StoryRecId = NewsBase->StoryStore->GetRecId(StoryUri);

	PJsonVal ResJson = TJsonVal::NewObj();
	// if first page then include story info and related concepts
	if (Page == 0) {
		ResJson->AddToObj("storyInfo", ConceptDailyTrends->GetStory(StoryRecId, 20, NewsConst.LangCoreV, ValidTypes));
		PRecSet RelatedConceptSet = NewsBase->StoryStore->GetRec(StoryRecId).DoJoin(NewsBase->Base, "hasConcept");
		ResJson->AddToObj("relatedConcepts", ConceptDailyTrends->GetConceptTrends(RelatedConceptSet, RelatedConceptCount, TIntSet(), ctEntities));
	}

	// add articles
	PRecSet ArticleRecSet = NewsBase->StoryStore->GetRec(StoryRecId).DoJoin(NewsBase->Base, "hasArticle");

	// sort by decreasing time
	if (SortBy == "date") ArticleRecSet->SortByField(false, NewsBase->ArticleStore->GetFieldId("DateTime"));
	else				  ArticleRecSet->SortByField(false, NewsBase->ArticleStore->GetFieldId("CosToCentroid"));

	ResJson->AddToObj("articles", ConceptDailyTrends->GetStoryArticles(ArticleRecSet, Page, Count));
	return ResJson;
}

PJsonVal TConceptDailyTrendsExSrv::GetArticle(const TStrKdV& FldNmValPrV)
{
	const TStr ArticleUri = GetFldVal(FldNmValPrV, "uri", "");
	const int ConceptCount = GetFldInt(FldNmValPrV, "conceptCount", 20);

	if (!NewsBase->ArticleStore->IsRecNm(ArticleUri))
		return TJsonVal::NewObj("error", TJsonVal::NewStr(TStr::Fmt("Unknown article uri %s", ArticleUri.CStr())));
	const uint64 ArticleRecId = NewsBase->ArticleStore->GetRecId(ArticleUri);

	PJsonVal ResJson = ConceptDailyTrends->GetNewsSrvUtil()->Article(ArticleRecId, -1, ctNone);

	// add related concepts
	PJsonVal ConcArr = TJsonVal::NewArr();
	PRecSet ConceptSet = NewsBase->ArticleStore->GetRec(ArticleRecId).DoJoin(NewsBase->Base, "hasConcept");
	ConceptSet->SortByFq(false);
	const int RelatedTotal = ConceptSet->GetRecs();
	const EnumConceptType ValidTypes = ctEntities;
	TUInt64V ConceptIdV;
	// take the daily trends computed for each day
	for (int ConcN = 0; ConcN < RelatedTotal && ConceptIdV.Len() < ConceptCount; ConcN++) {
		TConceptId ConceptId = ConceptSet->GetRecId(ConcN);
		// don't add the same concept or a hidden one
		if (ConceptDailyTrends->IsHidden(ConceptId))
			continue;
		TStr Type = NewsBase->ConceptStore->GetFieldNmStr((uint64) ConceptId, "Type");
		if (!NewsConst.IsValidType(Type, ValidTypes))
			continue;
		ConceptIdV.Add((uint64) ConceptId);
	}
	ResJson->AddToObj("concepts", ConceptDailyTrends->ConceptTrends(ConceptIdV, 30));
	return ResJson;
}

// get trends for individual concept types and individual concept classes
PJsonVal TConceptDailyTrendsExSrv::GetTrendGroups(const TStrKdV& FldNmValPrV)
{
	// number of concepts per group
	const int ConceptCount = GetFldInt(FldNmValPrV, "conceptCount", -1);
	// trends for individual concept types, such as person, org, location
	TStrV ConceptTypeV; GetFldValV(FldNmValPrV, "conceptType", ConceptTypeV);
	const EnumConceptType ValidTypes = NewsConst.GetValidTypes(ConceptTypeV, ctNone);
	// trends for specified concept classes
	TStrV ConceptClassUriV; GetFldValV(FldNmValPrV, "conceptClassUri", ConceptClassUriV);

	TJsonValV GroupArr;
	for (int N = 0; N < NewsConst.ConceptTypeV.Len(); N++) {
		const TStr Type = NewsConst.ConceptTypeV[N];
		if (NewsConst.IsValidType(Type, ValidTypes)) {
			TStrV SingleTypeV; SingleTypeV.Add(Type);
			EnumConceptType ValidType = NewsConst.GetValidTypes(SingleTypeV);
			TUInt64V ConceptIdV;  ConceptDailyTrends->GetTopConceptsByType(ValidType, ConceptCount, ConceptIdV);
			PJsonVal Trend = TJsonVal::NewObj("label", NewsConst.GetPluralTypeLabel(Type));
			Trend->AddToObj("type", TStr("Concept type"));
			Trend->AddToObj("trends", ConceptDailyTrends->ConceptTrends(ConceptIdV));
			GroupArr.Add(Trend);
		}
	}
	for (int N = 0; N < ConceptClassUriV.Len(); N++) {
		if (!NewsBase->ConceptClassStore->IsRecNm(ConceptClassUriV[N]))
			continue;
		const uint64 ConceptClassId = NewsBase->ConceptClassStore->GetRecId(ConceptClassUriV[N]);
		PRecSet ConceptSet = NewsBase->ConceptClassStore->GetRec(ConceptClassId).DoJoin(NewsBase->Base, "hasConcept");
		TUInt64V ConceptIdV; ConceptSet->GetRecIdV(ConceptIdV);
		TUInt64V TopConceptIdV;
		ConceptDailyTrends->GetTopConceptsFromList(ConceptIdV, ConceptCount, TopConceptIdV);
		PJsonVal Trend = TJsonVal::NewObj("label", NewsBase->ConceptClassStore->GetFieldNmStr(ConceptClassId, "Label"));
		Trend->AddToObj("uri", ConceptClassUriV[N]);
		Trend->AddToObj("type", TStr("Concept class"));
		Trend->AddToObj("trends", ConceptDailyTrends->ConceptTrends(TopConceptIdV));
		GroupArr.Add(Trend);
	}
	return TJsonVal::NewArr(GroupArr);
}

// get most concepts with highest popularity for the given concept classes
PJsonVal TConceptDailyTrendsExSrv::GetTopConcepts(const TStrKdV& FldNmValPrV)
{
	// number of concepts per group
	const int ConceptCount = GetFldInt(FldNmValPrV, "conceptCount", -1);
	// trends for specified concept classes
	TStrV ConceptClassUriV; GetFldValV(FldNmValPrV, "conceptClassUri", ConceptClassUriV);
	//const bool Normalize = GetFldBool(FldNmValPrV, "normalizeByIdf", true);

	TJsonValV GroupArr;
	for (int N = 0; N < ConceptClassUriV.Len(); N++) {
		if (!NewsBase->ConceptClassStore->IsRecNm(ConceptClassUriV[N]))
			continue;
		const uint64 ConceptClassId = NewsBase->ConceptClassStore->GetRecId(ConceptClassUriV[N]);
		PRecSet ConceptSet = NewsBase->ConceptClassStore->GetRec(ConceptClassId).DoJoin(NewsBase->Base, "hasConcept");
		THash<TUInt64, TInt> ConceptIdFqH;
		for (int C = 0; C < ConceptSet->GetRecs(); C++) {
			const uint64 Fq = NewsBase->NewsCounts->GetConceptFq(ConceptSet->GetRecId(C));
			ConceptIdFqH.AddDat(ConceptSet->GetRecId(C), (int) Fq);
		}
		ConceptIdFqH.SortByDat(false);
		TVec<TKeyDat<TUInt64, TInt> > ConceptV; ConceptIdFqH.GetKeyDatKdV(ConceptV);
		ConceptV.Trunc(ConceptCount);
		PRecSet TopConceptSet = TRecSet::New(NewsBase->ConceptStore, ConceptV, true);

		PJsonVal Trend = TJsonVal::NewObj("label", NewsBase->ConceptClassStore->GetFieldNmStr(ConceptClassId, "Label"));
		Trend->AddToObj("uri", ConceptClassUriV[N]);
		Trend->AddToObj("type", TStr("Concept class"));
		Trend->AddToObj("concepts", ConceptDailyTrends->GetNewsSrvUtil()->Concepts(TopConceptSet, TUInt64Set(), TInt::Mx, ctAll, NewsConst.LangCoreV));
		GroupArr.Add(Trend);
	}
	PJsonVal RetJson = TJsonVal::NewObj("topConcepts", TJsonVal::NewArr(GroupArr));
	RetJson->AddToObj("totalArticles", TJsonVal::NewNum(NewsBase->NewsCounts->GetArticleCount()));
	return RetJson;
}

// get trends for individual concept types and individual concept classes
PJsonVal TConceptDailyTrendsExSrv::GetTrendClusters(const TStrKdV& FldNmValPrV)
{
	const int TopConcepts = min(300, GetFldInt(FldNmValPrV, "topConcepts", 100));			// number of top trending concepts to consider
	const int ArticleSample = min(10000, GetFldInt(FldNmValPrV, "articleSample", 1000));	// latest articles to use when finding related concepts
	const TStr MinRatioStr = GetFldVal(FldNmValPrV, "minRatio", "0.4");						// min number of times the concept pair has to appear together in articles to be considered
	const int ClusterCount = GetFldInt(FldNmValPrV, "clusterCount", 5);						// num of clusters to return
	const double MinRatio = MinRatioStr.GetFlt(0.1);
	TStrV ConceptLangV; GetFldValV(FldNmValPrV, "conceptLang", ConceptLangV);
	if (ConceptLangV.Empty()) ConceptLangV.Add("");

	// get top 40 concepts
	TUInt64V TopConceptV; ConceptDailyTrends->GetTopTrends(ctEntities, TopConcepts, TopConceptV);
	TUInt64Set TopConceptH(TopConceptV);
	// compute how frequently each pair of top concepts appears together in the articles
	THash<TPair<TUInt64, TUInt64>, TFlt> ConceptPairToRatioH;
	for (int N = 0; N < TopConceptV.Len(); N++) {
		const uint64 Concept1 = TopConceptV[N];
		// find the articles for the concept
		PRecSet ArticleSet = NewsBase->ConceptStore->GetRec(Concept1).DoJoin(NewsBase->Base, "hasArticle");
		// take only the latest ArticleSample number of articles
		PRecSet ArticleSubSet = ArticleSet->GetLimit(ArticleSample, max(0, ArticleSet->GetRecs() - ArticleSample));
		// count how frequently the concepts from TopConceptH appear in these articles
		THash<TUInt64, TInt> ConceptFqH;
		for (int A = 0; A < ArticleSubSet->GetRecs(); A++) {
			PRecSet ConceptSet = ArticleSubSet->GetRec(A).DoJoin(NewsBase->Base, "hasConcept");
			for (int C = 0; C < ConceptSet->GetRecs(); C++) {
				if (TopConceptH.IsKey(ConceptSet->GetRecId(C)))
					ConceptFqH.AddDat(ConceptSet->GetRecId(C)) += 1;
			}
		}

		ConceptFqH.DelIfKey(Concept1);
		for (int KeyId = ConceptFqH.FFirstKeyId(); ConceptFqH.FNextKeyId(KeyId);) {
			const uint64 Concept2 = ConceptFqH.GetKey(KeyId);
			const int Fq = ConceptFqH[KeyId];
			TPair<TUInt64, TUInt64> Key = TPair<TUInt64, TUInt64>(min(Concept1, Concept2), max(Concept1, Concept2));
			const double Ratio = Fq / (double) ArticleSubSet->GetRecs();		// add the ratio of articles that contains both concepts
			ConceptPairToRatioH.AddDat(Key) += Ratio;
		}
	}

	// compute avg ratio (each concept is counted twice so we divide by 2)
	for (int KeyId = ConceptPairToRatioH.FFirstKeyId(); ConceptPairToRatioH.FNextKeyId(KeyId);) {
		ConceptPairToRatioH[KeyId] = ConceptPairToRatioH[KeyId] / 2.;
	}

	ConceptPairToRatioH.SortByDat(false);
	THash<TUInt64, TInt> ConceptToGroupH;	// concept -> index of the group
	TVec<TUInt64Set> ConceptGroupV;			// vector of concepts that belong to one group {c1,c2,...}
	for (int KeyId = ConceptPairToRatioH.FFirstKeyId(); ConceptPairToRatioH.FNextKeyId(KeyId);) {
		const double AvgRatio = ConceptPairToRatioH[KeyId];
		if (AvgRatio < MinRatio)
			break;
		TPair<TUInt64, TUInt64> Key = ConceptPairToRatioH.GetKey(KeyId);
		const uint64 Concept1 = Key.Val1;
		const uint64 Concept2 = Key.Val2;
		// first time we see any of the two concepts
		if (!ConceptToGroupH.IsKey(Concept1) && !ConceptToGroupH.IsKey(Concept2)) {
			TUInt64Set NewSet; NewSet.AddKey(Concept1); NewSet.AddKey(Concept2);
			ConceptToGroupH.AddDat(Concept1, ConceptGroupV.Len());
			ConceptToGroupH.AddDat(Concept2, ConceptGroupV.Len());
			ConceptGroupV.Add(NewSet);
		}
		// if one of the concepts is not yet seen - add the new concept to the group with the existing concept
		else if (!ConceptToGroupH.IsKey(Concept1) || !ConceptToGroupH.IsKey(Concept2)) {
			const uint64 NewConcept = ConceptToGroupH.IsKey(Concept1) ? Concept2 : Concept1;
			const uint64 ExistingConcept = ConceptToGroupH.IsKey(Concept1) ? Concept1 : Concept2;
			const int Index = ConceptToGroupH.GetDat(ExistingConcept);
			ConceptToGroupH.AddDat(NewConcept, Index);
			ConceptGroupV[Index].AddKey(NewConcept);
		}
		// if both concepts were already seen - merge the concepts from MaxInd to MinInd
		else {
			const int MinInd = min(ConceptToGroupH.GetDat(Concept1), ConceptToGroupH.GetDat(Concept2));
			const int MaxInd = max(ConceptToGroupH.GetDat(Concept1), ConceptToGroupH.GetDat(Concept2));
			if (MinInd == MaxInd)	// nothing to do if both concepts are already in same group
				continue;
			TUInt64Set Group = ConceptGroupV[MaxInd];
			for (int GroupKeyId = Group.FFirstKeyId(); Group.FNextKeyId(GroupKeyId);) {
				const uint64 Concept = Group.GetKey(GroupKeyId);
				ConceptToGroupH.AddDat(Concept, MinInd);	// reassing concept from MaxInd to MinInd
				ConceptGroupV[MinInd].AddKey(Concept);
			}
			ConceptGroupV[MaxInd] = TUInt64Set();
		}
	}

	TUInt64Set ConceptSet;
	TJsonValV ClusterArr;
	while (ClusterArr.Len() < ClusterCount && ConceptGroupV.Len() > 0) {
		// find the cluster with the most items
		int MaxInd = 0;
		for (int N = 1; N < ConceptGroupV.Len(); N++) {
			if (ConceptGroupV[N].Len() > ConceptGroupV[MaxInd].Len())
				MaxInd = N;
		}
		// ignore groups with less than 2 concepts
		// if we've come to the size less than 2 then we can break
		if (ConceptGroupV[MaxInd].Len() < 2)
			break;
		TUInt64Set& Set = ConceptGroupV[MaxInd];
		TJsonValV GroupInfoV;
		int Len = Set.Len();
		for (int i = 0; i < Len; i++) {
			for (int j = i + 1; j < Len; j++) {
				const TUInt64 Id1 = Set.GetKey(i), Id2 = Set.GetKey(j);
				// it's possible that we don't have a weight computed for a pair of concepts
				// in this case we just ignore the link
				if (!ConceptPairToRatioH.IsKey(TPair<TUInt64, TUInt64>(min(Id1, Id2), max(Id1, Id2))))
					continue;
				ConceptSet.AddKey(Id1); ConceptSet.AddKey(Id2);
				PJsonVal Item = TJsonVal::NewObj("id1", Id1.GetStr());
				Item->AddToObj("id2", Id2.GetStr());
				const double Wgt = ConceptPairToRatioH.GetDat(TPair<TUInt64, TUInt64>(min(Id1, Id2), max(Id1, Id2)));
				Item->AddToObj("wgt", Wgt);
				GroupInfoV.Add(Item);
			}
		}
		ClusterArr.Add(TJsonVal::NewArr(GroupInfoV));
		// remove the added cluster from the concept groups
		ConceptGroupV.Del(MaxInd);
	}

	PJsonVal ResJson = TJsonVal::NewObj("clusters", TJsonVal::NewArr(ClusterArr));

	PJsonVal ConceptJson = TJsonVal::NewObj();
	for (int KeyId = ConceptSet.FFirstKeyId(); ConceptSet.FNextKeyId(KeyId);) {
		const TUInt64 ConceptId = ConceptSet.GetKey(KeyId);
		ConceptJson->AddToObj(ConceptId.GetStr(), ConceptDailyTrends->GetNewsSrvUtil()->Concept(ConceptId, -1, ConceptLangV));
	}
	ResJson->AddToObj("concepts", ConceptJson);
	return ResJson;
}

// display source information + latest articles + related concepts
PJsonVal TConceptDailyTrendsExSrv::SourcePageInfo(const TStrKdV& FldNmValPrV) const
{
	TStr SourceUri = GetFldVal(FldNmValPrV, "sourceUri", "");
	if (!NewsBase->SourceStore->IsRecNm(SourceUri))
		return TJsonVal::NewObj("error", TStr::Fmt("Invalid source uri specified: %s", SourceUri.CStr()));
	uint64 SourceRecId = NewsBase->SourceStore->GetRecId(SourceUri);

	bool InclFullInfo = GetFldVal(FldNmValPrV, "inclFullInfo", "0") == "1";
	const int Page = GetFldInt(FldNmValPrV, "page", 0);
	const int Count = GetFldInt(FldNmValPrV, "count", 20);
	const int ConceptCount = GetFldInt(FldNmValPrV, "conceptCount", 20);

	PRecSet ArticleSet = NewsBase->SourceStore->GetRec(SourceRecId).DoJoin(NewsBase->Base, "hasArticle");

	int ArticleCount = ArticleSet->GetRecs();
	PRecSet ArticleSubSet = ArticleSet->GetLimit(Count, Page*Count);		// skip the first Offset articles and include only Count articles

	PJsonVal ResJson = TJsonVal::NewObj();
	// add articles
	ResJson->AddToObj("articles", ConceptDailyTrends->GetNewsSrvUtil()->Articles(ArticleSubSet, DefaultArticleBodyLen));

	ResJson->AddToObj("sourceInfo", ConceptDailyTrends->GetNewsSrvUtil()->Source(SourceRecId));

	if (InclFullInfo) {
		// get concepts from the last X articles
		PRecSet ArticleSubSet2 = ArticleSet->GetLimit(300, 0);
		ResJson->AddToObj("relatedConcepts", ConceptDailyTrends->GetRelatedConcepts(ArticleSubSet2));
	}

	return ResJson;
}

PJsonVal TConceptDailyTrendsExSrv::GetFeaturedEventsAndConcepts(const TStrKdV& FldNmValPrV)
{
	return ConceptDailyTrends->GetFeaturedEventsAndConcepts();
}

TStr TConceptDailyTrendsExSrv::GetConceptUri(const TStrKdV& FldNmValPrV) const
{
	TStr ConceptUri = GetFldVal(FldNmValPrV, "uri", "");
	if (ConceptUri != "")
		return ConceptUri;
	TStr ConceptName = GetFldVal(FldNmValPrV, "name", "");
	if (ConceptName != "")
		return "http://en.wikipedia.org/wiki/" + ConceptName;
	return "";
}


// /////////////////////
// TConceptDailyTrendsSrv
// /////////////////////
TStr TConceptDailyTrendsSrv::ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv)
{
	PJsonVal ResJson = NULL;
	try {
		TStr Action = GetFldVal(FldNmValPrV, "action", "");
		if (Action == "getTopTrends")
			ResJson = GetTopTrends(FldNmValPrV);
		else if (Action == "getTrendGroups")
			ResJson = GetTrendGroups(FldNmValPrV);
		else if (Action == "getFeaturedEventsAndConcepts")
			ResJson = GetFeaturedEventsAndConcepts(FldNmValPrV);
		else
			ResJson = TJsonVal::NewObj("error", TStr::Fmt("Unknown action '%s' provided. Known actions are getTopTrends, getCustomTopTrends, getArticlesForTopTrends, getConceptPage, getConceptStories, getStory, getSourcePageInfo, getFeaturedEventsAndConcepts.", Action.CStr()));

		// some method calls return null so we have to create an empty json object
		if (ResJson.Empty())
			ResJson = TJsonVal::NewObj();
	}
	catch (PExcept E) {
		ResJson = TJsonVal::NewObj();
		NewsBase->ErrorLogFile->NotifyErr("TConceptDailyTrendsExSrv::ExecJSon.", E);
		TNotify::StdNotify->OnStatus("TConceptDailyTrendsExSrv::ExecJSon. Error: " + E->GetMsgStr());
	}
	catch (...) {
		ResJson = TJsonVal::NewObj("error", TJsonVal::NewStr("General exception."));
		NewsBase->ErrorLogFile->NotifyErr("TConceptDailyTrendsExSrv::ExecJSon. General exception.");
	}

	// handle jsonP requests
	if (IsFldNm(FldNmValPrV, "callback")) {
		TStr Callback = GetFldVal(FldNmValPrV, "callback", "");
		return Callback + "(" + TJsonVal::GetStrFromVal(ResJson) + ")";
	}
	else
		return TJsonVal::GetStrFromVal(ResJson);
}


PJsonVal TConceptDailyTrendsSrv::GetTopTrends(const TStrKdV& FldNmValPrV)
{
	TStrV ConceptLangV; GetFldValV(FldNmValPrV, "conceptLang", ConceptLangV);
	if (ConceptLangV.Empty()) ConceptLangV.Add("eng");
	TStrV ConceptTypeV; GetFldValV(FldNmValPrV, "conceptType", ConceptTypeV);
	EnumConceptType ConceptTypes = NewsConst.GetValidTypes(ConceptTypeV, ctAll);
	const int ConceptCount = GetFldInt(FldNmValPrV, "conceptCount", 50);
	const TStr TrendName = GetFldVal(FldNmValPrV, "trendName", "default");
	EnumConceptInfoFlags ConceptFlags = NewsSrvUtil->GetConceptFlags(FldNmValPrV);
	ConceptFlags = (EnumConceptInfoFlags) ((int) ConceptFlags | picIncludeTrends);

	PJsonVal ArrJson = TJsonVal::NewArr();
	int Added = 0;
	PDailyTrendInfo TrendInfo = DailyTrendInfos->GetDailyTrendInfo(TrendName);
	if (TrendInfo.Empty())
		return TJsonVal::NewObj("error", TStr::Fmt("Trend name %s not found", TrendName.CStr()));
	const int ConcCount = TrendInfo->Trends->Len();
	for (int C = 0; C < ConcCount; C++) {
		TConceptId Id = TrendInfo->Trends->GetTopConceptId(C);
		// was the concept hidden by the admin?
		if (DailyTrendInfos->IsHidden(Id) || !NewsBase->ConceptStore->IsRecId((uint64) Id))
			continue;
		// do we want concepts of this type?
		TStr Type = NewsBase->ConceptStore->GetFieldNmStr((uint64) Id, "Type");
		if (!NewsConst.IsValidType(Type, ConceptTypes))
			continue;

		const TStr ConceptUri = NewsBase->ConceptStore->GetRecNm((uint64) Id);
		PJsonVal ConceptTrendJson = NewsSrvUtil->Concept((uint64) Id, -1, ConceptLangV, ConceptFlags);
		// send also info about test and null population frequencies so that we can see what kind of concepts we get as most popular
		ConceptTrendJson->AddToObj("testPopFq", TrendInfo->TestPop->GetConceptPop(Id));
		ConceptTrendJson->AddToObj("nullPopFq", TrendInfo->NullPop->GetConceptPop(Id));
		ArrJson->AddToArr(ConceptTrendJson);
		Added++;
		if (Added >= ConceptCount)
			break;
	}
	return ArrJson;
}

PJsonVal TConceptDailyTrendsSrv::GetTrendGroups(const TStrKdV& FldNmValPrV)
{
	// number of concepts per group
	const int ConceptCount = GetFldInt(FldNmValPrV, "conceptCount", -1);
	TStrV ConceptLangV; GetFldValV(FldNmValPrV, "conceptLang", ConceptLangV);
	if (ConceptLangV.Empty()) ConceptLangV.Add("eng");
	// trends for individual concept types, such as person, org, location
	TStrV ConceptTypeV; GetFldValV(FldNmValPrV, "conceptType", ConceptTypeV);
	const EnumConceptType ValidTypes = NewsConst.GetValidTypes(ConceptTypeV, ctNone);
	// trends for specified concept classes
	TStrV ConceptClassUriV; GetFldValV(FldNmValPrV, "conceptClassUri", ConceptClassUriV);
	const TStr TrendName = GetFldVal(FldNmValPrV, "trendName", "default");
	EnumConceptInfoFlags ConceptFlags = NewsSrvUtil->GetConceptFlags(FldNmValPrV);
	ConceptFlags = (EnumConceptInfoFlags) ((int) ConceptFlags | picIncludeTrends);

	PJsonVal ResJson = TJsonVal::NewObj();

	for (int N = 0; N < NewsConst.ConceptTypeV.Len(); N++) {
		const TStr Type = NewsConst.ConceptTypeV[N];
		if (NewsConst.IsValidType(Type, ValidTypes)) {
			TStrV SingleTypeV; SingleTypeV.Add(Type);
			EnumConceptType ValidType = NewsConst.GetValidTypes(SingleTypeV);
			TUInt64V ConceptIdV;  DailyTrendInfos->GetTopConceptsByType(TrendName, ValidType, ConceptCount, ConceptIdV);
			PJsonVal Trend = TJsonVal::NewObj("label", Type);
			Trend->AddToObj("type", TStr("Concept type"));
			Trend->AddToObj("trendingConcepts", NewsSrvUtil->Concepts(ConceptIdV, ConceptLangV, ConceptFlags));
			ResJson->AddToObj(Type, Trend);
		}
	}
	for (int N = 0; N < ConceptClassUriV.Len(); N++) {
		if (!NewsBase->ConceptClassStore->IsRecNm(ConceptClassUriV[N]))
			continue;
		const uint64 ConceptClassId = NewsBase->ConceptClassStore->GetRecId(ConceptClassUriV[N]);
		PRecSet ConceptSet = NewsBase->ConceptClassStore->GetRec(ConceptClassId).DoJoin(NewsBase->Base, "hasConcept");
		TUInt64V ConceptIdV; ConceptSet->GetRecIdV(ConceptIdV);
		TUInt64V TopConceptIdV;
		const TStr Label = NewsBase->ConceptClassStore->GetFieldNmStr(ConceptClassId, "Label");
		NewsBase->DailyTrendInfos->GetTopConceptsFromList(TrendName, ConceptIdV, ConceptCount, TopConceptIdV);
		PJsonVal Trend = TJsonVal::NewObj("label", Label);
		Trend->AddToObj("uri", ConceptClassUriV[N]);
		Trend->AddToObj("type", TStr("Concept class"));
		Trend->AddToObj("trendingConcepts", NewsSrvUtil->Concepts(TopConceptIdV, ConceptLangV, ConceptFlags));
		ResJson->AddToObj(Label, Trend);
	}
	return ResJson;
}

PJsonVal TConceptDailyTrendsSrv::GetFeaturedEventsAndConcepts(const TStrKdV& FldNmValPrV)
{
	TStrV ConceptLangV; GetFldValV(FldNmValPrV, "conceptLang", ConceptLangV);
	if (ConceptLangV.Empty()) ConceptLangV.Add("eng");
	TStrV ConceptTypeV; GetFldValV(FldNmValPrV, "conceptType", ConceptTypeV);
	EnumConceptType ConceptTypes = NewsConst.GetValidTypes(ConceptTypeV, ctAll);

	EnumEventInfoFlags EventFlags = NewsSrvUtil->GetEventFlags(FldNmValPrV);
	EnumConceptInfoFlags ConceptFlags = (EnumConceptInfoFlags) (picIncludePrimImage);

	TVec<TPair<TUInt64, TUInt64> > EventIdConceptIdPrV;
	DailyTrendInfos->GetFeaturedEventsAndConcepts(EventIdConceptIdPrV);

	TJsonValV FeaturedJsonV;
	for (int N = 0; N < EventIdConceptIdPrV.Len(); N++) {
		uint64 EventId = EventIdConceptIdPrV[N].Val1;
		const uint64 ConceptId = EventIdConceptIdPrV[N].Val2;

		if (!NewsBase->EventStore->IsRecId(EventId))
			continue;

		PRecSet StorySet = NewsBase->EventStore->GetRec(EventId).DoJoin(NewsBase->Base, "hasStory");
		// if no story replace EventId with a valid event id
		if (StorySet->GetRecs() == 0) {
			TRec PrimStoryRec = NewsBase->EventStore->GetRec(EventId).DoSingleJoin(NewsBase->Base, "hasPrimaryStory");
			TRec CurrEventRec = PrimStoryRec.DoSingleJoin(NewsBase->Base, "hasEvent");
			EventId = CurrEventRec.GetRecId();
		}
		PJsonVal FeaturedItem = TJsonVal::NewObj();
		FeaturedItem->AddToObj("event", NewsSrvUtil->Event(EventId, ConceptLangV, ConceptTypes, EventFlags));

		if (NewsBase->ConceptStore->IsRecId(ConceptId))
			FeaturedItem->AddToObj("concept", NewsSrvUtil->Concept(ConceptId, -1, ConceptLangV, ConceptFlags));

		FeaturedJsonV.Add(FeaturedItem);
	}
	return TJsonVal::NewArr(FeaturedJsonV);
}
