#ifndef CONCEPTDAILYTRENDSSRV
#define CONCEPTDAILYTRENDSSRV

#include "../Trends/ConceptDailyTrends.h"
#include "../Trends/ConceptTrendsManager.h"


// TConceptDailyTrendsExSrv web service
class TConceptDailyTrendsExSrv: public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PConceptDailyTrends ConceptDailyTrends;

public:
	static TConceptDailyTrendsExSrv* New(const PNewsBase& NewsBase, const PConceptDailyTrends& ConceptDailyTrends) {
		return new TConceptDailyTrendsExSrv(NewsBase, ConceptDailyTrends); }

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);

private:
	TConceptDailyTrendsExSrv(const PNewsBase& _NewsBase, const PConceptDailyTrends& _ConceptDailyTrends):
		TSAppSrvFun("conceptDailyTrendsEx", saotJSon), NewsBase(_NewsBase), ConceptDailyTrends(_ConceptDailyTrends) {	};

	// return an already-prepared json data
	PJsonVal GetTopTrends(const TStrKdV& FldNmValPrV);
	// get trends for custom concept types and count
	PJsonVal GetCustomTopTrends(const TStrKdV& FldNmValPrV);

	PJsonVal GetConceptDailyFreq(const TStrKdV& FldNmValPrV);
	// get concept info (name, desc, img) + trends for last month
	PJsonVal GetConceptPage(const TStrKdV& FldNmValPrV);
	// get story summaries for the concept
	PJsonVal GetConceptStories(const TStrKdV& FldNmValPrV);
	// get event summaries for the concept
	PJsonVal GetConceptEvents(const TStrKdV& FldNmValPrV);

	// get single event info (articles + other relevant info)
	PJsonVal GetEvent(const TStrKdV& FldNmValPrV);
	// get single story info (articles + other relevant info)
	PJsonVal GetStory(const TStrKdV& FldNmValPrV);
	PJsonVal GetArticle(const TStrKdV& FldNmValPrV);

	// get trends for individual concept types and individual concept classes
	PJsonVal GetTrendGroups(const TStrKdV& FldNmValPrV);
	PJsonVal GetTopConcepts(const TStrKdV& FldNmValPrV);
	PJsonVal GetTrendClusters(const TStrKdV& FldNmValPrV);

	// get articles for top trending concepts
	PJsonVal GetArticlesForTopTrends(const TStrKdV& FldNmValPrV);
	PJsonVal SourcePageInfo(const TStrKdV& FldNmValPrV) const;
	// return the user-set featured events and concepts
	PJsonVal GetFeaturedEventsAndConcepts(const TStrKdV& FldNmValPrV);

	TStr GetConceptUri(const TStrKdV& FldNmValPrV) const;
};

class TConceptDailyTrendsSrv : public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PNewsSrvUtil NewsSrvUtil;
	PDailyTrendInfos DailyTrendInfos;

public:
	static TConceptDailyTrendsSrv* New(const PNewsBase& NewsBase, const PNewsSrvUtil& NewsSrvUtil) {
		return new TConceptDailyTrendsSrv(NewsBase, NewsSrvUtil);
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);

private:
	TConceptDailyTrendsSrv(const PNewsBase& _NewsBase, const PNewsSrvUtil& _NewsSrvUtil) :
		TSAppSrvFun("conceptDailyTrends", saotJSon), NewsBase(_NewsBase), NewsSrvUtil(_NewsSrvUtil) {
		DailyTrendInfos = NewsBase->DailyTrendInfos;
	};

	// return an already-prepared json data
	PJsonVal GetTopTrends(const TStrKdV& FldNmValPrV);
	PJsonVal GetTrendGroups(const TStrKdV& FldNmValPrV);
	PJsonVal GetFeaturedEventsAndConcepts(const TStrKdV& FldNmValPrV);
};


#endif
