#ifndef EVENTCOMPARESRV_H
#define EVENTCOMPARESRV_H

#include <qminer_srv.h>
#include "../NewsMiner/NewsBase.h"
#include "../ServerArticles/EventRegBase.h"
#include "../ServerArticles/EventRegSrvUtil.h"

using namespace TQm;

// Event registry handler methods
class TEventCompareSrv : public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PEventRegBase EventRegBase;
	PEventRegSrvUtil EventRegSrvUtil;
	PNewsSrvUtil NewsSrvUtil;
	
	PStore ArticleStore;
	PStore ConceptStore;
	PStore StoryStore;
	PStore EventStore;

	uchar ArticleStoreId;
	uchar ConceptStoreId;
	uchar StoryStoreId;
	uchar EventStoreId;

	TEventCompareSrv(const PNewsBase& _NewsBase, const PEventRegBase _EventRegBase, const PNewsSrvUtil& _NewsSrvUtil, const PEventRegSrvUtil& _EventRegSrvUtil, const bool& _LogRqToFile) :
		TSAppSrvFun("eventCompare", saotJSon), NewsBase(_NewsBase), EventRegBase(_EventRegBase), NewsSrvUtil(_NewsSrvUtil), EventRegSrvUtil(_EventRegSrvUtil)
	{
		LogRqToFile = _LogRqToFile;
		ReportResponseSize = true;
		
		ArticleStore = NewsBase->ArticleStore;
		ConceptStore = NewsBase->ConceptStore;
		StoryStore = NewsBase->StoryStore;
		EventStore = EventRegBase->EventStore;

		ArticleStoreId = ArticleStore->GetStoreId();
		ConceptStoreId = ConceptStore->GetStoreId();
		StoryStoreId = StoryStore->GetStoreId();
		EventStoreId = EventStore->GetStoreId();
	}

public:
	static PSAppSrvFun New(const PNewsBase& NewsBase, const PEventRegBase EventRegBase, const PNewsSrvUtil& NewsSrvUtil, const PEventRegSrvUtil& EventRegSrvUtil, const bool& LogRqToFile) {
		return new TEventCompareSrv(NewsBase, EventRegBase, NewsSrvUtil, EventRegSrvUtil, LogRqToFile);
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);
	PJsonVal ProcessRequest(const TStrKdV& FldNmValPrV);

	bool GetEvents(const TStrKdV& FldNmValPrV, const int& GroupIndex, PRecSet& EventRecSet, PJsonVal& ErrorJson);

	PJsonVal CompareConcepts(const TStrKdV& FldNmValPrV, const PRecSet& EventRecSet0, const PRecSet& EventRecSet1);
	PJsonVal CompareLocations(const TStrKdV& FldNmValPrV, const PRecSet& EventRecSet0, const PRecSet& EventRecSet1);
	PJsonVal CompareLocationsWithTime(const TStrKdV& FldNmValPrV, const PRecSet& EventRecSet0, const PRecSet& EventRecSet1);
	PJsonVal CompareTimeline(const TStrKdV& FldNmValPrV, const PRecSet& EventRecSet0, const PRecSet& EventRecSet1);
	PJsonVal CompareCategories(const TStrKdV& FldNmValPrV, const PRecSet& EventRecSet0, const PRecSet& EventRecSet1);
};

#endif