#ifndef EVENTREGSRV_H
#define EVENTREGSRV_H

#include <qminer_srv.h>
#include "../NewsMiner/NewsBase.h"
#include "../NewsMiner/NewsSearch.h"
#include "../ServerArticles/EventRegBase.h"
#include "../ServerArticles/EventRegSrvUtil.h"
#include "../ServerArticles/EventStats.h"
//#include "../Srv/NewsClusterAdv.h"
#include "../Learning/ClusterEvaluation.h"

using namespace TQm;

///////////////////////////////////////////////
// Event registry handler methods
class TEventRegSrv: public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PEventRegBase EventRegBase;
	PNewsSearch NewsSearch;
	PEventRegSrvUtil EventRegSrvUtil;
	PNewsSrvUtil NewsSrvUtil;
	PStore ArticleStore;
	PStore StoryStore;
	PStore EventStore;
	uchar EventStoreId;

	TEventRegSrv(const PNewsBase& _NewsBase, const PEventRegBase _EventRegBase, const PNewsSearch& _NewsSearch, const PNewsSrvUtil& _NewsSrvUtil, const PEventRegSrvUtil& _EventRegSrvUtil, const bool& _LogRqToFile) :
		TSAppSrvFun("event", saotJSon), NewsBase(_NewsBase), EventRegBase(_EventRegBase), NewsSearch(_NewsSearch), NewsSrvUtil(_NewsSrvUtil), EventRegSrvUtil(_EventRegSrvUtil)
		{
			LogRqToFile = _LogRqToFile;
			ReportResponseSize = true;

			ArticleStore = NewsBase->ArticleStore;
			StoryStore = NewsBase->StoryStore;
			EventStore = EventRegBase->EventStore;
			EventStoreId = EventStore->GetStoreId();
		}

public:
	static PSAppSrvFun New(const PNewsBase& NewsBase, const PEventRegBase EventRegBase, const PNewsSearch& NewsSearch, const PNewsSrvUtil& NewsSrvUtil, const PEventRegSrvUtil& EventRegSrvUtil, const bool& LogRqToFile) {
		return new TEventRegSrv(NewsBase, EventRegBase, NewsSearch, NewsSrvUtil, EventRegSrvUtil, LogRqToFile);
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);

	uint64 GetEventId(const TStrKdV& FldNmValPrV);

	uint64 GetConceptId(const TStrKdV& FldNmValPrV);
	uint64 GetSourceId(const TStrKdV& FldNmValPrV);
	uint64 GetArticleId(const TStrKdV& FldNmValPrV);

	PJsonVal GetEvents(const TStrKdV& FldNmValPrV);
	PJsonVal GetEvent(const TStrKdV& FldNmValPrV);
	PJsonVal GetSingleEvent(const TStrKdV& FldNmValPrV, const TStr& EventUri);

	PJsonVal SimilarEvents(const TStrKdV& FldNmValPrV, const uint64& EventId);
	PJsonVal SimilarStories(const TStrKdV& FldNmValPrV, const uint64& EventId);

	PJsonVal GetSimilarConcepts(const TStrKdV& FldNmValPrV);

	PJsonVal GetConceptGraph(const TStrKdV& FldNmValPrV);

	PJsonVal GetUpdatedEvents(const TStrKdV& FldNmValPrV);

	PJsonVal GetEventIdForStory(const TStrKdV& FldNmValPrV);

	PJsonVal GetTrendingEvents(const TStrKdV& FldNmValPrV);

	PJsonVal GetConceptsDocAtlas(const TStrKdV& FldNmValPrV);

	PJsonVal GetDocAtlasOnConcepts(const TStrKdV& FldNmValPrV, const PRecSet& EventRecSet);
};

#endif
