#ifndef NEWSSRV_H
#define NEWSSRV_H

#include <base.h>

#include "../NewsMiner/NewsBase.h"
#include "../NewsMiner/NewsCounts.h"
#include "../NewsMiner/NewsSrvUtil.h"

///////////////////////////////////////////////
// Concept autocomplete administration
class TConceptAutoSuggestSrv: public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PNewsSrvUtil NewsSrvUtil;
	TInt CachedConceptCount;
	//PAutoCompConceptPrefixes AutoCompConceptPrefixes;

	typedef THash<TStr, TVec<TUIntIntKd> > PrefToSugg;
	THash<TStr, PrefToSugg > ConceptPrefixToIdWgtHH;
	THash<TUInt, TInt> ConceptIdToFqH;
	PNewsFixDb NewsFixDb;
	TInt NextLangInd;
	TUIntIntKdV ConceptIdFqV;			// sorted vector of all concepts
	
	// update data frequencies
	void UpdateDataCounts();

public:
	TConceptAutoSuggestSrv(const PNewsBase& _NewsBase, const PNewsSrvUtil& _NewsSrvUtil);
	static TConceptAutoSuggestSrv* New(const PNewsBase& NewsBase, const PNewsSrvUtil& NewsSrvUtil) {
		return new TConceptAutoSuggestSrv(NewsBase, NewsSrvUtil);
	}
	~TConceptAutoSuggestSrv();

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);

	void LoadOrUpdatePrefixes();
	void UpdatePrefixes(const bool& Report = true);
	PJsonVal FindConcepts(const TStrKdV& FldNmValPrV, EnumConceptType ValidTypes = ctAll);
	void GetConceptSuggestions(const TStr& Prefix, const TStr& Lang, TVec<TUIntIntKd>& SuggV) const;
};

///////////////////////////////////////////////
// Concept class autocomplete administration
class TConceptClassAutoSuggestSrv : public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PNewsSrvUtil NewsSrvUtil;

	typedef THash<TStr, TVec<TUInt64IntKd> > PrefToSugg;
	THash<TStr, PrefToSugg > ClassPrefixToIdWgtHH;

public:
	TConceptClassAutoSuggestSrv(const PNewsBase& _NewsBase, const PNewsSrvUtil& _NewsSrvUtil);
	static TConceptClassAutoSuggestSrv* New(const PNewsBase& NewsBase, const PNewsSrvUtil& NewsSrvUtil) {
		return new TConceptClassAutoSuggestSrv(NewsBase, NewsSrvUtil);
	}
	~TConceptClassAutoSuggestSrv();

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);

	void UpdatePrefixes();
	PJsonVal FindClasses(const TStrKdV& FldNmValPrV, const EnumConceptClassType& ClassType);
	void GetClassSuggestions(const TStr& Prefix, const TStr& Lang, TVec<TUInt64IntKd>& SuggV) const;
};

class TSourceAutoSuggestSrv: public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PNewsSrvUtil NewsSrvUtil;
	
	TVec<TUInt64> SourceIdV;			// sorted vector of all sources
	THash<TStr, TVec<TUInt64> > SourcePrefixToFqWIdV;

public:
	TSourceAutoSuggestSrv(const PNewsBase& _NewsBase, const PNewsSrvUtil& _NewsSrvUtil);
	static TSourceAutoSuggestSrv* New(const PNewsBase& NewsBase, const PNewsSrvUtil& NewsSrvUtil) {
		return new TSourceAutoSuggestSrv(NewsBase, NewsSrvUtil);
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);

	void UpdatePrefixes();
	void GetSourceSuggestions(TStr Prefix, TVec<TUInt64>& SuggV);
	TStr GetCleanSourceNameFromUri(const TStr& Source);
};


class TSourceLocationAutoSuggestSrv: public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PNewsSrvUtil NewsSrvUtil;
	PIndexVoc IndexVoc;
	
public:
	TSourceLocationAutoSuggestSrv(const PNewsBase& _NewsBase, const PNewsSrvUtil& _NewsSrvUtil);
		
	static PSAppSrvFun New(const PNewsBase& NewsBase, const PNewsSrvUtil& NewsSrvUtil) { 
		return new TSourceLocationAutoSuggestSrv(NewsBase, NewsSrvUtil);
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);
};

///////////////////////////////////////////////
// article server
class TPrintSrv : public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PNewsSrvUtil NewsSrvUtil;

	TStr GetPathName(const TStr& FName);
public:
	TPrintSrv(const PNewsBase& _NewsBase, const PNewsSrvUtil& _NewsSrvUtil);
	
	static PSAppSrvFun New(const PNewsBase& NewsBase, const PNewsSrvUtil& NewsSrvUtil) {
		return new TPrintSrv(NewsBase, NewsSrvUtil);
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);
	void PrintTopConcepts(const TStrKdV& FldNmValPrV);
};

///////////////////////////////////////////////
// overview server
class TOverviewSrv : public TSAppSrvFun {
private:
	PNewsBase NewsBase;
	PNewsSrvUtil NewsSrvUtil;

	TStr GetPathName(const TStr& FName);
public:
	TOverviewSrv(const PNewsBase& _NewsBase, const PNewsSrvUtil& _NewsSrvUtil);

	static PSAppSrvFun New(const PNewsBase& NewsBase, const PNewsSrvUtil& NewsSrvUtil) {
		return new TOverviewSrv(NewsBase, NewsSrvUtil);
	}

	TStr ExecJSon(const TStrKdV& FldNmValPrV, const PSAppSrvRqEnv& RqEnv);
	PJsonVal GetRecentStats(const TStrKdV& FldNmValPrV);
	
	PJsonVal GetArticleActivity(const TStrKdV& FldNmValPrV);
	PJsonVal GetEventActivity(const TStrKdV& FldNmValPrV);
	PJsonVal GetAllActivity(const TStrKdV& FldNmValPrV);
};

#endif