/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include <net.h>
#include "ConceptDailyTrendInfo.h"
#include "../NewsMiner/NewsBase.h"

// ///////////////////////
// TDailyTrendInfo
// ///////////////////////

// update trends with the specified values
PJsonVal TDailyTrendInfo::UpdateTrends()
{
	TNotify::StdNotify->OnStatus("Updating daily trends...");
	// clear cached TopTrendsJson - it should be recomputed since we have new concept trends
	//ClearCache();
	Trends = TConceptTrends::New();
	TestPop = TConceptPop::New();
	NullPop = TConceptPop::New();

	/*TStr Lang = TSAppSrvFun::GetFldVal(ParamsPrV, "labelLang", "eng");
	LabelFieldName = "Label" + Lang.GetCap();
	if (!NewsBase->ConceptClassStore->IsFieldNm(LabelFieldName))
		LabelFieldName = "Label";*/

	TStrV TestPopKeyV = TrendsManager->ParsePopKeys(TrendParams.TestPop);
	TStrV NullPopKeyV = TrendsManager->ParsePopKeys(TrendParams.NullPop);
	
	InfoLogger->NotifyInfo("Updating trends...");
	InfoLogger->NotifyInfo("Test population offsets: %s", TStr::GetStr(TestPopKeyV, ", ").CStr());
	InfoLogger->NotifyInfo("Null population offsets: %s", TStr::GetStr(NullPopKeyV, ", ").CStr());

	// validate the correctness of necessary params
	if (TestPopKeyV.Len() == 0) {
		TNotify::StdNotify->OnStatus("testPopOffs and testPopKeys did not contain any valid values for a population. Values should be of type 'Total', 'Y0', 'M1' or 'D2' or a comma separated list of them, like 'M1,M2,M3'.");
		return TJsonVal::NewObj("error", TStr("testPopOffs and testPopKeys did not contain any valid values for a population. Values should be of type 'Total', 'Y0', 'M1' or 'D2' or a comma separated list of them, like 'M1,M2,M3'."));
	}
	if (NullPopKeyV.Len() == 0) {
		TNotify::StdNotify->OnStatus("nullPopOffs and nullPopKeys did not contain any valid values for a population. Values should be of type 'Total', 'Y0', 'M1' or 'D2' or a comma separated list of them, like 'M1,M2,M3'.");
		return TJsonVal::NewObj("error", TStr("nullPopOffs and nullPopKeys did not contain any valid values for a population. Values should be of type 'Total', 'Y0', 'M1' or 'D2' or a comma separated list of them, like 'M1,M2,M3'."));
	}

	//const int ConceptCount = TSAppSrvFun::GetFldInt(ParamsPrV, "conceptCount", 30);
	/*TStrV ConceptTypeV; TSAppSrvFun::GetFldValV(ParamsPrV, "conceptType", ConceptTypeV);
	ValidTypes = NewsConst.GetValidTypes(ConceptTypeV);*/
	//InfoLogger->NotifyInfo("Ignore test concepts below fq = %d, Ignore null concepts below fq = %d, desired concept count = %d, concept types = %s", IgnoreTestConceptsBelowFq, IgnoreNullConceptsBelowFq, ConceptCount, TStr::GetStr(ConceptTypeV, ", ").CStr());

	// most popular trends computed using test and null distributions
	TestPop = TrendsManager->GetPopForKeys(TestPopKeyV);
	NullPop = TrendsManager->GetPopForKeys(NullPopKeyV);
	Trends = TrendsManager->ComparePops(TestPop, NullPop, TrendParams.IgnoreTestConceptsBelowFq, TrendParams.IgnoreNullConceptsBelowFq, true, tmPearsonResidual);

	const int ConcCount = Trends->Len();
	//PJsonVal ArrJson = TJsonVal::NewArr();
	//int Added = 0;
	//const int ConcCount = Trends->Len();
	//for (int C = 0; C < ConcCount; C++) {
	//	TConceptId Id = Trends->GetTopConceptId(C);
	//	// was the concept hidden by the admin?
	//	if (IsHidden(Id) || !NewsBase->ConceptStore->IsRecId((uint64) Id))
	//		continue;
	//	// do we want concepts of this type?
	//	TStr Type = NewsBase->ConceptStore->GetFieldNmStr((uint64) Id, "Type");
	//	if (!NewsConst.IsValidType(Type, ValidTypes))
	//		continue;

	//	const TStr ConceptUri = NewsBase->ConceptStore->GetRecNm((uint64) Id);
	//	PJsonVal ConceptTrendJson = ConceptTrend(Id, MonthlyTrendV);
	//	// send also info about test and null population frequencies so that we can see what kind of concepts we get as most popular
	//	ConceptTrendJson->AddToObj("testPopFq", TestPop->GetConceptPop(Id));
	//	ConceptTrendJson->AddToObj("nullPopFq", NullPop->GetConceptPop(Id));
	//	if (C < 10)
	//		InfoLogger->NotifyInfo("Top concept %d: testFq: %d, nullFq: %d, chi: %.3f, uri: %s", C, TestPop->GetConceptPop(Id), NullPop->GetConceptPop(Id), Trends->GetChiForConcept(Id, 0), NewsBase->ConceptStore->GetRecNm((uint64) Id).CStr());
	//	ArrJson->AddToArr(ConceptTrendJson);
	//	Added++;
	//	if (Added >= ConceptCount)
	//		break;
	//}
	//InfoLogger->NotifyInfo("Computed a set of %d top trending concepts", ArrJson->GetArrVals());
	//TopTrendsJson = TJsonVal::NewObj("trends", ArrJson);
	//// todo: maybe add some other info, if needed
	//TNotify::StdNotify->OnStatus("Daily trends updated.");
	return TJsonVal::NewObj("info", TStr::Fmt("Trends were updated. Trends contain %d concepts.", ConcCount));
}

// ///////////////////////
// TConceptDailyTrends
// ///////////////////////
PDailyTrendInfos TDailyTrendInfos::New(TNewsBase* NewsBase, const TStr& FName) 
{
	return new TDailyTrendInfos(NewsBase, FName);
}

TDailyTrendInfos::TDailyTrendInfos(TNewsBase* _NewsBase, const TStr& _FName) : NewsBase(_NewsBase), FName(_FName)
{
	InfoLogger = TLogger::New(3);
	InfoLogger->AddLogger(new TFileNotify("./Logs/Log-ConceptDailyTrends.txt", true, true));

	TrendsManager = NewsBase->TrendsManager;
}

//void TConceptDailyTrends::ClearCache() 
//{ 
//	for (int KeyId = InfoH.FFirstKeyId(); InfoH.FNextKeyId(KeyId);)
//		InfoH[KeyId]->ClearCache();
//}

PJsonVal TDailyTrendInfos::UpdateTrends()
{
	PJsonVal RetJson = TJsonVal::NewObj();
	for (int KeyId = InfoH.FFirstKeyId(); InfoH.FNextKeyId(KeyId); ) {
		const TStr Name = InfoH.GetKey(KeyId);
		RetJson->AddToObj(Name, InfoH.GetDat(Name)->UpdateTrends());
	}
	return RetJson;
}

bool TDailyTrendInfos::Load()
{
	if (TFile::Exists(FName)) {
		InfoLogger->NotifyInfo("Loading concept daily trends info.");
		TNotify::StdNotify->OnStatus("Loading concept daily trends info...");
		try {
			TFIn FIn(FName);
			HiddenConcepts.Load(FIn);
			EventIdConceptIdPrV.Load(FIn);
			TNotify::StdNotify->OnStatus("Loaded.");
			return true;
		}
		catch (...) {
			TNotify::StdNotify->OnStatus("Exception while loading daily trends.");
		}
	}
	else {
		InfoLogger->NotifyInfo("No concept daily trends data found.");
		TNotify::StdNotify->OnStatus("No concept daily trends data found.");
	}
	return false;
}

void TDailyTrendInfos::Save()
{
	try {
		TFOut FOut(FName);
		HiddenConcepts.Save(FOut);
		EventIdConceptIdPrV.Save(FOut);
	}
	catch (PExcept E) {
		TNotify::StdNotify->OnStatus("TDailyTrendInfos::Save. Error while saving daily trends: " + E->GetMsgStr());
		NewsBase->ErrorLogFile->NotifyErr("TDailyTrendInfos::Save.", E);
	}
}

PDailyTrendInfo TDailyTrendInfos::GetDailyTrendInfo(const TStr& TrendName)
{
	if (InfoH.IsKey(TrendName))
		return InfoH.GetDat(TrendName);
	return NULL;
}

void TDailyTrendInfos::AddTrendParams(const TTrendParams& TrendParams, const TStr& TrendName)
{
	if (TrendName == "") {
		TNotify::StdNotify->OnStatus("TDailyTrendInfos::AddTrendParams: Unable to add trend params - empty trend name received. Provide a valid name");
		return;
	}
	InfoH.AddDat(TrendName, TDailyTrendInfo::New(TrendsManager, TrendParams, InfoLogger));
}

//
// methods for settings the featured entities and events
// 
void TDailyTrendInfos::SetFeaturedEventsAndConcepts(const TVec<TPair<TUInt64, TUInt64> > & _EventIdConceptIdPrV)
{
	EventIdConceptIdPrV = TVec<TPair<TUInt64, TUInt64> >(_EventIdConceptIdPrV);
}


// return the list of top concepts that are of the desired type
void TDailyTrendInfos::GetTopConceptsByType(const TStr& TrendName, const EnumConceptType& ConceptType, const int& ConceptCount, TUInt64V& TopConceptV)
{
	TopConceptV.Clr();
	PDailyTrendInfo Info = GetDailyTrendInfo(TrendName);
	if (Info.Empty() || Info->Trends.Empty())
		return;
		
	const int ConcCount = Info->Trends->Len();
	for (int C = 0; C < ConcCount && TopConceptV.Len() < ConceptCount; C++) {
		TConceptId Id = Info->Trends->GetTopConceptId(C);
		// was the concept hidden by the admin?
		if (IsHidden(Id) || !NewsBase->ConceptStore->IsRecId((uint64) Id))
			continue;
		// do we want to ignore concepts of this type?
		TStr Type = NewsBase->ConceptStore->GetFieldNmStr((uint64) Id, "Type");
		if (!NewsConst.IsValidType(Type, ConceptType))
			continue;
		TopConceptV.Add((uint64) Id);
	}
}

// find the relevance of individual concepts in conceptV and the choose ConceptCount top concepts and return them in TopConceptV
void TDailyTrendInfos::GetTopConceptsFromList(const TStr& TrendName, const TUInt64V& ConceptV, const int& ConceptCount, TUInt64V& TopConceptV)
{
	TopConceptV.Clr();
	PDailyTrendInfo Info = GetDailyTrendInfo(TrendName);
	if (Info.Empty() || Info->Trends.Empty())
		return;

	// find trend values for specified concepts
	THash<TUInt64, TFlt> ConceptToTrendH;
	for (int N = 0; N < ConceptV.Len(); N++) {
		// debug:
		const uint64 ConceptId = (uint64) ConceptV[N];
		const TStr ConceptUri = NewsBase->ConceptStore->GetRecNm(ConceptId);

		const float Chi = Info->Trends->GetChiForConcept((TConceptId) ConceptV[N]);
		ConceptToTrendH.AddDat(ConceptV[N], Chi);
	}

	// sort by trend
	ConceptToTrendH.SortByDat(false);
	// return top ConceptCount concepts
	for (int KeyId = ConceptToTrendH.FFirstKeyId(); ConceptToTrendH.FNextKeyId(KeyId) && TopConceptV.Len() < ConceptCount;) {
		TConceptId Id = TConceptId(ConceptToTrendH.GetKey(KeyId));
		if (IsHidden(Id))
			continue;
		TopConceptV.Add((uint64) Id);
	}
}