#ifndef CONCEPTDAILYTRENDINFO
#define CONCEPTDAILYTRENDINFO

#include <base.h>
#include "logger.h"
#include "ConceptTrendsManager.h"
#include "../NewsMiner/NewsConst.h"

// forward declaration
class TNewsBase;

class TTrendParams
{
public:
	TStr TestPop;
	TStr NullPop;
	TInt IgnoreTestConceptsBelowFq;
	TInt IgnoreNullConceptsBelowFq;

	TTrendParams() {
		TestPop = "D0,D1";
		NullPop = "D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15";
		IgnoreTestConceptsBelowFq = 10;
		IgnoreNullConceptsBelowFq = 10;
	}
};

// concept trend info for particular set of parameters
class TDailyTrendInfo;
typedef TPt<TDailyTrendInfo> PDailyTrendInfo;

class TDailyTrendInfo {
public:
	friend class TPt<TDailyTrendInfo>;

	PConceptPop TestPop;
	PConceptPop NullPop;
	PConceptTrends Trends;			// trends computed by comparing train and test populations

private:
	TCRef CRef;

	PLogger InfoLogger;
	//TNewsBase* NewsBase;
	PConceptTrendsManager TrendsManager;
	TTrendParams TrendParams;
	PJsonVal TopTrendsJson;			// top trending concepts for different TrendName

	TDailyTrendInfo(const PConceptTrendsManager& _TrendsManager, const TTrendParams& _TrendParams, PLogger _InfoLogger) :
		TrendsManager(_TrendsManager), TrendParams(_TrendParams), InfoLogger(_InfoLogger) { 
		UpdateTrends();
	}

public:
	static PDailyTrendInfo New(PConceptTrendsManager& TrendsManager, const TTrendParams& TrendParams, PLogger InfoLogger) {
		return new TDailyTrendInfo(TrendsManager, TrendParams, InfoLogger);
	}
		
	PJsonVal UpdateTrends();

	//void ClearCache() { TopTrendsJson.Clr(); MonthlyTrendV.Clr(); }
};


// concept trend info for different sets of parameters
class TDailyTrendInfos;
typedef TPt<TDailyTrendInfos> PDailyTrendInfos;

class TDailyTrendInfos {
public:
	friend class TPt<TDailyTrendInfos>;

private:
	TCRef CRef;

	PLogger InfoLogger;
	TNewsBase* NewsBase;
	PConceptTrendsManager TrendsManager;
	TStr FName;
	THash<TStr, PDailyTrendInfo> InfoH;
	
	THashSet<TConceptId> HiddenConcepts;
	TVec<TPair<TUInt64, TUInt64> > EventIdConceptIdPrV;

	TDailyTrendInfos(TNewsBase* _NewsBase, const TStr& _FName);

public:
	static PDailyTrendInfos New(TNewsBase* NewsBase, const TStr& FName);

	void AddTrendParams(const TTrendParams& TrendParams, const TStr& TrendName = "default");
	bool HasTremdParams(const TStr& TrendName = "default") const { return InfoH.IsKey(TrendName); }
	
	//void ClearCache();
	PJsonVal UpdateTrends();
	
	bool Load();
	void Save();

	void HideConcept(const TConceptId& ConceptId) { HiddenConcepts.AddKey(ConceptId); }
	void ShowConcept(const TConceptId& ConceptId) { HiddenConcepts.DelIfKey(ConceptId); }
	bool IsHidden(const TConceptId& ConceptId) const { return HiddenConcepts.IsKey(ConceptId); }
	void GetHiddenConcepts(THashSet<TConceptId>& ConceptIdH) const { TVec<TConceptId> IdV; HiddenConcepts.GetKeyV(IdV); ConceptIdH.AddKeyV(IdV); }

	// methods for settings the featured entities and events
	void SetFeaturedEventsAndConcepts(const TVec<TPair<TUInt64, TUInt64> > & EventIdConceptIdPrV);
	void GetFeaturedEventsAndConcepts(TVec<TPair<TUInt64, TUInt64> >& _EventIdConceptIdPrV) { _EventIdConceptIdPrV = EventIdConceptIdPrV; }
	PJsonVal GetFeaturedEventsAndConcepts(const TStrV& LabelLanguageV);

	PDailyTrendInfo GetDailyTrendInfo(const TStr& TrendName);
	void GetTopConceptsByType(const TStr& TrendName, const EnumConceptType& ConceptType, const int& _ConceptCount, TUInt64V& TopConceptV);
	void GetTopConceptsFromList(const TStr& TrendName, const TUInt64V& ConceptV, const int& ConceptCount, TUInt64V& TopConceptV);
};



#endif