#ifndef CONCEPTDAILYTRENDS_H
#define CONCEPTDAILYTRENDS_H

#include <base.h>
#include <net.h>
#include "../NewsMiner/NewsBase.h"
#include "../NewsMiner/NewsSearch.h"
#include "ConceptTrendsManager.h"
#include "../NewsMiner/NewsSrvUtil.h"

// concept day trending class can display top X concepts (of chosen type) and their popularity across last X days
// it also provides methods to ignore/unignore specific concepts 
// note: if you just need to know which are the trending concepts for a specifc date then check the TConceptTrendsSrv class
class TConceptDailyTrends;
typedef TPt<TConceptDailyTrends> PConceptDailyTrends;
class TConceptDailyTrends : TSAppSrvFun{ 
public:
	friend class TPt<TConceptDailyTrends>;

private:
	TCRef CRef;

	PNewsBase NewsBase;
	PNewsSearch NewsSearch;
	PConceptTrendsManager TrendsManager;
	PNewsBaseFix NewsBaseFix;
	PNewsSrvUtil NewsSrvUtil;
	TStr FName;
	TStr LabelLang;

	THashSet<TConceptId> HiddenConcepts;
	TVec<TPair<TUInt64, TUInt64> > EventIdConceptIdPrV;
		
	PRecSet ArticleRecSet;
	TTm ArticleRecSetTimeStamp;

	TStrKdV ParamsPrV;
	// cached values	
	PJsonVal TopTrendsJson;			// top trending concepts for widths=1,2,3
	//TConceptTrendsV DailyTrendV;	// vec is for time dist. = 1,2,...14
	TConceptTrendsV MonthlyTrendV;	// trends of concepts over the whole month. len = 30
	PConceptTrends Trends;			// trends computed by comparing D0,D1 vs. M0
	PConceptPop TestPop;
	PConceptPop NullPop;
	//TStrSet IgnoreConceptTypeH;
	EnumConceptType ValidTypes;

	PLogger InfoLogger;

	TConceptDailyTrends(const PNewsBase& _NewsBase, const PNewsSearch& _NewsSearch, const PNewsSrvUtil& _NewsSrvUtil, const TStr& _FName);

	~TConceptDailyTrends() { Save(); }
	void Load();
	void Save() const;

public:
	static PConceptDailyTrends New(const PNewsBase& NewsBase, const PNewsSearch& NewsSearch, const PNewsSrvUtil& NewsSrvUtil, const TStr& FName) {
		return new TConceptDailyTrends(NewsBase, NewsSearch, NewsSrvUtil, FName);
	}

	// utils
	PNewsSrvUtil GetNewsSrvUtil() { return NewsSrvUtil; }
	TStr GetLabelLang() const { return LabelLang; }
	void HideConcept(const TConceptId& ConceptId) { HiddenConcepts.AddKey(ConceptId); }
	void ShowConcept(const TConceptId& ConceptId) { HiddenConcepts.DelIfKey(ConceptId); }
	bool IsHidden(const TConceptId& ConceptId) const { return HiddenConcepts.IsKey(ConceptId); }
	void GetHiddenConcepts(THashSet<TConceptId>& ConceptIdH) const { TVec<TConceptId> IdV; HiddenConcepts.GetKeyV(IdV); ConceptIdH.AddKeyV(IdV); }
	
	PConceptTrendsManager GetTrendsManager() { return TrendsManager; }
	
	void SetCustomParams(const TStr& TestPopOffs, const TStr& NullPopOffs, const int& IgnoreTestConceptsBelowFq = 100, const int& IgnoreNullConceptsBelowFq = 100, const int& RetCount = 30, const TStr& ConceptTypes = NewsConst.TypePerson + "," + NewsConst.TypeLocation + "," + NewsConst.TypeOrganization);
	void SetParams(const TStrKdV& _ParamsPrV) { ParamsPrV = _ParamsPrV; }
	bool HasParams() const { return ParamsPrV.Len() > 0; }
	PJsonVal UpdateTrends();
	bool HasTrends() const { return ! Trends.Empty(); }
	PConceptTrends GetTrends() const { return Trends; }
	TConceptTrendsV GetMonthlyTrendV() { return MonthlyTrendV; }
	
	void ClearCache() { TopTrendsJson.Clr(); MonthlyTrendV.Clr(); }

	void GetTopConceptsByType(const EnumConceptType& ConceptType, const int& ConceptCount, TUInt64V& TopConceptV);
	void GetTopConceptsFromList(const TUInt64V& ConceptIdV, const int& ConceptCount, TUInt64V& TopConceptV);

	
	PJsonVal GetConceptTrendArray(const uint64& ConceptRecId, const TConceptTrendsV& TrendV) const;
	// get trend over the last X days for a list of concepts
	PJsonVal ConceptTrends(const TUInt64V& ConceptIdV, const int& NrOfDays = 15);
	
	PJsonVal GetConceptTrends(const PRecSet& ConceptSet, const int& RelatedConceptCount = 20, const THashSet<TConceptId> IgnoreConceptH = THashSet<TConceptId>(), const EnumConceptType& ValidTypes = ctEntities) const;

	// get trend over the last X days for a concept
	PJsonVal ConceptTrend(const uint64& ConceptId, const TConceptTrendsV& ConceptTrendsV) const;

	// get the top trending concepts for the recent period including their trending info for the past days
	PJsonVal GetTopTrends() { return TopTrendsJson; }
	bool HasTopTrends() { return !TopTrendsJson.Empty(); }
	void GetTopTrends(const EnumConceptType& ValidTypes, const int& ConceptCount, TUInt64V& TopConceptV);
	// get articles for top trending concepts
	PRecSet GetArticlesForConcepts(const TUInt64V& ConceptV);
	PJsonVal GetArticles(const PRecSet& ArticleSet, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const int& Page = 0, const int& Count = 20);

	PJsonVal GetConceptInfo(const uint64& ConceptRecId, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const bool& IncludeDesc = true, const bool& IncludeImg = true) const;
	
	PJsonVal GetRelatedConceptsFromLatestArticles(const uint64& ConceptRecId, const int& RelatedConceptCount = 20, const int& LatestArticles = 500) const;
	PJsonVal GetRelatedConcepts(const PRecSet& ArticleRecSet, const int& RelatedConceptCount = 20, const THashSet<TConceptId> IgnoreConceptH = THashSet<TConceptId>(), const EnumConceptType& ValidTypes = ctEntities) const;

	PJsonVal GetConceptEvents(const uint64& ConceptRecId, const int& Page, const int& Count, const TStr& EventLang, const int& MinConceptScore, const int& MaxDaysBack = TInt::Mx, const int& MaxDaysAhead = TInt::Mx) const;
	PJsonVal GetConceptStories(const uint64& ConceptRecId, const int& Page, const int& Count, const int& MinConceptScore) const;
	PJsonVal GetConceptArticles(const uint64& ConceptRecId, const int& Page, const int& Count, const int& MinConceptScore) const;

	PJsonVal GetEvent(const uint64& EventRecId, const int& TopConcepts, const TStr& EventLang, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumConceptType& ConceptTypes = ctEntities) const;
	PJsonVal GetStory(const uint64& StoryRecId, const int& TopConcepts = 0, const TStrV& LabelLanguageV = NewsConst.LangCoreV, const EnumConceptType& ConceptTypes = ctEntities) const;
	PJsonVal GetStoryArticles(const uint64& StoryRecId, const int& Page = 0, const int& Count = 20) const;
	PJsonVal GetStoryArticles(const PRecSet& ArticleRecSet, const int& Page = 0, const int& Count = 20) const;

	PJsonVal GetConceptDailyFreq(const TUInt64V& ConceptIdV, const TStrV& LabelLanguageV = NewsConst.LangCoreV);

	// helper functions
	PRecSet GetNonDuplArticles(const PRecSet& ArtSet, const int& Page, const int& Count) const;
	
	TUInt64V GetValidConcepts(const PRecSet& ConceptRecSet, const int& Count, const EnumConceptType& ConceptTypes) const;

	// methods for settings the featured entities and events
	void SetFeaturedEventsAndConcepts(const TVec<TPair<TUInt64, TUInt64> > & EventIdConceptIdPrV);
	PJsonVal GetFeaturedEventsAndConcepts();
};


#endif