#ifndef ITERATIVEKMEANS
#define ITERATIVEKMEANS

#include <base.h>
#include <mine.h>

class TClusterInfo
{
public:
	// note: DIdV and DIdSimV contain the same lists of documents. the latter just contains additional info about sim to centroid
	TIntV DIdV;					// documents assigned to this cluster
	TVec<TIntFltKd> DIdSimV;	// vector of (DId, Sim to centroid)
	TVec<TClusterInfo> SubClusterV;
	PBowKWordSet KWordSet;

	// find all keywords in all subclusters in Info
	TStrSet GetAllKeywords() const;
};

class IterativeKMeans
{
private:
	PBowDocBs BowDocBs;
	PBowDocWgtBs BowDocWgtBs;
	int MinClusterSizeToSplit;
	int K;
	int ClustTrials;
	PBowSim BowSim;
	PNotify Notify;

	TClusterInfo ClusterInfo;

	void SplitDocsIntoClusters(const int& MaxDepth, TClusterInfo& Info, const TIntV& RemainingDIdV);
	void ComputeKeywords(const int& MxKeywords, const int& EventsUsedForKeywords, TClusterInfo& Info);

public:
	IterativeKMeans(const PBowDocBs& BowDocBs, const PBowDocWgtBs& BowDocWgtBs, const int& SampleWhenDocsAboveCnt = 10000, const int& K = 2, const int& ClustTrials = 5, const int& MinClusterSizeToSplit = 10, const int& MaxDepth = 5, const TBowSimType SimType = bstCos);
	
	const TClusterInfo& GetClusterInfo() { return ClusterInfo; }
	
	void ComputeKeywords(const int& MxKeywords, const int& EventsUsedForKeywords);
	
	
};

#endif