/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "JsonUtils.h"

PJsonVal JsonUtils::GetJsonForTIntHashSet(const THashSet<TInt>& HashSet)
{
	TChA Arr;
	int KeyId = HashSet.FFirstKeyId();
	while (HashSet.FNextKeyId(KeyId))
		Arr += HashSet.GetKey(KeyId).GetStr() + ",";
	if (Arr.Len() > 0)
		Arr.DelLastCh();
	return TJsonVal::NewStr(Arr);
}


//PJsonVal JsonUtils::GetJsonForBowSpV(const PBowSpV& BowSpV)
//{
//	TChA Arr;
//	for (int N=0; N<BowSpV->Len(); N++) {
//		TInt WId = BowSpV->GetWId(N);
//		TFlt Wgt = BowSpV->GetWgt(N);
//		Arr += WId.GetStr() + "," + Wgt.GetStr() + ",";
//	}
//	if (Arr.Len() > 0)
//		Arr.DelLastCh();
//	return TJsonVal::NewStr(Arr);
//}

PJsonVal JsonUtils::GetJsonForBowWgts(const PBowSpV& BowSpV)
{
	TFltV Arr;
	for (int N=0; N<BowSpV->Len(); N++) {
		TFlt Wgt = (double) BowSpV->GetWgt(N);
		Arr.Add(Wgt);
	}
	return TJsonVal::NewArr(Arr);
}


PJsonVal JsonUtils::GetJsonForTUInt64V(const TUInt64V& Vec)
{
	TChA Arr;
	for (int N=0; N < Vec.Len(); N++)
		Arr += Vec[N].GetStr() + ",";
	if (Arr.Len() > 0)
		Arr.DelLastCh();
	return TJsonVal::NewStr(Arr);
}

THashSet<TInt> JsonUtils::GetTIntHashSetForJson(const TStr& Text)
{
	THashSet<TInt> DataH;
	TStrV TokensV; Text.SplitOnAllCh(',', TokensV);
	for (int N=0; N < TokensV.Len(); N++) {
		TInt Val = TokensV[N].GetInt(TInt::Mx);
		if (Val != TInt::Mx)
			DataH.AddKey(Val);
	}
	return DataH;
}

//PBowSpV JsonUtils::GetBowSpVForJson(const TStr& Text)
//{
//	TIntFltKdV SpV;
//	TStrV TokensV; Text.SplitOnAllCh(',', TokensV);
//	for (int N=0; N < TokensV.Len() -1; N = N + 2) {
//		TInt WId = TokensV[N].GetInt(-1);
//		TFlt Wgt = TokensV[N+1].GetFlt(0);
//		if (WId != -1)
//			SpV.Add(TIntFltKd(WId, Wgt));
//	}
//	PBowSpV BowSpV = TBowSpV::New(-1, SpV);
//	return BowSpV;
//}

TFltV JsonUtils::GetTFltVForWgts(const PJsonVal& JsonData)
{

	TFltV WgtV;
	if (JsonData->IsArr()) {
		int Count = JsonData->GetArrVals();
		for (int N=0; N < Count; N++) {
			TFlt Wgt = JsonData->GetArrVal(N)->GetNum();
			WgtV.Add(Wgt);
		}
	}
	return WgtV;
}


TUInt64V JsonUtils::GetTUInt64VForJson(const TStr& Text)
{
	TUInt64V DataV;
	TStrV TokensV; Text.SplitOnAllCh(',', TokensV);
	for (int N=0; N < TokensV.Len(); N++) {
		TUInt64 Val = TokensV[N].GetUInt64(TUInt64::Mx);
		if (Val != TUInt64::Mx)
			DataV.Add(Val);
	}
	return DataV;
}

