/**
 * Author: Gregor Leban <gleban@gmail.com>
 * Copyright (c) 2015, Jozef Stefan Institute, Quintelligence d.o.o. and contributors
 * All rights reserved.
 *
 * This source code is licensed under the FreeBSD license found in the
 * LICENSE file in the root directory of this source tree.
 */
#include "ValidatedArticleLinks.h"
#include "../NewsMiner/NewsBase.h"

TValidatedArticleLinks::TValidatedArticleLinks(const TStr& _FName)
{
	FName = _FName;
}

TValidatedArticleLinks::~TValidatedArticleLinks()
{
	Save();
}


void TValidatedArticleLinks::Load()
{
	ArticleToStoryIdH.Clr();
	try {
		if (TFile::Exists(FName)) {
			TFIn FIn(FName);
			ArticleToStoryIdH.Load(FIn);
		}
	}
	catch (...) {
		TNotify::StdNotify->OnStatus("TValidatedArticleLinks::Load. Error while saving ValidatedStoryLinks.bin.");
	}
}

void TValidatedArticleLinks::Save() const
{
	try {
		TFOut FOut(FName);
		ArticleToStoryIdH.Save(FOut);
	}
	catch (...) {
		TNotify::StdNotify->OnStatus("TValidatedArticleLinks::Save. Error while saving ValidatedStoryLinks.bin.");
	}
}

// save information about all same and different stories into a json file
void TValidatedArticleLinks::SaveToJson(const TStr& OutFNm, const PNewsBase& NewsBase) const
{
	TFOut FOut(OutFNm);
	PStore ArticleStore = NewsBase->ArticleStore;
	for (int KeyId = ArticleToStoryIdH.FFirstKeyId(); ArticleToStoryIdH.FNextKeyId(KeyId);) {
		const uint64 ArticleId = ArticleToStoryIdH.GetKey(KeyId);
		const TUInt64 StoryId = ArticleToStoryIdH[KeyId];
		PJsonVal Json = TJsonVal::NewObj("articleUri", ArticleStore->GetRecNm(ArticleId));
		Json->AddToObj("storyId", StoryId.GetStr());
		FOut.PutStrLn(TJsonVal::GetStrFromVal(Json));
	}
}

// load the same and diff hash tables from json - used when we do data restoration
void TValidatedArticleLinks::LoadFromJson(const TStr& InFNm, const PNewsBase& NewsBase)
{
	if (TFile::Exists(InFNm)) {
		PStore ArticleStore = NewsBase->ArticleStore;
		PStore StoryStore = NewsBase->StoryStore;
		ArticleToStoryIdH.Clr();
		TFIn FIn(InFNm);
		TChA LnA;
		while (FIn.GetNextLn(LnA)) {
			PJsonVal Json = TJsonVal::GetValFromStr(LnA);
			const TStr ArticleUri = Json->GetObjStr("articleUri");
			const uint64 StoryId = Json->GetObjStr("storyId", "").GetUInt64(TUInt64::Mx);
			if (!ArticleStore->IsRecNm(ArticleUri) || !StoryStore->IsRecId(StoryId))
				continue;
			const uint64 ArticleId = ArticleStore->GetRecId(ArticleUri);
			ArticleToStoryIdH.AddDat(ArticleId, StoryId);
			if (StoryStore->IsRecId(StoryId))
				NewsBase->AddStory(ArticleId, StoryStore->GetRecNm(StoryId));
		}
	}
	else
		TNotify::StdNotify->OnStatusFmt("TValidatedArticleLinks: Unable to laod json file: %s. File does not exist.", InFNm.CStr());
}

// set story for article - once done, the clustering will not be able to modify the story id for it anymore
void TValidatedArticleLinks::SetArticleToStoryLink(const uint64& ArticleId, const uint64& StoryId)
{
	ArticleToStoryIdH.AddDat(ArticleId, StoryId);
}

// do we have a custom info for a given article
bool TValidatedArticleLinks::HasStoryForArticle(const uint64& ArticleId) const
{
	return ArticleToStoryIdH.IsKey(ArticleId);
}